@@ -1,7 +1,7 @@
 /* capture_loop.c
  * The actual capturing loop, getting packets and storing it
  *
- * $Id: capture_loop.c 13021 2005-01-13 20:19:49Z ulfl $
+ * $Id: capture_loop.c 14241 2005-04-30 15:39:40Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -72,7 +72,6 @@
 #include "pcap-util.h"
 
 #include "simple_dialog.h"
-#include "globals.h"
 #include "conditions.h"
 #include "capture_stop_conditions.h"
 #include "ringbuffer.h"
@@ -85,6 +84,8 @@
 /* XXX - try to remove this later */
 #include "ui_util.h"
 /* XXX - try to remove this later */
+#include "util.h"
+#include "alert_box.h"
 
 
 #include <epan/dissectors/packet-ap1394.h>
@@ -93,6 +94,7 @@
 #include <epan/dissectors/packet-clip.h>
 #include <epan/dissectors/packet-eth.h>
 #include <epan/dissectors/packet-fddi.h>
+#include <epan/dissectors/packet-fr.h>
 #include <epan/dissectors/packet-null.h>
 #include <epan/dissectors/packet-ppp.h>
 #include <epan/dissectors/packet-raw.h>
@@ -104,6 +106,10 @@
 #include <epan/dissectors/packet-ipfc.h>
 #include <epan/dissectors/packet-arcnet.h>
 
+/* Win32 needs the O_BINARY flag for open() */
+#ifndef O_BINARY
+#define O_BINARY	0
+#endif
 
 
 
@@ -559,14 +565,14 @@ static int capture_loop_open_input(capture_options *capture_opts, loop_data *ld,
      if they succeed; to tell if that's happened, we have to clear
      the error buffer, and check if it's still a null string.  */
   open_err_str[0] = '\0';
-  ld->pcap_h = pcap_open_live(cfile.iface,
+  ld->pcap_h = pcap_open_live(capture_opts->iface,
 		       capture_opts->has_snaplen ? capture_opts->snaplen :
 						  WTAP_MAX_PACKET_SIZE,
 		       capture_opts->promisc_mode, CAP_READ_TIMEOUT,
 		       open_err_str);
 
   if (ld->pcap_h != NULL) {
-    /* we've opened "cfile.iface" as a network device */
+    /* we've opened "iface" as a network device */
 #ifdef _WIN32
     /* try to set the capture buffer size */
     if (pcap_setbuff(ld->pcap_h, capture_opts->buffer_size * 1024 * 1024) != 0) {
@@ -583,7 +589,7 @@ static int capture_loop_open_input(capture_options *capture_opts, loop_data *ld,
 
     /* setting the data link type only works on real interfaces */
     if (capture_opts->linktype != -1) {
-      set_linktype_err_str = set_pcap_linktype(ld->pcap_h, cfile.iface,
+      set_linktype_err_str = set_pcap_linktype(ld->pcap_h, capture_opts->iface,
 	capture_opts->linktype);
       if (set_linktype_err_str != NULL) {
 	g_snprintf(errmsg, errmsg_len, "Unable to set data link type (%s).",
@@ -592,46 +598,38 @@ static int capture_loop_open_input(capture_options *capture_opts, loop_data *ld,
       }
     }
   } else {
-    /* We couldn't open "cfile.iface" as a network device. */
+    /* We couldn't open "iface" as a network device. */
 #ifdef _WIN32
-    /* On Windows, we don't support capturing on pipes, so we give up.
-       If this is a child process that does the capturing in sync
-       mode or fork mode, it shouldn't do any UI stuff until we pop up the
-       capture-progress window, and, since we couldn't start the
-       capture, we haven't popped it up. */
-    if (!capture_opts->capture_child) {
-      main_window_update();
-    }
+    /* On Windows, we don't support capturing on pipes, so we give up. */
 
     /* On Win32 OSes, the capture devices are probably available to all
        users; don't warn about permissions problems.
 
        Do, however, warn that WAN devices aren't supported. */
     g_snprintf(errmsg, errmsg_len,
-	"The capture session could not be initiated (%s).\n"
-	"Please check that you have the proper interface specified.\n"
-	"\n"
-	"Note that the WinPcap 2.x version of the driver Ethereal uses for packet\n"
-	"capture on Windows doesn't support capturing on PPP/WAN interfaces in\n"
-	"Windows NT/2000/XP/2003 Server, and that the WinPcap 3.0 and later versions\n"
-	"don't support capturing on PPP/WAN interfaces at all.",
-	open_err_str);
+"%sThe capture session could not be initiated!%s\n"
+"\n"
+"(%s)\n"
+"\n"
+"Please check that you have the proper interface specified.\n"
+"\n"
+"Note that version 3.0 of WinPcap, and earlier versions of WinPcap, don't\n"
+"support capturing on PPP/WAN interfaces in Windows NT 4.0 / 2000 / XP / Server 2003.\n"
+"\n"
+"WinPcap 3.1 has experimental support for it on Windows 2000 / XP / Server 2003,\n"
+"but has no support for it on Windows NT 4.0.  WinPcap 3.1 is currently in beta,\n"
+"so using it might introduce bugs not present in WinPcap 3.0; you should report\n"
+"all problems you see to the WinPcap developers, so they can try to fix\n"
+"them before the final WinPcap 3.1 release.",
+	simple_dialog_primary_start(), simple_dialog_primary_end(),
+    open_err_str);
     return FALSE;
 #else
-    /* try to open cfile.iface as a pipe */
-    ld->cap_pipe_fd = cap_pipe_open_live(cfile.iface, &ld->cap_pipe_hdr, ld, errmsg, errmsg_len);
+    /* try to open iface as a pipe */
+    ld->cap_pipe_fd = cap_pipe_open_live(capture_opts->iface, &ld->cap_pipe_hdr, ld, errmsg, errmsg_len);
 
     if (ld->cap_pipe_fd == -1) {
 
-      /* If this is a child process that does the capturing in sync
-       * mode or fork mode, it shouldn't do any UI stuff until we pop up the
-       * capture-progress window, and, since we couldn't start the
-       * capture, we haven't popped it up.
-       */
-      if (!capture_opts->capture_child) {
-          main_window_update();
-      }
-
       if (ld->cap_pipe_err == PIPNEXIST) {
 	/* Pipe doesn't exist, so output message for interface */
 
@@ -709,15 +707,15 @@ static void capture_loop_close_input(loop_data *ld) {
 
 
 /* init the capture filter */
-static int capture_loop_init_filter(loop_data *ld, char *errmsg, int errmsg_len) {
+static int capture_loop_init_filter(loop_data *ld, const gchar * iface, gchar * cfilter, char *errmsg, int errmsg_len) {
   bpf_u_int32 netnum, netmask;
   gchar       lookup_net_err_str[PCAP_ERRBUF_SIZE];
   struct bpf_program fcode;
 
   /* capture filters only work on real interfaces */
-  if (cfile.cfilter && !ld->from_cap_pipe) {
+  if (cfilter && !ld->from_cap_pipe) {
     /* A capture filter was specified; set it up. */
-    if (pcap_lookupnet(cfile.iface, &netnum, &netmask, lookup_net_err_str) < 0) {
+    if (pcap_lookupnet(iface, &netnum, &netmask, lookup_net_err_str) < 0) {
       /*
        * Well, we can't get the netmask for this interface; it's used
        * only for filters that check for broadcast IP addresses, so
@@ -729,10 +727,10 @@ static int capture_loop_init_filter(loop_data *ld, char *errmsg, int errmsg_len)
        */
       netmask = 0;
     }
-    if (pcap_compile(ld->pcap_h, &fcode, cfile.cfilter, 1, netmask) < 0) {
+    if (pcap_compile(ld->pcap_h, &fcode, cfilter, 1, netmask) < 0) {
       dfilter_t   *rfcode = NULL;
       /* filter string invalid, did the user tried a display filter? */
-      if (dfilter_compile(cfile.cfilter, &rfcode) && rfcode != NULL) {
+      if (dfilter_compile(cfilter, &rfcode) && rfcode != NULL) {
         g_snprintf(errmsg, errmsg_len,
           "%sInvalid capture filter: \"%s\"!%s\n"
           "\n"
@@ -743,7 +741,7 @@ static int capture_loop_init_filter(loop_data *ld, char *errmsg, int errmsg_len)
           "so you can't use most display filter expressions as capture filters.\n"
           "\n"
           "See the help for a description of the capture filter syntax.",
-          simple_dialog_primary_start(), cfile.cfilter, simple_dialog_primary_end(),
+          simple_dialog_primary_start(), cfilter, simple_dialog_primary_end(),
           pcap_geterr(ld->pcap_h));
 	dfilter_free(rfcode);
       } else {
@@ -752,7 +750,7 @@ static int capture_loop_init_filter(loop_data *ld, char *errmsg, int errmsg_len)
           "\n"
           "That string isn't a valid capture filter (%s).\n"
           "See the help for a description of the capture filter syntax.",
-          simple_dialog_primary_start(), cfile.cfilter, simple_dialog_primary_end(),
+          simple_dialog_primary_start(), cfilter, simple_dialog_primary_end(),
           pcap_geterr(ld->pcap_h));
       }
       return FALSE;
@@ -760,8 +758,14 @@ static int capture_loop_init_filter(loop_data *ld, char *errmsg, int errmsg_len)
     if (pcap_setfilter(ld->pcap_h, &fcode) < 0) {
       g_snprintf(errmsg, errmsg_len, "Can't install filter (%s).",
 	pcap_geterr(ld->pcap_h));
+#ifdef HAVE_PCAP_FREECODE
+      pcap_freecode(&fcode);
+#endif
       return FALSE;
     }
+#ifdef HAVE_PCAP_FREECODE
+    pcap_freecode(&fcode);
+#endif
   }
 
   return TRUE;
@@ -769,7 +773,7 @@ static int capture_loop_init_filter(loop_data *ld, char *errmsg, int errmsg_len)
 
 
 /* open the wiretap part of the capture output file */
-static int capture_loop_open_wiretap_output(capture_options *capture_opts, loop_data *ld, char *errmsg, int errmsg_len) {
+static int capture_loop_open_wiretap_output(capture_options *capture_opts, int save_file_fd, loop_data *ld, char *errmsg, int errmsg_len) {
   int         pcap_encap;
   int         file_snaplen;
   int         err;
@@ -783,7 +787,7 @@ static int capture_loop_open_wiretap_output(capture_options *capture_opts, loop_
   } else
 #endif
   {
-    pcap_encap = get_pcap_linktype(ld->pcap_h, cfile.iface);
+    pcap_encap = get_pcap_linktype(ld->pcap_h, capture_opts->iface);
     file_snaplen = pcap_snapshot(ld->pcap_h);
   }
 
@@ -799,7 +803,7 @@ static int capture_loop_open_wiretap_output(capture_options *capture_opts, loop_
     ld->wtap_pdh = ringbuf_init_wtap_dump_fdopen(WTAP_FILE_PCAP, ld->wtap_linktype,
       file_snaplen, &err);
   } else {
-    ld->wtap_pdh = wtap_dump_fdopen(cfile.save_file_fd, WTAP_FILE_PCAP,
+    ld->wtap_pdh = wtap_dump_fdopen(save_file_fd, WTAP_FILE_PCAP,
       ld->wtap_linktype, file_snaplen, &err);
   }
 
@@ -822,12 +826,12 @@ static int capture_loop_open_wiretap_output(capture_options *capture_opts, loop_
         g_snprintf(errmsg, errmsg_len,
 		     "The file to which the capture would be"
                      " saved (\"%s\") could not be opened: Error %d.",
- 			cfile.save_file, err);
+ 			capture_opts->save_file, err);
       } else {
         g_snprintf(errmsg, errmsg_len,
 		     "The file to which the capture would be"
                      " saved (\"%s\") could not be opened: %s.",
- 			cfile.save_file, strerror(err));
+ 			capture_opts->save_file, strerror(err));
       }
       break;
     }
@@ -840,7 +844,7 @@ static int capture_loop_open_wiretap_output(capture_options *capture_opts, loop_
 
 static gboolean capture_loop_close_output(capture_options *capture_opts, loop_data *ld, int *err_close) {
   if (capture_opts->multi_files_on) {
-    return ringbuf_wtap_dump_close(&cfile, err_close);
+    return ringbuf_wtap_dump_close(&capture_opts->save_file, err_close);
   } else {
     return wtap_dump_close(ld->wtap_pdh, err_close);
   }
@@ -956,6 +960,93 @@ capture_loop_dispatch(capture_options *capture_opts, loop_data *ld,
 }
 
 
+/* open the output file (temporary/specified name/ringbuffer) */
+/* Returns TRUE if the file opened successfully, FALSE otherwise. */
+static gboolean
+capture_loop_open_output(capture_options *capture_opts, int *save_file_fd,
+		      char *errmsg, int errmsg_len) {
+
+  char tmpname[128+1];
+  gchar *capfile_name;
+  gboolean is_tempfile;
+
+
+  if (capture_opts->save_file != NULL) {
+    /* We return to the caller while the capture is in progress.  
+     * Therefore we need to take a copy of save_file in
+     * case the caller destroys it after we return.
+     */
+    capfile_name = g_strdup(capture_opts->save_file);
+    if (capture_opts->multi_files_on) {
+      /* ringbuffer is enabled */
+      *save_file_fd = ringbuf_init(capfile_name,
+          (capture_opts->has_ring_num_files) ? capture_opts->ring_num_files : 0);
+
+      /* we need the ringbuf name */
+      if(*save_file_fd != -1) {
+          g_free(capfile_name);
+          capfile_name = g_strdup(ringbuf_current_filename());
+      }
+    } else {
+      /* Try to open/create the specified file for use as a capture buffer. */
+      *save_file_fd = open(capfile_name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT,
+				0600);
+    }
+    is_tempfile = FALSE;
+  } else {
+    /* Choose a random name for the temporary capture buffer */
+    *save_file_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
+    capfile_name = g_strdup(tmpname);
+    is_tempfile = TRUE;
+  }
+
+  /* did we fail to open the output file? */
+  if (*save_file_fd == -1) {
+    if (is_tempfile) {
+      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+	"The temporary file to which the capture would be saved (\"%s\") "
+	"could not be opened: %s.", capfile_name, strerror(errno));
+    } else {
+      if (capture_opts->multi_files_on) {
+        ringbuf_error_cleanup();
+      }
+
+      g_snprintf(errmsg, errmsg_len,
+	    "The file to which the capture would be saved (\"%s\") "
+        "could not be opened: %s.", capfile_name, 
+        strerror(errno));
+
+      /*open_failure_alert_box(capfile_name, errno, TRUE);*/
+    }
+    g_free(capfile_name);
+    return FALSE;
+  }
+
+  if(capture_opts->save_file != NULL) {
+    g_free(capture_opts->save_file);
+  }
+  capture_opts->save_file = capfile_name;
+  /* capture_opts.save_file is "g_free"ed later, which is equivalent to
+     "g_free(capfile_name)". */
+
+  return TRUE;
+}
+
+
+#ifndef _WIN32
+static void
+capture_loop_stop_signal_handler(int signo _U_)
+{
+  capture_loop_stop();
+}
+#endif
+
+#ifdef _WIN32
+#define TIME_GET() GetTickCount()
+#else
+#define TIME_GET() time(NULL)
+#endif
+
 /*
  * This needs to be static, so that the SIGUSR1 handler can clear the "go"
  * flag.
@@ -979,8 +1070,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   gboolean    close_ok;
   capture_info   capture_ui;
   char        errmsg[4096+1];
-
-  gboolean    show_info = capture_opts->show_info || !capture_opts->sync_mode;
+  int         save_file_fd;
 
 
   /* init the loop data */
@@ -1014,9 +1104,25 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   ld.pcap_fd            = 0;
 #endif
 
+#ifndef _WIN32
+  /*
+   * Catch SIGUSR1, so that we exit cleanly if the parent process
+   * kills us with it due to the user selecting "Capture->Stop".
+   */
+    signal(SIGUSR1, capture_loop_stop_signal_handler);
+#endif
+
   /* We haven't yet gotten the capture statistics. */
   *stats_known      = FALSE;
 
+  /*g_warning("capture_loop_start");
+  capture_opts_info(capture_opts);*/
+
+
+  /* open the output file (temporary/specified name/ringbuffer) */
+  if (!capture_loop_open_output(capture_opts, &save_file_fd, errmsg, sizeof(errmsg))) {
+    goto error;    
+  }
 
   /* open the "input file" from network interface or capture pipe */
   if (!capture_loop_open_input(capture_opts, &ld, errmsg, sizeof(errmsg))) {
@@ -1024,12 +1130,12 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   }
 
   /* init the input filter from the network interface (capture pipe will do nothing) */
-  if (!capture_loop_init_filter(&ld, errmsg, sizeof(errmsg))) {
+  if (!capture_loop_init_filter(&ld, capture_opts->iface, capture_opts->cfilter, errmsg, sizeof(errmsg))) {
     goto error;
   }
 
   /* open the wiretap part of the output file (the output file is already open) */
-  if (!capture_loop_open_wiretap_output(capture_opts, &ld, errmsg, sizeof(errmsg))) {
+  if (!capture_loop_open_wiretap_output(capture_opts, save_file_fd, &ld, errmsg, sizeof(errmsg))) {
     goto error;
   }
 
@@ -1039,18 +1145,15 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
      in other places as well - and I don't think that works all the
      time in any case, due to libpcap bugs. */
 
-  if (capture_opts->capture_child) {
-    /* Well, we should be able to start capturing.
+  /* Well, we should be able to start capturing.
 
-       This is the child process for a sync mode capture, so sync out
-       the capture file, so the header makes it to the file system,
-       and send a "capture started successfully and capture file created"
-       message to our parent so that they'll open the capture file and
-       update its windows to indicate that we have a live capture in
-       progress. */
-    fflush(wtap_dump_file(ld.wtap_pdh));
-    sync_pipe_capstart_to_parent();
-  }
+     Sync out the capture file, so the header makes it to the file system,
+     and send a "capture started successfully and capture file created"
+     message to our parent so that they'll open the capture file and
+     update its windows to indicate that we have a live capture in
+     progress. */
+  fflush(wtap_dump_file(ld.wtap_pdh));
+  sync_pipe_filename_to_parent(capture_opts->save_file);
 
   /* initialize capture stop (and alike) conditions */
   init_capture_stop_conditions();
@@ -1073,15 +1176,15 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   }
 
   /* start capture info dialog */
-  if(show_info) {
+  if(capture_opts->show_info) {
       capture_ui.callback_data  = &ld;
       capture_ui.counts         = &ld.counts;
-      capture_info_create(&capture_ui, cfile.iface);
+      capture_info_create(&capture_ui, capture_opts->iface);
   }
 
   /* init the time values */
-  start_time = time(NULL);
-  upd_time = time(NULL);
+  start_time = TIME_GET();
+  upd_time = TIME_GET();
 
   /* WOW, everything is prepared! */
   /* please fasten your seat belts, we will enter now the actual capture loop */
@@ -1091,6 +1194,26 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
     /* dispatch incoming packets */
     inpkts = capture_loop_dispatch(capture_opts, &ld, errmsg, sizeof(errmsg));
 
+    main_window_update();
+
+#ifdef _WIN32
+      /* some news from our parent (signal pipe)? -> just stop the capture */
+      {
+          HANDLE handle;
+          DWORD avail = 0;
+          gboolean result;
+
+
+          handle = (HANDLE) _get_osfhandle (0);
+          result = PeekNamedPipe(handle, NULL, 0, NULL, &avail, NULL);
+
+          if(!result || avail > 0) {
+            ld.go = FALSE;
+            /*g_warning("loop closing");*/
+          }
+      }
+#endif
+
     if (inpkts > 0) {
       ld.packets_sync_pipe += inpkts;
 
@@ -1106,12 +1229,14 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
           }
 
           /* Switch to the next ringbuffer file */
-          if (ringbuf_switch_file(&cfile, &ld.wtap_pdh, &ld.err)) {
+          if (ringbuf_switch_file(&ld.wtap_pdh, &capture_opts->save_file, &save_file_fd, &ld.err)) {
             /* File switch succeeded: reset the conditions */
             cnd_reset(cnd_autostop_size);
             if (cnd_file_duration) {
               cnd_reset(cnd_file_duration);
             }
+            fflush(wtap_dump_file(ld.wtap_pdh));
+            sync_pipe_filename_to_parent(capture_opts->save_file);
           } else {
             /* File switch failed: stop here */
             ld.go = FALSE;
@@ -1126,18 +1251,26 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
     } /* inpkts */
 
     /* Only update once a second so as not to overload slow displays */
-    cur_time = time(NULL);
-    if (cur_time > upd_time) {
-      upd_time = cur_time;
+    cur_time = TIME_GET();
+#ifdef _WIN32
+    if ( (cur_time - upd_time) > 500) {
+#else
+    if (cur_time - upd_time > 0) {
+#endif
+        upd_time = cur_time;
 
       /*if (pcap_stats(pch, stats) >= 0) {
         *stats_known = TRUE;
       }*/
 
       /* calculate and display running time */
-      if(show_info) {
+      if(capture_opts->show_info) {
           cur_time -= start_time;
+#ifdef _WIN32
+          capture_ui.running_time   = cur_time / 1000;
+#else
           capture_ui.running_time   = cur_time;
+#endif
           capture_ui.new_packets    = ld.packets_sync_pipe;
           capture_info_update(&capture_ui);
       }
@@ -1147,12 +1280,9 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
         /* do sync here */
         fflush(wtap_dump_file(ld.wtap_pdh));
 
-        if (capture_opts->capture_child) {
-	  /* This is the child process for a sync mode capture, so send
-	     our parent a message saying we've written out "ld.sync_packets"
+	  /* Send our parent a message saying we've written out "ld.sync_packets"
 	     packets to the capture file. */
         sync_pipe_packet_count_to_parent(ld.packets_sync_pipe);
-        }
 
         ld.packets_sync_pipe = 0;
       }
@@ -1175,11 +1305,13 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
           }
 
           /* Switch to the next ringbuffer file */
-          if (ringbuf_switch_file(&cfile, &ld.wtap_pdh, &ld.err)) {
+          if (ringbuf_switch_file(&ld.wtap_pdh, &capture_opts->save_file, &save_file_fd, &ld.err)) {
             /* file switch succeeded: reset the conditions */
             cnd_reset(cnd_file_duration);
             if(cnd_autostop_size)
               cnd_reset(cnd_autostop_size);
+            fflush(wtap_dump_file(ld.wtap_pdh));
+            sync_pipe_filename_to_parent(capture_opts->save_file);
           } else {
             /* File switch failed: stop here */
 	        ld.go = FALSE;
@@ -1196,7 +1328,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   } /* while (ld.go) */
 
   /* close capture info dialog */
-  if(show_info) {
+  if(capture_opts->show_info) {
     capture_info_destroy(&capture_ui);
   }
 
@@ -1225,7 +1357,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   if (ld.err == 0) {
     write_ok = TRUE;
   } else {
-    capture_loop_get_errmsg(errmsg, sizeof(errmsg), cfile.save_file, ld.err,
+    capture_loop_get_errmsg(errmsg, sizeof(errmsg), capture_opts->save_file, ld.err,
 			      FALSE);
     capture_loop_popup_errmsg(capture_opts, errmsg);
     write_ok = FALSE;
@@ -1237,7 +1369,7 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   /* If we've displayed a message about a write error, there's no point
      in displaying another message about an error on close. */
   if (!close_ok && write_ok) {
-    capture_loop_get_errmsg(errmsg, sizeof(errmsg), cfile.save_file, err_close,
+    capture_loop_get_errmsg(errmsg, sizeof(errmsg), capture_opts->save_file, err_close,
 		TRUE);
     capture_loop_popup_errmsg(capture_opts, errmsg);
   }
@@ -1258,10 +1390,8 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
        dropped. */
     if (pcap_stats(ld.pcap_h, stats) >= 0) {
       *stats_known = TRUE;
-      if (capture_opts->capture_child) {
-      	/* Let the parent process know. */
-        sync_pipe_drops_to_parent(stats->ps_drop);
-      }
+      /* Let the parent process know. */
+      sync_pipe_drops_to_parent(stats->ps_drop);
     } else {
       g_snprintf(errmsg, sizeof(errmsg),
 		"Can't get packet-drop statistics: %s",
@@ -1273,6 +1403,8 @@ capture_loop_start(capture_options *capture_opts, gboolean *stats_known, struct
   /* close the input file (pcap or capture pipe) */
   capture_loop_close_input(&ld);
 
+  /*g_warning("loop closed");*/
+
   /* ok, if the write and the close were successful. */
   return write_ok && close_ok;
 
@@ -1283,19 +1415,21 @@ error:
   } else {
     /* We can't use the save file, and we have no wtap_dump stream
        to close in order to close it, so close the FD directly. */
-    close(cfile.save_file_fd);
+    close(save_file_fd);
 
     /* We couldn't even start the capture, so get rid of the capture
        file. */
-    unlink(cfile.save_file); /* silently ignore error */
-    g_free(cfile.save_file);
+    unlink(capture_opts->save_file); /* silently ignore error */
+    g_free(capture_opts->save_file);
   }
-  cfile.save_file = NULL;
+  capture_opts->save_file = NULL;
   capture_loop_popup_errmsg(capture_opts, errmsg);
 
   /* close the input file (pcap or cap_pipe) */
   capture_loop_close_input(&ld);
 
+  /*g_warning("loop error");*/
+
   return FALSE;
 }
 
@@ -1365,17 +1499,11 @@ capture_loop_get_errmsg(char *errmsg, int errmsglen, const char *fname,
 }
 
 static void
-capture_loop_popup_errmsg(capture_options *capture_opts, const char *errmsg)
+capture_loop_popup_errmsg(capture_options *capture_opts _U_, const char *errmsg)
 {
-  if (capture_opts->capture_child) {
-    /* This is the child process for a sync mode capture.
-       Send the error message to our parent, so they can display a
+    /* Send the error message to our parent, so they can display a
        dialog box containing it. */
     sync_pipe_errmsg_to_parent(errmsg);
-  } else {
-    /* Display the dialog box ourselves; there's no parent. */
-    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s", errmsg);
-  }
 }
 
 
@@ -1471,6 +1599,10 @@ capture_loop_packet_cb(guchar *user, const struct pcap_pkthdr *phdr,
     case WTAP_ENCAP_APPLE_IP_OVER_IEEE1394:
       capture_ap1394(pd, 0, whdr.caplen, &ld->counts);
       break;
+    case WTAP_ENCAP_FRELAY:
+    case WTAP_ENCAP_FRELAY_WITH_PHDR:
+      capture_fr(pd, 0, whdr.caplen, &ld->counts);
+      break;
     /* XXX - some ATM drivers on FreeBSD might prepend a 4-byte ATM
        pseudo-header to DLT_ATM_RFC1483, with LLC header following;
        we might have to implement that at some point. */
