@@ -1,7 +1,7 @@
 /* capture_sync.c
  * Synchronisation between Ethereal capture parent and child instances
  *
- * $Id: capture_sync.c 12858 2004-12-29 21:20:42Z guy $
+ * $Id: capture_sync.c 14241 2005-04-30 15:39:40Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -22,10 +22,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-/* With MSVC and a libethereal.dll this file needs to import some variables 
-   in a special way. Therefore _NEED_VAR_IMPORT_ is defined. */
-#define _NEED_VAR_IMPORT_
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -81,6 +77,7 @@
 #include <epan/prefs.h>
 
 #include "globals.h"
+#include "file.h"
 
 #include "capture.h"
 #include "capture_sync.h"
@@ -95,8 +92,6 @@
 # include <io.h>
 #endif
 
-int fork_child = -1;	    /* If not -1, in parent, process ID of child */
-
 #ifdef _WIN32
 #include <process.h>    /* For spawning child process */
 #endif
@@ -113,7 +108,7 @@ static char *sync_pipe_signame(int);
 
 
 static gboolean sync_pipe_input_cb(gint source, gpointer user_data);
-static void sync_pipe_wait_for_child(gboolean always_report);
+static void sync_pipe_wait_for_child(capture_options *capture_opts, gboolean always_report);
 
 /* Size of buffer to hold decimal representation of
    signed/unsigned 64-bit int */
@@ -122,49 +117,159 @@ static void sync_pipe_wait_for_child(gboolean always_report);
 /*
  * Indications sent out on the sync pipe.
  */
-#define SP_CAPSTART	';'	    /* capture start message */
-#define SP_PACKET_COUNT	'*'	/* followed by count of packets captured since last message */
-#define SP_ERROR_MSG	'!'	/* followed by length of error message that follows */
-#define SP_DROPS	'#'	    /* followed by count of packets dropped in capture */
+#define SP_FILE	        'F'	    /* the name of the recently opened file */
+#define SP_ERROR_MSG    'E'     /* error message */
+#define SP_PACKET_COUNT 'P'     /* count of packets captured since last message */
+#define SP_DROPS        'D'	    /* count of packets dropped in capture */
+#define SP_QUIT         'Q'     /* capture quit message (from parent to child) */
 
 
-
-void
-sync_pipe_capstart_to_parent(void)
+/* write a message to the recipient pipe in the standard format 
+   (3 digit message length (excluding length and indicator field), 
+   1 byte message indicator and the rest is the message) */
+static void
+pipe_write_block(int pipe, char indicator, int len, const char *msg)
 {
-    static const char capstart_msg = SP_CAPSTART;
+    char lenbuf[3+1+1]; /* 3 digit len + indicator + zero terminator */
+    int ret;
 
-    write(1, &capstart_msg, 1);
+    /*g_warning("write %d enter", pipe);*/
+
+    g_assert(len < 1000);
+    g_assert(indicator < '0' || indicator > '9');
+
+    /* write header (3 digit len + indicator) */
+    g_snprintf(lenbuf, 5, "%03u%c", len, indicator);
+
+    ret = write(pipe, lenbuf, strlen(lenbuf));
+    if(ret == -1) {
+        return;
+    }
+
+    /* write value (if we have one) */
+    if(len) {
+        /*g_warning("write %d indicator: %c value len: %u msg: %s", pipe, indicator, len, msg);*/
+        ret = write(pipe, msg, len);
+        if(ret == -1) {
+            return;
+        }
+    } else {
+        /*g_warning("write %d indicator: %c no value", pipe, indicator);*/
+    }
+
+    /*g_warning("write %d leave", pipe);*/
+}
+
+
+/* read a message from the sending pipe in the standard format 
+   (3 digit message length (excluding length and indicator field), 
+   1 byte message indicator and the rest is the message) */
+int
+pipe_read_block(int pipe, char *indicator, int len, char *msg) {
+    int required;
+    int newly;
+    char header[4];
+    int offset;
+
+
+    /* read header (3 digit len and indicator) */
+    required = 4;
+    offset = 0;
+    while(required) {
+        newly = read(pipe, &header[offset], required);
+        if (newly == 0) {
+            /* EOF */
+            /*g_warning("read %d header empty (capture closed)", pipe);*/
+            return newly;
+        }
+        if (newly < 0) {
+            /* error */
+            /*g_warning("read %d header error: %s", pipe, strerror(errno));*/
+            return newly;
+        }
+
+        required -= newly;
+        offset += newly;
+    }
+
+    /* convert header values */
+    *indicator = header[3];
+    required = atoi(header);
+
+    /* only indicator with no value? */
+    if(required == 0) {
+        /*g_warning("read %d indicator: %c empty value", pipe, *indicator);*/
+        return 4;
+    }
+
+    g_assert(required <= len);
+    len = required;
+
+    /* read value */
+    offset = 0;
+    while(required) {
+        newly = read(pipe, &msg[offset], required);
+        if (newly == -1) {
+            /* error */
+            /*g_warning("read %d value error, indicator: %u", pipe, *indicator);*/
+            return newly;
+        }
+
+        required -= newly;
+        offset += newly;
+    }
+
+    /*g_warning("read %d ok indicator: %c len: %u msg: %s", pipe, *indicator, len, msg);*/
+    return len + 4;
 }
 
 void
 sync_pipe_packet_count_to_parent(int packet_count)
 {
     char tmp[SP_DECISIZE+1+1];
-    sprintf(tmp, "%d%c", packet_count, SP_PACKET_COUNT);
-    write(1, tmp, strlen(tmp));
+
+
+    g_snprintf(tmp, sizeof(tmp), "%d", packet_count);
+
+    pipe_write_block(1, SP_PACKET_COUNT, strlen(tmp)+1, tmp);
+}
+
+void
+sync_pipe_filename_to_parent(const char *filename)
+{
+    pipe_write_block(1, SP_FILE, strlen(filename)+1, filename);
 }
 
 void
 sync_pipe_errmsg_to_parent(const char *errmsg)
 {
-    int msglen = strlen(errmsg);
-    char lenbuf[SP_DECISIZE+1+1];
-
-    sprintf(lenbuf, "%u%c", msglen, SP_ERROR_MSG);
-    write(1, lenbuf, strlen(lenbuf));
-    write(1, errmsg, msglen);
+    pipe_write_block(1, SP_ERROR_MSG, strlen(errmsg)+1, errmsg);
 }
 
 void
 sync_pipe_drops_to_parent(int drops)
 {
 	char tmp[SP_DECISIZE+1+1];
-	sprintf(tmp, "%d%c", drops, SP_DROPS);
-	write(1, tmp, strlen(tmp));
+
+
+    g_snprintf(tmp, sizeof(tmp), "%d", drops);
+
+    pipe_write_block(1, SP_DROPS, strlen(tmp)+1, tmp);
 }
 
 
+#ifdef _WIN32
+
+static void
+signal_pipe_capquit_to_child(capture_options *capture_opts)
+{
+
+
+    pipe_write_block(capture_opts->signal_pipe_fd, SP_QUIT, 0, NULL);
+}
+#endif
+
+
 /* Add a string pointer to a NULL-terminated array of string pointers. */
 static char **
 sync_pipe_add_arg(char **args, int *argc, char *arg)
@@ -206,34 +311,37 @@ sync_pipe_quote_encapsulate(const char *string)
 
 
 
+#define ARGV_NUMBER_LEN 24
+
 gboolean
-sync_pipe_do_capture(capture_options *capture_opts, gboolean is_tempfile) {
-    guint byte_count;
-    int  i;
-    guchar  c;
-    char *msg;
-    int  err;
-    char ssnap[24];
-    char scount[24];			/* need a constant for len of numbers */
-    char sautostop_filesize[24];	/* need a constant for len of numbers */
-    char sautostop_duration[24];	/* need a constant for len of numbers */
-    char save_file_fd[24];
-#ifndef _WIN32
-    char errmsg[1024+1];
-#endif
-    int error;
-    int argc;
-    char **argv;
+sync_pipe_start(capture_options *capture_opts) {
+    char ssnap[ARGV_NUMBER_LEN];
+    char scount[ARGV_NUMBER_LEN];
+    char sfilesize[ARGV_NUMBER_LEN];
+    char sfile_duration[ARGV_NUMBER_LEN];
+    char sring_num_files[ARGV_NUMBER_LEN];
+    char sautostop_files[ARGV_NUMBER_LEN];
+    char sautostop_filesize[ARGV_NUMBER_LEN];
+    char sautostop_duration[ARGV_NUMBER_LEN];
 #ifdef _WIN32
-    char sync_pipe_fd[24];
+    char sync_pipe_fd[ARGV_NUMBER_LEN];
+    char signal_pipe_fd[ARGV_NUMBER_LEN];
     char *fontstring;
     char *filterstring;
+    int signal_pipe[2];                     /* pipe used to send messages from parent to child (currently only stop) */
+#else
+    char errmsg[1024+1];
 #endif
+    int argc;
+    char **argv;
     enum PIPES { PIPE_READ, PIPE_WRITE };   /* Constants 0 and 1 for PIPE_READ and PIPE_WRITE */
-    int sync_pipe[2];                       /* pipes used to sync between instances */
+    int sync_pipe[2];                       /* pipe used to send messages from child to parent */
 
 
-    fork_child = -1;
+    /*g_warning("sync_pipe_start");
+    capture_opts_info(capture_opts);*/
+
+    capture_opts->fork_child = -1;
 
     /* Allocate the string pointer array with enough space for the
        terminating NULL pointer. */
@@ -245,19 +353,11 @@ sync_pipe_do_capture(capture_options *capture_opts, gboolean is_tempfile) {
     argv = sync_pipe_add_arg(argv, &argc, CHILD_NAME);
 
     argv = sync_pipe_add_arg(argv, &argc, "-i");
-    argv = sync_pipe_add_arg(argv, &argc, cfile.iface);
+    argv = sync_pipe_add_arg(argv, &argc, capture_opts->iface);
 
-    argv = sync_pipe_add_arg(argv, &argc, "-w");
-    argv = sync_pipe_add_arg(argv, &argc, cfile.save_file);
-
-    argv = sync_pipe_add_arg(argv, &argc, "-W");
-    sprintf(save_file_fd,"%d",cfile.save_file_fd);	/* in lieu of itoa */
-    argv = sync_pipe_add_arg(argv, &argc, save_file_fd);
-
-    if (capture_opts->has_autostop_packets) {
-      argv = sync_pipe_add_arg(argv, &argc, "-c");
-      sprintf(scount,"%d",capture_opts->autostop_packets);
-      argv = sync_pipe_add_arg(argv, &argc, scount);
+    if(capture_opts->save_file) {
+      argv = sync_pipe_add_arg(argv, &argc, "-w");
+      argv = sync_pipe_add_arg(argv, &argc, capture_opts->save_file);
     }
 
     if (capture_opts->has_snaplen) {
@@ -277,10 +377,42 @@ sync_pipe_do_capture(capture_options *capture_opts, gboolean is_tempfile) {
       argv = sync_pipe_add_arg(argv, &argc, ssnap);
     }
 
-    if (capture_opts->has_autostop_filesize) {
-      argv = sync_pipe_add_arg(argv, &argc, "-a");
-      sprintf(sautostop_filesize,"filesize:%d",capture_opts->autostop_filesize);
-      argv = sync_pipe_add_arg(argv, &argc, sautostop_filesize);
+    if(capture_opts->multi_files_on) {
+      if (capture_opts->has_autostop_filesize) {
+        argv = sync_pipe_add_arg(argv, &argc, "-b");
+        sprintf(sfilesize,"filesize:%d",capture_opts->autostop_filesize);
+        argv = sync_pipe_add_arg(argv, &argc, sfilesize);
+      }
+
+      if (capture_opts->has_file_duration) {
+        argv = sync_pipe_add_arg(argv, &argc, "-b");
+        sprintf(sfile_duration,"duration:%d",capture_opts->file_duration);
+        argv = sync_pipe_add_arg(argv, &argc, sfile_duration);
+      }
+
+      if (capture_opts->has_ring_num_files) {
+        argv = sync_pipe_add_arg(argv, &argc, "-b");
+        sprintf(sring_num_files,"files:%d",capture_opts->ring_num_files);
+        argv = sync_pipe_add_arg(argv, &argc, sring_num_files);
+      }
+
+      if (capture_opts->has_autostop_files) {
+        argv = sync_pipe_add_arg(argv, &argc, "-a");
+        sprintf(sautostop_files,"files:%d",capture_opts->autostop_files);
+        argv = sync_pipe_add_arg(argv, &argc, sautostop_files);
+      }
+    } else {
+        if (capture_opts->has_autostop_filesize) {
+          argv = sync_pipe_add_arg(argv, &argc, "-a");
+          sprintf(sautostop_filesize,"filesize:%d",capture_opts->autostop_filesize);
+          argv = sync_pipe_add_arg(argv, &argc, sautostop_filesize);
+        }
+    }
+
+    if (capture_opts->has_autostop_packets) {
+      argv = sync_pipe_add_arg(argv, &argc, "-c");
+      sprintf(scount,"%d",capture_opts->autostop_packets);
+      argv = sync_pipe_add_arg(argv, &argc, scount);
     }
 
     if (capture_opts->has_autostop_duration) {
@@ -298,63 +430,78 @@ sync_pipe_do_capture(capture_options *capture_opts, gboolean is_tempfile) {
 
 #ifdef _WIN32
     /* Create a pipe for the child process */
-
-    if(_pipe(sync_pipe, 512, O_BINARY) < 0) {
+    /* (inrease this value if you have trouble while fast capture file switches) */
+    if(_pipe(sync_pipe, 5120, O_BINARY) < 0) {
       /* Couldn't create the pipe between parent and child. */
-      error = errno;
-      unlink(cfile.save_file);
-      g_free(cfile.save_file);
-      cfile.save_file = NULL;
       simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Couldn't create sync pipe: %s",
-                        strerror(error));
+                        strerror(errno));
+      g_free(argv);
       return FALSE;
     }
 
+    /* Create a pipe for the parent process */
+    if(_pipe(signal_pipe, 512, O_BINARY) < 0) {
+      /* Couldn't create the signal pipe between parent and child. */
+      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Couldn't create signal pipe: %s",
+                        strerror(errno));
+      close(sync_pipe[PIPE_READ]);
+      close(sync_pipe[PIPE_WRITE]);
+      g_free(argv);
+      return FALSE;
+    }
+
+    capture_opts->signal_pipe_fd = signal_pipe[PIPE_WRITE];
+
     /* Convert font name to a quote-encapsulated string and pass to child */
     argv = sync_pipe_add_arg(argv, &argc, "-m");
     fontstring = sync_pipe_quote_encapsulate(prefs.PREFS_GUI_FONT_NAME);
     argv = sync_pipe_add_arg(argv, &argc, fontstring);
 
-    /* Convert pipe write handle to a string and pass to child */
+    /* Convert sync pipe write handle to a string and pass to child */
     argv = sync_pipe_add_arg(argv, &argc, "-Z");
-    itoa(sync_pipe[PIPE_WRITE], sync_pipe_fd, 10);
+    sprintf(sync_pipe_fd,"sync:%d",sync_pipe[PIPE_WRITE]);
     argv = sync_pipe_add_arg(argv, &argc, sync_pipe_fd);
 
+    /* Convert signal pipe read handle to a string and pass to child */
+    argv = sync_pipe_add_arg(argv, &argc, "-Z");
+    sprintf(signal_pipe_fd,"signal:%d",signal_pipe[PIPE_READ]);
+    argv = sync_pipe_add_arg(argv, &argc, signal_pipe_fd);
+
     /* Convert filter string to a quote delimited string and pass to child */
     filterstring = NULL;
-    if (cfile.cfilter != NULL && strlen(cfile.cfilter) != 0) {
+    if (capture_opts->cfilter != NULL && strlen(capture_opts->cfilter) != 0) {
       argv = sync_pipe_add_arg(argv, &argc, "-f");
-      filterstring = sync_pipe_quote_encapsulate(cfile.cfilter);
+      filterstring = sync_pipe_quote_encapsulate(capture_opts->cfilter);
       argv = sync_pipe_add_arg(argv, &argc, filterstring);
     }
 
     /* Spawn process */
-    fork_child = spawnvp(_P_NOWAIT, ethereal_path, argv);
+    capture_opts->fork_child = spawnvp(_P_NOWAIT, ethereal_path, argv);
     g_free(fontstring);
     if (filterstring) {
       g_free(filterstring);
     }
+
+    /* child own's the read side now, close our handle */
+    close(signal_pipe[PIPE_READ]);
 #else
     if (pipe(sync_pipe) < 0) {
       /* Couldn't create the pipe between parent and child. */
-      error = errno;
-      unlink(cfile.save_file);
-      g_free(cfile.save_file);
-      cfile.save_file = NULL;
       simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Couldn't create sync pipe: %s",
-			strerror(error));
+			strerror(errno));
+      g_free(argv);
       return FALSE;
     }
 
     argv = sync_pipe_add_arg(argv, &argc, "-m");
     argv = sync_pipe_add_arg(argv, &argc, prefs.PREFS_GUI_FONT_NAME);
 
-    if (cfile.cfilter != NULL && strlen(cfile.cfilter) != 0) {
+    if (capture_opts->cfilter != NULL && capture_opts->cfilter != 0) {
       argv = sync_pipe_add_arg(argv, &argc, "-f");
-      argv = sync_pipe_add_arg(argv, &argc, cfile.cfilter);
+      argv = sync_pipe_add_arg(argv, &argc, capture_opts->cfilter);
     }
 
-    if ((fork_child = fork()) == 0) {
+    if ((capture_opts->fork_child = fork()) == 0) {
       /*
        * Child process - run Ethereal with the right arguments to make
        * it just pop up the live capture dialog box and capture with
@@ -394,116 +541,28 @@ sync_pipe_do_capture(capture_options *capture_opts, gboolean is_tempfile) {
        or by exiting abnormally). */
     close(sync_pipe[PIPE_WRITE]);
 
-    /* Close the save file FD, as we won't be using it - we'll be opening
-       it and reading the save file through Wiretap. */
-    close(cfile.save_file_fd);
-
-    if (fork_child == -1) {
+    if (capture_opts->fork_child == -1) {
       /* We couldn't even create the child process. */
-      error = errno;
-      close(sync_pipe[PIPE_READ]);
-      unlink(cfile.save_file);
-      g_free(cfile.save_file);
-      cfile.save_file = NULL;
       simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-			"Couldn't create child process: %s", strerror(error));
-      return FALSE;
-    }
-
-    /* Read a byte count from "sync_pipe[PIPE_READ]", terminated with a
-       colon; if the count is 0, the child process created the
-       capture file and we should start reading from it, otherwise
-       the capture couldn't start and the count is a count of bytes
-       of error message, and we should display the message. */
-    byte_count = 0;
-    for (;;) {
-      i = read(sync_pipe[PIPE_READ], &c, 1);
-      if (i == 0) {
-	/* EOF - the child process died.
-	   Close the read side of the sync pipe, remove the capture file,
-	   and report the failure. */
-	close(sync_pipe[PIPE_READ]);
-	unlink(cfile.save_file);
-	g_free(cfile.save_file);
-	cfile.save_file = NULL;
-	sync_pipe_wait_for_child(TRUE);
-	return FALSE;
-      }
-      if (c == SP_CAPSTART || c == SP_ERROR_MSG)
-	break;
-      if (!isdigit(c)) {
-	/* Child process handed us crap.
-	   Close the read side of the sync pipe, remove the capture file,
-	   and report the failure. */
-	close(sync_pipe[PIPE_READ]);
-	unlink(cfile.save_file);
-	g_free(cfile.save_file);
-	cfile.save_file = NULL;
-	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-			"Capture child process sent us a bad message");
-	return FALSE;
-      }
-      byte_count = byte_count*10 + c - '0';
-    }
-    if (c != SP_CAPSTART) {
-      /* Failure - the child process sent us a message indicating
-	 what the problem was. */
-      if (byte_count == 0) {
-	/* Zero-length message? */
-	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		"Capture child process failed, but its error message was empty.");
-      } else {
-	msg = g_malloc(byte_count + 1);
-	if (msg == NULL) {
-	  simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		"Capture child process failed, but its error message was too big.");
-	} else {
-	  i = read(sync_pipe[PIPE_READ], msg, byte_count);
-	  msg[byte_count] = '\0';
-	  if (i < 0) {
-	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		  "Capture child process failed: Error %s reading its error message.",
-		  strerror(errno));
-	  } else if (i == 0) {
-	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		  "Capture child process failed: EOF reading its error message.");
-	    sync_pipe_wait_for_child(FALSE);
-	  } else
-	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, msg);
-	  g_free(msg);
-	}
-
-	/* Close the sync pipe. */
-	close(sync_pipe[PIPE_READ]);
-
-	/* Get rid of the save file - the capture never started. */
-	unlink(cfile.save_file);
-	g_free(cfile.save_file);
-	cfile.save_file = NULL;
-      }
-      return FALSE;
-    }
-
-    /* The child process started a capture.
-       Attempt to open the capture file and set up to read it. */
-    err = cf_start_tail(cfile.save_file, is_tempfile, &cfile);
-    if (err != 0) {
-      /* We weren't able to open the capture file; user has been
-	 alerted. Close the sync pipe. */
-
+			"Couldn't create child process: %s", strerror(errno));
       close(sync_pipe[PIPE_READ]);
-
-      /* Don't unlink the save file - leave it around, for debugging
-	 purposes. */
-      g_free(cfile.save_file);
-      cfile.save_file = NULL;
+#ifdef _WIN32
+      close(signal_pipe[PIPE_WRITE]);
+#endif
       return FALSE;
     }
-    /* We were able to open and set up to read the capture file;
+
+    /* we might wait for a moment till child is ready, so update screen now */
+    main_window_update();
+
+    /* We were able to set up to read the capture file;
        arrange that our callback be called whenever it's possible
        to read from the sync pipe, so that it's called when
        the child process wants to tell us something. */
-    pipe_input_set_handler(sync_pipe[PIPE_READ], (gpointer) &cfile, &fork_child, sync_pipe_input_cb);
+
+    /* we have a running capture, now wait for the real capture filename */
+    pipe_input_set_handler(sync_pipe[PIPE_READ], (gpointer) capture_opts, 
+        &capture_opts->fork_child, sync_pipe_input_cb);
 
     return TRUE;
 }
@@ -515,146 +574,76 @@ sync_pipe_do_capture(capture_options *capture_opts, gboolean is_tempfile) {
 static gboolean 
 sync_pipe_input_cb(gint source, gpointer user_data)
 {
-  capture_file *cf = (capture_file *)user_data;
+  capture_options *capture_opts = (capture_options *)user_data;
 #define BUFSIZE	4096
-  char buffer[BUFSIZE+1], *p = buffer, *q = buffer, *msg, *r;
-  int  nread, msglen, chars_to_copy;
-  int  to_read = 0;
-  int  err;
+  char buffer[BUFSIZE+1];
+  int  nread;
+  char indicator;
 
 
-  if ((nread = read(source, buffer, BUFSIZE)) <= 0) {
+  nread = pipe_read_block(source, &indicator, BUFSIZE, buffer);
+  if(nread <= 0) {
     /* The child has closed the sync pipe, meaning it's not going to be
        capturing any more packets.  Pick up its exit status, and
        complain if it did anything other than exit with status 0. */
-    sync_pipe_wait_for_child(FALSE);
-
-    /* Read what remains of the capture file, and finish the capture.
-       XXX - do something if this fails? */
-    switch (cf_finish_tail(cf, &err)) {
-
-    case READ_SUCCESS:
-        if(cf->count == 0) {
-          simple_dialog(ESD_TYPE_INFO, ESD_BTN_OK, 
-          "%sNo packets captured!%s\n\n"
-          "As no data was captured, closing the %scapture file!",
-          simple_dialog_primary_start(), simple_dialog_primary_end(),
-          (cf->is_tempfile) ? "temporary " : "");
-          cf_close(cf);
-        }
-        break;
-    case READ_ERROR:
-      /* Just because we got an error, that doesn't mean we were unable
-         to read any of the file; we handle what we could get from the
-         file. */
-      break;
-
-    case READ_ABORTED:
-      /* Exit by leaving the main loop, so that any quit functions
-         we registered get called. */
-      main_window_quit();
-      return FALSE;
-    }
-
-    /* We're not doing a capture any more, so we don't have a save
-       file. */
-    g_free(cf->save_file);
-    cf->save_file = NULL;
-
+    sync_pipe_wait_for_child(capture_opts, FALSE);
+#ifdef _WIN32
+    close(capture_opts->signal_pipe_fd);
+#endif
+    capture_input_closed(capture_opts);
     return FALSE;
   }
 
-  buffer[nread] = '\0';
+  switch(indicator) {
+  case SP_FILE:
+      if(!capture_input_new_file(capture_opts, buffer)) {
+        /* We weren't able to open the new capture file; user has been
+           alerted. Close the sync pipe. */
+        /* XXX - is it safe to close the pipe inside this callback? */
+        close(source);
 
-  while (nread != 0) {
-    /* look for (possibly multiple) indications */
-    switch (*q) {
-    case SP_PACKET_COUNT :
-      to_read += atoi(p);
-      p = q + 1;
-      q++;
-      nread--;
-      break;
-    case SP_DROPS :
-      cf->drops_known = TRUE;
-      cf->drops = atoi(p);
-      p = q + 1;
-      q++;
-      nread--;
-      break;
-    case SP_ERROR_MSG :
-      msglen = atoi(p);
-      p = q + 1;
-      q++;
-      nread--;
-
-      /* Read the entire message.
-         XXX - if the child hasn't sent it all yet, this could cause us
-         to hang until they do. */
-      msg = g_malloc(msglen + 1);
-      r = msg;
-      while (msglen != 0) {
-      	if (nread == 0) {
-      	  /* Read more. */
-          if ((nread = read(source, buffer, BUFSIZE)) <= 0)
-            break;
-          p = buffer;
-          q = buffer;
-        }
-      	chars_to_copy = MIN(msglen, nread);
-        memcpy(r, q, chars_to_copy);
-        r += chars_to_copy;
-        q += chars_to_copy;
-        nread -= chars_to_copy;
-        msglen -= chars_to_copy;
+        /* the child has send us a filename which we couldn't open.
+           this probably means, the child is creating files faster than we can handle it.
+           this should only be the case for very fast file switches
+           we can't do much more than telling the child to stop
+           (this is the emergency brake if user e.g. wants to switch files every second) */
+        sync_pipe_stop(capture_opts);
       }
-      *r = '\0';
-      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, msg);
-      g_free(msg);
       break;
-    default :
-      q++;
-      nread--;
-      break;
-    }
-  }
-
-  /* Read from the capture file the number of records the child told us
-     it added.
-     XXX - do something if this fails? */
-  switch (cf_continue_tail(cf, to_read, &err)) {
-
-  case READ_SUCCESS:
-  case READ_ERROR:
-    /* Just because we got an error, that doesn't mean we were unable
-       to read any of the file; we handle what we could get from the
-       file.
-
-       XXX - abort on a read error? */
+  case SP_PACKET_COUNT:
+    nread = atoi(buffer);
+    capture_input_new_packets(capture_opts, nread);
     break;
-
-  case READ_ABORTED:
-    /* Kill the child capture process; the user wants to exit, and we
-       shouldn't just leave it running. */
-    kill_capture_child(TRUE /* sync_mode */);
+  case SP_ERROR_MSG:
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, buffer);
     break;
+  case SP_DROPS:
+    cf_set_drops_known(capture_opts->cf, TRUE);
+    cf_set_drops(capture_opts->cf, atoi(buffer));
+    break;
+  default:
+      g_assert_not_reached();
   }
 
   return TRUE;
 }
 
 
+
 /* the child process is going down, wait until it's completely terminated */
 static void
-sync_pipe_wait_for_child(gboolean always_report)
+sync_pipe_wait_for_child(capture_options *capture_opts, gboolean always_report)
 {
   int  wstatus;
 
+
+  g_assert(capture_opts->fork_child != -1);
+
 #ifdef _WIN32
   /* XXX - analyze the wait status and display more information
      in the dialog box?
      XXX - set "fork_child" to -1 if we find it exited? */
-  if (_cwait(&wstatus, fork_child, _WAIT_CHILD) == -1) {
+  if (_cwait(&wstatus, capture_opts->fork_child, _WAIT_CHILD) == -1) {
     simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		"Child capture process stopped unexpectedly");
   }
@@ -688,12 +677,13 @@ sync_pipe_wait_for_child(gboolean always_report)
   }
 
   /* No more child process. */
-  fork_child = -1;
+  capture_opts->fork_child = -1;
 #endif
 }
 
 
 #ifndef _WIN32
+/* convert signal to corresponding name */
 static char *
 sync_pipe_signame(int sig)
 {
@@ -788,53 +778,52 @@ sync_pipe_signame(int sig)
 #endif
 
 
+/* user wants to stop the capture run */
 void
-sync_pipe_stop(void)
+sync_pipe_stop(capture_options *capture_opts)
 {
-  if (fork_child != -1) {
+  /* XXX - in which cases this will be 0? */
+  if (capture_opts->fork_child != -1 && capture_opts->fork_child != 0) {
 #ifndef _WIN32
-      kill(fork_child, SIGUSR1);
+    /* send the SIGUSR1 signal to close the capture child gracefully. */
+    kill(capture_opts->fork_child, SIGUSR1);
 #else
-      /* XXX: this is not the preferred method of closing a process!
-       * the clean way would be getting the process id of the child process,
-       * then getting window handle hWnd of that process (using EnumChildWindows),
-       * and then do a SendMessage(hWnd, WM_CLOSE, 0, 0) 
-       *
-       * Unfortunately, I don't know how to get the process id from the handle */
-      /* Hint: OpenProcess will get an handle from the id, not vice versa :-(
-       *
-       * Hint: GenerateConsoleCtrlEvent() will only work, if both processes are 
-       * running in the same console, I don't know if that is true for our case.
-       * And this also will require to have the process id
-       */
-      TerminateProcess((HANDLE) fork_child, 0);
+    /* Win32 doesn't have the kill() system call, use the special signal pipe 
+       instead to close the capture child gracefully. */
+    signal_pipe_capquit_to_child(capture_opts);
 #endif
   }
 }
 
 
+/* Ethereal has to exit, force the capture child to close */
 void
-sync_pipe_kill(void)
+sync_pipe_kill(capture_options *capture_opts)
 {
-  if (fork_child != -1)
+  /* XXX - in which cases this will be 0? */
+  if (capture_opts->fork_child != -1 && capture_opts->fork_child != 0) {
 #ifndef _WIN32
-      kill(fork_child, SIGTERM);	/* SIGTERM so it can clean up if necessary */
+      kill(capture_opts->fork_child, SIGTERM);	/* SIGTERM so it can clean up if necessary */
 #else
       /* XXX: this is not the preferred method of closing a process!
        * the clean way would be getting the process id of the child process,
        * then getting window handle hWnd of that process (using EnumChildWindows),
        * and then do a SendMessage(hWnd, WM_CLOSE, 0, 0) 
        *
-       * Unfortunately, I don't know how to get the process id from the handle */
-      /* Hint: OpenProcess will get an handle from the id, not vice versa :-(
+       * Unfortunately, I don't know how to get the process id from the
+       * handle.  OpenProcess will get an handle (not a window handle)
+       * from the process ID; it will not get a window handle from the
+       * process ID.  (How could it?  A process can have more than one
+       * window.)
        *
-       * Hint: GenerateConsoleCtrlEvent() will only work, if both processes are 
-       * running in the same console, I don't know if that is true for our case.
-       * And this also will require to have the process id
+       * Hint: GenerateConsoleCtrlEvent() will only work if both processes are 
+       * running in the same console; that's not necessarily the case for
+       * us, as we might not be running in a console.
+       * And this also will require to have the process id.
        */
-      TerminateProcess((HANDLE) fork_child, 0);
+      TerminateProcess((HANDLE) (capture_opts->fork_child), 0);
 #endif
+  }
 }
 
-
 #endif /* HAVE_LIBPCAP */
