@@ -1,7 +1,7 @@
 /* column-utils.c
  * Routines for column utilities.
  *
- * $Id: column-utils.c 12889 2004-12-31 02:03:16Z guy $
+ * $Id: column-utils.c 13999 2005-04-01 10:17:05Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -574,20 +574,16 @@ static void
 col_set_addr(packet_info *pinfo, int col, address *addr, gboolean is_res,
 	     gboolean is_src)
 {
-  const char *addr_string;
   struct e_in6_addr ipv6_addr;
 
   pinfo->cinfo->col_expr[col][0] = '\0';
   pinfo->cinfo->col_expr_val[col][0] = '\0';
+  
   if (addr->type == AT_NONE)
     return;	/* no address, nothing to do */
+  
   if (is_res) {
-    addr_string = get_addr_name(addr);
-    if (addr_string != NULL) {
-      strncpy(pinfo->cinfo->col_buf[col], addr_string, COL_MAX_LEN);
-      pinfo->cinfo->col_buf[col][COL_MAX_LEN - 1] = '\0';
-    } else
-      address_to_str_buf(addr, pinfo->cinfo->col_buf[col]);
+    get_addr_name_buf(addr, pinfo->cinfo->col_buf[col],COL_MAX_LEN-1);
   } else {
     switch (addr->type) {
 
@@ -723,6 +719,17 @@ col_set_port(packet_info *pinfo, int col, gboolean is_res, gboolean is_src)
     pinfo->cinfo->col_expr_val[col][COL_MAX_LEN - 1] = '\0';
     break;
 
+  case PT_IDP:
+    /* XXX - resolve IDP socket numbers */
+    snprintf(pinfo->cinfo->col_buf[col], COL_MAX_LEN, "0x%04x", port);
+    if (is_src)
+      strcpy(pinfo->cinfo->col_expr[col], "idp.src.socket");
+    else
+      strcpy(pinfo->cinfo->col_expr[col], "idp.dst.socket");
+    snprintf(pinfo->cinfo->col_expr_val[col], COL_MAX_LEN, "0x%04x", port);
+    pinfo->cinfo->col_expr_val[col][COL_MAX_LEN - 1] = '\0';
+    break;
+
   default:
     break;
   }
@@ -994,6 +1001,10 @@ fill_in_columns(packet_info *pinfo)
       pinfo->cinfo->col_data[i] = pinfo->cinfo->col_buf[i];
       break;
         
+    case COL_HPUX_SUBSYS: /* done by nettl disector */
+    case COL_HPUX_DEVID:  /* done by nettl disector */
+      break;
+        
     case NUM_COL_FMTS:	/* keep compiler happy - shouldn't get here */
       g_assert_not_reached();
       break;
