@@ -2,7 +2,7 @@
  * Routines for AJP13 dissection
  * Copyright 2002, Christopher K. St. John <cks@distributopia.com>
  *
- * $Id: packet-ajp13.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-ajp13.c 13243 2005-02-02 20:07:03Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -640,10 +640,10 @@ dissect_ajp13_tcp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   /* conversational state really only does us good during the first
    * in-order traversal
    */
-  conv = find_conversation(&pinfo->src, &pinfo->dst, pinfo->ptype,
+  conv = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype,
                            pinfo->srcport, pinfo->destport, 0);
   if (!conv) {
-    conv = conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype,
+    conv = conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype,
                             pinfo->srcport, pinfo->destport, 0);
   }
   cd = (ajp13_conv_data*)conversation_get_proto_data(conv, proto_ajp13);
