@@ -2,7 +2,7 @@
  * Routines for dissecting the ATA over Ethernet protocol.
  *   Ronnie Sahlberg 2004
  *
- * $Id: packet-aoe.c 12890 2004-12-31 02:08:01Z guy $
+ * $Id: packet-aoe.c 13243 2005-02-02 20:07:03Z etxrab $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -226,12 +226,12 @@ dissect_ata_pdu(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset,
   conversation_t *conversation;
 
   /* only create a conversation for ATA commands */
-  conversation = find_conversation(&pinfo->src, &pinfo->dst,
+  conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                    pinfo->ptype, pinfo->srcport,
                                    pinfo->destport, 0);
   if (conversation == NULL) {
     /* We don't yet have a conversation, so create one. */
-    conversation = conversation_new(&pinfo->src, &pinfo->dst,
+    conversation = conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                     pinfo->ptype, pinfo->srcport,
                                     pinfo->destport, 0);
   }
