@@ -1,4 +1,4 @@
-/*#define DEBUG_BER 1*/
+/* #define DEBUG_BER 1 */
 /* TODO: change #.REGISTER signature to new_dissector_t and
  * update call_ber_oid_callback() accordingly.
  */
@@ -6,7 +6,7 @@
  * Helpers for ASN.1/BER dissection
  * Ronnie Sahlberg (C) 2004
  *
- * $Id: packet-ber.c 12743 2004-12-13 11:59:48Z sahlberg $
+ * $Id: packet-ber.c 14217 2005-04-28 09:54:03Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -47,6 +47,7 @@
 #include <epan/packet.h>
 
 #include <epan/strutil.h>
+#include <epan/to_str.h>
 #include <epan/prefs.h>
 #include "packet-ber.h"
 
@@ -289,7 +290,9 @@ int get_ber_identifier(tvbuff_t *tvb, int offset, gint8 *class, gboolean *pc, gi
 
 	id = tvb_get_guint8(tvb, offset);
 	offset += 1;
-	
+#ifdef DEBUG_BER
+printf ("BER ID=%02x", id);
+#endif
 	/* 8.1.2.2 */
 	tmp_class = (id>>6) & 0x03;
 	tmp_pc = (id>>5) & 0x01;
@@ -299,13 +302,19 @@ int get_ber_identifier(tvbuff_t *tvb, int offset, gint8 *class, gboolean *pc, gi
 		tmp_tag = 0;
 		while (tvb_length_remaining(tvb, offset) > 0) {
 			t = tvb_get_guint8(tvb, offset);
+#ifdef DEBUG_BER
+printf (" %02x", t);
+#endif
 			offset += 1;
 			tmp_tag <<= 7;       
 			tmp_tag |= t & 0x7F;
-			if (t & 0x80) break;
+			if (!(t & 0x80)) break;
 		}
 	}
 
+#ifdef DEBUG_BER
+printf ("\n");
+#endif
 	if (class)
 		*class = tmp_class;
 	if (pc)
@@ -350,10 +359,11 @@ int dissect_ber_identifier(packet_info *pinfo _U_, proto_tree *tree, tvbuff_t *t
  */
 /* 8.1.3 Length octets */
 int
-get_ber_length(tvbuff_t *tvb, int offset, guint32 *length, gboolean *ind) {
+get_ber_length(proto_tree *tree, tvbuff_t *tvb, int offset, guint32 *length, gboolean *ind) {
 	guint8 oct, len;
 	guint32 tmp_length;
 	gboolean tmp_ind;
+	int old_offset=offset;
 
 	tmp_length = 0;
 	tmp_ind = FALSE;
@@ -380,6 +390,13 @@ get_ber_length(tvbuff_t *tvb, int offset, guint32 *length, gboolean *ind) {
 		}
 	}
 
+	/* check that the length is sane */
+	if(tmp_length>(guint32)tvb_reported_length_remaining(tvb,offset)){
+		proto_tree_add_text(tree, tvb, old_offset, offset-old_offset, "BER: Error length:%d longer than tvb_reported_length_remaining:%d",tmp_length, tvb_reported_length_remaining(tvb, offset));
+		/* the ignorant mans way to generate [malformed packet] */
+		tvb_get_guint8(tvb, 99999999);
+	}
+
 	if (length)
 		*length = tmp_length;
 	if (ind)
@@ -398,7 +415,7 @@ dissect_ber_length(packet_info *pinfo _U_, proto_tree *tree, tvbuff_t *tvb, int
 	guint32 tmp_length;
 	gboolean tmp_ind;
 
-	offset = get_ber_length(tvb, offset, &tmp_length, &tmp_ind);
+	offset = get_ber_length(tree, tvb, offset, &tmp_length, &tmp_ind);
 	
 	if(show_internal_ber_fields){
 		if(tmp_ind){
@@ -423,6 +440,7 @@ dissect_ber_octet_string(gboolean implicit_tag, packet_info *pinfo, proto_tree *
 	guint32 len;
 	int end_offset;
 	proto_item *it;
+  guint32 i;
 
 #ifdef DEBUG_BER
 {
@@ -451,7 +469,10 @@ printf("OCTET STRING dissect_ber_octet_string(%s) entered\n",name);
 		/* sanity check: we only handle Constructed Universal Sequences */
 		if( (class!=BER_CLASS_UNI) 
 		  ||((tag<BER_UNI_TAG_NumericString)&&(tag!=BER_UNI_TAG_OCTETSTRING)&&(tag!=BER_UNI_TAG_UTF8String)) ){
+		    tvb_ensure_bytes_exist(tvb, offset-2, 2);
 	    	    proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: OctetString expected but Class:%d PC:%d Tag:%d was unexpected", class, pc, tag);
+			if(out_tvb)
+				*out_tvb=NULL;
 			return end_offset;
 		}
 	} else {
@@ -465,12 +486,24 @@ printf("OCTET STRING dissect_ber_octet_string(%s) entered\n",name);
 	if (pc) {
 		/* constructed */
 		/* TO DO */
+    printf("TODO: handle constructed type in packet-ber.c, dissect_ber_octet_string\n");
 	} else {
 		/* primitive */
 		if (hf_id != -1) {
+			tvb_ensure_bytes_exist(tvb, offset, len);
 			it = proto_tree_add_item(tree, hf_id, tvb, offset, len, FALSE);
 			ber_last_created_item = it;
-		}
+		} else {
+      proto_item *pi;
+      
+      pi=proto_tree_add_text(tree, tvb, offset, len, "Unknown OctetString: Length: 0x%02x, Value: 0x", len);
+      if(pi){
+        for(i=0;i<len;i++){
+          proto_item_append_text(pi,"%02x",tvb_get_guint8(tvb, offset));
+          offset++;
+        }
+      }
+    }
 		if (out_tvb) {
 			if(len<=(guint32)tvb_length_remaining(tvb, offset)){
 				*out_tvb = tvb_new_subset(tvb, offset, len, len);
@@ -484,10 +517,10 @@ printf("OCTET STRING dissect_ber_octet_string(%s) entered\n",name);
 
 int dissect_ber_octet_string_wcb(gboolean implicit_tag, packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset, gint hf_id, ber_callback func)
 {
-	tvbuff_t *out_tvb;
+	tvbuff_t *out_tvb = NULL;
 
 	offset = dissect_ber_octet_string(implicit_tag, pinfo, tree, tvb, offset, hf_id, (func)?&out_tvb:NULL);
-	if (func && (tvb_length(out_tvb)>0)) {
+	if (func && out_tvb && (tvb_length(out_tvb)>0)) {
 		if (hf_id != -1)
 			tree = proto_item_add_subtree(ber_last_created_item, ett_ber_octet_string);
 		func(pinfo, tree, out_tvb, 0);
@@ -495,6 +528,31 @@ int dissect_ber_octet_string_wcb(gboolean implicit_tag, packet_info *pinfo, prot
 	return offset;
 }
 
+/* 8.8 Encoding of a null value */
+int 
+dissect_ber_null(gboolean implicit_tag, packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset, gint hf_id) {
+  guint8 class;
+  gboolean pc;
+  guint32 tag, len;
+  int offset_old;
+
+  offset_old = offset;  
+  offset = dissect_ber_identifier(pinfo, tree, tvb, offset, &class, &pc, &tag);
+  if ((pc) ||
+      (!implicit_tag && ((class != BER_CLASS_UNI) || (tag != BER_UNI_TAG_NULL)))) {
+    proto_tree_add_text(tree, tvb, offset_old, offset - offset_old, "BER Error: NULL expected but Class:%d(%s) PC:%d Tag:%d was unexpected", class,val_to_str(class,ber_class_codes,"Unknown"), pc, tag);
+  }
+
+  offset_old = offset;  
+  offset = dissect_ber_length(pinfo, tree, tvb, offset, &len, NULL);
+  if (len) {
+    proto_tree_add_text(tree, tvb, offset_old, offset - offset_old, "BER Error: NULL expect zero length but Length=%d", len);
+    proto_tree_add_text(tree, tvb, offset, len, "BER Error: unexpected data in NULL type");
+    offset += len;
+  }
+
+  return offset;
+}
 
 int
 dissect_ber_integer(gboolean implicit_tag, packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset, gint hf_id, guint32 *value)
@@ -580,8 +638,12 @@ printf("INTEGERnew dissect_ber_integer(%s) entered implicit_tag:%d len:%d\n",nam
 	ber_last_created_item=NULL;
 
 	if(hf_id!=-1){	
-		/* XXX - what if "len" is not 1, 2, 3, or 4? */
-		ber_last_created_item=proto_tree_add_item(tree, hf_id, tvb, offset-len, len, FALSE);
+		/*  */
+		if (len < 1 || len > 4) {
+			proto_tree_add_text(tree, tvb, offset-len, len, "Can't handle integer length: %u", len);
+		} else {
+			ber_last_created_item=proto_tree_add_item(tree, hf_id, tvb, offset-len, len, FALSE);
+		}
 	}
 	if(value){
 		*value=val;
@@ -592,7 +654,7 @@ printf("INTEGERnew dissect_ber_integer(%s) entered implicit_tag:%d len:%d\n",nam
 
 
 int
-dissect_ber_boolean(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset, gint hf_id)
+dissect_ber_boolean(gboolean implicit_tag, packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset, gint hf_id)
 {
 	guint8 class;
 	gboolean pc;
@@ -601,10 +663,13 @@ dissect_ber_boolean(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int off
 	guint8 val;
 	header_field_info *hfi;
 
-	offset=dissect_ber_identifier(pinfo, tree, tvb, offset, &class, &pc, &tag);
-	offset=dissect_ber_length(pinfo, tree, tvb, offset, &len, NULL);
-
-/*	if(class!=BER_CLASS_UNI)*/
+	if(!implicit_tag){
+		offset=dissect_ber_identifier(pinfo, tree, tvb, offset, &class, &pc, &tag);
+		offset=dissect_ber_length(pinfo, tree, tvb, offset, &len, NULL);
+		/*if(class!=BER_CLASS_UNI)*/
+	} else {
+		/* nothing to do here, yet */
+	}
 	
 	val=tvb_get_guint8(tvb, offset);
 	offset+=1;
@@ -672,7 +737,8 @@ printf("SEQUENCE dissect_ber_sequence(%s) entered\n",name);
 		if ((!pc)
 		||(!implicit_tag&&((class!=BER_CLASS_UNI)
 							||(tag!=BER_UNI_TAG_SEQUENCE)))) {
-			proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: Sequence expected but Class:%d PC:%d Tag:%d was unexpected", class, pc, tag);
+			tvb_ensure_bytes_exist(tvb, offset-2, 2);
+			proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: Sequence expected but Class:%d(%s) PC:%d Tag:%d was unexpected", class,val_to_str(class,ber_class_codes,"Unknown"), pc, tag);
 			return end_offset;
 		}
 	} else {
@@ -708,7 +774,7 @@ printf("SEQUENCE dissect_ber_sequence(%s) entered\n",name);
 		hoffset = offset;
 		/* read header and len for next field */
 		offset = get_ber_identifier(tvb, offset, &class, &pc, &tag);
-		offset = get_ber_length(tvb, offset, &len, &ind_field);
+		offset = get_ber_length(tree, tvb, offset, &len, &ind_field);
 		eoffset = offset + len;
 ber_sequence_try_again:
 		/* have we run out of known entries in the sequence ?*/
@@ -740,8 +806,18 @@ ber_sequence_try_again:
 				seq++;
 				goto ber_sequence_try_again;
 			}
-
-			proto_tree_add_text(tree, tvb, offset, len, "BER Error: Wrong field in SEQUENCE  expected class:%d tag:%d but found class:%d tag:%d",seq->class,seq->tag,class,tag);
+			if ( seq->class == BER_CLASS_UNI){
+				proto_tree_add_text(tree, tvb, offset, len,
+				    "BER Error: Wrong field in SEQUENCE  expected class:%d (%s) tag:%d (%s) but found class:%d tag:%d",
+				    seq->class,val_to_str(seq->class,ber_class_codes,"Unknown"),
+				    seq->tag,val_to_str(seq->tag,ber_uni_tag_codes,"Unknown"),
+				    class,tag);
+			}else{
+				proto_tree_add_text(tree, tvb, offset, len,
+				    "BER Error: Wrong field in SEQUENCE  expected class:%d (%s) tag:%d but found class:%d tag:%d",
+				    seq->class,val_to_str(seq->class,ber_class_codes,"Unknown"),
+				    seq->tag,class,tag);
+			}
 			seq++;
 			offset=eoffset;
 			continue;
@@ -758,7 +834,11 @@ ber_sequence_try_again:
 				goto ber_sequence_try_again;
 			}
 
-			proto_tree_add_text(tree, tvb, offset, len, "BER Error: Wrong field in sequence  expected class:%d tag:%d but found class:%d tag:%d",seq->class,seq->tag,class,tag);
+			if ( seq->class == BER_CLASS_UNI){
+				proto_tree_add_text(tree, tvb, offset, len, "BER Error: Wrong field in sequence  expected class:%d (%s) tag:%d(%s) but found class:%d(%s) tag:%d",seq->class,val_to_str(seq->class,ber_class_codes,"Unknown"),seq->tag,val_to_str(seq->tag,ber_uni_tag_codes,"Unknown"),class,val_to_str(class,ber_class_codes,"Unknown"),tag);
+			}else{
+				proto_tree_add_text(tree, tvb, offset, len, "BER Error: Wrong field in sequence  expected class:%d (%s) tag:%d but found class:%d(%s) tag:%d",seq->class,val_to_str(seq->class,ber_class_codes,"Unknown"),seq->tag,class,val_to_str(class,ber_class_codes,"Unknown"),tag);
+			}
 			seq++;
 			offset=eoffset;
 			continue;
@@ -815,17 +895,21 @@ printf("SEQUENCE dissect_ber_sequence(%s) subdissector ate %d bytes\n",name,coun
 }
 #endif
 		seq++;
-		offset = hoffset+count;
-		/* if it was optional  and no bytes were eaten,
-		   just try again.
-		*/
-		if((count==0)&&(seq->flags&BER_FLAGS_OPTIONAL)){
-				goto ber_sequence_try_again;
+		if (len==0) {
+			offset = eoffset;
+		} else {
+			offset = hoffset+count;
+		}
+		/* if it was optional and no bytes were eaten and it was */
+		/* supposed to (len<>0), just try again. */
+		if((len!=0)&&(count==0)&&(seq->flags&BER_FLAGS_OPTIONAL)){
+			goto ber_sequence_try_again;
 		}
 	}
 
 	/* if we didnt end up at exactly offset, then we ate too many bytes */
 	if (offset != end_offset) {
+		tvb_ensure_bytes_exist(tvb, offset-2, 2);
 		proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: Sequence ate %d too many bytes", offset-end_offset);
 	}
 
@@ -875,7 +959,7 @@ printf("CHOICE dissect_ber_choice(%s) entered len:%d\n",name,tvb_length_remainin
 
 	/* read header and len for choice field */
 	offset=get_ber_identifier(tvb, offset, &class, &pc, &tag);
-	offset=get_ber_length(tvb, offset, &len, &ind);
+	offset=get_ber_length(parent_tree, tvb, offset, &len, &ind);
 	if(ind){
 	  /* if the length is indefinite we dont really know (yet) where the
 	   * object ends so assume it spans the rest of the tvb for now.
@@ -1034,6 +1118,7 @@ dissect_ber_GeneralString(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, i
 	/* sanity check: we only handle Universal GeneralString*/
 	if( (class!=BER_CLASS_UNI)
 	  ||(tag!=BER_UNI_TAG_GENSTR) ){
+		tvb_ensure_bytes_exist(tvb, offset-2, 2);
 	        proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: GeneralString expected but Class:%d PC:%d Tag:%d was unexpected", class, pc, tag);
 		return end_offset;
 	}
@@ -1081,12 +1166,13 @@ printf("RESTRICTED STRING dissect_ber_octet_string(%s) entered\n",name);
 
 	if (!implicit_tag) {
 		offset = get_ber_identifier(tvb, offset, &class, &pc, &tag);
-		offset = get_ber_length(tvb, offset, &len, NULL);
+		offset = get_ber_length(tree, tvb, offset, &len, NULL);
 		eoffset = offset + len;
 
 		/* sanity check */
 		if( (class!=BER_CLASS_UNI)
 		  ||(tag != type) ){
+	            tvb_ensure_bytes_exist(tvb, offset-2, 2);
 	    	    proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: String with tag=%d expected but Class:%d PC:%d Tag:%d was unexpected", type, class, pc, tag);
 			return eoffset;
 		}
@@ -1099,15 +1185,15 @@ printf("RESTRICTED STRING dissect_ber_octet_string(%s) entered\n",name);
 int
 dissect_ber_GeneralString(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset, gint hf_id, char *name_string, guint name_len)
 {
-	tvbuff_t *out_tvb;
+	tvbuff_t *out_tvb = NULL;
 
 	offset = dissect_ber_restricted_string(FALSE, BER_UNI_TAG_GeneralString, pinfo, tree, tvb, offset, hf_id, (name_string)?&out_tvb:NULL);
 
 	if (name_string) {
-		if (tvb_length(out_tvb) >= name_len) {
+		if (out_tvb && tvb_length(out_tvb) >= name_len) {
 			tvb_memcpy(out_tvb, name_string, 0, name_len-1);
 			name_string[name_len-1] = '\0';
-		} else {
+		} else if (out_tvb) {
 			tvb_memcpy(out_tvb, name_string, 0, -1);
 			name_string[tvb_length(out_tvb)] = '\0';
 		}
@@ -1121,11 +1207,9 @@ int dissect_ber_object_identifier(gboolean implicit_tag, packet_info *pinfo, pro
 	gint8 class;
 	gboolean pc;
 	gint32 tag;
-	guint32 i, len;
+	guint32 len;
 	int eoffset;
-	guint8 byte;
-	guint32 value;
-	char str[256],*strp, *name;
+	char str[MAX_OID_STR_LEN], *name;
 	proto_item *item;
 
 #ifdef DEBUG_BER
@@ -1157,6 +1241,7 @@ printf("OBJECT IDENTIFIER dissect_ber_object_identifier(%s) entered\n",name);
 		eoffset = offset + len;
 		if( (class!=BER_CLASS_UNI)
 		  ||(tag != BER_UNI_TAG_OID) ){
+	            tvb_ensure_bytes_exist(tvb, offset-2, 2);
 	    	    proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: Object Identifier expected but Class:%d PC:%d Tag:%d was unexpected", class, pc, tag);
 			return eoffset;
 		}
@@ -1165,31 +1250,8 @@ printf("OBJECT IDENTIFIER dissect_ber_object_identifier(%s) entered\n",name);
 		eoffset=offset+len;
 	}
 
-	value=0;
-	for (i=0,strp=str; i<len; i++){
-		byte = tvb_get_guint8(tvb, offset);
-		offset++;
-
-		if((strp-str)>200){
-    	    proto_tree_add_text(tree, tvb, offset, eoffset - offset, "BER Error: too long Object Identifier");
-			return offset;
-		}
-
-		/* 8.19.4 */
-		if (i == 0) {
-			strp += sprintf(strp, "%d.%d", byte/40, byte%40);
-			continue;
-		}
-
-		value = (value << 7) | (byte & 0x7F);
-		if (byte & 0x80) {
-			continue;
-		}
-
-		strp += sprintf(strp, ".%d", value);
-		value = 0;
-	}
-	*strp = '\0';
+	oid_to_str_buf(tvb_get_ptr(tvb, offset, len), len, str);
+	offset += len;
 
 	if (hf_id != -1) {
 		item=proto_tree_add_string(tree, hf_id, tvb, offset - len, len, str);
@@ -1211,7 +1273,7 @@ printf("OBJECT IDENTIFIER dissect_ber_object_identifier(%s) entered\n",name);
 
 static int dissect_ber_sq_of(gboolean implicit_tag, gint32 type, packet_info *pinfo, proto_tree *parent_tree, tvbuff_t *tvb, int offset, const ber_sequence_t *seq, gint hf_id, gint ett_id) {
 	gint8 class;
-	gboolean pc, ind, ind_field;
+	gboolean pc, ind = FALSE, ind_field;
 	gint32 tag;
 	guint32 len;
 	proto_tree *tree = parent_tree;
@@ -1254,6 +1316,7 @@ printf("SQ OF dissect_ber_sq_of(%s) entered\n",name);
 		if (!pc
 			||(!implicit_tag&&((class!=BER_CLASS_UNI)
 							||(tag!=type)))) {
+			tvb_ensure_bytes_exist(tvb, offset-2, 2);
 			proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: %s Of expected but Class:%d PC:%d Tag:%d was unexpected", 
 							(type==BER_UNI_TAG_SEQUENCE)?"Set":"Sequence", class, pc, tag);
 			return end_offset;
@@ -1277,7 +1340,7 @@ printf("SQ OF dissect_ber_sq_of(%s) entered\n",name);
 
 		/* read header and len for next field */
 		offset = get_ber_identifier(tvb, offset, NULL, NULL, NULL);
-		offset = get_ber_length(tvb, offset, &len, NULL);
+		offset = get_ber_length(tree, tvb, offset, &len, NULL);
 		offset += len;
 		cnt++;
 	}
@@ -1318,7 +1381,7 @@ printf("SQ OF dissect_ber_sq_of(%s) entered\n",name);
 		hoffset = offset;
 		/* read header and len for next field */
 		offset = get_ber_identifier(tvb, offset, &class, &pc, &tag);
-		offset = get_ber_length(tvb, offset, &len, &ind_field);
+		offset = get_ber_length(tree, tvb, offset, &len, &ind_field);
 		if(ind_field){
 			/* if the length is indefinite we dont really know (yet) where the
 			 * object ends so assume it spans the rest of the tvb for now.
@@ -1363,6 +1426,7 @@ printf("SQ OF dissect_ber_sq_of(%s) entered\n",name);
 
 	/* if we didnt end up at exactly offset, then we ate too many bytes */
 	if (offset != end_offset) {
+		tvb_ensure_bytes_exist(tvb, offset-2, 2);
 		proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: %s Of ate %d too many bytes", 
 							(type==BER_UNI_TAG_SEQUENCE)?"Set":"Sequence", offset-end_offset);
 	}
@@ -1397,6 +1461,7 @@ dissect_ber_GeneralizedTime(gboolean implicit_tag, packet_info *pinfo, proto_tre
 	  /* sanity check. we only handle universal/generalized time */
 	  if( (class!=BER_CLASS_UNI)
 	  ||(tag!=BER_UNI_TAG_GeneralizedTime)){
+		tvb_ensure_bytes_exist(tvb, offset-2, 2);
 	        proto_tree_add_text(tree, tvb, offset-2, 2, "BER Error: GeneralizedTime expected but Class:%d PC:%d Tag:%d was unexpected", class, pc, tag);
 		return end_offset;
 		end_offset=offset+len;
@@ -1446,6 +1511,7 @@ int dissect_ber_bitstring(gboolean implicit_tag, packet_info *pinfo, proto_tree
 	  if (!implicit_tag) {
 		if( (class!=BER_CLASS_UNI)
 		  ||(tag!=BER_UNI_TAG_BITSTRING) ){
+		    tvb_ensure_bytes_exist(tvb, offset-2, 2);
 	    	    proto_tree_add_text(parent_tree, tvb, offset-2, 2, "BER Error: BitString expected but Class:%d PC:%d Tag:%d was unexpected", class, pc, tag);
 			return end_offset;
 		}
@@ -1521,7 +1587,7 @@ int dissect_ber_bitstring(gboolean implicit_tag, packet_info *pinfo, proto_tree
 
 int dissect_ber_bitstring32(gboolean implicit_tag, packet_info *pinfo, proto_tree *parent_tree, tvbuff_t *tvb, int offset, int **bit_fields, gint hf_id, gint ett_id, tvbuff_t **out_tvb) 
 {
-	tvbuff_t *tmp_tvb;
+	tvbuff_t *tmp_tvb = NULL;
 	proto_tree *tree;
 	guint32 val;
 	int **bf;
@@ -1533,7 +1599,7 @@ int dissect_ber_bitstring32(gboolean implicit_tag, packet_info *pinfo, proto_tre
 	offset = dissect_ber_bitstring(implicit_tag, pinfo, parent_tree, tvb, offset, NULL, hf_id, ett_id, &tmp_tvb);
 	
 	tree = proto_item_get_subtree(ber_last_created_item);
-	if (bit_fields && tree) {
+	if (bit_fields && tree && tmp_tvb) {
 		/* tmp_tvb points to the actual bitstring (including any pad bits at the end.
 		 * note that this bitstring is not neccessarily always encoded as 4 bytes
 		 * so we have to read it byte by byte.
