@@ -1,8 +1,11 @@
+/* TODO:
+    dissect_ndr_nt_SID_with_options    see comment.
+*/
 /* packet-dcerpc-nt.c
  * Routines for DCERPC over SMB packet disassembly
  * Copyright 2001-2003, Tim Potter <tpot@samba.org>
  *
- * $Id: packet-dcerpc-nt.c 11592 2004-08-03 02:28:49Z guy $
+ * $Id: packet-dcerpc-nt.c 13413 2005-02-16 11:31:56Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -208,13 +211,131 @@ dissect_ndr_counted_byte_array_cb(tvbuff_t *tvb, int offset,
 	return offset;
 }
 
+void cb_byte_array_postprocess(packet_info *pinfo, proto_tree *tree _U_,
+			proto_item *item, tvbuff_t *tvb, 
+			int start_offset, int end_offset,
+			void *callback_args)
+{
+	gint options = GPOINTER_TO_INT(callback_args);
+	gint levels = CB_STR_ITEM_LEVELS(options);
+	char *s;
+
+	/* Align start_offset on 4-byte boundary. */
+
+	if (start_offset % 4)
+		start_offset += 4 - (start_offset % 4);
+
+	/* Get byte array value */
+
+	if ((end_offset - start_offset) <= 12)
+		return;
+
+	s = tvb_bytes_to_str(
+		tvb, start_offset + 12, (end_offset - start_offset - 12) );
+
+	/* Append string to COL_INFO */
+
+	if (options & CB_STR_COL_INFO) {
+		if (check_col(pinfo->cinfo, COL_INFO))
+			col_append_fstr(pinfo->cinfo, COL_INFO, ", %s", s);
+	}
+
+	/* Append string to upper-level proto_items */
+
+	if (levels > 0 && item && s && s[0]) {
+		proto_item_append_text(item, ": %s", s);
+		item = item->parent;
+		levels--;
+		if (levels > 0) {
+			proto_item_append_text(item, ": %s", s);
+			item = item->parent;
+			levels--;
+			while (levels > 0) {
+				proto_item_append_text(item, " %s", s);
+				item = item->parent;
+				levels--;
+			}
+		}
+	}
+}
+
 int
 dissect_ndr_counted_byte_array(tvbuff_t *tvb, int offset,
 			       packet_info *pinfo, proto_tree *tree,
-			       guint8 *drep, int hf_index)
+			       guint8 *drep, int hf_index, int levels)
 {
 	return dissect_ndr_counted_byte_array_cb(
-		tvb, offset, pinfo, tree, drep, hf_index, NULL, NULL);
+		tvb, offset, pinfo, tree, drep, hf_index, cb_byte_array_postprocess, GINT_TO_POINTER(2 + levels));
+}
+
+/* Dissect a counted ascii string in-line. */
+static gint ett_nt_counted_ascii_string = -1;
+
+int
+dissect_ndr_counted_ascii_string_cb(tvbuff_t *tvb, int offset,
+				  packet_info *pinfo, proto_tree *tree,
+				  guint8 *drep, int hf_index,
+				  dcerpc_callback_fnct_t *callback,
+				  void *callback_args)
+{
+	dcerpc_info *di = pinfo->private_data;
+	proto_item *item;
+	proto_tree *subtree;
+	guint16 len, size;
+
+        /* Structure starts with short, but is aligned for longs */
+
+	ALIGN_TO_4_BYTES;
+
+	if (di->conformant_run)
+		return offset;
+
+	item = proto_tree_add_text(tree, tvb, offset, 0, 
+		proto_registrar_get_name(hf_index));
+
+	subtree = proto_item_add_subtree(item, ett_nt_counted_ascii_string);
+	
+	/* 
+           struct {
+               short len;
+               short size;
+               [size_is(size), length_is(len), ptr] unsigned char *string;
+           } WHATEVER_THIS_IS_CALLED;
+
+         */
+
+	offset = dissect_ndr_uint16(tvb, offset, pinfo, subtree, drep,
+			hf_nt_cs_len, &len);
+
+	offset = dissect_ndr_uint16(tvb, offset, pinfo, subtree, drep,
+			hf_nt_cs_size, &size);	
+
+	offset = dissect_ndr_pointer_cb(tvb, offset, pinfo, subtree, drep,
+			dissect_ndr_char_cvstring, NDR_POINTER_UNIQUE,
+			"Ascii String", hf_index, callback, callback_args);
+
+	return offset;
+}
+
+int
+dissect_ndr_counted_ascii_string(tvbuff_t *tvb, int offset,
+			       packet_info *pinfo, proto_tree *tree,
+			       guint8 *drep, int hf_index, int levels)
+{
+	return dissect_ndr_counted_ascii_string_cb(
+		tvb, offset, pinfo, tree, drep, hf_index, cb_str_postprocess, GINT_TO_POINTER(2 + levels));
+}
+
+static int hf_nt_guid = -1;
+
+int
+dissect_nt_GUID(tvbuff_t *tvb, int offset,
+			packet_info *pinfo, proto_tree *tree,
+			guint8 *drep)
+{
+	offset=dissect_ndr_uuid_t(tvb, offset, pinfo, tree, drep, hf_nt_guid, NULL);
+
+	return offset;
 }
 
 /* This function is used to dissect a DCERPC encoded 64 bit time value.
@@ -754,15 +875,20 @@ dissect_nt_policy_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 	if (dcerpc_smb_fetch_pol(&hnd, &name, &open_frame, &close_frame,
 	    pinfo->fd->num)) {
 
-		if (open_frame)
-			proto_tree_add_uint(
+		if (open_frame) {
+			proto_item *item;
+			item=proto_tree_add_uint(
 				subtree, hf_nt_policy_open_frame, tvb,
 				old_offset, sizeof(e_ctx_hnd), open_frame);
-
-		if (close_frame)
-			proto_tree_add_uint(
+			PROTO_ITEM_SET_GENERATED(item);
+		}
+		if (close_frame) {
+			proto_item *item;
+			item=proto_tree_add_uint(
 				subtree, hf_nt_policy_close_frame, tvb,
 				old_offset, sizeof(e_ctx_hnd), close_frame);
+			PROTO_ITEM_SET_GENERATED(item);
+		}
 
 		/*
 		 * Don't append the handle name if pitem is null; that's
@@ -1021,7 +1147,7 @@ dissect_ndr_nt_SID(tvbuff_t *tvb, int offset, packet_info *pinfo,
 {
 	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
 	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-	char *sid_str;
+	char *sid_str=NULL;
 	char *name;
 
 	if(di->hf_index!=-1){
@@ -1053,6 +1179,56 @@ dissect_ndr_nt_SID(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	return offset;
 }
 
+/* same as dissect_ndr_nt_SID() but takes the same options as counted strings
+   do to prettify the dissect pane and the COL_INFO summary line
+*/
+int
+dissect_ndr_nt_SID_with_options(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint8 *drep, guint32 options)
+{
+	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
+	dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
+	gint levels = CB_STR_ITEM_LEVELS(options);
+	offset=dissect_ndr_nt_SID(tvb, offset, pinfo, tree, drep);
+
+	if(dcv && dcv->private_data){
+		char *s=dcv->private_data;
+		proto_item *item=(proto_item *)tree;
+
+		if ((options & CB_STR_COL_INFO)&&(!di->conformant_run)) {
+			/* kludge, ugly,   but this is called twice for all 
+			   dcerpc interfaces due to how we chase pointers 
+			   and putting the sid twice on the summary line
+			   looks even worse.
+			   Real solution would be to block updates to col_info
+			   while we just do a conformance run,   this might
+			   have sideeffects so it needs some more thoughts first.
+			*/
+			if (check_col(pinfo->cinfo, COL_INFO))
+				col_append_fstr(pinfo->cinfo, COL_INFO, ", %s", s);
+		}
+
+		/* Append string to upper-level proto_items */
+
+		if (levels > 0 && item && s && s[0]) {
+			proto_item_append_text(item, ": %s", s);
+			item = item->parent;
+			levels--;
+			if (levels > 0) {
+				proto_item_append_text(item, ": %s", s);
+				item = item->parent;
+				levels--;
+				while (levels > 0) {
+					proto_item_append_text(item, " %s", s);
+					item = item->parent;
+					levels--;
+				}
+			}
+		}
+	}
+
+	return offset;
+}
+
 static int
 dissect_ndr_nt_SID_hf_through_ptr(tvbuff_t *tvb, int offset, packet_info *pinfo, 
 		   proto_tree *tree, guint8 *drep)
@@ -1380,6 +1556,11 @@ void dcerpc_smb_init(int proto_dcerpc)
 		    NULL, 0x0, "Length of string in short integers", 
 		    HFILL }},
 		
+		/* GUIDs */
+		{ &hf_nt_guid,
+		  { "GUID", "nt.guid", FT_STRING, BASE_NONE, 
+		    NULL, 0x0, "GUID (uuid for groups?)", HFILL }},
+
 		/* Policy handles */
 
 		{ &hf_nt_policy_open_frame,
@@ -1497,6 +1678,7 @@ void dcerpc_smb_init(int proto_dcerpc)
                 &ett_nt_sid_array,
                 &ett_nt_sid_and_attributes_array,
                 &ett_nt_sid_and_attributes,
+		&ett_nt_counted_ascii_string,
 	};
 
 	/* Register ett's and hf's */
