@@ -1,7 +1,7 @@
 /* packet-dcom.c
  * Routines for DCOM generics
  *
- * $Id: packet-dcom.c 13006 2005-01-12 21:20:50Z ulfl $
+ * $Id: packet-dcom.c 14242 2005-04-30 16:08:29Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -670,30 +670,6 @@ gchar* dcom_uuid_to_str(e_uuid_t *uuid) {
 }
 
 
-/* dissect 64bits integer with alignment of 8 bytes (use this for VT_I8 type only) */
-int
-dissect_dcom_I8(tvbuff_t *tvb, gint offset, packet_info *pinfo,
-                    proto_tree *tree, guint8 *drep, 
-                    int hfindex, unsigned char *pdata)
-{
-    dcerpc_info *di;
-
-
-    di=pinfo->private_data;
-    if(di->conformant_run){
-      /* just a run to handle conformant arrays, no scalars to dissect */
-      return offset;
-    }
-
-    if (offset % 8) {
-        offset += 8 - (offset % 8);
-    }
-    return dissect_dcerpc_uint64(tvb, offset, pinfo, 
-                                  tree, drep, hfindex, pdata);
-}
-
-
-
 /* dissect a dcerpc array size */
 int
 dissect_dcom_dcerpc_array_size(tvbuff_t *tvb, gint offset, packet_info *pinfo,
@@ -1358,6 +1334,8 @@ dissect_dcom_BSTR(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 
 	/* update subtree header */
 	proto_item_append_text(sub_item, "\"%s\"", pszEscaped);
+	if ((int) (u32RealOffset - u32SubStart) <= 0)
+	    THROW(ReportedBoundsError);
 	proto_item_set_len(sub_item, u32RealOffset - u32SubStart);
 
     u32Max = MIN(u32Max, strlen(pszEscaped) + 1);
@@ -1469,8 +1447,6 @@ dissect_dcom_STDOBJREF(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 {
 	guint32	u32Flags;
 	guint32	u32PublicRefs;
-	unsigned char pu64Oxid[8];
-	unsigned char pu64Oid[8];
 	e_uuid_t ipid;
 	proto_item *sub_item;
 	proto_tree *sub_tree;
@@ -1488,9 +1464,9 @@ dissect_dcom_STDOBJREF(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                         hf_dcom_stdobjref_public_refs, &u32PublicRefs);
 	offset = dissect_dcom_ID(tvb, offset, pinfo, sub_tree, drep, 
-                        hf_dcom_stdobjref_oxid, pu64Oxid);
+                        hf_dcom_stdobjref_oxid, NULL);
 	offset = dissect_dcom_ID(tvb, offset, pinfo, sub_tree, drep, 
-                        hf_dcom_stdobjref_oid, pu64Oid);
+                        hf_dcom_stdobjref_oid, NULL);
 	offset = dissect_dcom_UUID(tvb, offset, pinfo, sub_tree, drep, 
                         hf_dcom_stdobjref_ipid, &ipid);
 
