@@ -5,11 +5,12 @@
  * AVPs.  If libxml is in the LD_LIBRARY_PATH, and dictionary.xml exists,
  * then it will not be used.
  *
- * $Id: packet-diameter-defs.h 12487 2004-11-03 23:19:09Z guy $
+ * $Id: packet-diameter-defs.h 14272 2005-05-02 18:31:15Z etxrab $
  * Generated: Fri Feb 23 13:04:15 2001
+ * * Copyright (c) 2005 by Anders Broman <anders.broman@ericsson.com>
  * References:
  * http://www.ietf.org/rfc/rfc3588.txt
- * http://www.iana.org/assignments/radius-types
+ * http://www.iana.org/assignments/radius-types updated 20 January 2005 (Not command codes)
  * http://www.ietf.org/internet-drafts/draft-ietf-aaa-diameter-cc-06.txt
  * http://www.ietf.org/internet-drafts/draft-ietf-aaa-diameter-nasreq-17.txt
  * http://www.ietf.org/internet-drafts/draft-ietf-aaa-diameter-mobileip-20.txt
@@ -269,13 +270,6 @@ static const value_string diameter_error_cause_attribute_vals[]= {
 	{507,"Request Initiated"},
 	{0,NULL}
 };
-/* Auth-Request-Type AVP Values (code 247) */
-static const value_string diameter_auth_request_type_vals[]= {
-	{1, "Authenticate Only"},
-	{2, "Authorize Only"},
-	{3, "Authorize Authenticate"},
-	{0,NULL}
-};
 /* Redirect-Host-Usage AVP Values (code 261) */
 static const value_string diameter_redirect_host_usage_vals[]= {
 	{0, "Don't Cache"},
@@ -312,6 +306,13 @@ static const value_string diameter_disconnect_cause_vals[]= {
 	{2, "Do Not Want To Talk To You"},
 	{0,NULL}
 };
+/* Auth-Request-Type AVP Values (code 274) */
+static const value_string diameter_auth_request_type_vals[]= {
+	{1, "Authenticate Only"},
+	{2, "Authorize Only"},
+	{3, "Authorize Authenticate"},
+	{0,NULL}
+};
 /* Auth-Session-State AVP Values (code 277) */
 static const value_string diameter_auth_session_state_vals[]= {
 	{0, "State Maintained"},
@@ -337,17 +338,30 @@ static const value_string diameter_termination_cause_vals[]= {
 	{8, "Session Timeout"},
 	{0,NULL}
 };
+/*   Inband-Security-Id AVP (code 299) */
+static const value_string diameter_inband_security_id_vals[]= {
+	{0, "NO_INBAND_SECURITY"},	/*[RFC3588]*/
+	{1, "TLS"},					/*[RFC3588]*/
+	{0,NULL}
+};
+
+/*
+ * XXX - this isn't an enumerated value list, it's a list of bits in a
+ * bitset.  The DIAMETER dissector doesn't have any mechanism to handle
+ * that; it would need one in order to dissect the value of this AVP
+ * correctly.
+ */
 /*    MIP-Feature-Vector AVP (code 337) */
 static const value_string diameter_mip_feature_vector_vals[]= {
 	{1, "Mobile-Node-Home-Address-Requested"},			/* RFC-ietf-aaa-diameter-mobileip-20.txt */
 	{2, "Home-Address-Allocatable-Only-in-Home-Realm"},	/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{3, "Home-Agent-Requested"},						/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{4, "Foreign-Home-Agent-Available"},				/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{5, "MN-HA-Key-Request"},							/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{6, "MN-FA-Key-Request"},							/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{7, "FA-HA-Key-Request"},							/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{8, "Home-Agent-In-Foreign-Network"},				/* RFC-ietf-aaa-diameter-mobileip-20.txt */
-	{9, "Co-Located-Mobile-Node"},						/* RFC-ietf-aaa-diameter-mobileip-20.txt */ 
+	{4, "Home-Agent-Requested"},						/* RFC-ietf-aaa-diameter-mobileip-20.txt */
+	{8, "Foreign-Home-Agent-Available"},				/* RFC-ietf-aaa-diameter-mobileip-20.txt */
+	{16, "MN-HA-Key-Request"},							/* RFC-ietf-aaa-diameter-mobileip-20.txt */
+	{32, "MN-FA-Key-Request"},							/* RFC-ietf-aaa-diameter-mobileip-20.txt */
+	{64, "FA-HA-Key-Request"},							/* RFC-ietf-aaa-diameter-mobileip-20.txt */
+	{128, "Home-Agent-In-Foreign-Network"},				/* RFC-ietf-aaa-diameter-mobileip-20.txt */
+	{256, "Co-Located-Mobile-Node"},						/* RFC-ietf-aaa-diameter-mobileip-20.txt */ 
 	{0,NULL}
 };
 /* MIP-Algorithm-Type AVP Values (code 345) */
@@ -362,6 +376,143 @@ static const value_string diameter_mip_replay_mode[] = {
 	{3, "Nonces"},
 	{0, NULL}
 };
+/*
+ * XXX - this isn't used; does there need to be an entry for AVP 406
+ * that would use it?
+ */
+/* Accounting-Auth-Method AVP Values (code 406) */
+static const value_string diameter_accounting_auth_method_vals[] = {
+	{1, "PAP"},										/* [RFC-ietf-aaa-diameter-nasreq-17.txt] */
+ 	{2, "CHAP"},									/* [RFC-ietf-aaa-diameter-nasreq-17.txt] */
+ 	{3, "MS-CHAP-1"},								/* [RFC-ietf-aaa-diameter-nasreq-17.txt] */
+	{4, "MS-CHAP-2"},								/* [RFC-ietf-aaa-diameter-nasreq-17.txt] */
+	{5, "EAP"},										/* [RFC-ietf-aaa-diameter-nasreq-17.txt] */
+	{7, "None"},									/* [RFC-ietf-aaa-diameter-nasreq-17.txt] */
+	{0, NULL}
+};
+
+/* CC-Request-Type  AVP Values (code 416)*/
+static const value_string diameter_cc_request_type_vals[]= {
+	{1, "INITIAL_REQUEST"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{2, "UPDATE_REQUEST"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{3, "TERMINATION_REQUEST"},						/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{4, "EVENT_REQUEST"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+ /* CC-Session-Failover AVP Values (code 418)*/
+static const value_string diameter_CC_Session_Failover_vals[]= {
+	{0, "FAILOVER_NOT_SUPPORTED"},					/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "FAILOVER_SUPPORTED"},						/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+ /* Check-Balance-Result AVP Values (code 422)*/
+static const value_string diameter_Check_Balance_Result_vals[]= {
+	{0, "ENOUGH_CREDIT"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "NO_CREDIT"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+ /* Credit-Control AVP Values (code 426)*/
+static const value_string diameter_credit_control_vals[]= {
+	{0, "CREDIT_AUTHORIZATION"},					/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "RE_AUTHORIZATION"},						/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+ /* Credit-Control-Failure-Handling AVP Values (code 427)*/
+static const value_string diameter_credit_control_failure_handling_vals[]= {
+	{0, "TERMINATE"},									/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "CONTINUE"},									/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{2, "RETRY_AND_TERMINATE"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/*  Direct-Debiting-Failure-Handling AVP Values (code 428)*/
+static const value_string diameter_direct_debiting_failure_handling_vals[]= {
+	{0, "TERMINATE_OR_BUFFER"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "CONTINUE"},									/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/* Redirect-Address-Type AVP Values (code 433)*/
+static const value_string diameter_redirect_address_type_vals[]= {
+	{0, "IPv4 Address"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "IPv6 Address"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{2, "URL"},											/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{3, "SIP URL"},										/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/*  Requested-Action AVP Values (436)*/
+static const value_string diameter_requested_action_vals[]= {
+	{0, "DIRECT_DEBITING"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "REFUND_ACCOUNT"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{2, "CHECK_BALANCE"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{3, "PRICE_ENQUIRY"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/*  Final-Unit-Action AVP Values (code 449)*/
+static const value_string diameter_final_unit_action_vals[]= {
+	{0, "TERMINATE"},									/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "REDIRECT"},									/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{2, "RESTRICT_ACCESS"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+ 
+/*  Subscription-Id-Type AVP Values (code 450)*/
+static const value_string diameter_subscription_id_type_vals[]= {
+	{0, "END_USER_E164"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "END_USER_IMSI"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{2, "END_USER_SIP_URI"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{3, "END_USER_NAI"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{4, "END_USER_PRIVATE"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/*  Tariff-Change-Usage AVP Values (code 452)*/
+static const value_string diameter_tariff_change_usage_vals[]= {
+	{0, "UNIT_BEFORE_TARIFF_CHANGE"},					/* [RFC-ietf-aaa-diameter-cc-06.txt] */  
+	{1, "UNIT_AFTER_TARIFF_CHANGE"},					/* [RFC-ietf-aaa-diameter-cc-06.txt]  */
+	{2, "UNIT_INDETERMINATE"},							/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/*  CC-Unit-Type AVP Values (code 454) */
+static const value_string diameter_cc_Unit_type_vals[]= {
+	{0, "TIME"},										/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "MONEY"},										/* [RFC-ietf-aaa-diameter-cc-06.txt]  */
+	{2, "TOTAL-OCTETS"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */ 
+	{3, "INPUT-OCTETS"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{4, "OUTPUT-OCTETS"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{5, "SERVICE-SPECIFIC-UNITS"},						/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+/*  Multiple-Services-Indicator AVP Values (code 455)*/ 
+static const value_string diameter_multiple_services_indicator_vals[]= {
+	{0, "MULTIPLE_SERVICES_NOT_SUPPORTED"},				/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{1, "MULTIPLE_SERVICES_SUPPORTED"},					/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+
+/*
+ * XXX - this isn't used; does there need to be an entry for AVP 459
+ * that would use it?
+ */
+/*  User-Equipment-Info-Type AVP Values (code 459)*/
+static const value_string diameter_user_equipment_info_type_vals[]= {
+	{0, "IMEISV"},										/* [RFC-ietf-aaa-diameter-cc-06.txt]  */
+	{1, "MAC"},											/* [RFC-ietf-aaa-diameter-cc-06.txt]  */
+	{2, "EUI64"},										/* [RFC-ietf-aaa-diameter-cc-06.txt] */ 
+	{3, "MODIFIED_EUI64"},								/* [RFC-ietf-aaa-diameter-cc-06.txt] */
+	{0, NULL}
+};
+
+
 /* Accounting-Record-Type AVP Values (code 480) */
 static const value_string diameter_accounting_record_type_vals[]= {
 	{1, "Event Record"},
@@ -370,37 +521,36 @@ static const value_string diameter_accounting_record_type_vals[]= {
 	{4, "Stop Record"},
 	{0,NULL}
 };
-
+/* Accounting-Realtime-Required AVP Values (code 483) */
+static const value_string diameter_accounting_realtime_required_vals[]= {
+	{1, "DELIVER_AND_GRANT"},
+	{2, "GRANT_AND_STORE"},
+	{3, "GRANT_AND_LOSE"},
+	{0,NULL}
+};
 static const value_string diameter_application_id_vals[] = {
 	{0, "Diameter Common Messages"},
 	{1, "Diameter NASREQ Application"},
 	{2, "Diameter Mobile IPv4 Application"},
 	{3, "Diameter Base Accounting"},
 	{4, "Diameter Credit-Control Application"},	/* draft-ietf-aaa-diameter-cc-06 */
-	{16777216, "3GPP Cx"},		/* 3GPP TS 29.228 and 29.229 */
-	{16777217, "3GPP Sh"},		/* 3GPP TS 29.328 and 29.329 */
-	{16777218, "3GPP Rf/Ro"},	/* 3GPP TS 32.225 */
+	{5, "Diameter EAP"},							/* [RFC-ietf-aaa-eap-10.txt] */
+	{16777216, "3GPP Cx"},						/* 3GPP TS 29.228 and 29.229 */
+	{16777217, "3GPP Sh"},						/* 3GPP TS 29.328 and 29.329 */
+	{16777218, "3GPP Re/Rf"},					/* 3GPP TS 32.225 */
+	{16777219, "3GPP Wx"},						/* 3GPP TS 29.234 */
+	{16777220, "3GPP Zn"},						/* 3GPP TS 29.109 */
+	{16777221, "3GPP Zh"},						/* 3GPP TS 29.109 */
+	{16777222, "3GPP Gq"},						/* 3GPP TS 29.209 */
+	{16777223, "3GPP Gmb"},						/* 3GPP TS 29.061 */
 	{4294967295U, "Relay Application"},
 	
 	{0, NULL}
 };
 
+
 /* Diameter Session Initiation Protocol (SIP) Application value strings */
 /* Remove comment when IANA assigned values are avalable 
-static const value_string SIP_user_data_request_type[] = {
-	{0, "COMPLETE_PROFILE"},
-	{1, "REGISTERED_PROFILE"},
-	{2, "UNREGISTERED_PROFILE"},
-	{0, NULL}
-
-};
-static const value_string SIP_user_authorization_type[] = {
-	{0, "REGISTRATION"},
-	{1, "DE_REGISTRATION"},
-	{2, "REGISTRATION_AND_CAPABILITIES"},
-	{0, NULL}
-
-};
 
 static const value_string SIP_reason_code_vals[] = {
 	{0, "PERMANENT_TERMINATION"},
@@ -410,14 +560,12 @@ static const value_string SIP_reason_code_vals[] = {
 	{0, NULL}
 
 };
-static const value_string SIP_user_data_already_available_vals[] = {
-	{0, "USER_DATA_NOT_AVAILABLE"},
-	{1, "USER_DATA_ALREADY_AVAILABLE"},
-	{0, NULL}
 
-};
+Remove comment when IANA assigned values are avalable */ 
 
-static const value_string SIP_server_assignment_type[] ={
+/* Used both in Diameter Session Initiation Protocol (SIP) Application and 3GPP Vendor
+ * Specific AVP:s TS 29 299 5.7.0 */
+static const value_string SIP_server_assignment_type_vals[] ={
 	{0, "NO_ASSIGNMENT"},
 	{1, "REGISTRATION"},
 	{2, "RE_REGISTRATION"},
@@ -431,10 +579,35 @@ static const value_string SIP_server_assignment_type[] ={
 	{10, "AUTHENTICATION_TIMEOUT"},
 	{11, "DEREGISTRATION_TOO_MUCH_DATA"},
 	{0, NULL}
+};
+static const value_string SIP_user_authorization_type[] = {
+	{0, "REGISTRATION"},
+	{1, "DE_REGISTRATION"},
+	{2, "REGISTRATION_AND_CAPABILITIES"},
+	{0, NULL}
 
 };
+static const value_string SIP_user_data_request_type[] = {
+	{0, "COMPLETE_PROFILE"},
+	{1, "REGISTERED_PROFILE"},
+	{2, "UNREGISTERED_PROFILE"},
+	{0, NULL}
 
- Remove comment when IANA assigned values are avalable */ 
+};
+static const value_string ThreeGPP_reason_code_vals[] = {
+	{0, "PERMANENT_TERMINATION"},
+	{1, "NEW_SIP_SERVER_ASSIGNED "},
+	{2, "SERVER_CHANGE"},
+	{3, "REMOVE_S-CSCF "},
+	{0, NULL}
+
+};
+static const value_string SIP_user_data_already_available_vals[] = {
+	{0, "USER_DATA_NOT_AVAILABLE"},
+	{1, "USER_DATA_ALREADY_AVAILABLE"},
+	{0, NULL}
+
+};
 
 /*
  * The Result-Code data field contains an IANA-managed 32-bit address
@@ -524,7 +697,6 @@ static const value_string diameter_result_code_vals[] = {
 	/* draft-ietf-aaa-diameter-cc-03.txt */
 	{5030, "DIAMETER_USER_UNKNOWN"},
 	{5031, "DIAMETER_RATING_FAILED"},
-	{5032, "DIAMETER_CREDIT_LIMIT_REACHED"},
 
 	/* draft-ietf-aaa-diameter-sip-app-01.txt numbers not yet allocated by IANA 
 
@@ -540,7 +712,7 @@ static const value_string diameter_result_code_vals[] = {
 	*/
 	{0, NULL}
 };
-
+/* TS 29.229 6.4.0 */
 static const value_string diameter_exp_result_code_vals[] = {
         {2001, "DIAMETER_FIRST_REGISTRATION"},
         {2002, "DIAMETER_SUBSEQUENT_REGISTRATION"},
@@ -551,12 +723,13 @@ static const value_string diameter_exp_result_code_vals[] = {
         {5002, "DIAMETER_ERROR_IDENTITIES_DONT_MATCH"},
         {5003, "DIAMETER_ERROR_IDENTITY_NOT_REGISTERED"},
         {5004, "DIAMETER_ERROR_ROAMING_NOT_ALLOWED"},
-        {5005, "DIAMETER_ERROR_ROAMING_IDENTITY_ALREADY_REGISTERED"},
+        {5005, "DIAMETER_ERROR_IDENTITY_ALREADY_REGISTERED "},
         {5006, "DIAMETER_ERROR_ROAMING_AUTH_SCHEME_NOT_SUPPORTED"},
         {5007, "DIAMETER_ERROR_IN_ASSIGNMENT_TYPE"},
         {5008, "DIAMETER_ERROR_TOO_MUCH_DATA"},
         {5009, "DIAMETER_ERROR_NOT_SUPPORTED_USER_DATA"},
-
+        {5010, "DIAMETER_MISSING_USER_ID"},
+        {5011, "DIAMETER_ERROR_FEATURE_UNSUPPORTED"},
 	{0, NULL}
 };
 	 
@@ -672,49 +845,47 @@ static struct old_avp_info old_diameter_avps[] = {
    241-255	Reserved				 [RFC2058]   
 */
 	/* Diameter AVPs */
-    { 482, "Accounting-Interim-Interval", DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 483, "Accounting-Realtime-Required",DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 485, "Accounting-Record-Number",    DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 480, "Accounting-Record-Type",      DIAMETER_ENUMERATED,  diameter_accounting_record_type_vals},
-    { 287, "Accounting-Sub-Session-Id",   DIAMETER_UNSIGNED64,  (value_string *)NULL},
-    { 259, "Acct-Application-Id",         DIAMETER_UNSIGNED32,  diameter_application_id_vals},
-    { 275, "Alternate-Peer",              DIAMETER_IDENTITY,    (value_string *)NULL},
-    { 258, "Auth-Application-Id",         DIAMETER_UNSIGNED32,  diameter_application_id_vals},
-    { 247, "Auth-Request-Type",           DIAMETER_ENUMERATED,  diameter_auth_request_type_vals},
-    { 291, "Authorization-Lifetime",      DIAMETER_INTEGER32,   (value_string *)NULL},
-    { 276, "Auth-Grace-Period",           DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 277, "Auth-Session-State",          DIAMETER_ENUMERATED,  diameter_auth_session_state_vals},
-    { 285, "Re-Auth-Request-Type",        DIAMETER_ENUMERATED,  diameter_re_auth_request_type_vals},
-    { 293, "Destination-Host",            DIAMETER_IDENTITY,    (value_string *)NULL},
-    { 283, "Destination-Realm",           DIAMETER_UTF8STRING,  (value_string *)NULL},
-    { 273, "Disconnect-Cause",            DIAMETER_ENUMERATED,  diameter_disconnect_cause_vals},
-    { 281, "Error-Message",               DIAMETER_UTF8STRING,  (value_string *)NULL},
-    { 294, "Error-Reporting-Host",        DIAMETER_IDENTITY,    (value_string *)NULL},
-    { 279, "Failed-AVP",                  DIAMETER_OCTET_STRING,(value_string *)NULL},
-    { 267, "Firmware-Revision",           DIAMETER_UNSIGNED32,  (value_string *)NULL},
     { 257, "Host-IP-Address",             DIAMETER_IP_ADDRESS,  (value_string *)NULL},
-    { 272, "Multi-Round-Time-Out",        DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 264, "Origin-Host",                 DIAMETER_IDENTITY,    (value_string *)NULL},
-    { 296, "Origin-Realm",                DIAMETER_UTF8STRING,  (value_string *)NULL},
-    { 278, "Origin-State-Id",             DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 269, "Product-Name",                DIAMETER_UTF8STRING,  (value_string *)NULL},
-    { 280, "Proxy-Host",                  DIAMETER_IDENTITY,    (value_string *)NULL},
-    { 284, "Proxy-Info",                  DIAMETER_GROUPED,     (value_string *)NULL},
-    { 292, "Redirect-Host",               DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 258, "Auth-Application-Id",         DIAMETER_UNSIGNED32,  diameter_application_id_vals},
+    { 259, "Acct-Application-Id",         DIAMETER_UNSIGNED32,  diameter_application_id_vals},
+    { 260, "Vendor-Specific-Application-Id", DIAMETER_GROUPED, (value_string *)NULL},
     { 261, "Redirect-Host-Usage",         DIAMETER_ENUMERATED,  diameter_redirect_host_usage_vals},
     { 262, "Redirect-Max-Cache-Time",     DIAMETER_UNSIGNED32,  (value_string *)NULL},
-    { 268, "Result-Code",                 DIAMETER_ENUMERATED,  diameter_result_code_vals},
-    { 282, "Route-Record",                DIAMETER_IDENTITY,    (value_string *)NULL},
     { 263, "Session-Id",                  DIAMETER_SESSION_ID,  (value_string *)NULL},
+    { 264, "Origin-Host",                 DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 265, "Supported-Vendor-Id",         DIAMETER_UNSIGNED32,  (value_string *)NULL},
+    { 266, "Vendor-Id",                   DIAMETER_ENUMERATED,  sminmpec_values},
+    { 267, "Firmware-Revision",           DIAMETER_UNSIGNED32,  (value_string *)NULL},
+    { 268, "Result-Code",                 DIAMETER_ENUMERATED,  diameter_result_code_vals},
+    { 269, "Product-Name",                DIAMETER_UTF8STRING,  (value_string *)NULL},
     { 270, "Session-Binding",             DIAMETER_UNSIGNED32,  diameter_session_binding_vals},
     { 271, "Session-Server-Failover",     DIAMETER_ENUMERATED,  diameter_session_server_failover_vals},
+    { 272, "Multi-Round-Time-Out",        DIAMETER_UNSIGNED32,  (value_string *)NULL},
+    { 273, "Disconnect-Cause",            DIAMETER_ENUMERATED,  diameter_disconnect_cause_vals},
+    { 274, "Auth-Request-Type",           DIAMETER_ENUMERATED,  diameter_auth_request_type_vals},
+    { 275, "Alternate-Peer",              DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 276, "Auth-Grace-Period",           DIAMETER_UNSIGNED32,  (value_string *)NULL},
+    { 277, "Auth-Session-State",          DIAMETER_ENUMERATED,  diameter_auth_session_state_vals},
+    { 278, "Origin-State-Id",             DIAMETER_UNSIGNED32,  (value_string *)NULL},
+    { 279, "Failed-AVP",                  DIAMETER_OCTET_STRING,(value_string *)NULL},
+    { 280, "Proxy-Host",                  DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 281, "Error-Message",               DIAMETER_UTF8STRING,  (value_string *)NULL},
+    { 282, "Route-Record",                DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 283, "Destination-Realm",           DIAMETER_UTF8STRING,  (value_string *)NULL},
+    { 284, "Proxy-Info",                  DIAMETER_GROUPED,     (value_string *)NULL},
+    { 285, "Re-Auth-Request-Type",        DIAMETER_ENUMERATED,  diameter_re_auth_request_type_vals},
     { 286, "Source-Route",                DIAMETER_IDENTITY,    (value_string *)NULL},
-    { 265, "Supported-Vendor-Id",         DIAMETER_UNSIGNED32,  (value_string *)NULL},
+    { 287, "Accounting-Sub-Session-Id",   DIAMETER_UNSIGNED64,  (value_string *)NULL},
+    { 291, "Authorization-Lifetime",      DIAMETER_INTEGER32,   (value_string *)NULL},
+    { 292, "Redirect-Host",               DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 293, "Destination-Host",            DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 294, "Error-Reporting-Host",        DIAMETER_IDENTITY,    (value_string *)NULL},
+    { 296, "Origin-Realm",                DIAMETER_UTF8STRING,  (value_string *)NULL},
     { 295, "Termination-Cause",           DIAMETER_ENUMERATED,  diameter_termination_cause_vals},
-    { 266, "Vendor-Id",                   DIAMETER_ENUMERATED,  sminmpec_values},
-    { 260, "Vendor-Specific-Application-Id", DIAMETER_GROUPED, (value_string *)NULL},
-    { 297, "Experimental-Result",          DIAMETER_GROUPED,     (value_string *)NULL},
-    { 298, "Experimental-Result-Code",     DIAMETER_ENUMERATED,  diameter_exp_result_code_vals},
+    { 297, "Experimental-Result",			DIAMETER_GROUPED,     (value_string *)NULL},
+    { 298, "Experimental-Result-Code",		DIAMETER_ENUMERATED,	diameter_exp_result_code_vals},
+    { 299, "Inband-Security-Id",			DIAMETER_ENUMERATED,	diameter_inband_security_id_vals},	/* [RFC3588] */
+    { 300, "E2E-Sequence",					DIAMETER_GROUPED,     (value_string *)NULL},				/* [RFC3588] */
 
 /* Diameter Mobile IP AVPs */
     { 318, "MIP-FA-to-HA-SPI",            DIAMETER_UNSIGNED32,     (value_string *)NULL},
@@ -722,6 +893,7 @@ static struct old_avp_info old_diameter_avps[] = {
     { 320, "MIP-Reg-Request",             DIAMETER_MIP_REG_REQ,    (value_string *)NULL},
     { 321, "MIP-Reg-Reply",               DIAMETER_OCTET_STRING,   (value_string *)NULL},
     { 322, "MIP-MN-AAA-Auth",             DIAMETER_GROUPED,        (value_string *)NULL},
+    { 323, "MIP-HA-to-FA-SPI",				DIAMETER_UNSIGNED32,	(value_string *)NULL},
     { 325, "MIP-MN-to-FA-MSA",            DIAMETER_GROUPED,        (value_string *)NULL},
     { 326, "MIP-FA-to-MN-MSA",            DIAMETER_GROUPED,        (value_string *)NULL},
     { 328, "MIP-FA-to-HA-MSA",            DIAMETER_GROUPED,        (value_string *)NULL},
@@ -751,33 +923,27 @@ static struct old_avp_info old_diameter_avps[] = {
 	{ 413, "CC-Money",                          DIAMETER_GROUPED    ,		(value_string *)NULL}, 
 	{ 414, "CC-Output-Octets",                  DIAMETER_UNSIGNED64 ,		(value_string *)NULL}, 
 	{ 415, "CC-Request-Number",                 DIAMETER_UNSIGNED32 ,		(value_string *)NULL}, 
-	{ 416, "CC-Request-Type",                   DIAMETER_ENUMERATED ,		(value_string *)NULL}, 
+	{ 416, "CC-Request-Type",                   DIAMETER_ENUMERATED , diameter_cc_request_type_vals}, 
 	{ 417, "CC-Service-Specific-Units",         DIAMETER_UNSIGNED64 ,		(value_string *)NULL},  
-	{ 418, "CC-Session-Failover",				DIAMETER_ENUMERATED ,		(value_string *)NULL}, 
+	{ 418, "CC-Session-Failover",				DIAMETER_ENUMERATED , diameter_CC_Session_Failover_vals}, 
 	{ 419, "CC-Sub-Session-Id",                 DIAMETER_UNSIGNED64 ,		(value_string *)NULL},
 	{ 420, "CC-Time",                           DIAMETER_UNSIGNED32 ,		(value_string *)NULL},
 	{ 421, "CC-Total-Octets",                   DIAMETER_UNSIGNED64 ,		(value_string *)NULL},
-	{ 454, "CC-Unit-Type",                      DIAMETER_ENUMERATED ,		(value_string *)NULL},
-	{ 422, "Check-Balance-Result",              DIAMETER_ENUMERATED ,		(value_string *)NULL},
+	{ 422, "Check-Balance-Result",              DIAMETER_ENUMERATED , diameter_Check_Balance_Result_vals},
 	{ 423, "Cost-Information",                  DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 424, "Cost-Unit",                         DIAMETER_UTF8STRING ,		(value_string *)NULL},
-	{ 426, "Credit-Control",                    DIAMETER_ENUMERATED ,		(value_string *)NULL},
-	{ 427, "Credit-Control-Failure-Handling",   DIAMETER_ENUMERATED ,		(value_string *)NULL},
+	{ 426, "Credit-Control",                    DIAMETER_ENUMERATED , diameter_credit_control_vals},
+	{ 427, "Credit-Control-Failure-Handling",   DIAMETER_ENUMERATED , diameter_credit_control_failure_handling_vals},
 	{ 425, "Currency-Code",                     DIAMETER_UNSIGNED32 ,		(value_string *)NULL},
-	{ 428, "Direct-Debiting-Failure-Handling",  DIAMETER_ENUMERATED ,		(value_string *)NULL},
+	{ 428, "Direct-Debiting-Failure-Handling",  DIAMETER_ENUMERATED , diameter_direct_debiting_failure_handling_vals},
 	{ 429, "Exponent",                          DIAMETER_INTEGER32  ,		(value_string *)NULL},
-	{ 449, "Final-Unit-Action",                 DIAMETER_ENUMERATED ,		(value_string *)NULL},
 	{ 430, "Final-Unit-Indication",             DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 431, "Granted-Service-Unit",              DIAMETER_GROUPED    ,		(value_string *)NULL},
-	{ 453, "G-S-U-Pool-Identifier",             DIAMETER_UNSIGNED32 ,		(value_string *)NULL},
-	{ 457, "G-S-U-Pool-Reference",              DIAMETER_GROUPED    ,		(value_string *)NULL},
-	{ 456, "Multiple-Services-Credit-Control",  DIAMETER_GROUPED    ,		(value_string *)NULL},
-	{ 455, "Multiple-Services-Indicator",       DIAMETER_ENUMERATED ,		(value_string *)NULL},
 	{ 432, "Rating-Group",                      DIAMETER_UNSIGNED32 ,		(value_string *)NULL},
-	{ 433, "Redirect-Address-Type",             DIAMETER_ENUMERATED ,		(value_string *)NULL},
+	{ 433, "Redirect-Address-Type",             DIAMETER_ENUMERATED , diameter_redirect_address_type_vals},
 	{ 434, "Redirect-Server",                   DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 435, "Redirect-Server-Address",           DIAMETER_UTF8STRING ,		(value_string *)NULL},
-	{ 436, "Requested-Action",                  DIAMETER_ENUMERATED ,		(value_string *)NULL},
+	{ 436, "Requested-Action",                  DIAMETER_ENUMERATED , diameter_requested_action_vals},
 	{ 437, "Requested-Service-Unit",            DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 438, "Restriction-Filter-Rule",           DIAMETER_IP_FILTER_RULE,	(value_string *)NULL},
 	{ 439, "Service-Identifier",                DIAMETER_UTF8STRING ,		(value_string *)NULL},
@@ -786,13 +952,26 @@ static struct old_avp_info old_diameter_avps[] = {
 	{ 442, "Service-Parameter-Value",           DIAMETER_OCTET_STRING,		(value_string *)NULL},
 	{ 443, "Subscription-Id",                   DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 444, "Subscription-Id-Data",              DIAMETER_UTF8STRING ,		(value_string *)NULL},
-	{ 450, "Subscription-Id-Type",              DIAMETER_ENUMERATED ,		(value_string *)NULL},
-	{ 452, "Tariff-Change-Usage",               DIAMETER_ENUMERATED ,		(value_string *)NULL},
-	{ 451, "Tariff-Time-Change",                DIAMETER_TIME,				(value_string *)NULL},
 	{ 445, "Unit-Value",                        DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 446, "Used-Service-Unit",                 DIAMETER_GROUPED    ,		(value_string *)NULL},
 	{ 447, "Value-Digits",                      DIAMETER_INTEGER64  ,		(value_string *)NULL},
 	{ 448, "Validity-Time",                     DIAMETER_UNSIGNED32 ,		(value_string *)NULL},
+	{ 449, "Final-Unit-Action",                 DIAMETER_ENUMERATED ,		diameter_final_unit_action_vals},
+	{ 450, "Subscription-Id-Type",              DIAMETER_ENUMERATED ,		diameter_subscription_id_type_vals},
+	{ 451, "Tariff-Time-Change",                DIAMETER_TIME,				(value_string *)NULL},
+	{ 452, "Tariff-Change-Usage",               DIAMETER_ENUMERATED ,		diameter_tariff_change_usage_vals},
+	{ 453, "G-S-U-Pool-Identifier",             DIAMETER_UNSIGNED32 ,		(value_string *)NULL},
+	{ 454, "CC-Unit-Type",                      DIAMETER_ENUMERATED ,		diameter_cc_Unit_type_vals},
+	{ 455, "Multiple-Services-Indicator",       DIAMETER_ENUMERATED ,		diameter_multiple_services_indicator_vals},
+	{ 456, "Multiple-Services-Credit-Control",  DIAMETER_GROUPED    ,		(value_string *)NULL},
+	{ 457, "G-S-U-Pool-Reference",              DIAMETER_GROUPED    ,		(value_string *)NULL},
+    { 482, "Accounting-Interim-Interval",		DIAMETER_UNSIGNED32,		(value_string *)NULL},
+    { 483, "Accounting-Realtime-Required",		DIAMETER_UNSIGNED32,		(value_string *)NULL},
+    { 485, "Accounting-Record-Number",			DIAMETER_UNSIGNED32,		(value_string *)NULL},
+    { 480, "Accounting-Record-Type",			DIAMETER_ENUMERATED,		diameter_accounting_record_type_vals},
+    { 483, "Accounting-Realtime-Required",      DIAMETER_ENUMERATED,		diameter_accounting_realtime_required_vals},
+    { 485, "Accounting-Record-Number",			DIAMETER_UNSIGNED32,		(value_string *)NULL},
+    { 487, "Accounting-Sub-Session-Id",			DIAMETER_ENUMERATED,		(value_string *)NULL},
 
 
 /* draft-ietf-aaa-diameter-sip-app-01.txt AVP codes to be allocated
@@ -813,7 +992,7 @@ static struct old_avp_info old_diameter_avps[] = {
 	{ xx15, "SIP-Authentication-Context",		DIAMETER_GROUPED,			(value_string *)NULL},
 	{ xx16, "SIP-Confidentiality-Key",			DIAMETER_OCTET_STRING,		(value_string *)NULL},
 	{ xx17, "SIP-Integrity-Key",				DIAMETER_OCTET_STRING,		(value_string *)NULL},
-	{ xx18, "SIP-Server-Assignment-Type",		DIAMETER_ENUMERATED,		SIP_server_assignment_type},
+	{ xx18, "SIP-Server-Assignment-Type",		DIAMETER_ENUMERATED,		SIP_server_assignment_type_vals},
 	{ xx19, "SIP-Deregistration-Reason",		DIAMETER_GROUPED,			(value_string *)NULL},
 	{ xx20, "SIP-Reason-Code",					DIAMETER_ENUMERATED,		SIP_reason_code_vals},
 	{ xx21, "SIP-Reason-Info",					DIAMETER_UTF8STRING,		(value_string *)NULL},
@@ -829,30 +1008,65 @@ static struct old_avp_info old_diameter_avps[] = {
 	{0, (char *)NULL, 0, (value_string*)NULL}
 };
 
-
-
+ /* The following table describes the Diameter AVPs defined for the Cx interface protocol.
+  * The Vendor-Id header of all AVPs defined in this specification shall be set to 
+  * 3GPP (10415).
+  * TS 29.299 5.7.0
+  */
+static struct old_avp_info ThreeGPP_vendor_diameter_avps[] = {
+	{ 1, "Visited-Network-Identifier",				DIAMETER_OCTET_STRING,		(value_string *)NULL},
+	{ 2, "Public-Identity",							DIAMETER_PUBLIC_ID,			(value_string *)NULL},
+	{ 3, "Server-Name",								DIAMETER_UTF8STRING,		(value_string *)NULL},
+	{ 4, "Server-Capabilities",						DIAMETER_GROUPED,			(value_string *)NULL},
+	{ 5, "Mandatory-Capability",					DIAMETER_UNSIGNED32,		(value_string *)NULL},
+	{ 6, "Optional-Capability",						DIAMETER_UNSIGNED32,		(value_string *)NULL},
+	{ 7, "User-Data",								DIAMETER_OCTET_STRING,		(value_string *)NULL},
+	{ 8, "SIP-Number-Auth-Items",					DIAMETER_UNSIGNED32,		(value_string *)NULL},
+	{ 9, "SIP-Authentication-Scheme",				DIAMETER_UTF8STRING,		(value_string *)NULL},
+	{ 10, "SIP-Authenticate",						DIAMETER_OCTET_STRING,		(value_string *)NULL},
+	{ 11, "SIP-Authorization",						DIAMETER_OCTET_STRING,		(value_string *)NULL},
+	{ 12, "SIP-Authentication-Context",				DIAMETER_UTF8STRING,		(value_string *)NULL},
+	{ 13, "SIP-Auth-Data-Item",						DIAMETER_GROUPED,			(value_string *)NULL},
+	{ 14, "SIP-Item-Number",						DIAMETER_UNSIGNED32,		(value_string *)NULL},
+	{ 15, "Server-Assignment-Type",					DIAMETER_ENUMERATED,		SIP_server_assignment_type_vals},
+	{ 16, "Deregistration-Reason",					DIAMETER_GROUPED,			(value_string *)NULL},
+	{ 17, "Reason-Code",							DIAMETER_ENUMERATED,		ThreeGPP_reason_code_vals},
+	{ 18, "Reason-Info",							DIAMETER_UTF8STRING,		(value_string *)NULL},
+	{ 19, "Charging-Information",					DIAMETER_GROUPED,			(value_string *)NULL},
+	{ 20, "Primary-Event-Charging-Function-Name",	DIAMETER_URI,				(value_string *)NULL},
+	{ 21, "Secondary-Event-Charging-Function-Name",	DIAMETER_URI,				(value_string *)NULL},
+	{ 22, "Primary-Charging-Collection-Function-Name",	 DIAMETER_URI,			(value_string *)NULL},
+	{ 23, "Secondary-Charging-Collection-Function-Name", DIAMETER_URI,			(value_string *)NULL},
+	{ 24, "User-Authorization-Type",				DIAMETER_ENUMERATED,		SIP_user_authorization_type},
+	{ 25, "User-Data-Request-Type",					DIAMETER_ENUMERATED,		SIP_user_data_request_type},
+	{ 26, "User-Data-Already-Available",			DIAMETER_ENUMERATED,		SIP_user_data_already_available_vals},
+	{ 27, "Confidentiality-Key",					DIAMETER_OCTET_STRING,		(value_string *)NULL},
+	{ 28, "Integrity-Key",							DIAMETER_OCTET_STRING,		(value_string *)NULL},
+	{0, (char *)NULL, 0, (value_string*)NULL}
+};
 static const value_string diameter_command_code_vals[] = {
 
 	/* Base Protocol */
 	{257, "Capabilities-Exchange"},
 	{258, "Re-Auth"},
+	{260, "AA-Mobile-Node"},			/* [RFC-ietf-aaa-diameter-mobileip-20.txt]  */
+	{262, "Home-Agent-MIP"},			/* [RFC-ietf-aaa-diameter-mobileip-20.txt]  */
+	{265, "AAR / AAA"},					/* [RFC-ietf-aaa-diameter-nasreq-17.txt]	*/
+	{265, "DER / DEA"},					/* [RFC-ietf-aaa-eap-10.txt]				*/
 	{271, "Accounting"},
+	{272, "Credit-Control"},					/* [RFC-ietf-aaa-diameter-cc-06.txt]		*/
 	{274, "Abort-Session"},
 	{275, "Session-Termination"},
 	{280, "Device-Watchdog"},
-	{282, "Disconnect-Peer"},
-	{300, "Test-Auth"},
-	/* Mip Protocol */
-	{260, "AA-Mobile-Node"},
-	{262, "Home-Agent-MIP"},
-	/* Nasreq Protocol */
-	{265, "AA"},
-	{268, "Diameter-EAP"},
-	/* Credit-Control Application */
-	{272, "Credit-Control"},
-	/* draft-ietf-aaa-diameter-cms-sec-04 */
-	{304,  "Diameter-Security-Association"},
-	{305,  "Proxy-Diameter-Security-Association"},
+	{282, "Disconnect-Peer"},			/* [RFC3588] */
+	/* 300-313   Allocated for 3GPP (TS 29.229 6.40)*/
+	{300, "User-Authorization"},
+	{301, "Server-Assignment"},
+	{302, "Location-Info"},
+	{303, "Multimedia-Auth"},
+	{304, "Registration-Termination"},
+	{305, "Push-Profile"},
+
 	/* Session Initiation Protocol (SIP) Application, numbers not yet assigned by IANA 
 	{aaa, "User-Authorization"},
 	{bbb, "Server-Assignment"},
