@@ -2,7 +2,7 @@
  * Routines for FC Extended Link Services
  * Copyright 2001, Dinesh G Dutt <ddutt@cisco.com>
  *
- * $Id: packet-fcels.c 12890 2004-12-31 02:08:01Z guy $
+ * $Id: packet-fcels.c 14027 2005-04-08 10:28:44Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -476,6 +476,9 @@ construct_rcptctl_string (guint16 flag, gchar *flagstr, guint8 opcode)
     }
 }
 
+/* Maximum length of possible string from, construct_*_string
+ * 296 bytes, FIX possible buffer overflow */
+#define FCELS_LOGI_MAXSTRINGLEN 512
 
 static void
 dissect_fcels_logi (tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
@@ -487,7 +490,7 @@ dissect_fcels_logi (tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
         class;
     proto_tree *logi_tree, *cmnsvc_tree;
     proto_item *subti;
-    gchar flagstr[256];
+    gchar flagstr[FCELS_LOGI_MAXSTRINGLEN];
     guint16 flag;
     
     if (tree) {
@@ -1064,6 +1067,10 @@ dissect_fcels_lsts (tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
     }
 }
 
+/* Maximum length of possible string from, dissect_fcels_prlilo_payload
+ * 119 bytes, FIX possible buffer overflow */
+#define FCELS_PRLILO_MAXSTRINGLEN 256
+
 static void
 dissect_fcels_prlilo_payload (tvbuff_t *tvb, packet_info *pinfo _U_,
                               guint8 isreq, proto_item *ti, guint8 opcode)
@@ -1074,7 +1081,7 @@ dissect_fcels_prlilo_payload (tvbuff_t *tvb, packet_info *pinfo _U_,
     proto_tree *prli_tree, *svcpg_tree;
     int num_svcpg, payload_len, i, flag;
     proto_item *subti;
-    gchar flagstr[100];
+    gchar flagstr[FCELS_PRLILO_MAXSTRINGLEN];
 
     /* We're assuming that we're invoked only if tree is not NULL i.e.
      * we don't do the usual "if (tree)" check here, the caller must.
@@ -1531,12 +1538,12 @@ dissect_fcels (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         else {
             options = NO_PORT2;
         }
-        conversation = find_conversation (&pinfo->dst, &pinfo->src,
+        conversation = find_conversation (pinfo->fd->num, &pinfo->dst, &pinfo->src,
                                           pinfo->ptype, pinfo->oxid,
                                           pinfo->rxid, options);
             
         if (!conversation) {
-            conversation = conversation_new (&pinfo->dst, &pinfo->src,
+            conversation = conversation_new (pinfo->fd->num, &pinfo->dst, &pinfo->src,
                                              pinfo->ptype, pinfo->oxid,
                                              pinfo->rxid, options);
         }
@@ -1566,7 +1573,7 @@ dissect_fcels (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         isreq = FC_ELS_RPLY;
 
         options = NO_PORT2;
-        conversation = find_conversation (&pinfo->dst, &pinfo->src,
+        conversation = find_conversation (pinfo->fd->num, &pinfo->dst, &pinfo->src,
                                           pinfo->ptype, pinfo->oxid,
                                           pinfo->rxid, options);
         if (!conversation) {
@@ -1576,7 +1583,7 @@ dissect_fcels (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             addrdata[0] = addrdata[1] = 0;
             addrdata[2] = pinfo->dst.data[2];
             SET_ADDRESS (&dstaddr, AT_FC, 3, addrdata);
-            conversation = find_conversation (&dstaddr, &pinfo->src,
+            conversation = find_conversation (pinfo->fd->num, &dstaddr, &pinfo->src,
                                               pinfo->ptype, pinfo->oxid,
                                               pinfo->rxid, options);
         }
@@ -1584,7 +1591,7 @@ dissect_fcels (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         if (!conversation) {
             /* Finally check for FLOGI with both NO_PORT2 and NO_ADDR2 set */
             options = NO_ADDR2 | NO_PORT2;
-            conversation = find_conversation (&pinfo->src, &pinfo->dst,
+            conversation = find_conversation (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                               pinfo->ptype, pinfo->oxid,
                                               pinfo->rxid, options);
             if (!conversation) {
