@@ -2,18 +2,12 @@
  * Routines for Fibre Channel Protocol for SCSI (FCP)
  * Copyright 2001, Dinesh G Dutt <ddutt@cisco.com>
  *
- * $Id: packet-fcp.c 12890 2004-12-31 02:08:01Z guy $
+ * $Id: packet-fcp.c 13917 2005-03-26 03:56:54Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
  *
- * Copied from WHATEVER_FILE_YOU_USED (where "WHATEVER_FILE_YOU_USED"
- * is a dissector file; if you just copied this from README.developer,
- * don't bother with the "Copied from" - you don't even need to put
- * in a "Copied from" if you copied an existing dissector, especially
- * if the bulk of the code in the new dissector is your code)
- * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -245,6 +239,7 @@ dissect_fcp_cmnd (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     fcp_conv_data_t *cdata;
     fcp_conv_key_t ckey, *req_key;
     scsi_task_id_t task_key;
+    guint16 lun=0xffff;
 
     /* Determine the length of the FCP part of the packet */
     flags = tvb_get_guint8 (tvb, offset+10);
@@ -262,11 +257,11 @@ dissect_fcp_cmnd (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* by the data that is sent back or sent next by the initiator as part */
     /* of this command. The state is destroyed in the response dissector */
     
-    conversation = find_conversation (&pinfo->src, &pinfo->dst,
+    conversation = find_conversation (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                       pinfo->ptype, pinfo->oxid,
                                       pinfo->rxid, NO_PORT2);
     if (!conversation) {
-        conversation = conversation_new (&pinfo->src, &pinfo->dst,
+        conversation = conversation_new (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                          pinfo->ptype, pinfo->oxid,
                                          pinfo->rxid, NO_PORT2);
     }
@@ -278,6 +273,17 @@ dissect_fcp_cmnd (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     
     cdata = (fcp_conv_data_t *)g_hash_table_lookup (fcp_req_hash,
                                                     &ckey);
+    /*
+     * XXX - the fetch of the fcp_dl value will throw an exception on
+     * a short frame before we get a chance to dissect the stuff before
+     * it.
+     *
+     * XXX - this doesn't appear to store the data length with the
+     * FCP packet with the data, so this might not work correctly
+     * if you select a command packet, select the corresponding data
+     * packet, and then select another data packet with a different
+     * length.
+     */
     if (cdata) {
         /* Since we never free the memory used by an exchange, this maybe a
          * case of another request using the same exchange as a previous
@@ -299,47 +305,53 @@ dissect_fcp_cmnd (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
         g_hash_table_insert (fcp_req_hash, req_key, cdata);
     }
     
+    /* XXX this one is redundant  right?  ronnie
     dissect_scsi_cdb (tvb, pinfo, fcp_tree, offset+12, 16+add_len,
-                      SCSI_DEV_UNKNOWN);
+                      SCSI_DEV_UNKNOWN, lun);
+    */
 
     if (tree) {
         ti = proto_tree_add_protocol_format (tree, proto_fcp, tvb, 0, len,
                                              "FCP_CMND");
         fcp_tree = proto_item_add_subtree (ti, ett_fcp);
-        proto_tree_add_uint_hidden (fcp_tree, hf_fcp_type, tvb, offset, 0, 0);
-                                    
-        lun0 = tvb_get_guint8 (tvb, offset);
-
-        /* Display single-level LUNs in decimal for clarity */
-        /* I'm taking a shortcut here by assuming that if the first byte of the
-         * LUN field is 0, it is a single-level LUN. This is not true. For a
-         * real single-level LUN, all 8 bytes except byte 1 must be 0.
-         */
-        if (lun0) {
-            cdata->fcp_lun = -1;
-            proto_tree_add_item (fcp_tree, hf_fcp_multilun, tvb, offset, 8, 0);
-        }
-        else {
-            cdata->fcp_lun = tvb_get_guint8 (tvb, offset+1);
-            proto_tree_add_item (fcp_tree, hf_fcp_singlelun, tvb, offset+1,
-                                 1, 0);
-        }
-
-        proto_tree_add_item (fcp_tree, hf_fcp_crn, tvb, offset+8, 1, 0);
-        proto_tree_add_item (fcp_tree, hf_fcp_taskattr, tvb, offset+9, 1, 0);
-        proto_tree_add_uint_format (fcp_tree, hf_fcp_taskmgmt, tvb, offset+10,
-                                    1, flags,
-                                    "Task Management Flags: 0x%x (%s)",
-                                    flags,
-                                    task_mgmt_flags_to_str (flags, str));
-        proto_tree_add_item (fcp_tree, hf_fcp_addlcdblen, tvb, offset+11, 1, 0);
-        proto_tree_add_item (fcp_tree, hf_fcp_rddata, tvb, offset+11, 1, 0);
-        proto_tree_add_item (fcp_tree, hf_fcp_wrdata, tvb, offset+11, 1, 0);
-        dissect_scsi_cdb (tvb, pinfo, tree, offset+12, 16+add_len,
-                          SCSI_DEV_UNKNOWN);
-        proto_tree_add_item (fcp_tree, hf_fcp_dl, tvb, offset+12+16+add_len,
-                             4, 0);
+    }    
+    proto_tree_add_uint_hidden (fcp_tree, hf_fcp_type, tvb, offset, 0, 0);
+    
+    lun0 = tvb_get_guint8 (tvb, offset);
+    
+    /* Display single-level LUNs in decimal for clarity */
+    /* I'm taking a shortcut here by assuming that if the first byte of the
+     * LUN field is 0, it is a single-level LUN. This is not true. For a
+     * real single-level LUN, all 8 bytes except byte 1 must be 0.
+     */
+    if (lun0) {
+      cdata->fcp_lun = -1;
+      proto_tree_add_item (fcp_tree, hf_fcp_multilun, tvb, offset, 8, 0);
+      lun=tvb_get_guint8(tvb, offset)&0x3f;
+      lun<<=8;
+      lun|=tvb_get_guint8(tvb, offset+1);
     }
+    else {
+      cdata->fcp_lun = tvb_get_guint8 (tvb, offset+1);
+      proto_tree_add_item (fcp_tree, hf_fcp_singlelun, tvb, offset+1,
+			   1, 0);
+      lun=tvb_get_guint8(tvb, offset+1);
+    }
+
+    proto_tree_add_item (fcp_tree, hf_fcp_crn, tvb, offset+8, 1, 0);
+    proto_tree_add_item (fcp_tree, hf_fcp_taskattr, tvb, offset+9, 1, 0);
+    proto_tree_add_uint_format (fcp_tree, hf_fcp_taskmgmt, tvb, offset+10,
+				1, flags,
+				"Task Management Flags: 0x%x (%s)",
+				flags,
+				task_mgmt_flags_to_str (flags, str));
+    proto_tree_add_item (fcp_tree, hf_fcp_addlcdblen, tvb, offset+11, 1, 0);
+    proto_tree_add_item (fcp_tree, hf_fcp_rddata, tvb, offset+11, 1, 0);
+    proto_tree_add_item (fcp_tree, hf_fcp_wrdata, tvb, offset+11, 1, 0);
+    dissect_scsi_cdb (tvb, pinfo, tree, offset+12, 16+add_len,
+		      SCSI_DEV_UNKNOWN, lun);
+    proto_tree_add_item (fcp_tree, hf_fcp_dl, tvb, offset+12+16+add_len,
+			 4, 0);
 }
 
 static void
@@ -353,7 +365,7 @@ dissect_fcp_data (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     scsi_task_id_t task_key;
 
     /* Retrieve conversation state to determine expected payload */
-    conversation = find_conversation (&pinfo->src, &pinfo->dst,
+    conversation = find_conversation (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                       pinfo->ptype, pinfo->oxid,
                                       pinfo->rxid, NO_PORT2);
     if (conversation) {
@@ -377,10 +389,10 @@ dissect_fcp_data (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             proto_tree_add_uint_hidden (fcp_tree, hf_fcp_singlelun, tvb,
                                         0, 0, cdata->fcp_lun);
 
-        dissect_scsi_payload (tvb, pinfo, tree, 0, FALSE, cdata->fcp_dl);
+        dissect_scsi_payload (tvb, pinfo, tree, 0, FALSE, cdata->fcp_dl, (guint16) cdata->fcp_lun);
     }
     else {
-        dissect_scsi_payload (tvb, pinfo, tree, 0, FALSE, 0);
+        dissect_scsi_payload (tvb, pinfo, tree, 0, FALSE, 0, 0xffff);
     }
 }
 
@@ -389,9 +401,8 @@ dissect_fcp_rsp (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
     guint32 offset = 0,
         del_usecs = 0;
-    guint len = 0,
-          add_len = 0,
-          rsplen = 0;
+    guint32 snslen = 0,
+           rsplen = 0;
     gchar str[128];
     guint8 flags;
     proto_item *ti;
@@ -410,7 +421,7 @@ dissect_fcp_rsp (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
     /* Response marks the end of the conversation. So destroy state */
-    conversation = find_conversation (&pinfo->src, &pinfo->dst,
+    conversation = find_conversation (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                       pinfo->ptype, pinfo->oxid,
                                       pinfo->rxid, NO_PORT2);
     if (conversation) {
@@ -423,20 +434,7 @@ dissect_fcp_rsp (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
     
     if (tree) {
-        /* Determine the length of the FCP part of the packet */
-        len = FCP_DEF_RSP_LEN;
-
-        flags = tvb_get_guint8 (tvb, offset+10);
-        if (flags & 0x2) {
-            add_len = tvb_get_ntohl (tvb, offset+20);
-            len += add_len;
-        }
-        if (flags & 0x1) {
-            add_len = tvb_get_ntohl (tvb, offset+16);
-            len += add_len;
-        }
-
-        ti = proto_tree_add_protocol_format (tree, proto_fcp, tvb, 0, len,
+        ti = proto_tree_add_protocol_format (tree, proto_fcp, tvb, 0, -1,
                                              "FCP_RSP");
         fcp_tree = proto_item_add_subtree (ti, ett_fcp);
         proto_tree_add_uint_hidden (fcp_tree, hf_fcp_type, tvb, offset, 0, 0);
@@ -456,25 +454,42 @@ dissect_fcp_rsp (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                 proto_tree_add_uint_hidden (fcp_tree, hf_fcp_singlelun, tvb,
                                             offset, 0, cdata->fcp_lun);
         }
+        flags = tvb_get_guint8 (tvb, offset+10);
         proto_tree_add_uint_format (fcp_tree, hf_fcp_rspflags, tvb, offset+10,
                                     1, flags, "Flags: 0x%02x (%s)", flags,
                                     rspflags_to_str (flags, str));
         proto_tree_add_item (fcp_tree, hf_fcp_scsistatus, tvb, offset+11, 1, 0);
         if (flags & 0xC)
             proto_tree_add_item (fcp_tree, hf_fcp_resid, tvb, offset+12, 4, 0);
-        if (flags & 0x2)
-            proto_tree_add_item (fcp_tree, hf_fcp_snslen, tvb, offset+16, 4, 0);
+        if (flags & 0x2) {
+            snslen = tvb_get_ntohl (tvb, offset+16);
+            proto_tree_add_uint (fcp_tree, hf_fcp_snslen, tvb, offset+16, 4,
+                                 snslen);
+        }
         if (flags & 0x1) {
             rsplen = tvb_get_ntohl (tvb, offset+20);
-            proto_tree_add_item (fcp_tree, hf_fcp_rsplen, tvb, offset+20, 4, 0);
+            proto_tree_add_uint (fcp_tree, hf_fcp_rsplen, tvb, offset+20, 4,
+                                 rsplen);
+            /* XXX - must rsplen be >= 4?  What other than the code is there? */
             proto_tree_add_item (fcp_tree, hf_fcp_rspcode, tvb, offset+27, 1,
                                  0);
         }
+        /* This handles too-large rsplen values (including ones > 2^31-1) */
+        tvb_ensure_bytes_exist (tvb, offset+24, rsplen);
+        offset += 24+rsplen;
         if (flags & 0x2) {
-            dissect_scsi_snsinfo (tvb, pinfo, tree, offset+24+rsplen,
-                                  tvb_get_ntohl (tvb, offset+16));
+            dissect_scsi_snsinfo (tvb, pinfo, tree, offset,
+                                  snslen,
+				  (guint16) (cdata?cdata->fcp_lun:0xffff) );
         }
+        /* This handles too-large snslen values (including ones > 2^31-1) */
+        tvb_ensure_bytes_exist (tvb, offset, snslen);
+        offset += snslen;
+        proto_item_set_end (ti, tvb, offset);
         if (cdata) {
+            /*
+             * XXX - this isn't done if an exception is thrown.
+             */
             g_mem_chunk_free (fcp_req_vals, cdata);
             g_hash_table_remove (fcp_req_hash, &ckey);
         }
@@ -494,11 +509,11 @@ dissect_fcp_xfer_rdy (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     fcp_conv_key_t ckey, *req_key;
 
     /* Retrieve conversation state to determine expected payload */
-    conversation = find_conversation (&pinfo->src, &pinfo->dst,
+    conversation = find_conversation (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                       pinfo->ptype, pinfo->oxid,
                                       pinfo->rxid, NO_PORT2);
     if (!conversation) {
-        conversation = conversation_new (&pinfo->src, &pinfo->dst,
+        conversation = conversation_new (pinfo->fd->num, &pinfo->src, &pinfo->dst,
                                          pinfo->ptype, pinfo->oxid,
                                          pinfo->rxid, NO_PORT2);
     }
