@@ -2,7 +2,7 @@
  *
  * Top-most dissector. Decides dissector based on Wiretap Encapsulation Type.
  *
- * $Id: packet-frame.c 13078 2005-01-16 23:30:55Z guy $
+ * $Id: packet-frame.c 14265 2005-05-02 14:07:33Z tuexen $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -46,6 +46,8 @@ static int hf_frame_p2p_dir = -1;
 static int hf_frame_file_off = -1;
 static int hf_frame_marked = -1;
 static int hf_frame_ref_time = -1;
+static int hf_link_number = -1;
+static int hf_frame_protocols = -1;
 
 static int proto_short = -1;
 int proto_malformed = -1;
@@ -72,12 +74,15 @@ static const value_string p2p_dirs[] = {
 static dissector_table_t wtap_encap_dissector_table;
 
 static void
-dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 {
-	proto_tree	*fh_tree;
-	proto_item	*ti;
+	proto_tree	*fh_tree=NULL;
+	proto_item	*volatile ti = NULL;
 	nstime_t	ts;
 	int		cap_len, pkt_len;
+	proto_tree	*tree;
+
+	tree=parent_tree;
 
 	pinfo->current_proto = "Frame";
 
@@ -103,6 +108,14 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			pinfo->p2p_dir = pinfo->pseudo_header->isdn.uton ?
 			    P2P_DIR_SENT : P2P_DIR_RECV;
 			break;
+		case WTAP_ENCAP_MTP2_WITH_PHDR:
+			pinfo->p2p_dir = pinfo->pseudo_header->mtp2.sent ?
+			    P2P_DIR_SENT : P2P_DIR_RECV;
+			pinfo->link_number  = pinfo->pseudo_header->mtp2.link_number;
+			pinfo->annex_a_used = pinfo->pseudo_header->mtp2.annex_a_used ?
+			    MTP2_ANNEX_A_USED : MTP2_ANNEX_A_NOT_USED;
+			break;
+		
 		}
 	}
 
@@ -122,7 +135,6 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	/* Put in frame header information. */
 	if (tree) {
-
 	  cap_len = tvb_length(tvb);
 	  pkt_len = tvb_reported_length(tvb);
 
@@ -130,7 +142,19 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    "Frame %u (%u bytes on wire, %u bytes captured)", pinfo->fd->num, pkt_len, cap_len);
 
 	  fh_tree = proto_item_add_subtree(ti, ett_frame);
+	}
 
+	/* if IP is not referenced from any filters we dont need to worry about
+	   generating any tree items.  We must do this after we created the actual
+	   protocol above so that proto hier stat still works though.
+	*/
+	if(!proto_field_is_referenced(tree, proto_frame)){
+		tree=NULL;
+		fh_tree = NULL;
+	}
+
+       
+	if (fh_tree) {
 	  proto_tree_add_boolean_hidden(fh_tree, hf_frame_marked, tvb, 0, 0,pinfo->fd->flags.marked);
 
 	  if(pinfo->fd->flags.ref_time){
@@ -166,12 +190,22 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		0, 0, cap_len, "Capture Length: %d byte%s", cap_len,
 		plurality(cap_len, "", "s"));
 
+	  ti = proto_tree_add_string(fh_tree, hf_frame_protocols, tvb,
+	  	0, 0, "");
+	  pinfo->layer_names = g_string_new("");
+
 	  /* Check for existences of P2P pseudo header */
 	  if (pinfo->p2p_dir != P2P_DIR_UNKNOWN) {
 		  proto_tree_add_uint(fh_tree, hf_frame_p2p_dir, tvb,
 				  0, 0, pinfo->p2p_dir);
 	  }
 
+	  /* Check for existences of MTP2 link number */
+	  if ((pinfo->pseudo_header != NULL ) && (pinfo->fd->lnk_t == WTAP_ENCAP_MTP2_WITH_PHDR)) {
+		  proto_tree_add_uint(fh_tree, hf_link_number, tvb,
+				  0, 0, pinfo->link_number);
+	  }
+
 	  if (show_file_off) {
 		  proto_tree_add_int_format(fh_tree, hf_frame_file_off, tvb,
 				  0, 0, pinfo->fd->file_off,
@@ -180,27 +214,32 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  }
 	}
 
-
 	TRY {
 		if (!dissector_try_port(wtap_encap_dissector_table, pinfo->fd->lnk_t,
-					tvb, pinfo, tree)) {
+					tvb, pinfo, parent_tree)) {
 
 			if (check_col(pinfo->cinfo, COL_PROTOCOL))
 				col_set_str(pinfo->cinfo, COL_PROTOCOL, "UNKNOWN");
 			if (check_col(pinfo->cinfo, COL_INFO))
 				col_add_fstr(pinfo->cinfo, COL_INFO, "WTAP_ENCAP = %u",
 				    pinfo->fd->lnk_t);
-			call_dissector(data_handle,tvb, pinfo, tree);
+			call_dissector(data_handle,tvb, pinfo, parent_tree);
 		}
 	}
 	CATCH_ALL {
-		show_exception(tvb, pinfo, tree, EXCEPT_CODE, GET_MESSAGE);
+		show_exception(tvb, pinfo, parent_tree, EXCEPT_CODE, GET_MESSAGE);
 	}
 	ENDTRY;
 
+	if (tree) {
+		proto_item_append_string(ti, pinfo->layer_names->str);
+		g_string_free(pinfo->layer_names, TRUE);
+		pinfo->layer_names = NULL;
+	}
+
 	tap_queue_packet(frame_tap, pinfo, NULL);
-	
-	if (mate_handle) call_dissector(mate_handle,tvb, pinfo, tree);
+
+	if (mate_handle) call_dissector(mate_handle,tvb, pinfo, parent_tree);
 
 }
 
@@ -302,6 +341,10 @@ proto_register_frame(void)
 		{ "Point-to-Point Direction",	"frame.p2p_dir", FT_UINT8, BASE_DEC, VALS(p2p_dirs), 0x0,
 			"", HFILL }},
 
+		{ &hf_link_number,
+		{ "Link Number",		"frame.link_nr", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"", HFILL }},
+
 		{ &hf_frame_file_off,
 		{ "File Offset",	"frame.file_off", FT_INT32, BASE_DEC, NULL, 0x0,
 			"", HFILL }},
@@ -313,6 +356,10 @@ proto_register_frame(void)
 		{ &hf_frame_ref_time,
 		{ "This is a Ref Time frame",	"frame.ref_time", FT_NONE, 0, NULL, 0x0,
 			"This frame is a Reference Time frame", HFILL }},
+
+		{ &hf_frame_protocols,
+		{ "Protocols in frame",	"frame.protocols", FT_STRING, 0, NULL, 0x0,
+			"Protocols carried by this frame", HFILL }},
 	};
 	static gint *ett[] = {
 		&ett_frame,
