@@ -9,7 +9,7 @@
  * Copyright 2002, Tim Potter <tpot@samba.org>
  * Copyright 1999, Andrew Tridgell <tridge@samba.org>
  *
- * $Id: packet-http.c 13145 2005-01-20 14:50:41Z gerald $
+ * $Id: packet-http.c 13815 2005-03-20 00:07:07Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -63,6 +63,8 @@ static int hf_http_response = -1;
 static int hf_http_request = -1;
 static int hf_http_basic = -1;
 static int hf_http_request_method = -1;
+static int hf_http_request_uri = -1;
+static int hf_http_version = -1;
 static int hf_http_response_code = -1;
 static int hf_http_authorization = -1;
 static int hf_http_proxy_authenticate = -1;
@@ -138,6 +140,12 @@ static gboolean http_decompress_body = FALSE;
 #define TCP_PORT_SSDP			1900
 #define UDP_PORT_SSDP			1900
 
+/* 
+ * tcp alternate port
+ */
+static guint http_alternate_tcp_port = 0;
+static guint alternate_tcp_port = 0;
+
 /*
  * Protocols implemented atop HTTP.
  */
@@ -147,7 +155,8 @@ typedef enum {
 	PROTO_DAAP		/* Digital Audio Access Protocol */
 } http_proto_t;
 
-typedef void (*RequestDissector)(tvbuff_t*, proto_tree*, int);
+typedef void (*ReqRespDissector)(tvbuff_t*, proto_tree*, int, const guchar*,
+    const guchar*);
 
 /*
  * Structure holding information from headers needed by main
@@ -162,8 +171,8 @@ typedef struct {
 	char	*transfer_encoding;
 } headers_t;
 
-static int is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
-		RequestDissector *req_dissector, int *req_strlen);
+static int is_http_request_or_reply(const gchar *data, int linelen,
+    http_type_t *type, ReqRespDissector *reqresp_dissector);
 static int chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
 		proto_tree *tree, int offset);
 static void process_header(tvbuff_t *tvb, int offset, int next_offset,
@@ -240,13 +249,19 @@ cleanup_headers(void *arg)
 
 /*
  * TODO: remove this ugly global variable.
- *
- * XXX - we leak "http_info_value_t" structures.
- * XXX - this gets overwritten if there's more than one HTTP request or
- * reply in the tvbuff.
+ * XXX: do we really want to have to pass this from one function to another?
  */
 static http_info_value_t	*stat_info;
 
+/*
+ * As the http_info_value_t will contain much more allocated data than it
+ * used to, we'll keep the stat_infos for this frame in this array.
+ *
+ * we'll clean it at every call of dissect_http_message() leaving in the array
+ * only those stat_infos that belong to the current frame
+ */
+static GPtrArray* stat_infos;
+
 static int
 dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
     proto_tree *tree)
@@ -264,9 +279,8 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	gboolean	saw_req_resp_or_header;
 	guchar		c;
 	http_type_t     http_type;
-	proto_item	*hdr_item;
-	RequestDissector req_dissector;
-	int		req_strlen;
+	proto_item	*hdr_item = NULL;
+	ReqRespDissector reqresp_dissector;
 	proto_tree	*req_tree;
 	int		colon_offset;
 	headers_t	headers;
@@ -274,6 +288,9 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	int		reported_datalen = -1;
 	dissector_handle_t handle;
 	gboolean	dissected;
+	guint		i;
+	guint32 framenum = pinfo->fd->num;
+	http_info_value_t *si;
 
 	/*
 	 * Is this a request or response?
@@ -291,7 +308,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	line = tvb_get_ptr(tvb, offset, first_linelen);
 	http_type = HTTP_OTHERS;	/* type not known yet */
 	is_request_or_reply = is_http_request_or_reply((const gchar *)line,
-			first_linelen, &http_type, NULL, NULL);
+	    first_linelen, &http_type, NULL);
 	if (is_request_or_reply) {
 		/*
 		 * Yes, it's a request or response.
@@ -308,9 +325,32 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 	}
 
+	/* we first allocate and initialize the current stat_info */ 
 	stat_info = g_malloc(sizeof(http_info_value_t));
+	stat_info->framenum = framenum;
 	stat_info->response_code = 0;
 	stat_info->request_method = NULL;
+	stat_info->request_uri = NULL;
+	stat_info->http_host = NULL;
+
+	/* We'll then delete from the array all the stat_infos that do not belong to this frame */ 
+	i = stat_infos->len;
+	while (i) {		
+		--i;
+		
+		si = g_ptr_array_index(stat_infos,i);
+		
+		if ( si->framenum != framenum ) {
+			g_ptr_array_remove_index_fast(stat_infos,i);
+			if (si->request_method) g_free(si->request_method);
+			if (si->request_uri) g_free(si->request_uri);
+			if (si->http_host) g_free(si->http_host);
+			g_free(si);
+		}
+	}
+	
+	/* then we'll add the current stat_info to the array */
+	g_ptr_array_add(stat_infos,stat_info);
 
 	switch (pinfo->match_port) {
 
@@ -372,6 +412,10 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	while (tvb_reported_length_remaining(tvb, offset) != 0) {
 		/*
 		 * Find the end of the line.
+		 * XXX - what if we don't find it because the packet
+		 * is cut short by a snapshot length or the header is
+		 * split across TCP segments?  How much dissection should
+		 * we do on it?
 		 */
 		linelen = tvb_find_line_end(tvb, offset,
 		    tvb_ensure_length_remaining(tvb, offset), &next_offset,
@@ -389,9 +433,10 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		/*
 		 * OK, does it look like an HTTP request or response?
 		 */
-		req_dissector = NULL;
-		is_request_or_reply = is_http_request_or_reply((const gchar *)line,
-				linelen, &http_type, &req_dissector, &req_strlen);
+		reqresp_dissector = NULL;
+		is_request_or_reply =
+		    is_http_request_or_reply((const gchar *)line,
+		    linelen, &http_type, &reqresp_dissector);
 		if (is_request_or_reply)
 			goto is_http;
 
@@ -536,12 +581,12 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				    offset, next_offset - offset, "%s",
 				    tvb_format_text(tvb, offset,
 				      next_offset - offset));
-				if (req_dissector) {
-					req_tree = proto_item_add_subtree(
-					    hdr_item, ett_http_request);
-					req_dissector(tvb, req_tree,
-					    req_strlen);
-				}
+			}
+			if (reqresp_dissector) {
+				if (tree) req_tree = proto_item_add_subtree(hdr_item, ett_http_request);
+				else req_tree = NULL;
+				
+				reqresp_dissector(tvb, req_tree, offset, line, lineend);
 			}
 		} else {
 			/*
@@ -910,31 +955,71 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 /* This can be used to dissect an HTTP request until such time
  * that a more complete dissector is written for that HTTP request.
- * This simple dissectory only puts http.request_method into a sub-tree.
+ * This simple dissector only puts the request method, URI, and
+ * protocol version into a sub-tree.
  */
 static void
-basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int req_strlen)
+basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
+    const guchar *line, const guchar *lineend)
 {
-	proto_tree_add_item(tree, hf_http_request_method, tvb, 0, req_strlen, FALSE);
+	const guchar *next_token;
+	int tokenlen;
+	
+	/* The first token is the method. */
+	tokenlen = get_token_len(line, lineend, &next_token);
+	if (tokenlen == 0)
+		return;
+	proto_tree_add_item(tree, hf_http_request_method, tvb, offset, tokenlen,
+	    FALSE);
+	offset += next_token - line;
+	line = next_token;
+
+	/* The next token is the URI. */
+	tokenlen = get_token_len(line, lineend, &next_token);
+	if (tokenlen == 0)
+		return;
+	stat_info->request_uri = tvb_get_string(tvb, offset, tokenlen);
+	proto_tree_add_string(tree, hf_http_request_uri, tvb, offset, tokenlen,
+	    stat_info->request_uri);
+	offset += next_token - line;
+	line = next_token;
+
+	/* Everything to the end of the line is the version. */
+	tokenlen = lineend - line;
+	if (tokenlen == 0)
+		return;
+	proto_tree_add_item(tree, hf_http_version, tvb, offset, tokenlen,
+	    FALSE);
 }
 
 static void
-basic_response_dissector(tvbuff_t *tvb, proto_tree *tree, int resp_strlen)
+basic_response_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
+    const guchar *line, const guchar *lineend)
 {
-	gchar *data;
-	int minor, major, status_code;
+	const guchar *next_token;
+	int tokenlen;
+	gchar response_chars[4];
+	
+	/* The first token is the version. */
+	tokenlen = get_token_len(line, lineend, &next_token);
+	if (tokenlen == 0)
+		return;
+	proto_tree_add_item(tree, hf_http_version, tvb, offset, tokenlen,
+	    FALSE);
+	offset += next_token - line;
+	line = next_token;
 
-	/* BEWARE - sscanf() only operates on C strings.
-	 * The pointer returned by tvb_get_ptr points into the real data,
-	 * which is not necessarily NULL terminated. For this reason,
-	 * the sscanf() call is only applied to a buffer guaranteed to
-	 * only contain a NULL terminated string. */
-	data = g_strndup((const gchar *)tvb_get_ptr(tvb, 5, resp_strlen), resp_strlen);
-	if (sscanf((const gchar *)data, "%d.%d %d", &minor, &major, &status_code) == 3) {
-		proto_tree_add_uint(tree, hf_http_response_code, tvb, 9, 3, status_code);
-		stat_info->response_code = status_code;
-	}
-	g_free(data);
+	/* The next token is the status code. */
+	tokenlen = get_token_len(line, lineend, &next_token);
+	if (tokenlen < 3)
+		return;
+	memcpy(response_chars, line, 3);
+	response_chars[3] = '\0';
+	
+	stat_info->response_code = strtoul(response_chars,NULL,10);
+	
+	proto_tree_add_uint(tree, hf_http_response_code, tvb, offset, 3,
+	    stat_info->response_code);
 }
 
 /*
@@ -1127,7 +1212,7 @@ chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
  */
 static int
 is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
-		RequestDissector *req_dissector, int *req_strlen)
+		ReqRespDissector *reqresp_dissector)
 {
 	int isHttpRequestOrReply = FALSE;
 	int prefix_len = 0;
@@ -1154,10 +1239,8 @@ is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
 	if (linelen >= 5 && strncmp(data, "HTTP/", 5) == 0) {
 		*type = HTTP_RESPONSE;
 		isHttpRequestOrReply = TRUE;	/* response */
-		if (req_dissector) {
-			*req_dissector = basic_response_dissector;
-			*req_strlen = linelen - 5;
-		}
+		if (reqresp_dissector)
+			*reqresp_dissector = basic_response_dissector;
 	} else {
 		const guchar * ptr = (const guchar *)data;
 		int		 index = 0;
@@ -1296,11 +1379,8 @@ is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
 			break;
 		}
 
-		if (isHttpRequestOrReply && req_dissector) {
-			*req_dissector = basic_request_dissector;
-			*req_strlen = index + prefix_len;
-		}
-		if (isHttpRequestOrReply && req_dissector) {
+		if (isHttpRequestOrReply && reqresp_dissector) {
+			*reqresp_dissector = basic_request_dissector;
 			if (!stat_info->request_method)
 				stat_info->request_method = g_malloc( index+1 );
 				strncpy( stat_info->request_method, data, index);
@@ -1327,6 +1407,7 @@ typedef struct {
 #define HDR_CONTENT_LENGTH	4
 #define HDR_CONTENT_ENCODING	5
 #define HDR_TRANSFER_ENCODING	6
+#define HDR_HOST  7
 
 static const header_info headers[] = {
 	{ "Authorization", &hf_http_authorization, HDR_AUTHORIZATION },
@@ -1338,7 +1419,7 @@ static const header_info headers[] = {
 	{ "Content-Encoding", &hf_http_content_encoding, HDR_CONTENT_ENCODING },
 	{ "Transfer-Encoding", &hf_http_transfer_encoding, HDR_TRANSFER_ENCODING },
 	{ "User-Agent",	&hf_http_user_agent, HDR_NO_SPECIAL },
-	{ "Host", &hf_http_host, HDR_NO_SPECIAL },
+	{ "Host", &hf_http_host, HDR_HOST },
 	{ "Connection", &hf_http_connection, HDR_NO_SPECIAL },
 	{ "Cookie", &hf_http_cookie, HDR_NO_SPECIAL },
 	{ "Accept", &hf_http_accept, HDR_NO_SPECIAL },
@@ -1507,6 +1588,10 @@ process_header(tvbuff_t *tvb, int offset, int next_offset,
 			memcpy(eh_ptr->transfer_encoding, value, value_len);
 			eh_ptr->transfer_encoding[value_len] = '\0';
 			break;
+			
+		case HDR_HOST:
+			stat_info->http_host = g_strdup(value);
+			break;
 		}
 
 		/*
@@ -1632,6 +1717,19 @@ dissect_http_udp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	dissect_http_message(tvb, 0, pinfo, tree);
 }
 
+void reinit_http(void) {
+	if ( http_alternate_tcp_port != alternate_tcp_port ) {
+		
+		if (alternate_tcp_port)
+			dissector_delete("tcp.port", alternate_tcp_port, http_handle );
+		
+		if (http_alternate_tcp_port)
+			dissector_add("tcp.port", http_alternate_tcp_port, http_handle);
+		
+		alternate_tcp_port = http_alternate_tcp_port;
+	}	
+}
+
 void
 proto_register_http(void)
 {
@@ -1655,6 +1753,14 @@ proto_register_http(void)
 	      { "Request Method",	"http.request.method",
 		FT_STRING, BASE_NONE, NULL, 0x0,
 		"HTTP Request Method", HFILL }},
+	    { &hf_http_request_uri,
+	      { "Request URI",	"http.request.uri",
+		FT_STRING, BASE_NONE, NULL, 0x0,
+		"HTTP Request-URI", HFILL }},
+	    { &hf_http_version,
+	      { "Request Version",	"http.request.version",
+		FT_STRING, BASE_NONE, NULL, 0x0,
+		"HTTP Request HTTP-Version", HFILL }},
 	    { &hf_http_response_code,
 	      { "Response Code",	"http.response.code",
 		FT_UINT16, BASE_DEC, NULL, 0x0,
@@ -1762,7 +1868,7 @@ proto_register_http(void)
 	    "HTTP", "http");
 	proto_register_field_array(proto_http, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
-	http_module = prefs_register_protocol(proto_http, NULL);
+	http_module = prefs_register_protocol(proto_http, reinit_http);
 	prefs_register_bool_preference(http_module, "desegment_headers",
 	    "Reassemble HTTP headers spanning multiple TCP segments",
 	    "Whether the HTTP dissector should reassemble headers "
@@ -1791,7 +1897,11 @@ proto_register_http(void)
 	    "using \"Content-Encoding: \"",
 	    &http_decompress_body);
 #endif
-
+	prefs_register_uint_preference(http_module, "tcp_alternate_port",
+								   "Alternate TCP port",
+								   "Decode packets on this TCP port as HTTP",
+								   10,&http_alternate_tcp_port);
+	
 	http_handle = create_dissector_handle(dissect_http, proto_http);
 
 	/*
@@ -1826,6 +1936,8 @@ proto_register_http(void)
 	 * Register for tapping
 	 */
 	http_tap = register_tap("http");
+	
+	stat_infos = g_ptr_array_new();
 }
 
 /*
@@ -1855,8 +1967,8 @@ proto_reg_handoff_http(void)
 	media_handle = find_dissector("media");
 
 	dissector_add("tcp.port", TCP_PORT_HTTP, http_handle);
-	dissector_add("tcp.port", TCP_ALT_PORT_HTTP, http_handle);
 	dissector_add("tcp.port", TCP_PORT_PROXY_HTTP, http_handle);
+	dissector_add("tcp.port", TCP_ALT_PORT_HTTP, http_handle);
 	dissector_add("tcp.port", TCP_PORT_PROXY_ADMIN_HTTP, http_handle);
 	dissector_add("tcp.port", TCP_PORT_HKP, http_handle);
 
