@@ -8,7 +8,7 @@
  * IAX2 is a VoIP protocol for the open source PBX Asterisk. Please see
  * http://www.asterisk.org for more information.
  *
- * $Id: packet-iax2.c 11477 2004-07-23 06:37:58Z guy $
+ * $Id: packet-iax2.c 14222 2005-04-28 19:16:54Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -43,7 +43,7 @@
 #include "iax2_codec_type.h"
 
 #define IAX2_PORT		4569
-#define PROTO_TAG_IAX2	"IAX2"
+#define PROTO_TAG_IAX2		"IAX2"
 
 /* #define DEBUG_HASHING */
 
@@ -121,6 +121,30 @@ static int hf_IAX_IE_AUTOANSWER = -1;
 static int hf_IAX_IE_MUSICONHOLD = -1;
 static int hf_IAX_IE_TRANSFERID = -1;
 static int hf_IAX_IE_RDNIS = -1;
+static int hf_IAX_IE_PROVISIONING = -1;
+static int hf_IAX_IE_AESPROVISIONING = -1;
+static int hf_IAX_IE_DATETIME = -1;
+static int hf_IAX_IE_DEVICETYPE = -1;
+static int hf_IAX_IE_SERVICEIDENT = -1;
+static int hf_IAX_IE_FIRMWAREVER = -1;
+static int hf_IAX_IE_FWBLOCKDESC = -1;
+static int hf_IAX_IE_FWBLOCKDATA = -1;
+static int hf_IAX_IE_PROVVER = -1;
+static int hf_IAX_IE_CALLINGPRES = -1;
+static int hf_IAX_IE_CALLINGTON = -1;
+static int hf_IAX_IE_CALLINGTNS = -1;
+static int hf_IAX_IE_SAMPLINGRATE = -1;
+static int hf_IAX_IE_CAUSECODE = -1;
+static int hf_IAX_IE_ENCRYPTION = -1;
+static int hf_IAX_IE_ENCKEY = -1;
+static int hf_IAX_IE_CODEC_PREFS = -1;
+static int hf_IAX_IE_RR_JITTER = -1;
+static int hf_IAX_IE_RR_LOSS = -1;
+static int hf_IAX_IE_RR_PKTS = -1;
+static int hf_IAX_IE_RR_DELAY = -1;
+static int hf_IAX_IE_RR_DROPPED = -1;
+static int hf_IAX_IE_RR_OOO = -1;
+
 static int hf_IAX_IE_DATAFORMAT = -1;
 static int hf_IAX_IE_UNKNOWN_BYTE = -1;
 static int hf_IAX_IE_UNKNOWN_I16 = -1;
@@ -193,6 +217,9 @@ static const value_string iax_iax_subclasses[] = {
   {32, "MWI"},
   {33, "UNSUPPORTED"},
   {34, "TRANSFER"},
+  {35, "PROVISION"},
+  {36, "FWDOWNL"},
+  {37, "FWDATA"},
   {0,NULL}
 };
 
@@ -243,6 +270,26 @@ static const value_string iax_ies_type[] = {
   {IAX_IE_PROVISIONING, "Provisioning info"},
   {IAX_IE_AESPROVISIONING, "AES Provisioning info"},
   {IAX_IE_DATETIME,"Date/Time"},
+  {IAX_IE_DEVICETYPE, "Device type"},
+  {IAX_IE_SERVICEIDENT, "Service Identifier"},
+  {IAX_IE_FIRMWAREVER, "Firmware revision"},
+  {IAX_IE_FWBLOCKDESC, "Firmware block description"},
+  {IAX_IE_FWBLOCKDATA, "Firmware block of data"},
+  {IAX_IE_PROVVER, "Provisioning version"},
+  {IAX_IE_CALLINGPRES, "Calling presentation"},
+  {IAX_IE_CALLINGTON, "Calling type of number"},
+  {IAX_IE_CALLINGTNS, "Calling transit network select"},
+  {IAX_IE_SAMPLINGRATE, "Supported sampling rates"},
+  {IAX_IE_CAUSECODE, "Hangup cause"},
+  {IAX_IE_ENCRYPTION, "Encryption format"},
+  {IAX_IE_ENCKEY, "Raw encryption key"},
+  {IAX_IE_CODEC_PREFS, "Codec preferences"},
+  {IAX_IE_RR_JITTER, "Received jitter"},
+  {IAX_IE_RR_LOSS, "Received loss"},
+  {IAX_IE_RR_PKTS, "Received frames"},
+  {IAX_IE_RR_DELAY, "Max playout delay in ms for received frames"},
+  {IAX_IE_RR_DROPPED, "Dropped frames"},
+  {IAX_IE_RR_OOO, "Frames received out of order"},
   {IAX_IE_DATAFORMAT, "Data call format"},
   {0,NULL}
 };
@@ -288,6 +335,54 @@ static const value_string iax_packet_types[] = {
   {0,NULL}
 };
   
+static const value_string iax_causecodes[] = {
+  {AST_CAUSE_UNALLOCATED, "Unallocated"},
+  {AST_CAUSE_NO_ROUTE_TRANSIT_NET, "No route transit net"},
+  {AST_CAUSE_NO_ROUTE_DESTINATION, "No route to destination"},
+  {AST_CAUSE_CHANNEL_UNACCEPTABLE, "Channel unacceptable"},
+  {AST_CAUSE_CALL_AWARDED_DELIVERED, "Call awarded delivered"},
+  {AST_CAUSE_NORMAL_CLEARING, "Normal clearing"},
+  {AST_CAUSE_USER_BUSY, "User busy"},
+  {AST_CAUSE_NO_USER_RESPONSE, "No user response"},
+  {AST_CAUSE_NO_ANSWER, "No answer"},
+  {AST_CAUSE_CALL_REJECTED, "Call rejected"},
+  {AST_CAUSE_NUMBER_CHANGED, "Number changed"},
+  {AST_CAUSE_DESTINATION_OUT_OF_ORDER, "Destination out of order"},
+  {AST_CAUSE_INVALID_NUMBER_FORMAT, "Invalid number format"},
+  {AST_CAUSE_FACILITY_REJECTED, "Facility rejected"},
+  {AST_CAUSE_RESPONSE_TO_STATUS_ENQUIRY, "Response to status inquiry"},
+  {AST_CAUSE_NORMAL_UNSPECIFIED, "Normal unspecified"},
+  {AST_CAUSE_NORMAL_CIRCUIT_CONGESTION, "Normal circuit congestion"},
+  {AST_CAUSE_NETWORK_OUT_OF_ORDER, "Network out of order"},
+  {AST_CAUSE_NORMAL_TEMPORARY_FAILURE, "Normal temporary failure"},
+  {AST_CAUSE_SWITCH_CONGESTION, "Switch congestion"},
+  {AST_CAUSE_ACCESS_INFO_DISCARDED, "Access info discarded"},
+  {AST_CAUSE_REQUESTED_CHAN_UNAVAIL, "Requested channel unavailable"},
+  {AST_CAUSE_PRE_EMPTED, "Preempted"},
+  {AST_CAUSE_FACILITY_NOT_SUBSCRIBED, "Facility not subscribed"},
+  {AST_CAUSE_OUTGOING_CALL_BARRED, "Outgoing call barred"},
+  {AST_CAUSE_INCOMING_CALL_BARRED, "Incoming call barred"},
+  {AST_CAUSE_BEARERCAPABILITY_NOTAUTH, "Bearer capability not authorized"},
+  {AST_CAUSE_BEARERCAPABILITY_NOTAVAIL, "Bearer capability not available"},
+  {AST_CAUSE_BEARERCAPABILITY_NOTIMPL, "Bearer capability not implemented"},
+  {AST_CAUSE_CHAN_NOT_IMPLEMENTED, "Channel not implemented"},
+  {AST_CAUSE_FACILITY_NOT_IMPLEMENTED, "Facility not implemented"},
+  {AST_CAUSE_INVALID_CALL_REFERENCE, "Invalid call reference"},
+  {AST_CAUSE_INCOMPATIBLE_DESTINATION, "Incompatible destination"},
+  {AST_CAUSE_INVALID_MSG_UNSPECIFIED, "Invalid message unspecified"},
+  {AST_CAUSE_MANDATORY_IE_MISSING, "Mandatory IE missing"},
+  {AST_CAUSE_MESSAGE_TYPE_NONEXIST, "Message type nonexistent"},
+  {AST_CAUSE_WRONG_MESSAGE, "Wrong message"},
+  {AST_CAUSE_IE_NONEXIST, "IE nonexistent"},
+  {AST_CAUSE_INVALID_IE_CONTENTS, "Invalid IE contents"},
+  {AST_CAUSE_WRONG_CALL_STATE, "Wrong call state"},
+  {AST_CAUSE_RECOVERY_ON_TIMER_EXPIRE, "Recovery on timer expire"},
+  {AST_CAUSE_MANDATORY_IE_LENGTH_ERROR, "Mandatory IE length error"},
+  {AST_CAUSE_PROTOCOL_ERROR, "Protocol error"},
+  {AST_CAUSE_INTERWORKING, "Interworking"},
+  {0, NULL}
+};
+
 
 /* ************************************************************************* */
 
@@ -514,6 +609,10 @@ static iax_call_data *iax_lookup_circuit_details_from_dest( guint src_circuit_id
 #ifdef DEBUG_HASHING
     g_message( "++ destination circuit not found, must have missed NEW packet" );
 #endif
+    if( reversed_p )
+      *reversed_p = FALSE;
+    if( circuit_p )
+      *circuit_p = NULL;
     return NULL;
   }
 
@@ -565,6 +664,10 @@ static iax_call_data *iax_lookup_circuit_details_from_dest( guint src_circuit_id
 		 src_circuit_id,dst_circuit_id,
 		 iax_call->forward_circuit_id,
 		 iax_call->reverse_circuit_id);
+      if( reversed_p )
+        *reversed_p = FALSE;
+      if( circuit_p )
+        *circuit_p = NULL;
       return NULL;
     }
   } else if ( dst_circuit_id == iax_call -> reverse_circuit_id ) {
@@ -581,6 +684,10 @@ static iax_call_data *iax_lookup_circuit_details_from_dest( guint src_circuit_id
 		 src_circuit_id,dst_circuit_id,
 		 iax_call->forward_circuit_id,
 		 iax_call->reverse_circuit_id);
+      if( reversed_p )
+        *reversed_p = FALSE;
+      if( circuit_p )
+        *circuit_p = NULL;
       return NULL;
     }
   } else {
@@ -972,6 +1079,7 @@ static guint32 dissect_ies (tvbuff_t * tvb, guint32 offset,
 	      proto_tree *codec_tree;
 	      proto_item *codec_base;
 
+	      if (ies_len != 4) THROW(ReportedBoundsError);
 	      codec_base =
 		proto_tree_add_item (ies_tree, hf_IAX_IE_CAPABILITY,
 				     tvb, offset + 2, ies_len, FALSE);
@@ -996,18 +1104,22 @@ static guint32 dissect_ies (tvbuff_t * tvb, guint32 offset,
 	      break;
 	    }
 	    case IAX_IE_FORMAT:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_FORMAT, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_VERSION:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_VERSION, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_ADSICPE:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_ADSICPE, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_AUTHMETHODS:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_AUTHMETHODS, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
@@ -1018,14 +1130,17 @@ static guint32 dissect_ies (tvbuff_t * tvb, guint32 offset,
 	      proto_tree_add_item(sockaddr_tree, hf_IAX_IE_APPARENTADDR_SINPORT, tvb, offset + 4, 2, FALSE);
 	      break;
 	    case IAX_IE_REFRESH:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_REFRESH, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_DPSTATUS:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_DPSTATUS, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_CALLNO:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_CALLNO, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
@@ -1038,6 +1153,7 @@ static guint32 dissect_ies (tvbuff_t * tvb, guint32 offset,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_MSGCOUNT:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_MSGCOUNT, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
@@ -1050,11 +1166,121 @@ static guint32 dissect_ies (tvbuff_t * tvb, guint32 offset,
 				   offset + 2, ies_len, FALSE);
 	      break;
 	    case IAX_IE_TRANSFERID:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
 	      proto_tree_add_item (ies_tree, hf_IAX_IE_TRANSFERID, tvb,
 				   offset + 2, ies_len, FALSE);
 	      break;
+	    case IAX_IE_PROVISIONING:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_PROVISIONING, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_AESPROVISIONING:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_AESPROVISIONING, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_DATETIME:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_DATETIME, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_DEVICETYPE:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_DEVICETYPE, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_SERVICEIDENT:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_SERVICEIDENT, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_FIRMWAREVER:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_FIRMWAREVER, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_FWBLOCKDESC:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_FWBLOCKDESC, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_FWBLOCKDATA:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_FWBLOCKDATA, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_PROVVER:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_PROVVER, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_CALLINGPRES:
+	      if (ies_len != 1) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_CALLINGPRES, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_CALLINGTON:
+	      if (ies_len != 1) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_CALLINGTON, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_CALLINGTNS:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_CALLINGTNS, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_SAMPLINGRATE:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_SAMPLINGRATE, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_CAUSECODE:
+	      if (ies_len != 1) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_CAUSECODE, tvb,
+				   offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_ENCRYPTION:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_ENCRYPTION, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_ENCKEY:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_ENCKEY, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_CODEC_PREFS:
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_CODEC_PREFS, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_RR_JITTER:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_RR_JITTER, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_RR_LOSS:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_RR_LOSS, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_RR_PKTS:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_RR_PKTS, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_RR_DELAY:
+	      if (ies_len != 2) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_RR_DELAY, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_RR_DROPPED:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_RR_DROPPED, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
+	    case IAX_IE_RR_OOO:
+	      if (ies_len != 4) THROW(ReportedBoundsError);
+	      proto_tree_add_item (ies_tree, hf_IAX_IE_RR_OOO, tvb,
+				  offset +2, ies_len, FALSE);
+	      break;
 	      
       case IAX_IE_DATAFORMAT:
+	if (ies_len != 4) THROW(ReportedBoundsError);
 	proto_tree_add_item (ies_tree, hf_IAX_IE_DATAFORMAT, tvb,
 			     offset + 2, ies_len, FALSE);
 
@@ -1125,15 +1351,15 @@ dissect_fullpacket (tvbuff_t * tvb, guint32 offset,
 
   circuit_t *circuit;
 
-      /*
-       * remove the top bit for retransmission detection 
-       */
-      dcallno = tvb_get_ntohs(tvb, offset);
-      retransmission = dcallno & 0x8000;
-      dcallno = dcallno & 0x7FFF;
-      ts = tvb_get_ntohl(tvb, offset+2);
-      type = tvb_get_guint8(tvb, offset + 8);
-      csub = tvb_get_guint8(tvb, offset + 9);
+  /*
+   * remove the top bit for retransmission detection 
+   */
+  dcallno = tvb_get_ntohs(tvb, offset);
+  retransmission = dcallno & 0x8000;
+  dcallno = dcallno & 0x7FFF;
+  ts = tvb_get_ntohl(tvb, offset+2);
+  type = tvb_get_guint8(tvb, offset + 8);
+  csub = tvb_get_guint8(tvb, offset + 9);
 
   /* see if we've seen this packet before */
   iax_packet = (iax_packet_data *)p_get_proto_data(pinfo->fd,proto_iax2);
@@ -1153,6 +1379,15 @@ dissect_fullpacket (tvbuff_t * tvb, guint32 offset,
     p_add_proto_data(pinfo->fd,proto_iax2,iax_packet);
   } else {
     iax_call = iax_packet->call_data;
+
+    /*
+     * XXX - the code needs to set "circuit" and "reversed" somehow here,
+     * by determining them based on values in "iax_call" or "iax_packet".
+     * I leave that as an exercise for somebody who wants to maintain
+     * this code.
+     */
+    circuit = NULL;
+    reversed = FALSE;
   }
    
   if( iax2_tree ) {
@@ -1715,6 +1950,79 @@ proto_register_iax2 (void)
       0x0, "",
       HFILL}},
 
+    {&hf_IAX_IE_PROVISIONING,
+     {"Provisioning info","iax2.iax.provisioning", FT_STRING, BASE_NONE,
+       NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_AESPROVISIONING,
+     {"AES Provisioning info","iax2.iax.aesprovisioning", FT_STRING, BASE_NONE,
+       NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_DATETIME,
+     {"Date/Time", "iax2.iax.datetime", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_DEVICETYPE,
+     {"Device type", "iax2.iax.devicetype", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_SERVICEIDENT,
+     {"Service identifier", "iax2.iax.serviceident", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_FIRMWAREVER,
+     {"Firmware version", "iax2.iax.firmwarever", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_FWBLOCKDESC,
+     {"Firmware block description", "iax2.iax.fwblockdesc", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_FWBLOCKDATA,
+     {"Firmware block of data", "iax2.iax.fwblockdata", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_PROVVER,
+     {"Provisioning version", "iax2.iax.provver", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_CALLINGPRES,
+     {"Calling presentation", "iax2.iax.callingpres", FT_UINT8, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_CALLINGTON,
+     {"Calling type of number", "iax2.iax.callington", FT_UINT8, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_CALLINGTNS,
+     {"Calling transit network select", "iax2.iax.callingtns", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_SAMPLINGRATE,
+     {"Supported sampling rates", "iax2.iax.samplingrate", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_CAUSECODE,
+     {"Hangup cause", "iax2.iax.causecode", FT_UINT8, BASE_HEX, VALS(iax_causecodes),
+       0x0, "", HFILL}},
+
+    {&hf_IAX_IE_ENCRYPTION,
+     {"Encryption format", "iax2.iax.encryption", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_ENCKEY,
+     {"Encryption key", "iax2.iax.enckey", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_CODEC_PREFS,
+     {"Codec negotiation", "iax2.iax.codecprefs", FT_STRING, BASE_NONE, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_RR_JITTER,
+     {"Received jitter (as in RFC1889)", "iax2.iax.rrjitter", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_RR_LOSS,
+     {"Received loss (high byte loss pct, low 24 bits loss count, as in rfc1889)", "iax2.iax.rrloss",
+       FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_RR_PKTS,
+     {"Total frames received", "iax2.iax.rrpkts", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_RR_DELAY,
+     {"Max playout delay in ms for received frames", "iax2.iax.rrdelay", FT_UINT16, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_RR_DROPPED,
+     {"Dropped frames (presumably by jitterbuffer)", "iax2.iax.rrdropped", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
+    {&hf_IAX_IE_RR_OOO,
+     {"Frame received out of order", "iax2.iax.rrooo", FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL}},
+
     {&hf_IAX_IE_DATAFORMAT,
      {"Data call format", "iax2.iax.dataformat", FT_UINT32, BASE_HEX,
       VALS(iax_dataformats), 0x0, "", HFILL}},
