@@ -1,7 +1,7 @@
 /* packet-ip.c
  * Routines for IP and miscellaneous IP protocol packet disassembly
  *
- * $Id: packet-ip.c 12934 2005-01-03 00:08:40Z gerald $
+ * $Id: packet-ip.c 14051 2005-04-11 08:43:51Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,7 +39,7 @@
 #include <epan/ipproto.h>
 #include "ip_opts.h"
 #include <epan/prefs.h>
-#include "reassemble.h"
+#include <epan/reassemble.h>
 #include "etypes.h"
 #include "greproto.h"
 #include "ppptypes.h"
@@ -814,7 +814,7 @@ static guint16 ip_checksum(const guint8 *ptr, int len)
 }
 
 static void
-dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 {
   proto_tree *ip_tree = NULL, *field_tree;
   proto_item *ti = NULL, *tf;
@@ -831,6 +831,11 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   static e_ip eip_arr[4];
   static int eip_current=0;
   e_ip *iph;
+  const guchar		*src_addr, *dst_addr;
+  guint32 		src32, dst32;
+  proto_tree *tree;
+
+  tree=parent_tree;
 
   eip_current++;
   if(eip_current==4){
@@ -854,6 +859,14 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	hi_nibble(iph->ip_v_hl));
   }
 
+  /* if IP is not referenced from any filters we dont need to worry about
+     generating any tree items.  We must do this after we created the actual
+     protocol above so that proto hier stat still works though.
+  */
+  if(!proto_field_is_referenced(parent_tree, proto_ip)){
+    tree=NULL;
+  }
+
   if (hlen < IPH_MIN_LEN) {
     if (check_col(pinfo->cinfo, COL_INFO))
       col_add_fstr(pinfo->cinfo, COL_INFO, "Bogus IP header length (%u, must be at least %u)",
@@ -974,10 +987,11 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     if (tree)
       proto_tree_add_uint(ip_tree, hf_ip_checksum, tvb, offset + 10, 2, iph->ip_sum);
   }
-
-  SET_ADDRESS(&pinfo->net_src, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_SRC, 4));
-  SET_ADDRESS(&pinfo->src, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_SRC, 4));
-  SET_ADDRESS(&iph->ip_src, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_SRC, 4));
+  src_addr = tvb_get_ptr(tvb, offset + IPH_SRC, 4);
+  src32 = tvb_get_ntohl(tvb, offset + IPH_SRC);
+  SET_ADDRESS(&pinfo->net_src, AT_IPv4, 4, src_addr);
+  SET_ADDRESS(&pinfo->src, AT_IPv4, 4, src_addr);
+  SET_ADDRESS(&iph->ip_src, AT_IPv4, 4, src_addr);
   if (tree) {
     memcpy(&addr, iph->ip_src.data, 4);
     if (ip_summary_in_tree) {
@@ -987,10 +1001,11 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     proto_tree_add_ipv4(ip_tree, hf_ip_src, tvb, offset + 12, 4, addr);
     proto_tree_add_ipv4_hidden(ip_tree, hf_ip_addr, tvb, offset + 12, 4, addr);
   }
-
-  SET_ADDRESS(&pinfo->net_dst, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_DST, 4));
-  SET_ADDRESS(&pinfo->dst, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_DST, 4));
-  SET_ADDRESS(&iph->ip_dst, AT_IPv4, 4, tvb_get_ptr(tvb, offset + IPH_DST, 4));
+  dst_addr = tvb_get_ptr(tvb, offset + IPH_DST, 4);
+  dst32 = tvb_get_ntohl(tvb, offset + IPH_DST);
+  SET_ADDRESS(&pinfo->net_dst, AT_IPv4, 4, dst_addr);
+  SET_ADDRESS(&pinfo->dst, AT_IPv4, 4, dst_addr);
+  SET_ADDRESS(&iph->ip_dst, AT_IPv4, 4, dst_addr);
 
   if (tree) {
     memcpy(&addr, iph->ip_dst.data, 4);
@@ -1034,7 +1049,8 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (ip_defragment && (iph->ip_off & (IP_MF|IP_OFFSET)) &&
       tvb_bytes_exist(tvb, offset, pinfo->iplen - pinfo->iphdrlen) &&
       ipsum == 0) {
-    ipfd_head = fragment_add_check(tvb, offset, pinfo, iph->ip_id,
+    ipfd_head = fragment_add_check(tvb, offset, pinfo, 
+			     iph->ip_id ^ src32 ^ dst32,
 			     ip_fragment_table,
 			     ip_reassembled_table,
 			     (iph->ip_off & IP_OFFSET)*8,
@@ -1083,7 +1099,7 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     }
 
     call_dissector(data_handle, tvb_new_subset(tvb, offset, -1, -1), pinfo,
-                   tree);
+                   parent_tree);
     pinfo->fragmented = save_fragmented;
     goto end_of_ip;
   }
@@ -1095,13 +1111,13 @@ dissect_ip(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      even be labelled as an IP frame; ideally, if a frame being dissected
      throws an exception, it'll be labelled as a mangled frame of the
      type in question. */
-  if (!dissector_try_port(ip_dissector_table, nxt, next_tvb, pinfo, tree)) {
+  if (!dissector_try_port(ip_dissector_table, nxt, next_tvb, pinfo, parent_tree)) {
     /* Unknown protocol */
     if (update_col_info) {
       if (check_col(pinfo->cinfo, COL_INFO))
         col_add_fstr(pinfo->cinfo, COL_INFO, "%s (0x%02x)", ipprotostr(iph->ip_p), iph->ip_p);
     }
-    call_dissector(data_handle,next_tvb, pinfo, tree);
+    call_dissector(data_handle,next_tvb, pinfo, parent_tree);
   }
   pinfo->fragmented = save_fragmented;
 
@@ -1762,10 +1778,12 @@ proto_reg_handoff_ip(void)
 	dissector_add("ip.proto", IP_PROTO_IPIP, ip_handle);
 	dissector_add("null.type", BSD_AF_INET, ip_handle);
 	dissector_add("chdlctype", ETHERTYPE_IP, ip_handle);
+	dissector_add("osinl.excl", NLPID_IP, ip_handle);
 	dissector_add("fr.ietf", NLPID_IP, ip_handle);
 	dissector_add("x.25.spi", NLPID_IP, ip_handle);
         dissector_add("arcnet.protocol_id", ARCNET_PROTO_IP_1051, ip_handle);
         dissector_add("arcnet.protocol_id", ARCNET_PROTO_IP_1201, ip_handle);
+	dissector_add_handle("udp.port", ip_handle);
 }
 
 void
@@ -1897,5 +1915,4 @@ proto_reg_handoff_icmp(void)
 
   icmp_handle = create_dissector_handle(dissect_icmp, proto_icmp);
   dissector_add("ip.proto", IP_PROTO_ICMP, icmp_handle);
-  dissector_add("wtap_encap", WTAP_ENCAP_RAW_ICMP, icmp_handle);
 }
