@@ -1,6 +1,6 @@
 /* packet-isup.h
  *
- * $Id: packet-isup.h 12688 2004-12-08 05:58:59Z guy $
+ * $Id: packet-isup.h 14086 2005-04-15 08:14:44Z etxrab $
  *
  * Copyright 2003, Michael Lum <mlum [AT] telostech.com>,
  * In association with Telos Technology Inc.
@@ -28,6 +28,10 @@
 
 typedef struct _isup_tap_rec_t {
     guint8		message_type;
+    /* added for VoIP calls analysis, see gtk/voip_calls.c*/
+    gchar           *called_number;
+    gchar           *calling_number;
+    guint8			cause_value;
 } isup_tap_rec_t;
 
 
@@ -38,7 +42,7 @@ typedef struct _isup_tap_rec_t {
  */
 ETH_VAR_IMPORT const value_string isup_message_type_value[];
 ETH_VAR_IMPORT const value_string isup_message_type_value_acro[];
-
+ETH_VAR_IMPORT const value_string q850_cause_code_vals[];
 /*
  * Export some definitions and value_string tables for other dissectors
  */
@@ -129,8 +133,11 @@ ETH_VAR_IMPORT const value_string isup_message_type_value_acro[];
 
 extern const value_string isup_parameter_type_value[]; 
 extern const value_string isup_transmission_medium_requirement_value[];
-
 /*
  * Export dissection of some parameters
  */
 void dissect_nsap(tvbuff_t *parameter_tvb,gint offset,gint len, proto_tree *parameter_tree);
+void dissect_isup_called_party_number_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item);
+void dissect_isup_calling_party_number_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item);
+void dissect_isup_cause_indicators_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item);
+
