@@ -1,6 +1,6 @@
 /* packet-kerberos.h
  *
- * $Id: packet-kerberos.h 12426 2004-10-28 22:06:55Z gerald $
+ * $Id: packet-kerberos.h 13850 2005-03-21 10:52:57Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,7 +32,8 @@
    to future DCERPC decryption and other uses.
    The list is terminated by {0, NULL }
 */
-#define KRB_CBTAG_SAFE_USER_DATA	1
+#define KRB_CBTAG_SAFE_USER_DATA	        1
+#define KRB_CBTAG_PRIV_USER_DATA	        2
 typedef struct _kerberos_callbacks {
 	int tag;
 	int (*callback)(packet_info *pinfo, tvbuff_t *tvb, proto_tree *tree);
@@ -49,4 +50,24 @@ dissect_krb5_Checksum(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int o
 int
 dissect_krb5_ctime(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset);
 
+int dissect_krb5_cname(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset);
+int dissect_krb5_realm(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset);
+
+#ifdef HAVE_KERBEROS
+#define KRB_MAX_ORIG_LEN	256
+
+#if defined(HAVE_HEIMDAL_KERBEROS) || defined(HAVE_MIT_KERBEROS)
+typedef struct _enc_key_t {
+	struct _enc_key_t	*next;
+	int keytype;
+	int keylength;
+	char *keyvalue;
+	char 			key_origin[KRB_MAX_ORIG_LEN+1];
+} enc_key_t;
+extern enc_key_t *enc_key_list;
+
+#endif /* HAVE_HEIMDAL_KERBEROS || HAVE_MIT_KERBEROS */
+
+#endif /* HAVE_KERBEROS */
+
 #endif /* __PACKET_KERBEROS_H */
