@@ -3,7 +3,7 @@
  * Copyright 2001, Tom Uijldert <tom.uijldert@cmg.nl>
  * Copyright 2004, Olivier Biot
  *
- * $Id: packet-mmse.c 12959 2005-01-05 10:27:22Z guy $
+ * $Id: packet-mmse.c 14163 2005-04-22 04:26:23Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -821,6 +821,7 @@ dissect_mmse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint8 pdut,
 			    length += 1;
 			}
 			if (tree) {
+			    tvb_ensure_bytes_exist(tvb, offset - 1, length + 1);
 			    proto_tree_add_string(mmse_tree,
 				    hf_mmse_content_location,
 				    tvb, offset - 1, length + 1,
@@ -846,6 +847,7 @@ dissect_mmse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint8 pdut,
 			tmptime.secs = tval;
 			tmptime.nsecs = 0;
 			if (tree) {
+			    tvb_ensure_bytes_exist(tvb, offset - 1, count + 1);
 			    proto_tree_add_time(mmse_tree, hf_mmse_date, tvb,
 				    offset - 1, count + 1, &tmptime);
 			}
@@ -876,6 +878,7 @@ dissect_mmse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint8 pdut,
 			tmptime.secs = tval;
 			tmptime.nsecs = 0;
 
+			tvb_ensure_bytes_exist(tvb, offset - 1, length + count + 1);
 			if (field == 0x80)
 			    proto_tree_add_time(mmse_tree,
 				    hf_mmse_delivery_time_abs,
@@ -905,6 +908,7 @@ dissect_mmse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint8 pdut,
 			tmptime.secs = tval;
 			tmptime.nsecs = 0;
 
+			tvb_ensure_bytes_exist(tvb, offset - 1, length + count + 1);
 			if (field == 0x80)
 			    proto_tree_add_time(mmse_tree, hf_mmse_expiry_abs,
 				    tvb, offset - 1,
@@ -924,6 +928,7 @@ dissect_mmse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint8 pdut,
 		    length = get_value_length(tvb, offset, &count);
 		    if (tree) {
 			field = tvb_get_guint8(tvb, offset + count);
+			tvb_ensure_bytes_exist(tvb, offset - 1, length + count + 1);
 			if (field == 0x81) {
 			    proto_tree_add_string(mmse_tree, hf_mmse_from, tvb,
 				    offset-1, length + count + 1,
@@ -1206,6 +1211,7 @@ dissect_mmse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint8 pdut,
 			tmptime.nsecs = 0;
 			strval = abs_time_to_str(&tmptime);
 			/* Now render the fields */
+			tvb_ensure_bytes_exist(tvb, offset - 1, length + count + 1);
 			ti = proto_tree_add_string_format(mmse_tree,
 				hf_mmse_prev_sent_date,
 				tvb, offset - 1, 1 + count + length,
