@@ -10,7 +10,7 @@
  *
  * Copyright 2003, Jeff Morriss <jeff.morriss[AT]ulticom.com>
  *
- * $Id: packet-mtp3mg.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: packet-mtp3mg.c 12585 2004-11-24 02:59:49Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -325,8 +325,8 @@ static const value_string service_indicator_code_vals[] = {
 	{ 0x6,	"DUP (call and circuit related messages)" },
 	{ 0x7,	"DUP (facility registration and cancellation message)" },
 	{ 0x8,	"MTP testing user part" },
-	{ 0x9,	"Spare" },
-	{ 0xa,	"Spare" },
+	{ 0x9,	"ISUP-b" },
+	{ 0xa,	"ISUP-s" },
 	{ 0xb,	"Spare" },
 	{ 0xc,	"Spare" },
 	{ 0xd,	"Spare" },
