@@ -3,7 +3,7 @@
  * Devin Heitmueller <dheitmueller@netilla.com>
  * Copyright 2003, Tim Potter <tpot@samba.org>
  *
- * $Id: packet-ntlmssp.c 12116 2004-09-27 23:29:22Z guy $
+ * $Id: packet-ntlmssp.c 13690 2005-03-10 10:16:49Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,14 +36,14 @@
 
 #include "packet-windows-common.h"
 #include "packet-smb-common.h"
-#include "asn1.h"		/* XXX - needed for subid_t */
-#include "packet-gssapi.h"
+#include <epan/asn1.h>		/* XXX - needed for subid_t */
 #include "packet-frame.h"
 #include <epan/prefs.h>
 #include <epan/crypt-rc4.h>
 #include <epan/crypt-md4.h>
 #include <epan/crypt-des.h>
 #include "packet-dcerpc.h"
+#include "packet-gssapi.h"
 
 #include "packet-ntlmssp.h"
 
@@ -926,11 +926,11 @@ dissect_ntlmssp_challenge (tvbuff_t *tvb, packet_info *pinfo, int offset,
    * Store the flags and the RC4 state information with the conversation,
    * as they're needed in order to dissect subsequent messages.
    */
-  conversation = find_conversation(&pinfo->src, &pinfo->dst,
+  conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 				   pinfo->ptype, pinfo->srcport,
 				   pinfo->destport, 0);
   if (!conversation) { /* Create one */
-    conversation = conversation_new(&pinfo->src, &pinfo->dst, pinfo->ptype, 
+    conversation = conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype, 
 				    pinfo->srcport, pinfo->destport, 0);
   }
 
@@ -1019,7 +1019,7 @@ dissect_ntlmssp_auth (tvbuff_t *tvb, packet_info *pinfo, int offset,
      * it means this is the first time we've dissected this frame, so
      * we should give it flag info.
      */
-    conversation = find_conversation(&pinfo->src, &pinfo->dst,
+    conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 				     pinfo->ptype, pinfo->srcport,
 				     pinfo->destport, 0);
     if (conversation != NULL) {
@@ -1203,7 +1203,7 @@ get_encrypted_state(packet_info *pinfo, int cryptpeer)
   conversation_t *conversation;
   ntlmssp_info *conv_ntlmssp_info;
 
-  conversation = find_conversation(&pinfo->src, &pinfo->dst,
+  conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 				   pinfo->ptype, pinfo->srcport,
 				   pinfo->destport, 0);
   if (conversation == NULL) {
@@ -1258,7 +1258,7 @@ decrypt_verifier(tvbuff_t *tvb, int offset, guint32 encrypted_block_length,
     return;
   }
   if (!packet_ntlmssp_info->verifier_decrypted) {
-    conversation = find_conversation(&pinfo->src, &pinfo->dst,
+    conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 				     pinfo->ptype, pinfo->srcport,
 				     pinfo->destport, 0);
     if (conversation == NULL) {
@@ -1411,7 +1411,9 @@ dissect_ntlmssp_verf(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 }
 
 static tvbuff_t *
-dissect_ntlmssp_encrypted_payload(tvbuff_t *tvb, int offset,
+dissect_ntlmssp_encrypted_payload(tvbuff_t *data_tvb, 
+				  tvbuff_t *auth_tvb _U_,
+				  int offset,
 				  packet_info *pinfo, 
 				  dcerpc_auth_info *auth_info _U_)
 {
@@ -1424,7 +1426,7 @@ dissect_ntlmssp_encrypted_payload(tvbuff_t *tvb, int offset,
   ntlmssp_info *conv_ntlmssp_info = NULL;
   ntlmssp_packet_info *packet_ntlmssp_info = NULL;
 
-  encrypted_block_length = tvb_length_remaining (tvb, offset);
+  encrypted_block_length = tvb_length_remaining (data_tvb, offset);
 
   /* Check to see if we already have state for this packet */
   packet_ntlmssp_info = p_get_proto_data(pinfo->fd, proto_ntlmssp);
@@ -1437,7 +1439,7 @@ dissect_ntlmssp_encrypted_payload(tvbuff_t *tvb, int offset,
   
   if (!packet_ntlmssp_info->payload_decrypted) {
     /* Pull the challenge info from the conversation */
-    conversation = find_conversation(&pinfo->src, &pinfo->dst,
+    conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 				     pinfo->ptype, pinfo->srcport,
 				     pinfo->destport, 0);
     if (conversation == NULL) {
@@ -1470,7 +1472,7 @@ dissect_ntlmssp_encrypted_payload(tvbuff_t *tvb, int offset,
 
     /* Store the decrypted contents in the packet state struct
        (of course at this point, they aren't decrypted yet) */
-    packet_ntlmssp_info->decrypted_payload = tvb_memdup(tvb, offset,
+    packet_ntlmssp_info->decrypted_payload = tvb_memdup(data_tvb, offset,
                                                         encrypted_block_length);
     decrypted_payloads = g_slist_prepend(decrypted_payloads,
                                          packet_ntlmssp_info->decrypted_payload);
@@ -1495,7 +1497,7 @@ dissect_ntlmssp_encrypted_payload(tvbuff_t *tvb, int offset,
 			       encrypted_block_length,
 			       encrypted_block_length);
 
-  tvb_set_child_real_data_tvbuff(tvb, decr_tvb);
+  tvb_set_child_real_data_tvbuff(data_tvb, decr_tvb);
   
   offset += encrypted_block_length;
 
