@@ -7,7 +7,7 @@ proper helper routines
  * Routines for dissection of ASN.1 Aligned PER
  * 2003  Ronnie Sahlberg
  *
- * $Id: packet-per.c 13337 2005-02-07 08:57:47Z etxrab $
+ * $Id: packet-per.c 14217 2005-04-28 09:54:03Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -38,6 +38,7 @@ proper helper routines
 #include <stdio.h>
 #include <string.h>
 
+#include <epan/to_str.h>
 #include <epan/prefs.h>
 #include "packet-per.h"
 #include "packet-ber.h"
@@ -205,6 +206,17 @@ dissect_per_GeneralString(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, pro
 	return offset;
 }
 
+/* 17 Encoding the null type */
+guint32
+dissect_per_null(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tree *tree, int hf_index) {
+  proto_item *ti_tmp;
+
+  ti_tmp = proto_tree_add_item(tree, hf_index, tvb, offset>>8, 0, FALSE);
+  proto_item_append_text(ti_tmp, ": NULL");
+
+  return offset;
+}
+
 /* 19 this function dissects a sequence of */
 static guint32
 dissect_per_sequence_of_helper(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tree *tree, int (*func)(tvbuff_t *, int , packet_info *, proto_tree *), guint32 length)
@@ -409,21 +421,21 @@ DEBUG_ENTRY("dissect_per_restricted_character_string");
 guint32
 dissect_per_NumericString(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tree *tree, int hf_index, int min_len, int max_len)
 {
-	offset=dissect_per_restricted_character_string(tvb, offset, pinfo, tree, hf_index, min_len, max_len, " 0123456789", 11, NULL, NULL);
+	offset=dissect_per_restricted_character_string(tvb, offset, pinfo, tree, hf_index, min_len, max_len, " 0123456789", 11, NULL, 0);
 
 	return offset;
 }
 guint32
 dissect_per_PrintableString(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tree *tree, int hf_index, int min_len, int max_len)
 {
-	offset=dissect_per_restricted_character_string(tvb, offset, pinfo, tree, hf_index, min_len, max_len, " '()+,-.*0123456789:=?ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", 74, NULL, NULL);
+	offset=dissect_per_restricted_character_string(tvb, offset, pinfo, tree, hf_index, min_len, max_len, " '()+,-.*0123456789:=?ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", 74, NULL, 0);
 	return offset;
 }
 guint32
 dissect_per_VisibleString(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tree *tree, int hf_index, int min_len, int max_len)
 {
 	offset=dissect_per_restricted_character_string(tvb, offset, pinfo, tree, hf_index, min_len, max_len,
-		" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~", 95, NULL, NULL);
+		" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~", 95, NULL, 0);
 	return offset;
 }
 guint32
@@ -550,14 +562,12 @@ DEBUG_ENTRY("dissect_per_set_of");
 
 
 
-/* this function reads a OBJECT IDENTIFIER */
+/* 23 Encoding the object identifier type */
 guint32
 dissect_per_object_identifier(tvbuff_t *tvb, guint32 offset, packet_info *pinfo _U_, proto_tree *tree, int hf_index, char *value_string)
 {
-	int i,count;
-	char str[256],*strp,*name;
-	guint8 byte;
-	guint32 value;
+	gint length;
+	char str[MAX_OID_STR_LEN], *name;
 	proto_tree *etr=NULL;
 	proto_item *item;
 
@@ -567,48 +577,12 @@ DEBUG_ENTRY("dissect_per_object_identifier");
 		etr=tree;
 	}
 
-	/* first byte is the count and it is byte aligned */
-	BYTE_ALIGN_OFFSET(offset);
-	count=tvb_get_guint8(tvb, offset>>3);
+	offset = dissect_per_length_determinant(tvb, offset, pinfo, etr, hf_per_object_identifier_length, &length);
 
+	oid_to_str_buf(tvb_get_ptr(tvb, offset>>3, length), length, str);
+	item = proto_tree_add_string(tree, hf_index, tvb, offset>>3, length, str);
+	offset += 8 * length;
 
-	proto_tree_add_uint(etr, hf_per_object_identifier_length, tvb, offset>>3, 1, count);
-	offset+=8;
-
-	value=0;
-	for(i=0,strp=str;i<count;i++){
-		byte=tvb_get_guint8(tvb,offset>>3);
-		offset+=8;
-
-		if((strp-str)>200){
-PER_NOT_DECODED_YET("too long octet_string");
-			/*XXX assert here */
-			return offset;
-		}
-
-		if(i==0){
-			/* the first byte contains the first two object identifier components */
-			if(byte<40){
-				strp+=sprintf(strp,"0.%d",byte);
-			} else if (byte<80){
-				strp+=sprintf(strp,"1.%d",byte-40);
-			} else {
-				strp+=sprintf(strp,"2.%d",byte-80);
-			}
-			continue;
-		}
-
-		value=(value<<7)|(byte&0x7f);
-		if(byte&0x80){
-			continue;
-		}
-
-		strp+=sprintf(strp,".%d",value);
-		value=0;
-	}
-	*strp=0;
-
-	item=proto_tree_add_string(tree, hf_index, tvb, (offset>>3)-count, count, str);
 	/* see if we know the name of this oid */
 	if(item){
 		name = get_ber_oid_name(str);
