@@ -2,7 +2,7 @@
 *
 * Routine to dissect ISO 8823 OSI Presentation Protocol packets
 *
-* $Id: packet-pres.c 13074 2005-01-16 16:06:15Z ulfl $
+* $Id: packet-pres.c 14307 2005-05-04 19:55:17Z gerald $
 *
 * Yuriy Sidelnikov <YSidelnikov@hotmail.com>
 *
@@ -41,7 +41,7 @@
 
 #include <epan/strutil.h>
 
-#include "asn1.h"
+#include <epan/asn1.h>
 #include "format-oid.h"
 
 #include "packet-ses.h"
@@ -247,33 +247,6 @@ call_acse_dissector(tvbuff_t *tvb, gint offset, gint param_len,
 		ENDTRY;
 	}
 }
-static char*
-string_to_hex(const unsigned char * in,char * out,int len)
-{
-	char ascii[MAXSTRING];
-	int  i;
-	memset(&ascii,0x00,sizeof(ascii));
-for(i=0;i<len;i++)
-			{
-	unsigned char o_out = *(in+i);
-    sprintf(out+(i<<1),"%.2x",*  (in+i));
-	if(  ( (o_out) >= 'a') & ( (o_out) <='z')  ||
-		 ( (o_out) >= 'A') & ( (o_out) <='Z')  ||
-		 ( (o_out) >= '0') & ( (o_out) <='9')
-	   )
-					{
-					ascii[i] = o_out;
-					}
-				else
-					{
-					ascii[i] = '.';
-					}
-
-			}
-		strcat(out," ");
-		strcat(out,ascii);
-    return out;
-}
 
 static int read_length(ASN1_SCK *a, proto_tree *tree, int hf_id, guint *len)
 {
@@ -498,17 +471,18 @@ display_string);
 }
 
 static void
-print_value(ASN1_SCK *asn,proto_tree *pres_tree,tvbuff_t *tvb,int 
-*offset,int item_len)
+print_value(ASN1_SCK *asn,proto_tree *pres_tree, tvbuff_t *tvb, int *offset, int item_len)
 {
-	  gint    start = *offset;
-	  char    tmp[MAXSTRING];
-		*offset = asn->offset;  /* align to data*/
-		string_to_hex(tvb_get_ptr(tvb,*offset,item_len),tmp,item_len);
-		proto_tree_add_text(pres_tree, tvb, *offset, item_len, tmp);
-		(*offset)=start+item_len;
-		asn->offset = (*offset);
+    gint    start = *offset;
+    gchar  *tmp;
+
+    *offset = asn->offset;  /* align to data*/
+    tmp = tvb_bytes_to_str(tvb, *offset, item_len);
+    proto_tree_add_text(pres_tree, tvb, *offset, item_len, tmp);
+    (*offset) = start+item_len;
+    asn->offset = (*offset);
 }
+
 static int
 get_integer_value(ASN1_SCK *asn,int length,int *offset)
 {
@@ -1371,7 +1345,7 @@ show_connection_request_confirm(ASN1_SCK *asn,proto_tree *pres_tree,tvbuff_t
 				proto_tree_add_uint(pres_tree_ms, hf_pres_ms_type, tvb, (*offset)-1, 1, 
 type);
 				proto_tree_add_text(pres_tree_ms, tvb, *offset, (asn->offset-*offset),
-										"Lenght:%u",*item_len);
+										"Length:%u",*item_len);
 				*offset=asn->offset;
 				show_integer(asn,pres_tree_ms,tvb,offset,*item_len);
 					break;
@@ -1380,7 +1354,7 @@ type);
 				proto_tree_add_uint(pres_tree_ms, hf_pres_seq_type, tvb, (*offset)-1, 1, 
 type);
 				proto_tree_add_text(pres_tree_ms, tvb, *offset, (asn->offset-*offset),
-										"Lenght:%u",*item_len);
+										"Length:%u",*item_len);
 				*offset=asn->offset;
 				show_sequence_top(asn,pres_tree_ms,tvb,pinfo,offset,*item_len);
 					break;
@@ -1577,10 +1551,12 @@ dissect_ppdu(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree
 						}
 					}
 				}
+                if( ((gint)rest_len) > 0) {
 				ms = proto_tree_add_text(pres_tree, tvb, offset, rest_len,
 										val_to_str(session->spdu_type, ses_vals, "Unknown Ppdu type (0x%02x)"));
 				pres_tree_ms = proto_item_add_subtree(ms, ett_pres_ms);
 				show_user_data(&asn,pres_tree_ms,tvb,&offset,rest_len,s_type);
+                }
 			}
 	}
 /*    close asn.1 stream    */
