@@ -3,7 +3,7 @@
  * disassembly
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-q931.h 13098 2005-01-17 22:26:33Z etxrab $
+ * $Id: packet-q931.h 13263 2005-02-03 21:51:35Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,7 +31,7 @@ extern void dissect_q931_bearer_capability_ie(tvbuff_t *, int, int,
     proto_tree *);
 
 extern void dissect_q931_cause_ie(tvbuff_t *, int, int,
-    proto_tree *, int);
+    proto_tree *, int, guint8 *);
 
 extern void dissect_q931_progress_indicator_ie(tvbuff_t *, int, int,
     proto_tree *);
@@ -44,7 +44,19 @@ extern void dissect_q931_user_user_ie(tvbuff_t *tvb, int offset, int len,
 
 extern const value_string q931_cause_location_vals[];
 
-extern const value_string q931_cause_code_vals[];
+typedef struct _q931_packet_info {
+       gchar *calling_number;
+       gchar *called_number;
+       guint8 cause_value;
+       gint32 crv;
+} q931_packet_info;
+
+/*
+ * the following allows TAP code access to the messages
+ * without having to duplicate it. With MSVC and a
+ * libethereal.dll, we need a special declaration.
+ */
+ETH_VAR_IMPORT const value_string q931_cause_code_vals[];
 
 extern const value_string q931_protocol_discriminator_vals[];
 
