@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c 12332 2004-10-17 22:14:47Z guy $
+ * $Id: packet-rtp.c 14308 2005-05-04 19:57:42Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -69,10 +69,12 @@
 #include <epan/prefs.h>
 
 static dissector_handle_t rtp_handle;
+static dissector_handle_t stun_handle;
 
 static int rtp_tap = -1;
 
 static dissector_table_t rtp_pt_dissector_table;
+static dissector_table_t rtp_dyn_pt_dissector_table;
 
 /* RTP header fields             */
 static int proto_rtp           = -1;
@@ -106,6 +108,17 @@ static gint ett_csrc_list = -1;
 static gint ett_hdr_ext   = -1;
 static gint ett_rtp_setup = -1;
 
+
+#define RTP0_INVALID 0
+#define RTP0_STUN    1
+
+static enum_val_t rtp_version0_types[] = {
+	{ "invalid", "Invalid RTP packets", RTP0_INVALID },
+	{ "stun", "STUN packets", RTP0_STUN },
+	{ NULL, NULL, 0 }
+};
+static guint global_rtp_version0_type = 0;
+
 static dissector_handle_t data_handle;
 
 static gboolean dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo,
@@ -113,6 +126,7 @@ static gboolean dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo,
 static void dissect_rtp( tvbuff_t *tvb, packet_info *pinfo,
     proto_tree *tree );
 static void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+static void get_conv_info(packet_info *pinfo, struct _rtp_info *rtp_info);
 
 /* Preferences bool to control whether or not setup info should be shown */
 static gboolean global_rtp_show_setup_info = TRUE;
@@ -190,11 +204,59 @@ const value_string rtp_payload_type_vals[] =
 	{ 0,		NULL },
 };
 
+const value_string rtp_payload_type_short_vals[] =
+{
+       { PT_PCMU,      "g711U" },
+       { PT_1016,      "fs-1016" },
+       { PT_G721,      "g721" },
+       { PT_GSM,       "GSM" },
+       { PT_G723,      "g723" },
+       { PT_DVI4_8000, "DVI4 8k" },
+       { PT_DVI4_16000, "DVI4 16k" },
+       { PT_LPC,       "Exp. from Xerox PARC" },
+       { PT_PCMA,      "g711A" },
+       { PT_G722,      "g722" },
+       { PT_L16_STEREO, "16-bit audio, stereo" },
+       { PT_L16_MONO,  "16-bit audio, monaural" },
+       { PT_QCELP,     "Qualcomm" },
+       { PT_CN,        "CN" },
+       { PT_MPA,       "MPEG-I/II Audio"},
+       { PT_G728,      "g728" },
+       { PT_DVI4_11025, "DVI4 11k" },
+       { PT_DVI4_22050, "DVI4 22k" },
+       { PT_G729,      "g729" },
+       { PT_CN_OLD,    "CN(old)" },
+       { PT_CELB,      "CellB" },
+       { PT_JPEG,      "JPEG" },
+       { PT_NV,        "NV" },
+       { PT_H261,      "h261" },
+       { PT_MPV,       "MPEG-I/II Video"},
+       { PT_MP2T,      "MPEG-II streams"},
+       { PT_H263,      "h263" },
+       { 0,            NULL },
+};
+
+static void
+free_hash_item( gpointer key _U_ , gpointer value, gpointer user_data _U_ )
+{
+        g_free(value);
+		g_free(key);
+}
+
+void
+rtp_free_hash_dyn_payload(GHashTable *rtp_dyn_payload)
+{
+	if (rtp_dyn_payload == NULL) return;
+	g_hash_table_foreach(rtp_dyn_payload, (GHFunc)free_hash_item, NULL);
+	g_hash_table_destroy(rtp_dyn_payload);
+	rtp_dyn_payload = NULL;
+}
+
 /* Set up an RTP conversation */
 void rtp_add_address(packet_info *pinfo,
                      address *addr, int port,
                      int other_port,
-                     gchar *setup_method, guint32 setup_frame_number)
+                     gchar *setup_method, guint32 setup_frame_number, GHashTable *rtp_dyn_payload)
 {
 	address null_addr;
 	conversation_t* p_conv;
@@ -216,14 +278,14 @@ void rtp_add_address(packet_info *pinfo,
 	 * Check if the ip address and port combination is not
 	 * already registered as a conversation.
 	 */
-	p_conv = find_conversation( addr, &null_addr, PT_UDP, port, other_port,
+	p_conv = find_conversation( setup_frame_number, addr, &null_addr, PT_UDP, port, other_port,
                                 NO_ADDR_B | (!other_port ? NO_PORT_B : 0));
 
 	/*
 	 * If not, create a new conversation.
 	 */
-	if ( ! p_conv ) {
-		p_conv = conversation_new( addr, &null_addr, PT_UDP,
+	if ( !p_conv || p_conv->setup_frame != setup_frame_number) {
+		p_conv = conversation_new( setup_frame_number, addr, &null_addr, PT_UDP,
 		                           (guint32)port, (guint32)other_port,
 								   NO_ADDR2 | (!other_port ? NO_PORT2 : 0));
 	}
@@ -242,6 +304,7 @@ void rtp_add_address(packet_info *pinfo,
 	if ( ! p_conv_data ) {
 		/* Create conversation data */
 		p_conv_data = g_mem_chunk_alloc(rtp_conversations);
+		p_conv_data->rtp_dyn_payload = NULL;
 
 		conversation_add_proto_data(p_conv, proto_rtp, p_conv_data);
 	}
@@ -249,9 +312,13 @@ void rtp_add_address(packet_info *pinfo,
 	/*
 	 * Update the conversation data.
 	 */
+	/* Free the hash if already exists */
+	rtp_free_hash_dyn_payload(p_conv_data->rtp_dyn_payload);
+
 	strncpy(p_conv_data->method, setup_method, MAX_RTP_SETUP_METHOD_SIZE);
 	p_conv_data->method[MAX_RTP_SETUP_METHOD_SIZE] = '\0';
 	p_conv_data->frame_number = setup_frame_number;
+	p_conv_data->rtp_dyn_payload = rtp_dyn_payload;
 }
 
 static void rtp_init( void )
@@ -287,7 +354,17 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	octet1 = tvb_get_guint8( tvb, offset );
 	version = RTP_VERSION( octet1 );
 
-	if (version != 2) {
+	if (version == 0) {
+		switch (global_rtp_version0_type) {
+		case RTP0_STUN:
+			call_dissector(stun_handle, tvb, pinfo, tree);
+			return TRUE;
+
+		case RTP0_INVALID:
+		default:
+			return FALSE; /* Unknown or unsupported version */
+		}
+	} else if (version != 2) {
 		/* Unknown or unsupported version */
 		return FALSE;
 	}
@@ -314,11 +391,27 @@ dissect_rtp_data( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     unsigned int data_reported_len, unsigned int payload_type )
 {
 	tvbuff_t *newtvb;
+	struct _rtp_conversation_info *p_conv_data = NULL;
+	gboolean found_match = FALSE;
 
 	newtvb = tvb_new_subset( tvb, offset, data_len, data_reported_len );
-	if (!dissector_try_port(rtp_pt_dissector_table, payload_type, newtvb,
-	    pinfo, tree))
-		proto_tree_add_item( rtp_tree, hf_rtp_data, newtvb, 0, -1, FALSE );
+
+	/* if the payload type is dynamic (96 to 127), we check if the conv is set and we look for the pt definition */
+	if ( (payload_type >=96) && (payload_type <=127) ) {
+		p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
+		if (p_conv_data && p_conv_data->rtp_dyn_payload) {
+			gchar *payload_type_str = NULL;
+			payload_type_str = g_hash_table_lookup(p_conv_data->rtp_dyn_payload, &payload_type);
+			if (payload_type_str)
+				found_match = dissector_try_string(rtp_dyn_pt_dissector_table,
+													payload_type_str, newtvb, pinfo, tree);
+		}
+	}
+	/* if we don't found, it is static OR could be set static from the preferences */
+	if (found_match == FALSE)
+		if (!dissector_try_port(rtp_pt_dissector_table, payload_type, newtvb, pinfo, tree))
+			proto_tree_add_item( rtp_tree, hf_rtp_data, newtvb, 0, -1, FALSE );
+
 }
 
 static void
@@ -334,6 +427,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	unsigned int csrc_count;
 	gboolean    marker_set;
 	unsigned int payload_type;
+	gchar *payload_type_str = NULL;
 	unsigned int i            = 0;
 	unsigned int hdr_extension= 0;
 	unsigned int padding_count;
@@ -344,15 +438,37 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	guint32     timestamp;
 	guint32     sync_src;
 	guint32     csrc_item;
+	struct _rtp_conversation_info *p_conv_data = NULL;
 
-	static struct _rtp_info rtp_info;
+	/* Can tap up to 4 RTP packets within same packet */
+	static struct _rtp_info rtp_info_arr[4];
+	static int rtp_info_current=0;
+	struct _rtp_info *rtp_info;
+
+	rtp_info_current++;
+	if (rtp_info_current==4) {
+		rtp_info_current=0;
+	}
+	rtp_info = &rtp_info_arr[rtp_info_current];
 
 	/* Get the fields in the first octet */
 	octet1 = tvb_get_guint8( tvb, offset );
 	version = RTP_VERSION( octet1 );
 
+	if (version == 0) {
+		switch (global_rtp_version0_type) {
+		case RTP0_STUN:
+			call_dissector(stun_handle, tvb, pinfo, tree);
+			return;
+
+		case RTP0_INVALID:
+		default:
+			; /* Unknown or unsupported version (let it fall through */
+		}
+	}
+
 	/* fill in the rtp_info structure */
-	rtp_info.info_version = version;
+	rtp_info->info_version = version;
 	if (version != 2) {
 		/*
 		 * Unknown or unsupported version.
@@ -391,13 +507,14 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	sync_src = tvb_get_ntohl( tvb, offset + 8 );
 
 	/* fill in the rtp_info structure */
-	rtp_info.info_padding_set = padding_set;
-	rtp_info.info_padding_count = 0;
-	rtp_info.info_marker_set = marker_set;
-	rtp_info.info_payload_type = payload_type;
-	rtp_info.info_seq_num = seq_num;
-	rtp_info.info_timestamp = timestamp;
-	rtp_info.info_sync_src = sync_src;
+	rtp_info->info_padding_set = padding_set;
+	rtp_info->info_padding_count = 0;
+	rtp_info->info_marker_set = marker_set;
+	rtp_info->info_payload_type = payload_type;
+	rtp_info->info_seq_num = seq_num;
+	rtp_info->info_timestamp = timestamp;
+	rtp_info->info_sync_src = sync_src;
+	rtp_info->info_setup_frame_num = 0;
 
 	/*
 	 * Do we have all the data?
@@ -408,8 +525,8 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		/*
 		 * Yes.
 		 */
-		rtp_info.info_all_data_present = TRUE;
-		rtp_info.info_data_len = reported_length;
+		rtp_info->info_all_data_present = TRUE;
+		rtp_info->info_data_len = reported_length;
 
 		/*
 		 * Save the pointer to raw rtp data (header + payload incl.
@@ -422,31 +539,44 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		 * their data.)
 		 * See "add_packet_to_packet_list()" for details.
 		 */
-		rtp_info.info_data = tvb_get_ptr(tvb, 0, -1);
+		rtp_info->info_data = tvb_get_ptr(tvb, 0, -1);
 	} else {
 		/*
 		 * No - packet was cut short at capture time.
 		 */
-		rtp_info.info_all_data_present = FALSE;
-		rtp_info.info_data_len = 0;
-		rtp_info.info_data = NULL;
+		rtp_info->info_all_data_present = FALSE;
+		rtp_info->info_data_len = 0;
+		rtp_info->info_data = NULL;
 	}
 
+	/* Look for conv and add to the frame if found */
+	get_conv_info(pinfo, rtp_info);
+
 	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )   {
 		col_set_str( pinfo->cinfo, COL_PROTOCOL, "RTP" );
 	}
 
+	/* if it is dynamic payload, let use the conv data to see if it is defined */
+	if ( (payload_type>95) && (payload_type<128) ) {
+		/* Use existing packet info if available */
+		p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
+		if (p_conv_data && p_conv_data->rtp_dyn_payload)
+			payload_type_str = g_hash_table_lookup(p_conv_data->rtp_dyn_payload, &payload_type);
+	}
+
 	if ( check_col( pinfo->cinfo, COL_INFO) ) {
 		col_add_fstr( pinfo->cinfo, COL_INFO,
 		    "Payload type=%s, SSRC=%u, Seq=%u, Time=%u%s",
-		    val_to_str( payload_type, rtp_payload_type_vals,
-		        "Unknown (%u)" ),
+			payload_type_str ? payload_type_str : val_to_str( payload_type, rtp_payload_type_vals,"Unknown (%u)" ),
 		    sync_src,
 		    seq_num,
 		    timestamp,
 		    marker_set ? ", Mark" : "");
 	}
+
+
 	if ( tree ) {
+		proto_tree *item;
 		/* Create RTP protocol tree */
 		ti = proto_tree_add_item(tree, proto_rtp, tvb, offset, -1, FALSE );
 		rtp_tree = proto_item_add_subtree(ti, ett_rtp );
@@ -469,8 +599,12 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 
 		proto_tree_add_boolean( rtp_tree, hf_rtp_marker, tvb, offset,
 		    1, octet2 );
-		proto_tree_add_uint( rtp_tree, hf_rtp_payload_type, tvb,
-		    offset, 1, octet2 );
+
+		item = proto_tree_add_uint_format( rtp_tree, hf_rtp_payload_type, tvb,
+		    offset, 1, octet2, "Payload type: %s (%u)", 
+			payload_type_str ? payload_type_str : val_to_str( payload_type, rtp_payload_type_vals,"Unknown"),
+			payload_type);
+
 		offset++;
 
 		/* Sequence number 16 bits (2 octets) */
@@ -489,7 +623,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	}
 	/* CSRC list*/
 	if ( csrc_count > 0 ) {
-        	if ( tree ) {
+		if ( tree ) {
 			ti = proto_tree_add_text(rtp_tree, tvb, offset, csrc_count * 4, "Contributing Source identifiers");
 			rtp_csrc_tree = proto_item_add_subtree( ti, ett_csrc_list );
 		}
@@ -560,9 +694,9 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		data_len =
 		    tvb_reported_length_remaining( tvb, offset ) - padding_count;
 
-		rtp_info.info_payload_offset = offset;
-		rtp_info.info_payload_len = tvb_length_remaining(tvb, offset);
-		rtp_info.info_padding_count = padding_count;
+		rtp_info->info_payload_offset = offset;
+		rtp_info->info_payload_len = tvb_length_remaining(tvb, offset);
+		rtp_info->info_padding_count = padding_count;
 
 		if (data_len > 0) {
 			/*
@@ -612,16 +746,15 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		    tvb_length_remaining( tvb, offset ),
 		    tvb_reported_length_remaining( tvb, offset ),
 		    payload_type );
-		rtp_info.info_payload_offset = offset;
-		rtp_info.info_payload_len = tvb_length_remaining(tvb, offset);
+		rtp_info->info_payload_offset = offset;
+		rtp_info->info_payload_len = tvb_length_remaining(tvb, offset);
 	}
 	if (!pinfo->in_error_pkt)
-		tap_queue_packet(rtp_tap, pinfo, &rtp_info);
+		tap_queue_packet(rtp_tap, pinfo, rtp_info);
 }
 
-
-/* Look for conversation info and display any setup info found */
-void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+/* Look for conversation info */
+static void get_conv_info(packet_info *pinfo, struct _rtp_info *rtp_info)
 {
 	/* Conversation and current data */
 	conversation_t *p_conv = NULL;
@@ -633,9 +766,9 @@ void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (!p_conv_data)
 	{
 		/* First time, get info from conversation */
-		p_conv = find_conversation(&pinfo->net_dst, &pinfo->net_src,
-                                   pinfo->ptype,
-                                   pinfo->destport, pinfo->srcport, NO_ADDR_B);
+		p_conv = find_conversation(pinfo->fd->num, &pinfo->net_dst, &pinfo->net_src,
+		                           pinfo->ptype,
+		                           pinfo->destport, pinfo->srcport, NO_ADDR_B);
 		if (p_conv)
 		{
 			/* Create space for packet info */
@@ -647,16 +780,30 @@ void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				p_conv_packet_data = g_mem_chunk_alloc(rtp_conversations);
 				strcpy(p_conv_packet_data->method, p_conv_data->method);
 				p_conv_packet_data->frame_number = p_conv_data->frame_number;
+				p_conv_packet_data->rtp_dyn_payload = p_conv_data->rtp_dyn_payload;
 				p_add_proto_data(pinfo->fd, proto_rtp, p_conv_packet_data);
 			}
 		}
 	}
+	if (p_conv_data) rtp_info->info_setup_frame_num = p_conv_data->frame_number;
+}
+
+
+/* Display setup info */
+static void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	/* Conversation and current data */
+	struct _rtp_conversation_info *p_conv_data = NULL;
+		proto_tree *rtp_setup_tree;
+	proto_item *ti;
+
+	/* Use existing packet info if available */
+	p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
+
+	if (!p_conv_data) return;
 
 	/* Create setup info subtree with summary info. */
-	if (p_conv_data)
-	{
-		proto_tree *rtp_setup_tree;
-		proto_item *ti =  proto_tree_add_string_format(tree, hf_rtp_setup, tvb, 0, 0,
+	ti =  proto_tree_add_string_format(tree, hf_rtp_setup, tvb, 0, 0,
 		                                               "",
 		                                               "Stream setup by %s (frame %d)",
 		                                               p_conv_data->method,
@@ -673,7 +820,6 @@ void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			                             tvb, 0, 0, p_conv_data->method);
 			PROTO_ITEM_SET_GENERATED(item);
 		}
-	}
 }
 
 
@@ -749,7 +895,7 @@ proto_register_rtp(void)
 				"rtp.p_type",
 				FT_UINT8,
 				BASE_DEC,
-				VALS(rtp_payload_type_vals),
+				NULL,
 				0x7F,
 				"", HFILL
 			}
@@ -930,25 +1076,34 @@ proto_register_rtp(void)
 	proto_register_subtree_array(ett, array_length(ett));
 
 	register_dissector("rtp", dissect_rtp, proto_rtp);
+
 	rtp_tap = register_tap("rtp");
 
 	rtp_pt_dissector_table = register_dissector_table("rtp.pt",
-	    "RTP payload type", FT_UINT8, BASE_DEC);
+	                                                  "RTP payload type", FT_UINT8, BASE_DEC);
+	rtp_dyn_pt_dissector_table = register_dissector_table("rtp_dyn_payload_type",
+												    "Dynamic RTP payload type", FT_STRING, BASE_NONE);
+
 
 	rtp_module = prefs_register_protocol(proto_rtp, NULL);
 
 	prefs_register_bool_preference(rtp_module, "show_setup_info",
-		"Show stream setup information",
-		"Where available, show which protocol and frame caused "
-		"this RTP stream to be created",
-		&global_rtp_show_setup_info);
+	                               "Show stream setup information",
+	                               "Where available, show which protocol and frame caused "
+	                               "this RTP stream to be created",
+	                               &global_rtp_show_setup_info);
 
 	prefs_register_bool_preference(rtp_module, "heuristic_rtp",
-		"Try to decode RTP outside of conversations ",
-                "If call control SIP/H323/RTSP/.. messages are missing in the trace, "
-                "RTP isn't decoded without this",
-		&global_rtp_heur);
-   
+	                               "Try to decode RTP outside of conversations",
+	                               "If call control SIP/H323/RTSP/.. messages are missing in the trace, "
+	                               "RTP isn't decoded without this",
+	                               &global_rtp_heur);
+
+	prefs_register_enum_preference(rtp_module, "version0_type",
+	                               "Treat RTP version 0 packets as",
+	                               "If an RTP version 0 packet is encountered, it can be treated as an invalid packet or a STUN packet",
+	                               &global_rtp_version0_type,
+	                               rtp_version0_types, FALSE);
 	register_init_routine( &rtp_init );
 }
 
@@ -956,7 +1111,7 @@ void
 proto_reg_handoff_rtp(void)
 {
 	data_handle = find_dissector("data");
-
+	stun_handle = find_dissector("stun");
 	/*
 	 * Register this dissector as one that can be selected by a
 	 * UDP port number.
