@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-sdp.c 12963 2005-01-05 10:51:05Z guy $
+ * $Id: packet-sdp.c 14180 2005-04-24 18:43:42Z lroland $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -57,6 +57,9 @@
 #include <epan/conversation.h>
 #include <epan/strutil.h>
 
+#include "tap.h"
+#include "packet-sdp.h"
+
 #include "packet-rtp.h"
 #include "rtp_pt.h"
 #include "packet-rtcp.h"
@@ -68,6 +71,9 @@ static dissector_handle_t rtcp_handle=NULL;
 
 static dissector_handle_t t38_handle=NULL;
 
+static int sdp_tap = -1;
+static sdp_packet_info *sdp_pi;
+
 static int proto_sdp = -1;
 
 /* Top level fields */
@@ -157,12 +163,20 @@ static int ett_sdp_media_attribute = -1;
 
 
 #define SDP_MAX_RTP_CHANNELS 4
+#define SDP_MAX_RTP_PAYLOAD_TYPES 20
+
+typedef struct {
+	gint32 pt[SDP_MAX_RTP_PAYLOAD_TYPES];
+	gint8 pt_count;
+	GHashTable *rtp_dyn_payload;
+} transport_media_pt_t;
 
 typedef struct {
 	char *connection_address;
 	char *connection_type;
 	char *media_port[SDP_MAX_RTP_CHANNELS];
 	char *media_proto[SDP_MAX_RTP_CHANNELS];
+	transport_media_pt_t media[SDP_MAX_RTP_CHANNELS];
 	gint8 media_count;
 } transport_info_t;
 
@@ -183,7 +197,7 @@ static void dissect_sdp_encryption_key(tvbuff_t *tvb, proto_item * ti);
 static void dissect_sdp_session_attribute(tvbuff_t *tvb, proto_item *ti);
 static void dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
 			      transport_info_t *transport_info);
-static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item *ti);
+static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item *ti, transport_info_t *transport_info);
 
 static void
 dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
@@ -207,11 +221,16 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 	guint32 	port=0;
 	gboolean 	is_rtp=FALSE;
-	gboolean        is_t38=FALSE;
+	gboolean 	is_t38=FALSE;
+	gboolean 	set_rtp=FALSE;
 	gboolean 	is_ipv4_addr=FALSE;
 	gboolean	is_ipv6_addr=FALSE;
     guint32 	ipaddr[4];
-	gint		n;
+	gint		n,i;
+
+    /* Initialise packet info for passing to tap */
+    sdp_pi = g_malloc(sizeof(sdp_packet_info));
+    sdp_pi->summary_str[0] = '\0';
 
 	/* Initialise RTP channel info */
 	transport_info.connection_address=NULL;
@@ -220,10 +239,11 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	{
 	    transport_info.media_port[n]=NULL;
 	    transport_info.media_proto[n]=NULL;
+		transport_info.media[n].pt_count = 0;
+		transport_info.media[n].rtp_dyn_payload = g_hash_table_new( g_int_hash, g_int_equal);
 	}
 	transport_info.media_count = 0;
 
-
 	/*
 	 * As RFC 2327 says, "SDP is purely a format for session
 	 * description - it does not incorporate a transport protocol,
@@ -399,27 +419,53 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				}
 		    }
 	    }
-	    /* Add rtp and rtcp conversation, if available */
+		set_rtp = FALSE;
+	    /* Add rtp and rtcp conversation, if available (overrides t38 if conversation already set) */
 	    if((!pinfo->fd->flags.visited) && port!=0 && is_rtp && (is_ipv4_addr || is_ipv6_addr)){
 		    src_addr.data=(char *)&ipaddr;
 		    if(rtp_handle){
 				rtp_add_address(pinfo, &src_addr, port, 0,
-	                "SDP", pinfo->fd->num);
+	                "SDP", pinfo->fd->num, transport_info.media[n].rtp_dyn_payload);
+				set_rtp = TRUE;
 		    }
 		    if(rtcp_handle){
 				port++;
 				rtcp_add_address(pinfo, &src_addr, port, 0,
 	                 "SDP", pinfo->fd->num);
 		    }
-	    }
+		} 
 			
-	    /* Add t38 conversation, if available */
-	    if((!pinfo->fd->flags.visited) && port!=0 && is_t38 && is_ipv4_addr){
+	    /* Add t38 conversation, if available and only if no rtp */
+	    if((!pinfo->fd->flags.visited) && port!=0 && !set_rtp && is_t38 && is_ipv4_addr){
                     src_addr.data=(char *)&ipaddr;
                     if(t38_handle){
                                 t38_add_address(pinfo, &src_addr, port, 0, "SDP", pinfo->fd->num);
                     }
 	    }
+
+		/* Create the summary str for the Voip Call analysis */
+		for (i = 0; i < transport_info.media[n].pt_count; i++)
+		{
+			/* if the payload type is dynamic (96 to 127), check the hash table to add the desc in the SDP summary */
+			if ( (transport_info.media[n].pt[i] >=96) && (transport_info.media[n].pt[i] <=127) ) {
+				gchar *str_dyn_pt = g_hash_table_lookup(transport_info.media[n].rtp_dyn_payload, &transport_info.media[n].pt[i]);
+				if (str_dyn_pt)
+					g_snprintf(sdp_pi->summary_str, 50, "%s %s", sdp_pi->summary_str, str_dyn_pt);
+				else
+					g_snprintf(sdp_pi->summary_str, 50, "%s %d", sdp_pi->summary_str, transport_info.media[n].pt[i]);
+			} else 
+				g_snprintf(sdp_pi->summary_str, 50, "%s %s", sdp_pi->summary_str, val_to_str(transport_info.media[n].pt[i], rtp_payload_type_short_vals, "%u"));
+		}
+
+		/* Free the hash table if we did't assigned it to a conv use it */
+		if (set_rtp == FALSE) 
+			rtp_free_hash_dyn_payload(transport_info.media[n].rtp_dyn_payload);
+	}
+
+	/* Free the remainded hash tables not used */
+	for (n = transport_info.media_count; n < SDP_MAX_RTP_CHANNELS; n++)
+	{
+		rtp_free_hash_dyn_payload(transport_info.media[n].rtp_dyn_payload);
 	}
 
 	/* Free up 'connection info' strings */
@@ -430,12 +476,14 @@ dissect_sdp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		g_free(transport_info.connection_type);
 	}
 
-
 	datalen = tvb_length_remaining(tvb, offset);
 	if (datalen > 0) {
 		proto_tree_add_text(sdp_tree, tvb, offset, datalen,
 		    "Data (%d bytes)", datalen);
 	}
+
+    /* Report this packet to the tap */
+    tap_queue_packet(sdp_tap, pinfo, sdp_pi);
 }
 
 static void
@@ -459,7 +507,7 @@ call_sdp_subdissector(tvbuff_t *tvb, int hf, proto_tree* ti, transport_info_t *t
   } else if ( hf == hf_media ) {
     dissect_sdp_media(tvb,ti,transport_info);
   } else if ( hf == hf_media_attribute ){
-    dissect_sdp_media_attribute(tvb,ti);
+    dissect_sdp_media_attribute(tvb,ti,transport_info);
   }
 }
 
@@ -797,7 +845,7 @@ static void
 dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
 		  transport_info_t *transport_info){
   proto_tree *sdp_media_tree;
-  gint offset, next_offset, tokenlen;
+  gint offset, next_offset, tokenlen, index;
   guint8 *media_format;
 
   offset = 0;
@@ -884,6 +932,10 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
       media_format = tvb_get_string(tvb, offset, tokenlen);
       proto_tree_add_string(sdp_media_tree, hf_media_format, tvb, offset,
                              tokenlen, val_to_str(atol(media_format), rtp_payload_type_vals, "%u"));
+	  index = transport_info->media[transport_info->media_count].pt_count;
+	  transport_info->media[transport_info->media_count].pt[index] = atol(media_format);
+	  if (index < (SDP_MAX_RTP_PAYLOAD_TYPES-1))
+		  transport_info->media[transport_info->media_count].pt_count++;
       g_free(media_format);
     } else {
       proto_tree_add_item(sdp_media_tree, hf_media_format, tvb,
@@ -906,9 +958,12 @@ dissect_sdp_media(tvbuff_t *tvb, proto_item *ti,
 
 }
 
-static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti){
+static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti, transport_info_t *transport_info){
   proto_tree *sdp_media_attribute_tree;
-  gint offset, next_offset, tokenlen;
+  gint offset, next_offset, tokenlen, n;
+  guint8 *field_name;
+  guint8 *payload_type;
+  guint8 *encoding_name;
 
   offset = 0;
   next_offset = 0;
@@ -928,11 +983,78 @@ static void dissect_sdp_media_attribute(tvbuff_t *tvb, proto_item * ti){
 		      hf_media_attribute_field,
 		      tvb, offset, tokenlen, FALSE);
 
+  field_name = tvb_get_string(tvb, offset, tokenlen);
+
   offset = next_offset + 1;
   proto_tree_add_item(sdp_media_attribute_tree,
 		      hf_media_attribute_value,
 		      tvb, offset, -1, FALSE);
 
+  /* decode the rtpmap to see if it is DynamicPayload to dissect them automatic */
+  if (strcmp(field_name, "rtpmap") == 0) {
+	gint *key;
+
+	next_offset = tvb_find_guint8(tvb,offset,-1,' ');
+	g_free(field_name);
+
+    if(next_offset == -1)
+		return;
+
+    tokenlen = next_offset - offset;
+
+	payload_type = tvb_get_string(tvb, offset, tokenlen);
+
+    offset = next_offset + 1;
+
+    next_offset = tvb_find_guint8(tvb,offset,-1,'/');
+
+    if(next_offset == -1){
+		g_free(payload_type);
+        return;
+    }
+
+    tokenlen = next_offset - offset;
+
+    encoding_name = tvb_get_string(tvb, offset, tokenlen);
+
+	key=g_malloc( sizeof(gint) );
+	*key=atol(payload_type);
+
+	/* As per RFC2327 it is possible to have multiple Media Descriptions ("m=") For example:
+
+			a=rtpmap:101 G726-32/8000
+			m=audio 49170 RTP/AVP 0 97
+			a=rtpmap:97 telephone-event/8000
+			m=audio 49172 RTP/AVP 97 101
+			a=rtpmap:97 G726-24/8000
+
+	The Media attributes ("a="s) after the "m=" only apply for that "m=". 
+	If there is an "a=" before the first "m=", that attribute apply for all the session	(all the "m="s).
+	*/
+
+	/* so, if this "a=" appear before any "m=", we add it to all the dynamic hash tables */ 
+	if (transport_info->media_count == 0) {
+		for (n=0; n < SDP_MAX_RTP_CHANNELS; n++) {
+			if (n==0)
+				g_hash_table_insert(transport_info->media[n].rtp_dyn_payload, key, encoding_name);
+			else {	/* we create a new key and encoding_name to assign to the other hash tables */
+				gint *key2;
+				key2=g_malloc( sizeof(gint) );
+				*key2=atol(payload_type);
+				g_hash_table_insert(transport_info->media[n].rtp_dyn_payload, key2, g_strdup(encoding_name));
+			}
+		}
+
+	/* if the "a=" is after an "m=", only apply to this "m=" */
+	} else 
+		/* in case there is an overflow in SDP_MAX_RTP_CHANNELS, we keep always the last "m=" */
+		if (transport_info->media_count == SDP_MAX_RTP_CHANNELS-1)
+			g_hash_table_insert(transport_info->media[ transport_info->media_count ].rtp_dyn_payload, key, encoding_name);		
+		else
+			g_hash_table_insert(transport_info->media[ transport_info->media_count-1 ].rtp_dyn_payload, key, encoding_name);
+
+    g_free(payload_type);
+  } else g_free(field_name);
 }
 
 void
@@ -1167,6 +1289,9 @@ proto_register_sdp(void)
    * on Windows without stuffing it into the Big Transfer Vector).
    */
   register_dissector("sdp", dissect_sdp, proto_sdp);
+
+  /* Register for tapping */
+  sdp_tap = register_tap("sdp");
 }
 
 void
