@@ -2,7 +2,7 @@
 *
 * Routine to dissect ISO 8327-1 OSI Session Protocol packets
 *
-* $Id: packet-ses.c 12115 2004-09-27 22:55:15Z guy $
+* $Id: packet-ses.c 13074 2005-01-16 16:06:15Z ulfl $
 *
 * Yuriy Sidelnikov <YSidelnikov@hotmail.com>
 *
@@ -298,7 +298,7 @@ call_pres_dissector(tvbuff_t *tvb, int offset, guint16 param_len,
 		}
 		CATCH_ALL
 		{
-			show_exception(tvb, pinfo, tree, EXCEPT_CODE);
+			show_exception(tvb, pinfo, tree, EXCEPT_CODE, GET_MESSAGE);
 		}
 		ENDTRY;
 	}
