@@ -5,7 +5,7 @@
  * Copyright 2002, Richard Sharpe <rsharpe@ns.aus.com>
  * Copyright 2003, Richard Sharpe <rsharpe@richardsharpe.com>
  *
- * $Id: packet-spnego.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-spnego.c 13850 2005-03-21 10:52:57Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -25,7 +25,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
-
+/* The heimdal code for decryption of GSSAPI wrappers using heimdal comes from
+   Heimdal 1.6 and has been modified for ethereal's requirements.
+*/
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -34,13 +36,16 @@
 # include <sys/types.h>
 #endif
 
+#include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
 
-#include "asn1.h"
+#include <epan/asn1.h>
 #include "format-oid.h"
+#include "packet-dcerpc.h"
 #include "packet-gssapi.h"
 #include "packet-kerberos.h"
+#include <epan/crypt-rc4.h>
 #include <epan/conversation.h>
 
 #define SPNEGO_negTokenInit 0
@@ -226,7 +231,7 @@ static const value_string spnego_krb5_seal_alg_vals[] = {
 static int
 dissect_spnego_krb5_getmic_base(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
 static int
-dissect_spnego_krb5_wrap_base(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
+dissect_spnego_krb5_wrap_base(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree, guint16 token_id);
 
 static void
 dissect_spnego_krb5(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
@@ -383,7 +388,7 @@ dissect_spnego_krb5(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  break;
 
 	case KRB_TOKEN_WRAP:
-          offset = dissect_spnego_krb5_wrap_base(tvb, offset, pinfo, subtree);
+          offset = dissect_spnego_krb5_wrap_base(tvb, offset, pinfo, subtree, token_id);
 	  break;
 
 	case KRB_TOKEN_DELETE_SEC_CONTEXT:
@@ -399,13 +404,363 @@ dissect_spnego_krb5(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	return;
 }
 
+#ifdef HAVE_KERBEROS
+#include <epan/crypt-md5.h>
+
+#ifndef KEYTYPE_ARCFOUR_56
+# define KEYTYPE_ARCFOUR_56 24
+#endif
+/* XXX - We should probably do a configure-time check for this instead */
+#ifndef KRB5_KU_USAGE_SEAL
+# define KRB5_KU_USAGE_SEAL 22
+#endif
+
+static int
+arcfour_mic_key(void *key_data, size_t key_size, int key_type,
+		void *cksum_data, size_t cksum_size,
+		void *key6_data)
+{
+    char k5_data[16];
+    char T[4];
+
+    memset(T, 0, 4);
+
+    if (key_type == KEYTYPE_ARCFOUR_56) {
+	char L40[14] = "fortybits";
+
+	memcpy(L40 + 10, T, sizeof(T));
+	md5_hmac(
+                L40, 14,  
+                key_data,
+                key_size, 
+	    	k5_data);
+	memset(&k5_data[7], 0xAB, 9);
+    } else {
+	md5_hmac(
+                T, 4,  
+                key_data,
+                key_size,
+	    	k5_data);
+    }
+
+    md5_hmac(
+	cksum_data, cksum_size,  
+	k5_data,
+	16, 
+	key6_data);
+
+    return 0;
+}
+
+static int
+usage2arcfour(int usage)
+{
+    switch (usage) {
+    case 3: /*KRB5_KU_AS_REP_ENC_PART 3 */
+    case 9: /*KRB5_KU_TGS_REP_ENC_PART_SUB_KEY 9 */
+	return 8;
+    case 22: /*KRB5_KU_USAGE_SEAL 22 */
+	return 13;
+    case 23: /*KRB5_KU_USAGE_SIGN 23 */
+        return 15;
+    case 24: /*KRB5_KU_USAGE_SEQ 24 */
+	return 0;
+    default :
+	return 0;
+    }
+}
+
+static int
+arcfour_mic_cksum(char *key_data, int key_length,
+		  unsigned usage,
+		  u_char sgn_cksum[8],
+		  const char *v1, size_t l1,
+		  const void *v2, size_t l2,
+		  const void *v3, size_t l3)
+{
+    const char signature[] = "signaturekey";
+    char ksign_c[16];
+    unsigned char t[4];
+    md5_state_t ms;
+    unsigned char digest[16];
+    int rc4_usage;
+    char cksum[16];
+    
+    rc4_usage=usage2arcfour(usage);
+    md5_hmac(signature, sizeof(signature), 
+		key_data, key_length, 
+		ksign_c);
+    md5_init(&ms);
+    t[0] = (rc4_usage >>  0) & 0xFF;
+    t[1] = (rc4_usage >>  8) & 0xFF;
+    t[2] = (rc4_usage >> 16) & 0xFF;
+    t[3] = (rc4_usage >> 24) & 0xFF;
+    md5_append(&ms, t, 4);
+    md5_append(&ms, v1, l1);
+    md5_append(&ms, v2, l2);
+    md5_append(&ms, v3, l3);
+    md5_finish(&ms, digest);
+    md5_hmac(digest, 16, ksign_c, 16, cksum);
+
+    memcpy(sgn_cksum, cksum, 8);
+
+    return 0;
+}
+
+/*
+ * Verify padding of a gss wrapped message and return its length.
+ */
+static int
+gssapi_verify_pad(unsigned char *wrapped_data, int wrapped_length, 
+		   size_t datalen,
+		   size_t *padlen)
+{
+    unsigned char *pad;
+    size_t padlength;
+    int i;
+
+    pad = wrapped_data + wrapped_length - 1;
+    padlength = *pad;
+
+    if (padlength > datalen)
+	return 1;
+
+    for (i = padlength; i > 0 && *pad == padlength; i--, pad--)
+	;
+    if (i != 0)
+	return 2;
+
+    *padlen = padlength;
+
+    return 0;
+}
+
+static int
+decrypt_arcfour(packet_info *pinfo,
+	 char *input_message_buffer,
+	 char *output_message_buffer,
+	 char *key_value, int key_size, int key_type)
+{
+    unsigned char Klocaldata[16];
+    int ret;
+    int32_t seq_number;
+    size_t datalen;
+    char k6_data[16], SND_SEQ[8], Confounder[8];
+    char cksum_data[8];
+    int cmp;
+    int conf_flag;
+    size_t padlen = 0;
+
+    datalen = tvb_length(pinfo->gssapi_encrypted_tvb);
+
+    if(tvb_get_ntohs(pinfo->gssapi_wrap_tvb, 4)==0x1000){
+	conf_flag=1;
+    } else if (tvb_get_ntohs(pinfo->gssapi_wrap_tvb, 4)==0xffff){
+	conf_flag=0;
+    } else {
+	return -3;
+    }
+
+    if(tvb_get_ntohs(pinfo->gssapi_wrap_tvb, 6)!=0xffff){
+	return -4;
+    }
+
+    ret = arcfour_mic_key(key_value, key_size, key_type,
+			  (void *)tvb_get_ptr(pinfo->gssapi_wrap_tvb, 16, 8),
+			  8, /* SGN_CKSUM */
+			  k6_data);
+    if (ret) {
+	return -5;
+    }
+
+    {
+	rc4_state_struct rc4_state;
+	
+	crypt_rc4_init(&rc4_state, k6_data, sizeof(k6_data));
+	memcpy(SND_SEQ, (unsigned char *)tvb_get_ptr(pinfo->gssapi_wrap_tvb, 8, 8), 8);
+	crypt_rc4(&rc4_state, SND_SEQ, 8);
+
+	memset(k6_data, 0, sizeof(k6_data));
+    }
+
+    seq_number=g_ntohl(*((guint32 *)SND_SEQ));
+
+    cmp = memcmp(&SND_SEQ[4], "\xff\xff\xff\xff", 4);
+    if(cmp){
+	cmp = memcmp(&SND_SEQ[4], "\x00\x00\x00\x00", 4);
+    }
+
+    if (cmp != 0) {
+	return -6;
+    }
+
+    {
+	int i;
+
+	for (i = 0; i < 16; i++)
+	    Klocaldata[i] = ((u_char *)key_value)[i] ^ 0xF0;
+    }
+    ret = arcfour_mic_key(Klocaldata,sizeof(Klocaldata),key_type,
+			  SND_SEQ, 4,
+			  k6_data);
+    memset(Klocaldata, 0, sizeof(Klocaldata));
+    if (ret) {
+	return -7;
+    }
+
+    if(conf_flag) {
+	rc4_state_struct rc4_state;
+
+	crypt_rc4_init(&rc4_state, k6_data, sizeof(k6_data));
+	memcpy(Confounder, (unsigned char *)tvb_get_ptr(pinfo->gssapi_wrap_tvb, 24, 8), 8);
+	crypt_rc4(&rc4_state, Confounder, 8);
+	memcpy(output_message_buffer, input_message_buffer, datalen);
+	crypt_rc4(&rc4_state, output_message_buffer, datalen);
+    } else {
+	memcpy(Confounder, 
+		tvb_get_ptr(pinfo->gssapi_wrap_tvb, 24, 8), 
+		8); /* Confounder */
+	memcpy(output_message_buffer, 
+		input_message_buffer, 
+	        datalen);
+    }
+    memset(k6_data, 0, sizeof(k6_data));
+
+    /* only normal (i.e. non DCE style  wrapping use padding ? */
+    if(pinfo->decrypt_gssapi_tvb==DECRYPT_GSSAPI_NORMAL){
+	ret = gssapi_verify_pad(output_message_buffer,datalen,datalen, &padlen);
+    	if (ret) {
+	    return -9;
+	}
+	datalen -= padlen;
+    }
+
+    /* dont know what the checksum looks like for dce style gssapi */
+    if(pinfo->decrypt_gssapi_tvb==DECRYPT_GSSAPI_NORMAL){
+	ret = arcfour_mic_cksum(key_value, key_size,
+			    KRB5_KU_USAGE_SEAL,
+			    cksum_data, 
+			    tvb_get_ptr(pinfo->gssapi_wrap_tvb, 0, 8), 8,
+			    Confounder, sizeof(Confounder),
+			    output_message_buffer, 
+			    datalen + padlen);
+	if (ret) {
+	    return -10;
+	}
+
+	cmp = memcmp(cksum_data, 
+	    tvb_get_ptr(pinfo->gssapi_wrap_tvb, 16, 8),
+	    8); /* SGN_CKSUM */
+	if (cmp) {
+	    return -11;
+	}
+    }
+
+    return datalen;
+}
+
+
+
+#if defined(HAVE_HEIMDAL_KERBEROS) || defined(HAVE_MIT_KERBEROS)
+
+static void
+decrypt_gssapi_krb_arcfour_wrap(proto_tree *tree, packet_info *pinfo, tvbuff_t *tvb, int keytype)
+{
+	int ret;
+	enc_key_t *ek;
+	int length;
+	const guint8 *original_data;
+
+	static int omb_index=0;
+	static guint8 *omb_arr[4]={NULL,NULL,NULL,NULL};
+	static guint8 *cryptocopy=NULL; /* workaround for pre-0.6.1 heimdal bug */
+	guint8 *output_message_buffer;
+
+	omb_index++;
+	if(omb_index>=4){
+		omb_index=0;
+	}
+	output_message_buffer=omb_arr[omb_index];
+
+
+	length=tvb_length(pinfo->gssapi_encrypted_tvb);
+	original_data=tvb_get_ptr(pinfo->gssapi_encrypted_tvb, 0, length);
+
+	/* dont do anything if we are not attempting to decrypt data */
+/*
+	if(!krb_decrypt){
+		return;
+	}
+*/
+	/* XXX we should only do this for first time, then store somewhere */
+	/* XXX We also need to re-read the keytab when the preference changes */
+
+	if(cryptocopy){
+		g_free(cryptocopy);
+		cryptocopy=NULL;
+	}		
+	cryptocopy=g_malloc(length);
+	if(output_message_buffer){
+		g_free(output_message_buffer);
+		output_message_buffer=NULL;
+	}
+	output_message_buffer=g_malloc(length);
+
+	for(ek=enc_key_list;ek;ek=ek->next){
+		/* shortcircuit and bail out if enctypes are not matching */
+		if(ek->keytype!=keytype){
+			continue;
+		}
+
+		/* pre-0.6.1 versions of Heimdal would sometimes change
+		  the cryptotext data even when the decryption failed.
+		  This would obviously not work since we iterate over the
+		  keys. So just give it a copy of the crypto data instead.
+		  This has been seen for RC4-HMAC blobs.
+		*/
+		memcpy(cryptocopy, original_data, length);
+		ret=decrypt_arcfour(pinfo,
+				cryptocopy,
+				output_message_buffer,
+				ek->keyvalue,
+				ek->keylength,
+				ek->keytype
+					    );
+		if (ret >= 0) {
+			proto_tree_add_text(tree, NULL, 0, 0, "[Decrypted using: %s]", ek->key_origin);
+			pinfo->gssapi_decrypted_tvb=tvb_new_real_data(
+				output_message_buffer,
+				ret, ret);
+			tvb_set_child_real_data_tvbuff(tvb, pinfo->gssapi_decrypted_tvb);
+			add_new_data_source(pinfo, pinfo->gssapi_decrypted_tvb, "Decrypted GSS-Krb5");
+			return;
+		}
+	}
+	return;
+}
+#endif /* HAVE_HEIMDAL_KERBEROS || HAVE_MIT_KERBEROS */
+
+
+#endif
+
 /*
  * XXX - This is for GSSAPI Wrap tokens ...
  */
 static int
-dissect_spnego_krb5_wrap_base(tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tree)
+dissect_spnego_krb5_wrap_base(tvbuff_t *tvb, int offset, packet_info *pinfo
+#ifndef HAVE_KERBEROS
+	_U_
+#endif
+    , proto_tree *tree, guint16 token_id
+#ifndef HAVE_KERBEROS
+	_U_
+#endif
+    )
 {
-	guint16 sgn_alg;
+	guint16 sgn_alg, seal_alg;
+#ifdef HAVE_KERBEROS
+	int start_offset=offset;
+#endif
 
 	/*
 	 * The KRB5 blob conforms to RFC1964:
@@ -421,8 +776,9 @@ dissect_spnego_krb5_wrap_base(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 
 	offset += 2;
 
-	proto_tree_add_item(tree, hf_spnego_krb5_seal_alg, tvb, offset, 2,
-			    TRUE);
+	seal_alg = tvb_get_letohs(tvb, offset);
+	proto_tree_add_uint(tree, hf_spnego_krb5_seal_alg, tvb, offset, 2,
+			    seal_alg);
 
 	offset += 2;
 
@@ -454,15 +810,61 @@ dissect_spnego_krb5_wrap_base(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 	if (sgn_alg == KRB_SGN_ALG_HMAC) {
 	  proto_tree_add_item(tree, hf_spnego_krb5_confounder, tvb, offset, 8,
 			      TRUE);
-
 	  offset += 8;
 	}
 
+	/* Is the data encrypted? */
+	pinfo->gssapi_data_encrypted=(seal_alg!=KRB_SEAL_ALG_NONE);
+
+#ifdef HAVE_KERBEROS
+#define GSS_ARCFOUR_WRAP_TOKEN_SIZE 32
+	if(pinfo->decrypt_gssapi_tvb){
+		/* if the caller did not provide a tvb, then we just use
+		   whatever is left of our current tvb.
+		*/
+		if(!pinfo->gssapi_encrypted_tvb){
+			int len;
+			len=tvb_reported_length_remaining(tvb,offset);
+			if(len>tvb_length_remaining(tvb, offset)){
+				/* no point in trying to decrypt, 
+				   we dont have the full pdu.
+				*/
+				return offset;
+			}
+			pinfo->gssapi_encrypted_tvb = tvb_new_subset(
+					tvb, offset, len, len);
+		}
+			
+		/* if this is KRB5 wrapped rc4-hmac */
+		if((token_id==KRB_TOKEN_WRAP)
+		 &&(sgn_alg==KRB_SGN_ALG_HMAC)
+		 &&(seal_alg==KRB_SEAL_ALG_RC4)){
+			/* do we need to create a tvb for the wrapper
+			   as well ?
+			*/
+			if(!pinfo->gssapi_wrap_tvb){
+				pinfo->gssapi_wrap_tvb = tvb_new_subset(
+					tvb, start_offset-2,
+					GSS_ARCFOUR_WRAP_TOKEN_SIZE,
+					GSS_ARCFOUR_WRAP_TOKEN_SIZE);
+			}
+#if defined(HAVE_HEIMDAL_KERBEROS) || defined(HAVE_MIT_KERBEROS)
+			decrypt_gssapi_krb_arcfour_wrap(tree,
+				pinfo,
+				tvb,
+				23 /* rc4-hmac */);
+#endif /* HAVE_HEIMDAL_KERBEROS || HAVE_MIT_KERBEROS */
+		}
+	}	
+#endif
 	/*
 	 * Return the offset past the checksum, so that we know where
 	 * the data we're wrapped around starts.  Also, set the length
 	 * of our top-level item to that offset, so it doesn't cover
 	 * the data we're wrapped around.
+	 * 
+	 * Note that for DCERPC the GSSAPI blobs comes after the data it wraps,
+	 * not before.
 	 */
 	return offset;
 }
@@ -544,6 +946,7 @@ dissect_spnego_krb5_wrap(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree
 	proto_item *item;
 	proto_tree *subtree;
 	int offset = 0;
+	guint16 token_id;
 
 	item = proto_tree_add_item(tree, hf_spnego_krb5, tvb, 0, -1, FALSE);
 
@@ -557,12 +960,13 @@ dissect_spnego_krb5_wrap(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree
 
 	/* First, the token ID ... */
 
-	proto_tree_add_item(subtree, hf_spnego_krb5_tok_id, tvb, offset, 2,
-			    TRUE);
+	token_id = tvb_get_letohs(tvb, offset);
+	proto_tree_add_uint(subtree, hf_spnego_krb5_tok_id, tvb, offset, 2,
+			    token_id);
 
 	offset += 2;
 
-        offset = dissect_spnego_krb5_wrap_base(tvb, offset, pinfo, subtree);
+        offset = dissect_spnego_krb5_wrap_base(tvb, offset, pinfo, subtree, token_id);
 
         /*
 	 * Return the offset past the checksum, so that we know where
@@ -1082,7 +1486,7 @@ dissect_spnego_supportedMech(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 	 * could override that. :-(
 	 */
 
-	if ((conversation = find_conversation(&pinfo->src, &pinfo->dst,
+	if ((conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 					     pinfo->ptype, pinfo->srcport,
 					     pinfo->destport, 0))) {
 
@@ -1294,7 +1698,7 @@ dissect_spnego(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	     * If we have a conversation, try to get the handle,
 	     * and if we get one, attach it to the frame.
 	     */
-	    conversation = find_conversation(&pinfo->src, &pinfo->dst,
+	    conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 					     pinfo->ptype, pinfo->srcport,
 					     pinfo->destport, 0);
 
@@ -1427,7 +1831,7 @@ dissect_spnego_wrap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	     * If we have a conversation, try to get the handle,
 	     * and if we get one, attach it to the frame.
 	     */
-	    conversation = find_conversation(&pinfo->src, &pinfo->dst,
+	    conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst,
 					     pinfo->ptype, pinfo->srcport,
 					     pinfo->destport, 0);
 
