@@ -9,7 +9,7 @@
  *
  * (append your name here for newer version)
  *
- * $Id: packet-tcap.c 12900 2004-12-31 03:18:13Z guy $
+ * $Id: packet-tcap.c 14297 2005-05-03 18:40:06Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -61,7 +61,7 @@
 
 #include <epan/packet.h>
 #include <epan/prefs.h>
-#include "asn1.h"
+#include <epan/asn1.h>
 #include "packet-tcap.h"
 #include "packet-ber.h"
 
@@ -196,6 +196,7 @@ int
 tcap_find_eoc(ASN1_SCK *asn1)
 {
     guint	saved_offset;
+    gint        prev_offset;
     guint	tag;
     guint	len;
     gboolean	def_len;
@@ -204,6 +205,7 @@ tcap_find_eoc(ASN1_SCK *asn1)
 
     while (!asn1_eoc(asn1, -1))
     {
+	prev_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
 	asn1_length_decode(asn1, &def_len, &len);
 
@@ -216,6 +218,8 @@ tcap_find_eoc(ASN1_SCK *asn1)
 	    asn1->offset += tcap_find_eoc(asn1);
 	    asn1_eoc_decode(asn1, -1);
 	}
+	if (prev_offset >= asn1->offset)
+	    THROW(ReportedBoundsError);
     }
 
     len = asn1->offset - saved_offset;
@@ -501,7 +505,7 @@ dissect_tcap_opr_code(ASN1_SCK *asn1, proto_tree *tree)
 static int
 dissect_tcap_param(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 {
-    guint orig_offset, saved_offset, len_offset;
+    gint orig_offset, saved_offset, len_offset;
     guint tag, len;
     gboolean def_len;
     proto_item *item;
@@ -592,6 +596,8 @@ dissect_tcap_param(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	    asn1->offset, len, "Parameter Data");
 
 	asn1->offset += len;
+	if (saved_offset >= asn1->offset)
+	    THROW(ReportedBoundsError);
     }
 
     return TC_DS_OK;
@@ -1496,7 +1502,7 @@ dissect_tcap_components(ASN1_SCK *asn1, proto_tree *tcap_tree)
 {
     proto_tree	*subtree;
     proto_item	*comps_item;
-    guint	saved_offset, comps_start;
+    gint	saved_offset, comps_start;
     guint	len, comp_len;
     gint	keep_len;
     gboolean	comps_def_len, def_len;
@@ -1598,6 +1604,8 @@ dissect_tcap_components(ASN1_SCK *asn1, proto_tree *tcap_tree)
 		break;
 	    }
 	}
+	if (saved_offset >= asn1->offset)
+	    THROW(ReportedBoundsError);
     }
 
     if (!comps_def_len)
@@ -1647,6 +1655,7 @@ dissect_tcap_dlg_application_context_name(ASN1_SCK *asn1, proto_tree *tcap_tree,
     guint name_len, len, len2;
     guint tag;
     subid_t *oid;
+    char oid_str[BER_MAX_OID_STR_LEN];
     int ret;
     gboolean def_len;
 
@@ -1663,7 +1672,7 @@ dissect_tcap_dlg_application_context_name(ASN1_SCK *asn1, proto_tree *tcap_tree,
 	 * TODO Probbably more can be removed here but I'm uncertain about the lengths 
     proto_tree_add_bytes(tcap_tree, hf_tcap_app_con_name, asn1->tvb, saved_offset, len2, tvb_get_ptr(asn1->tvb, saved_offset, len2));
 	*/
-	asn1->offset = dissect_ber_object_identifier(FALSE, pinfo, tcap_tree, asn1->tvb, saved_offset, hf_tcap_oid, NULL);
+	asn1->offset = dissect_ber_object_identifier(FALSE, pinfo, tcap_tree, asn1->tvb, saved_offset, hf_tcap_oid, oid_str);
     if (ret == ASN1_ERR_NOERROR) g_free(oid);
 
     if (!def_len)
@@ -1671,6 +1680,7 @@ dissect_tcap_dlg_application_context_name(ASN1_SCK *asn1, proto_tree *tcap_tree,
 	/* for Application Context Name Tag */
 	dissect_tcap_eoc(asn1, tcap_tree);
     }
+	pinfo->private_data = g_strdup(oid_str);
 
     return TC_DS_OK;
 }
@@ -1816,7 +1826,7 @@ dissect_tcap_dlg_user_info(ASN1_SCK *asn1, proto_tree *tree, packet_info *pinfo)
 	int ret;
 	proto_item *para_item;
 	proto_tree *sub_tree;
-	char oid_str[64]; /*64 chars should be long enough? */
+	char oid_str[BER_MAX_OID_STR_LEN];
 	tvbuff_t *next_tvb;
 
 #define TC_USR_INFO_TAG 0xbe
@@ -2039,6 +2049,7 @@ dissect_tcap_dialog_portion(ASN1_SCK *asn1, proto_tree *tcap_tree, proto_item *t
 
     if (portion_def_len)
     {
+	tvb_ensure_bytes_exist(asn1->tvb, saved_offset, len);
 	proto_item_set_len(dlg_item, len);
     }
 
@@ -2752,8 +2763,10 @@ dissect_tcap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
     proto_item *ti;
     proto_tree *tcap_tree;
+	void *save_private_data = pinfo->private_data;
 
     g_pinfo = pinfo;
+	pinfo->private_data = NULL;
 
     if (check_col(pinfo->cinfo, COL_PROTOCOL))
 	col_set_str(pinfo->cinfo, COL_PROTOCOL, "TCAP");
@@ -2772,6 +2785,9 @@ dissect_tcap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     {
 	dissect_ansi_tcap_message(tvb, pinfo, tcap_tree);
     }
+	if ( pinfo->private_data != NULL)
+		g_free(pinfo->private_data);
+	pinfo->private_data = save_private_data;
 }
 
 
