@@ -1,7 +1,7 @@
 /* filesystem.c
  * Filesystem utility routines
  *
- * $Id: filesystem.c,v 1.29 2004/01/24 02:01:43 guy Exp $
+ * $Id: filesystem.c 14063 2005-04-12 23:48:47Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -49,7 +49,7 @@
 #include <direct.h>		/* to declare "mkdir()" on Windows */
 #endif
 
-#ifndef WIN32
+#ifndef _WIN32
 #include <pwd.h>
 #endif
 
@@ -60,19 +60,19 @@
  * character in the pathname, or NULL if the pathname contains no
  * separators.
  */
-char *
-find_last_pathname_separator(char *path)
+static char *
+find_last_pathname_separator(const char *path)
 {
 	char *separator;
 
-#ifdef WIN32
+#ifdef _WIN32
 	char c;
 
 	/*
 	 * We have to scan for '\' or '/'.
 	 * Get to the end of the string.
 	 */
-	separator = path + strlen(path);	/* points to ending '\0' */
+	separator = strchr(path, '\0');		/* points to ending '\0' */
 	while (separator > path) {
 		c = *--separator;
 		if (c == '\\' || c == '/')
@@ -93,10 +93,10 @@ find_last_pathname_separator(char *path)
 /*
  * Given a pathname, return the last component.
  */
-char *
-get_basename(char *path)
+const char *
+get_basename(const char *path)
 {
-	char *filename;
+	const char *filename;
 
 	g_assert(path != NULL);
 	filename = find_last_pathname_separator(path);
@@ -235,7 +235,7 @@ test_for_fifo(const char *path)
 const char *
 get_datafile_dir(void)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	char prog_pathname[_MAX_PATH+2];
 	char *dir_end;
 	size_t datafile_dir_len;
@@ -316,7 +316,7 @@ get_datafile_dir(void)
 const char *
 get_systemfile_dir(void)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	return get_datafile_dir();
 #else
 	return "/etc";
@@ -327,7 +327,7 @@ get_systemfile_dir(void)
  * Name of directory, under the user's home directory, in which
  * personal configuration files are stored.
  */
-#ifdef WIN32
+#ifdef _WIN32
 #define PF_DIR "Ethereal"
 #else
 /*
@@ -347,7 +347,7 @@ get_systemfile_dir(void)
 static const char *
 get_persconffile_dir(void)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	char *appdatadir;
 	char *userprofiledir;
 #else
@@ -360,7 +360,7 @@ get_persconffile_dir(void)
 	if (pf_dir != NULL)
 		return pf_dir;
 
-#ifdef WIN32
+#ifdef _WIN32
 	/*
 	 * Use %APPDATA% or %USERPROFILE%, so that configuration files are
 	 * stored in the user profile, rather than in the home directory.
@@ -436,7 +436,7 @@ int
 create_persconffile_dir(char **pf_dir_path_return)
 {
 	const char *pf_dir_path;
-#ifdef WIN32
+#ifdef _WIN32
 	char *pf_dir_path_copy, *pf_dir_parent_path;
 	size_t pf_dir_parent_path_len;
 #endif
@@ -445,7 +445,7 @@ create_persconffile_dir(char **pf_dir_path_return)
 
 	pf_dir_path = get_persconffile_dir();
 	if (stat(pf_dir_path, &s_buf) != 0 && errno == ENOENT) {
-#ifdef WIN32
+#ifdef _WIN32
 		/*
 		 * Does the parent directory of that directory
 		 * exist?  %APPDATA% may not exist even though
@@ -491,7 +491,7 @@ create_persconffile_dir(char **pf_dir_path_return)
 	return ret;
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 /*
  * Returns the user's home directory on Win32.
  */
@@ -562,13 +562,13 @@ get_home_dir(void)
  */
 char *
 get_persconffile_path(const char *filename, gboolean for_writing
-#ifndef WIN32
+#ifndef _WIN32
 	_U_
 #endif
 )
 {
 	char *path;
-#ifdef WIN32
+#ifdef _WIN32
 	struct stat s_buf;
 	char *old_path;
 #endif
@@ -577,7 +577,7 @@ get_persconffile_path(const char *filename, gboolean for_writing
 	    strlen(filename) + 2);
 	sprintf(path, "%s" G_DIR_SEPARATOR_S "%s", get_persconffile_dir(),
 	    filename);
-#ifdef WIN32
+#ifdef _WIN32
 	if (!for_writing) {
 		if (stat(path, &s_buf) != 0 && errno == ENOENT) {
 			/*
@@ -657,16 +657,16 @@ file_open_error_message(int err, gboolean for_writing)
 
 	case ENOENT:
 		if (for_writing)
-			errmsg = "The path to the file \"%s\" does not exist.";
+			errmsg = "The path to the file \"%s\" doesn't exist.";
 		else
-			errmsg = "The file \"%s\" does not exist.";
+			errmsg = "The file \"%s\" doesn't exist.";
 		break;
 
 	case EACCES:
 		if (for_writing)
-			errmsg = "You do not have permission to create or write to the file \"%s\".";
+			errmsg = "You don't have permission to create or write to the file \"%s\".";
 		else
-			errmsg = "You do not have permission to read the file \"%s\".";
+			errmsg = "You don't have permission to read the file \"%s\".";
 		break;
 
 	case EISDIR:
@@ -726,3 +726,81 @@ file_write_error_message(int err)
 	return errmsg;
 }
 
+
+gboolean
+file_exists(const char *fname)
+{
+  struct stat   file_stat;
+
+
+  /*
+   * This is a bit tricky on win32. The st_ino field is documented as:
+   * "The inode, and therefore st_ino, has no meaning in the FAT, ..."
+   * but it *is* set to zero if stat() returns without an error,
+   * so this is working, but maybe not quite the way expected. ULFL
+   */
+   file_stat.st_ino = 1;   /* this will make things work if an error occured */
+   stat(fname, &file_stat);
+   if (file_stat.st_ino == 0) {
+       return TRUE;
+   } else {
+       return FALSE;
+   }
+
+}
+
+
+gboolean
+files_identical(const char *fname1, const char *fname2)
+{
+    /* Two different implementations, because:
+     * - _fullpath is not available on unix 
+     * - the stat inode will not work as expected on Win32, so two different implementations.
+     *
+     * XXX - will _fullpath work with UNC?
+     */
+#ifdef _WIN32
+    char full1[MAX_PATH], full2[MAX_PATH];
+
+
+    if( _fullpath( full1, fname1, MAX_PATH ) == NULL ) {
+        return FALSE;
+    }
+
+    if( _fullpath( full2, fname2, MAX_PATH ) == NULL ) {
+        return FALSE;
+    }
+    
+    if(strcmp(full1, full2) == 0) {
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+#else
+  struct stat   infile, outfile;
+
+  /*
+   * Check that the from file is not the same as to file
+   * We do it here so we catch all cases ...
+   * Unfortunately, the file requester gives us an absolute file
+   * name and the read file name may be relative (if supplied on
+   * the command line). From Joerg Mayer.
+   *
+   * This is a bit tricky on win32. The st_ino field is documented as:
+   * "The inode, and therefore st_ino, has no meaning in the FAT, ..."
+   * but it *is* set to zero if stat() returns without an error,
+   * so this is not working, as it only checks if both files existing. ULFL
+   */
+   infile.st_ino = 1;   /* These prevent us from getting equality         */
+   outfile.st_ino = 2;  /* If one or other of the files is not accessible */
+   stat(fname1, &infile);
+   stat(fname2, &outfile);
+   if (infile.st_ino == outfile.st_ino) {
+       return TRUE;
+   } else {
+       return FALSE;
+   }
+
+#endif
+}
+
