@@ -1,5 +1,5 @@
 /*
- * $Id: ftypes.c,v 1.26 2004/07/04 02:29:43 guy Exp $
+ * $Id: ftypes.c 11796 2004-08-22 00:31:58Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -427,6 +427,13 @@ fvalue_set_integer(fvalue_t *fv, guint32 value)
 	fv->ftype->set_value_integer(fv, value);
 }
 
+void
+fvalue_set_integer64(fvalue_t *fv, guint64 value)
+{
+	g_assert(fv->ftype->set_value_integer64);
+	fv->ftype->set_value_integer64(fv, value);
+}
+
 void
 fvalue_set_floating(fvalue_t *fv, gdouble value)
 {
@@ -449,6 +456,13 @@ fvalue_get_integer(fvalue_t *fv)
 	return fv->ftype->get_value_integer(fv);
 }
 
+guint64
+fvalue_get_integer64(fvalue_t *fv)
+{
+	g_assert(fv->ftype->get_value_integer64);
+	return fv->ftype->get_value_integer64(fv);
+}
+
 double
 fvalue_get_floating(fvalue_t *fv)
 {
