@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c 12914 2005-01-01 23:11:46Z guy $
+ * $Id: packet.c 14265 2005-05-02 14:07:33Z tuexen $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -53,7 +53,7 @@
 #include "plugins.h"
 #include "epan_dissect.h"
 
-#include "../reassemble.h"
+#include <epan/reassemble.h>
 
 static gint proto_malformed = -1;
 static dissector_handle_t frame_handle = NULL;
@@ -118,6 +118,14 @@ call_init_routine(gpointer routine, gpointer dummy _U_)
 	(*func)();
 }
 
+/*
+ * XXX - for now, these are the same; the "init" routines free whatever
+ * stuff is left over from any previous dissection, and then initialize
+ * their tables.
+ *
+ * We should probably split that into "init" and "cleanup" routines, for
+ * cleanliness' sake.
+ */
 void
 init_dissection(void)
 {
@@ -137,6 +145,12 @@ init_dissection(void)
 	reassemble_init();
 }
 
+void
+cleanup_dissection(void)
+{
+	init_dissection();
+}
+
 /* Allow protocols to register a "cleanup" routine to be
  * run after the initial sequential run through the packets.
  * Note that the file can still be open after this; this is not
@@ -291,14 +305,21 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 	edt->pi.want_pdu_tracking = 0;
 	edt->pi.p2p_dir = P2P_DIR_UNKNOWN;
 	edt->pi.private_data = NULL;
-        edt->pi.oxid = 0;
-        edt->pi.rxid = 0;
-        edt->pi.r_ctl = 0;
-        edt->pi.src_idx = 0;
-        edt->pi.dst_idx = 0;
-        edt->pi.vsan = 0;
-        edt->pi.dcectxid = 0;
-        edt->pi.dcetransporttype = -1;
+	edt->pi.oxid = 0;
+	edt->pi.rxid = 0;
+	edt->pi.r_ctl = 0;
+	edt->pi.src_idx = 0;
+	edt->pi.dst_idx = 0;
+	edt->pi.vsan = 0;
+	edt->pi.dcectxid = 0;
+	edt->pi.dcetransporttype = -1;
+	edt->pi.decrypt_gssapi_tvb = 0;
+	edt->pi.gssapi_wrap_tvb = NULL;
+	edt->pi.gssapi_encrypted_tvb = NULL;
+	edt->pi.gssapi_decrypted_tvb = NULL;
+	edt->pi.layer_names = NULL;
+	edt->pi.link_number = 0;
+	edt->pi.annex_a_used = MTP2_ANNEX_A_USED_UNKNOWN;
 
 	TRY {
 		edt->tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
@@ -317,13 +338,12 @@ dissect_packet(epan_dissect_t *edt, union wtap_pseudo_header *pseudo_header,
 		g_assert_not_reached();
 	}
 	CATCH(ReportedBoundsError) {
-	  if(proto_malformed != -1){
-		proto_tree_add_protocol_format(edt->tree, proto_malformed, edt->tvb, 0, 0,
-				"[Malformed Frame: Packet Length]" );
-	  }
-	  else {
-	    g_assert_not_reached();
-	  }
+		if(proto_malformed != -1){
+			proto_tree_add_protocol_format(edt->tree, proto_malformed, edt->tvb, 0, 0,
+			                               "[Malformed Frame: Packet Length]" );
+		} else {
+			g_assert_not_reached();
+		}
 	}
 	ENDTRY;
 
@@ -402,6 +422,7 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 	volatile address save_net_dst;
 	volatile address save_src;
 	volatile address save_dst;
+	volatile gint saved_layer_names_len = 0;
 
 	if (handle->protocol != NULL &&
 	    !proto_is_protocol_enabled(handle->protocol)) {
@@ -414,6 +435,9 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 	saved_proto = pinfo->current_proto;
 	saved_can_desegment = pinfo->can_desegment;
 
+	if (pinfo->layer_names != NULL)
+		saved_layer_names_len = pinfo->layer_names->len;
+
 	/*
 	 * can_desegment is set to 2 by anyone which offers the
 	 * desegmentation api/service.
@@ -431,6 +455,16 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 	if (handle->protocol != NULL) {
 		pinfo->current_proto =
 		    proto_get_protocol_short_name(handle->protocol);
+
+		/*
+		 * Add the protocol name to the layers
+		 */
+		if (pinfo->layer_names) {
+			if (pinfo->layer_names->len > 0)
+				g_string_append(pinfo->layer_names, ":");
+			g_string_append(pinfo->layer_names, 
+			    proto_get_protocol_filter_name(proto_get_id(handle->protocol)));
+		}
 	}
 
 	if (pinfo->in_error_pkt) {
@@ -525,6 +559,18 @@ call_dissector_work(dissector_handle_t handle, tvbuff_t *tvb,
 		 */
 		ret = call_dissector_through_handle(handle, tvb, pinfo, tree);
 	}
+
+	if (ret == 0) {
+		/*
+		 * That dissector didn't accept the packet, so
+		 * remove its protocol's name from the list
+		 * of protocols.
+		 */
+		if (pinfo->layer_names != NULL) {
+			g_string_truncate(pinfo->layer_names,
+			    saved_layer_names_len);
+		}
+	}
 	pinfo->current_proto = saved_proto;
 	pinfo->can_desegment = saved_can_desegment;
 	return ret;
@@ -575,7 +621,7 @@ find_dissector_table(const char *name)
 	return g_hash_table_lookup( dissector_tables, name );
 }
 
-/* Find an entry in a uint dissector table. */ 
+/* Find an entry in a uint dissector table. */
 static dtbl_entry_t *
 find_uint_dtbl_entry(dissector_table_t sub_dissectors, guint32 pattern)
 {
@@ -821,7 +867,7 @@ dissector_get_port_handle(dissector_table_t sub_dissectors, guint32 port)
 		return NULL;
 }
 
-/* Find an entry in a string dissector table. */ 
+/* Find an entry in a string dissector table. */
 static dtbl_entry_t *
 find_string_dtbl_entry(dissector_table_t sub_dissectors, const gchar *pattern)
 {
@@ -1426,6 +1472,7 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 	GSList *entry;
 	heur_dtbl_entry_t *dtbl_entry;
 	guint16 saved_can_desegment;
+	gint saved_layer_names_len = 0;
 
 	/* can_desegment is set to 2 by anyone which offers this api/service.
 	   then everytime a subdissector is called it is decremented by one.
@@ -1442,10 +1489,15 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 
 	status = FALSE;
 	saved_proto = pinfo->current_proto;
+
+	if (pinfo->layer_names != NULL)
+		saved_layer_names_len = pinfo->layer_names->len;
+
 	for (entry = sub_dissectors; entry != NULL; entry = g_slist_next(entry)) {
 		/* XXX - why set this now and above? */
 		pinfo->can_desegment = saved_can_desegment-(saved_can_desegment>0);
 		dtbl_entry = (heur_dtbl_entry_t *)entry->data;
+
 		if (dtbl_entry->protocol != NULL &&
 		    !proto_is_protocol_enabled(dtbl_entry->protocol)) {
 			/*
@@ -1458,9 +1510,31 @@ dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
 			pinfo->current_proto =
 			    proto_get_protocol_short_name(dtbl_entry->protocol);
 		}
+
+		/*
+		 * Add the protocol name to the layers; we'll remove it
+		 * if the dissector fails.
+		 */
+		if (pinfo->layer_names) {
+			if (pinfo->layer_names->len > 0)
+				g_string_append(pinfo->layer_names, ":");
+			g_string_append(pinfo->layer_names,
+			    proto_get_protocol_filter_name(proto_get_id(dtbl_entry->protocol)));
+		}
+
 		if ((*dtbl_entry->dissector)(tvb, pinfo, tree)) {
 			status = TRUE;
 			break;
+		} else {
+			/*
+			 * That dissector didn't accept the packet, so
+			 * remove its protocol's name from the list
+			 * of protocols.
+			 */
+			if (pinfo->layer_names != NULL) {
+				g_string_truncate(pinfo->layer_names,
+				    saved_layer_names_len);
+			}
 		}
 	}
 	pinfo->current_proto = saved_proto;
@@ -1637,3 +1711,58 @@ call_dissector(dissector_handle_t handle, tvbuff_t *tvb,
 	}
 	return ret;
 }
+
+/*
+ * Dumps the "layer type"/"decode as" associations to stdout, similar
+ * to the proto_registrar_dump_*() routines.
+ *
+ * There is one record per line. The fields are tab-delimited.
+ *
+ * Field 1 = layer type, e.g. "tcp.port"
+ * Field 2 = selector in decimal
+ * Field 3 = "decode as" name, e.g. "http"
+ */
+
+
+static void
+dissector_dump_decodes_display(gchar *table_name, ftenum_t selector_type _U_,
+    gpointer key, gpointer value, gpointer user_data _U_)
+{
+	guint32 selector = (guint32) key;
+	dissector_table_t sub_dissectors = find_dissector_table(table_name);
+	dtbl_entry_t *dtbl_entry;
+	dissector_handle_t handle;
+	gint proto_id;
+	gchar *decode_as;
+
+	g_assert(sub_dissectors);
+	switch (sub_dissectors->type) {
+
+		case FT_UINT8:
+		case FT_UINT16:
+		case FT_UINT24:
+		case FT_UINT32:
+			dtbl_entry = value;
+			g_assert(dtbl_entry);
+
+			handle = dtbl_entry->current;
+			g_assert(handle);
+
+			proto_id = dissector_handle_get_protocol_index(handle);
+
+			if (proto_id != -1) {
+				decode_as = proto_get_protocol_filter_name(proto_id);
+				g_assert(decode_as != NULL);
+				printf("%s\t%d\t%s\n", table_name, selector, decode_as);
+			}
+			break;
+
+	default:
+		break;
+	}
+}
+
+void
+dissector_dump_decodes() {
+	dissector_all_tables_foreach(dissector_dump_decodes_display, NULL);
+}
