@@ -1,7 +1,7 @@
 /* packet.h
  * Definitions for packet disassembly structures and routines
  *
- * $Id: packet.h 11400 2004-07-18 00:24:25Z guy $
+ * $Id: packet.h 13881 2005-03-23 12:58:59Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -75,6 +75,9 @@ typedef struct _packet_counts {
   gint           arp;
 } packet_counts;
 
+/** Number of packet counts. */
+#define PACKET_COUNTS_SIZE sizeof(packet_counts) / sizeof (gint)
+
 /* Types of character encodings */
 typedef enum {
 	CHAR_ASCII	 = 0,	/* ASCII */
@@ -87,6 +90,10 @@ typedef struct true_false_string {
 	char	*false_string;
 } true_false_string;
 
+/**
+* A default set of true/false strings that dissectors can use for 
+* FT_BOOLEAN header fields.
+**/
 extern const true_false_string flags_set_truth;
 
 extern void packet_init(void);
@@ -315,6 +322,9 @@ extern void register_init_routine(void (*func)(void));
 /* Initialize all data structures used for dissection. */
 extern void init_dissection(void);
 
+/* Free data structures allocated for dissection. */
+extern void cleanup_dissection(void);
+
 /* Allow protocols to register a "cleanup" routine to be
  * run after the initial sequential run through the packets.
  * Note that the file can still be open after this; this is not
@@ -364,4 +374,10 @@ extern void ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_ethertype,
 		packet_info *pinfo, proto_tree *tree, proto_tree *fh_tree,
 		int etype_id, int trailer_id, int fcs_len);
 
+/*
+ * Dump layer/selector/dissector records in a fashion similar to the
+ * proto_registrar_dump_* routines.
+ */
+extern void dissector_dump_decodes(void);
+
 #endif /* packet.h */
