@@ -1,7 +1,7 @@
 /* packet_info.h
  * Definitions for packet info structures and routines
  *
- * $Id: packet_info.h 11912 2004-09-06 15:04:27Z tuexen $
+ * $Id: packet_info.h 14265 2005-05-02 14:07:33Z tuexen $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,6 +33,10 @@
 #define P2P_DIR_SENT	0
 #define P2P_DIR_RECV	1
 
+#define MTP2_ANNEX_A_USED_UNKNOWN -1
+#define MTP2_ANNEX_A_NOT_USED      0
+#define MTP2_ANNEX_A_USED          1
+
 #define PINFO_SOF_FIRST_FRAME   0x1
 #define PINFO_SOF_SOFF          0x2
 #define PINFO_EOF_LAST_FRAME    0x80
@@ -85,13 +89,13 @@ typedef struct _packet_info {
   guint16 want_pdu_tracking;	/* >0 if the subdissector has specified
 				   a value in 'bytes_until_next_pdu'.
 				   When a dissector detects that the next PDU
-				   will start beyond the start of the next 
-				   segment, it can set this value to 2 
-				   and 'bytes_until_next_pdu' to the number of 
-				   bytes beyond the next segment where the 
+				   will start beyond the start of the next
+				   segment, it can set this value to 2
+				   and 'bytes_until_next_pdu' to the number of
+				   bytes beyond the next segment where the
 				   next PDU starts.
 
-				   If the protocol dissector below this 
+				   If the protocol dissector below this
 				   one is capable of PDU tracking it can
 				   use this hint to detect PDUs that starts
 				   unaligned to the segment boundaries.
@@ -99,7 +103,7 @@ typedef struct _packet_info {
 				   (some) protocols to detect when a new PDU
 				   starts in the middle of a tcp segment.
 
-				   There is intelligence in the glue between 
+				   There is intelligence in the glue between
 				   dissector layers to make sure that this
 				   request is only passed down to the protocol
 				   immediately below the current one and not
@@ -107,7 +111,7 @@ typedef struct _packet_info {
 				*/
   guint32 bytes_until_next_pdu;
 
-				    
+
   int     iplen;
   int     iphdrlen;
   int	  p2p_dir;
@@ -123,14 +127,39 @@ typedef struct _packet_info {
   guint16 src_idx;              /* Source port index (Cisco MDS-specific) */
   guint16 dst_idx;              /* Dest port index (Cisco MDS-specific) */
   guint16 vsan;                 /* Fibre channel/Cisco MDS-specific */
+
+  /* Extra data for DCERPC handling and tracking of context ids */
   guint16 dcectxid;             /* Context ID (DCERPC-specific) */
-  int     dcetransporttype;     /* Transport type, tag for private_data (DCERPC-specific)
+  int     dcetransporttype;     /* Transport type
                                  * Value -1 means "not a DCERPC packet"
                                  */
+  guint16 dcetransportsalt;	/* fid: if transporttype==DCE_CN_TRANSPORT_SMBPIPE */
+
+  /* Extra data for handling of decryption of GSSAPI wrapped tvbuffs.
+     Caller sets decrypt_gssapi_tvb if this service is requested.
+     If gssapi_encrypted_tvb is NULL, then the rest of the tvb data following
+     the gssapi blob itself is decrypted othervise the gssapi_encrypted_tvb
+     tvb will be decrypted (DCERPC has the data before the gssapi blob)
+     If, on return, gssapi_data_encrypted is FALSE, the wrapped tvbuff
+     was signed (i.e., an encrypted signature was present, to check
+     whether the data was modified by a man in the middle) but not sealed
+     (i.e., the data itself wasn't encrypted).
+  */
+#define DECRYPT_GSSAPI_NORMAL	1
+#define DECRYPT_GSSAPI_DCE	2
+  guint16 decrypt_gssapi_tvb;
+  tvbuff_t *gssapi_wrap_tvb;
+  tvbuff_t *gssapi_encrypted_tvb;
+  tvbuff_t *gssapi_decrypted_tvb;
+  gboolean gssapi_data_encrypted;
+ 
   guint32 ppid[MAX_NUMBER_OF_PPIDS]; /* The first NUMBER_OF_PPIDS PPIDS which are present
                                       * in the SCTP packet
                                       */
   void    *private_data;	/* pointer to data passed from one dissector to another */
+  GString *layer_names; 	/* layers of each protocol */
+  guint16 link_number;
+  gchar   annex_a_used;
 } packet_info;
 
 #endif /* __PACKET_INFO_H__ */
