@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c 12863 2004-12-30 02:10:24Z guy $
+ * $Id: prefs.c 13889 2005-03-24 03:10:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -60,8 +60,8 @@ static gchar *put_string_list(GList *);
 static void   clear_string_list(GList *);
 static void   free_col_info(e_prefs *);
 
-#define GPF_NAME	"ethereal.conf"
 #define PF_NAME		"preferences"
+#define OLD_GPF_NAME	"ethereal.conf"	/* old name for global preferences file */
 
 static gboolean init_prefs = TRUE;
 static gchar *gpf_path = NULL;
@@ -895,7 +895,7 @@ print.file: /a/very/long/path/
  *
  */
 
-#define MAX_VAR_LEN    48
+#define MAX_VAR_LEN    100
 
 #define DEF_NUM_COLS    6
 
@@ -1065,13 +1065,38 @@ read_prefs(int *gpf_errno_return, int *gpf_read_errno_return,
     prefs.name_resolve_concurrency = 500;
   }
 
-  /* Construct the pathname of the global preferences file. */
-  if (! gpf_path)
-    gpf_path = get_datafile_path(GPF_NAME);
+  /*
+   * If we don't already have the pathname of the global preferences
+   * file, construct it.  Then, in either case, try to open the file.
+   */
+  if (gpf_path == NULL) {
+    /*
+     * We don't have the path; try the new path first, and, if that
+     * file doesn't exist, try the old path.
+     */
+    gpf_path = get_datafile_path(PF_NAME);
+    if ((pf = fopen(gpf_path, "r")) == NULL && errno == ENOENT) {
+      /*
+       * It doesn't exist by the new name; try the old name.
+       */
+      g_free(gpf_path);
+      gpf_path = get_datafile_path(OLD_GPF_NAME);
+      pf = fopen(gpf_path, "r");
+    }
+  } else {
+    /*
+     * We have the path; try it.
+     */
+    pf = fopen(gpf_path, "r");
+  }
 
-  /* Read the global preferences file, if it exists. */
+  /*
+   * If we were able to open the file, read it.
+   * XXX - if it failed for a reason other than "it doesn't exist",
+   * report the error.
+   */
   *gpf_path_return = NULL;
-  if ((pf = fopen(gpf_path, "r")) != NULL) {
+  if (pf != NULL) {
     /*
      * Start out the counters of "mgcp.{tcp,udp}.port" entries we've
      * seen.
@@ -1132,7 +1157,8 @@ read_prefs(int *gpf_errno_return, int *gpf_read_errno_return,
       *pf_errno_return = errno;
       *pf_read_errno_return = 0;
       *pf_path_return = pf_path;
-    }
+    } else
+      g_free(pf_path);
   }
 
   return &prefs;
@@ -1989,6 +2015,9 @@ set_pref(gchar *pref_name, gchar *value)
           pref = find_preference(module, "udp_ports");
         else if (strcmp(dotp, "sctp_port") == 0)
           pref = find_preference(module, "sctp_ports");
+      } else if (strcmp(module->name, "llcgprs") == 0) {
+        if (strcmp(dotp, "ignore_cipher_bit") == 0)
+          pref = find_preference(module, "autodetect_cipher_bit");
       }
     }
     if (pref == NULL)
