@@ -1,7 +1,7 @@
 /* proto.h
  * Definitions for protocol display
  *
- * $Id: proto.h 11954 2004-09-10 15:21:20Z gram $
+ * $Id: proto.h 14051 2005-04-11 08:43:51Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -78,6 +78,43 @@ typedef struct _protocol protocol_t;
 	}								\
   }
 
+/** Macro used for reporting errors in dissectors; it throws a
+ * DissectorError exception, with the string passed as an argument
+ * as the message for the exception, so that it can show up in
+ * the Info column and the protocol tree.
+ *
+ * That string should be allocated with g_malloc(); using
+ * "g_strdup_printf()" would work.
+ *
+ * @param message string to use as the message
+ */
+#define REPORT_DISSECTOR_BUG(message)  \
+  (THROW_MESSAGE(DissectorError, message))
+
+/** Macro used for assertions in dissectors; it doesn't abort, it just
+ * throws a DissectorError exception, with the assertion failure
+ * message as a parameter, so that it can show up in the protocol tree.
+ *
+ * @param expression expression to test in the assertion
+ */
+#define DISSECTOR_ASSERT(expression)  \
+  ((void) ((expression) ? 0 : \
+   __DISSECTOR_ASSERT (expression, __FILE__, __LINE__)))
+
+/** Same as DISSECTOR_ASSERT(), but will throw DissectorError exception
+ * unconditionally, much like GLIB's g_assert_not_reached works.
+ */
+#define DISSECTOR_ASSERT_NOT_REACHED()  \
+  (REPORT_DISSECTOR_BUG( \
+    g_strdup_printf("%s:%u: failed assertion \"DISSECTOR_ASSERT_NOT_REACHED\"", \
+     __FILE__, __LINE__)))
+
+#define __DISSECTOR_ASSERT_STRINGIFY(s)	# s
+
+#define __DISSECTOR_ASSERT(expression, file, lineno)  \
+  (REPORT_DISSECTOR_BUG( \
+    g_strdup_printf("%s:%u: failed assertion \"%s\"", \
+     file, lineno, __DISSECTOR_ASSERT_STRINGIFY(expression))))
 
 /** GNUC has the ability to check format strings that follow the syntax used in printf and others.
  Hide the differences between different compilers in this GNUC_FORMAT_CHECK macro.
@@ -110,13 +147,19 @@ struct _header_field_info {
 	char				*abbrev;    /**< abbreviated name of this field */
 	enum ftenum			type;       /**< field type, one of FT_ (from ftypes.h) */
 	int					display;	/**< one of BASE_, or number of field bits for FT_BOOLEAN */
-	const void			*strings;	/**< _value_string (or true_false_string for FT_BOOLEAN), typically converted by VALS() or TFS() */
+	const void			*strings;	/**< _value_string (or true_false_string for FT_BOOLEAN), typically converted by VALS() or TFS() If this is an FT_PROTOCOL then it points to the associated protocol_t structure*/
 	guint32				bitmask;    /**< FT_BOOLEAN only: bitmask of interesting bits */
 	char				*blurb;		/**< Brief description of field. */
 
 	/* ------- set by proto routines (prefilled by HFILL macro, see below) ------ */
 	int				id;		/**< Field ID */
 	int				parent;		/**< parent protocol tree */
+		/* This field keeps track of whether a field is 
+		 * referenced in any filter or not and if so how 
+		 * many times. If a filter is being referenced the 
+		 * refcount for the parent protocol is updated as well 
+		 */
+	int				ref_count;	/**< is this field referenced by a filter or not */
 	int				bitshift;	/**< bits to shift (FT_BOOLEAN only) */
 	header_field_info		*same_name_next; /**< Link to next hfinfo with same abbrev*/
 	header_field_info		*same_name_prev; /**< Link to previous hfinfo with same abbrev*/
@@ -127,7 +170,7 @@ struct _header_field_info {
  * _header_field_info. If new fields are added or removed, it should
  * be changed as necessary.
  */
-#define HFILL 0, 0, 0, NULL, NULL
+#define HFILL 0, 0, 0, 0, NULL, NULL
 
 /** Used when registering many fields at once, using proto_register_field_array() */
 typedef struct hf_register_info {
@@ -228,6 +271,21 @@ extern void proto_init(const char *plugin_dir,
 /** Frees memory used by proto routines. Called at program shutdown */
 extern void proto_cleanup(void);
 
+/** This function takes a tree and a protocol id as parameter and
+    will return TRUE/FALSE for whether the protocol or any of the filterable
+    fields in the protocol is referenced by any fitlers.
+    If this function returns FALSE then it is safe to skip any 
+	proto_tree_add_...() calls and just treat the call as if the 
+    dissector was called with tree==NULL.
+    If you reset the tree to NULL by this dissector returning FALSE,
+    you will still need to call any subdissector with the original value of 
+    tree or filtering will break.
+
+    The purpose of this is to optimize ethereal for speed and make it
+    faster for when filters are being used.
+*/
+extern gboolean proto_field_is_referenced(proto_tree *tree, int proto_id);
+
 
 
 /** Create a subtree under an existing item.
@@ -860,7 +918,7 @@ proto_item_fill_label(field_info *fi, gchar *label_str);
 
 
 /** Register a new protocol.
- @param name the name of the new protocol
+ @param name the full name of the new protocol
  @param short_name abbreviated name of the new protocol
  @param filter_name protocol name used for a display filter string 
  @return the new protocol handle */
@@ -878,7 +936,7 @@ proto_register_field_array(int parent, hf_register_info *hf, int num_records);
  @param indices array of ett indices
  @param num_indices the number of records in indices */
 extern void
-proto_register_subtree_array(gint **indices, int num_indices);
+proto_register_subtree_array(gint *const *indices, int num_indices);
 
 /** Returns number of items (protocols or header fields) registered.
  @return the number of items */
@@ -902,7 +960,7 @@ extern header_field_info* proto_registrar_get_nth(guint hfindex);
 /** Get the header_field information based upon a field name.
  @param field_name the field name to search for
  @return the registered item */
-extern header_field_info* proto_registrar_get_byname(char *field_name);
+extern header_field_info* proto_registrar_get_byname(const char *field_name);
 
 /** Get enum ftenum FT_ of registered header_field number n.
  @param n item # n (0-indexed)
