@@ -1,7 +1,7 @@
 /* to_str.c
  * Routines for utilities to convert various other types to strings.
  *
- * $Id: to_str.c 12911 2005-01-01 12:29:26Z guy $
+ * $Id: to_str.c 14216 2005-04-28 09:51:55Z kukosa $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -141,7 +141,7 @@ ether_to_str(const guint8 *ad)
 gchar *
 ip_to_str(const guint8 *ad) {
   static gchar  str[4][16];
-  static int   cur_idx=0;
+  static guint32   cur_idx=0;
   gchar *cur;
 
   cur_idx++;
@@ -197,34 +197,33 @@ ip_to_str_buf(const guint8 *ad, gchar *buf)
 {
 	register gchar const *p;
 	register gchar *b=buf;
-	register gchar c;
 
 	p=fast_strings[*ad++];
-	while((c=*p)){
-		*b++=c;
+	do {
+		*b++=*p;
 		p++;
-	}
+	} while(*p);
 	*b++='.';
 
 	p=fast_strings[*ad++];
-	while((c=*p)){
-		*b++=c;
+	do {
+		*b++=*p;
 		p++;
-	}
+	} while(*p);
 	*b++='.';
 
 	p=fast_strings[*ad++];
-	while((c=*p)){
-		*b++=c;
+	do {
+		*b++=*p;
 		p++;
-	}
+	} while(*p);
 	*b++='.';
 
-	p=fast_strings[*ad++];
-	while((c=*p)){
-		*b++=c;
+	p=fast_strings[*ad];
+	do {
+		*b++=*p;
 		p++;
-	}
+	} while(*p);
 	*b=0;
 }
 
@@ -363,8 +362,9 @@ vines_addr_to_str_buf(const guint8 *addrp, gchar *buf)
 /*
  * Maximum length of a string showing days/hours/minutes/seconds.
  * (Does not include the terminating '\0'.)
+ * Includes space for a '-' sign for any negative compunents.
  */
-#define TIME_SECS_LEN	(8+1+4+2+2+5+2+2+7+2+2+7)
+#define TIME_SECS_LEN	(10+1+4+2+2+5+2+2+7+2+2+7+4)
 
 /*
  * Convert a value in seconds and fractions of a second to a string,
@@ -372,15 +372,27 @@ vines_addr_to_str_buf(const guint8 *addrp, gchar *buf)
  * into a buffer.
  * "is_nsecs" says that "frac" is microseconds if true and milliseconds
  * if false.
+ * If time is negative, add a '-' to all non-null components.
  */
 static void
-time_secs_to_str_buf(guint32 time, guint32 frac, gboolean is_nsecs,
+time_secs_to_str_buf(gint32 time, guint32 frac, gboolean is_nsecs,
 			   gchar *buf)
 {
   static gchar *p;
   int hours, mins, secs;
+  gchar *msign = "";
   gboolean do_comma = FALSE;
 
+  if(time<0){
+    time= -time;
+    msign="-";
+  }
+
+  if(time<0){	/* We've overflowed. */
+    sprintf(buf, "Unable to cope with time value %d", time);
+    return;
+  }
+
   secs = time % 60;
   time /= 60;
   mins = time % 60;
@@ -388,35 +400,36 @@ time_secs_to_str_buf(guint32 time, guint32 frac, gboolean is_nsecs,
   hours = time % 24;
   time /= 24;
 
+  /* This would probably be cleaner if we used GStrings instead. */
   p = buf;
   if (time != 0) {
-    sprintf(p, "%u day%s", time, PLURALIZE(time));
+    sprintf(p, "%s%u day%s", time?msign:"", time, PLURALIZE(time));
     p += strlen(p);
     do_comma = TRUE;
   }
   if (hours != 0) {
-    sprintf(p, "%s%u hour%s", COMMA(do_comma), hours, PLURALIZE(hours));
+    sprintf(p, "%s%s%u hour%s", COMMA(do_comma), hours?msign:"", hours, PLURALIZE(hours));
     p += strlen(p);
     do_comma = TRUE;
   }
   if (mins != 0) {
-    sprintf(p, "%s%u minute%s", COMMA(do_comma), mins, PLURALIZE(mins));
+    sprintf(p, "%s%s%u minute%s", COMMA(do_comma), mins?msign:"", mins, PLURALIZE(mins));
     p += strlen(p);
     do_comma = TRUE;
   }
   if (secs != 0 || frac != 0) {
     if (frac != 0) {
       if (is_nsecs)
-        sprintf(p, "%s%u.%09u seconds", COMMA(do_comma), secs, frac);
+        sprintf(p, "%s%s%u.%09u seconds", COMMA(do_comma), msign, secs, frac);
       else
-        sprintf(p, "%s%u.%03u seconds", COMMA(do_comma), secs, frac);
+        sprintf(p, "%s%s%u.%03u seconds", COMMA(do_comma), msign, secs, frac);
     } else
-      sprintf(p, "%s%u second%s", COMMA(do_comma), secs, PLURALIZE(secs));
+      sprintf(p, "%s%s%u second%s", COMMA(do_comma), msign, secs, PLURALIZE(secs));
   }
 }
 
 gchar *
-time_secs_to_str(guint32 time)
+time_secs_to_str(gint32 time)
 {
   static gchar  str[3][TIME_SECS_LEN+1];
   static gchar *cur;
@@ -439,7 +452,7 @@ time_secs_to_str(guint32 time)
 }
 
 gchar *
-time_msecs_to_str(guint32 time)
+time_msecs_to_str(gint32 time)
 {
   static gchar  str[3][TIME_SECS_LEN+1+3+1];
   static gchar *cur;
@@ -458,8 +471,16 @@ time_msecs_to_str(guint32 time)
     return cur;
   }
 
-  msecs = time % 1000;
-  time /= 1000;
+  if(time<0){
+    /* oops we got passed a negative time */
+    time= -time;
+    msecs = time % 1000;
+    time /= 1000;
+    time= -time;
+  } else {
+    msecs = time % 1000;
+    time /= 1000;
+  }
 
   time_secs_to_str_buf(time, msecs, FALSE, cur);
   return cur;
@@ -581,7 +602,7 @@ rel_time_to_str(nstime_t *rel_time)
 	static char str[3][1+TIME_SECS_LEN+1+6+1];
 	char *p;
 	char *sign;
-	guint32 time;
+	gint32 time;
 	gint32 nsec;
 
 	if (cur == &str[0][0]) {
@@ -871,3 +892,41 @@ address_to_str_buf(const address *addr, gchar *buf)
     g_assert_not_reached();
   }
 }
+
+gchar* oid_to_str(guint8 *oid, gint oid_len) {
+  /* static buffer */
+  static int cnt = 0;
+  static gchar strbuf[8][MAX_OID_STR_LEN];
+
+  cnt = (cnt + 1) % 8;
+  return oid_to_str_buf(oid, oid_len, strbuf[cnt]);
+}
+
+gchar* oid_to_str_buf(guint8 *oid, gint oid_len, gchar *buf) {
+  gint i;
+  guint8 byte;
+  guint32 value;
+  gchar *bufp;
+
+  bufp = buf; value=0;
+  for (i=0; i<oid_len; i++){
+    byte = oid[i];
+    if ((bufp - buf) > (MAX_OID_STR_LEN - 12)) {
+      bufp += sprintf(bufp, ".>>>");
+      break;
+    }
+    if (i == 0) {
+      bufp += sprintf(bufp, "%d.%d", byte/40, byte%40);
+      continue;
+    }
+    value = (value << 7) | (byte & 0x7F);
+    if (byte & 0x80) {
+      continue;
+    }
+    bufp += sprintf(bufp, ".%d", value);
+    value = 0;
+  }
+  *bufp = '\0';
+
+  return buf;
+}
