@@ -9,7 +9,7 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.c 12957 2005-01-05 10:19:36Z guy $
+ * $Id: tvbuff.c 14220 2005-04-28 18:07:40Z gerald $
  *
  * Copyright (c) 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -1711,12 +1711,15 @@ guint8 *
 tvb_get_string(tvbuff_t *tvb, gint offset, gint length)
 {
 	const guint8 *ptr;
-	guint8 *strbuf;
+	guint8 *strbuf = NULL;
+
+	tvb_ensure_bytes_exist(tvb, offset, length);
 
 	ptr = ensure_contiguous(tvb, offset, length);
 	strbuf = g_malloc(length + 1);
-	if (length != 0)
+	if (length != 0) {
 		memcpy(strbuf, ptr, length);
+	}
 	strbuf[length] = '\0';
 	return strbuf;
 }
