@@ -1,7 +1,7 @@
 /* etypes.h
  * Defines ethernet packet types, similar to tcpdump's ethertype.h
  *
- * $Id: etypes.h 12967 2005-01-06 08:37:43Z guy $
+ * $Id: etypes.h 13577 2005-03-03 08:34:48Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -80,6 +80,10 @@
 #define ETHERTYPE_3C_NBP_DGRAM	0x3c07
 #endif
 
+#ifndef ETHERTYPE_XNS_IDP
+#define ETHERTYPE_XNS_IDP	0x0600
+#endif
+
 #ifndef ETHERTYPE_IP
 #define ETHERTYPE_IP		0x0800
 #endif
@@ -252,6 +256,10 @@
 #define ETHERTYPE_RSN_PREAUTH	0x88c7  /* 802.11i Pre-Authentication */
 #endif
 
+#ifndef ETHERTYPE_LLDP
+#define ETHERTYPE_LLDP	        0x88cc  /* IEEE 802.1 Link Layer Discovery Protocol (LLDP) */
+#endif
+
 #ifndef ETHERTYPE_LOOP
 #define ETHERTYPE_LOOP		0x9000 	/* used for layer 2 testing (do i see my own frames on the wire) */
 #endif
