@@ -5,7 +5,7 @@
  *
  * MUCH code modified from service_response_time_table.c.
  *
- * $Id: ansi_a_stat.c,v 1.18 2004/05/23 23:24:05 ulfl Exp $
+ * $Id: ansi_a_stat.c 14098 2005-04-16 10:53:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,12 +45,12 @@
 #include "image/clist_descend.xpm"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
-#include "tap.h"
+#include <epan/tap.h>
 #include "../register.h"
-#include "packet-bssap.h"
-#include "packet-ansi_a.h"
+#include <epan/dissectors/packet-bssap.h>
+#include <epan/dissectors/packet-ansi_a.h>
 #include "../globals.h"
-#include "filter_prefs.h"
+#include "filter_dlg.h"
 #include "compat_macros.h"
 #include "ui_util.h"
 
@@ -83,33 +83,30 @@ static void
 ansi_a_stat_reset(
     void		*tapdata)
 {
-    tapdata = tapdata;
+    ansi_a_stat_t	*stat_p = tapdata;
 
-    memset((void *) &stat, 0, sizeof(ansi_a_stat_t));
+    memset(stat_p, 0, sizeof(ansi_a_stat_t));
 }
 
 
 static int
 ansi_a_stat_packet(
     void		*tapdata,
-    packet_info		*pinfo,
+    packet_info		*pinfo _U_,
     epan_dissect_t	*edt _U_,
-    void		*data)
+    const void		*data)
 {
-    ansi_a_tap_rec_t	*data_p = data;
-
-
-    tapdata = tapdata;
-    pinfo = pinfo;
+    ansi_a_stat_t	*stat_p = tapdata;
+    const ansi_a_tap_rec_t	*data_p = data;
 
     switch (data_p->pdu_type)
     {
     case BSSAP_PDU_TYPE_BSMAP:
-	stat.bsmap_message_type[data_p->message_type]++;
+	stat_p->bsmap_message_type[data_p->message_type]++;
 	break;
 
     case BSSAP_PDU_TYPE_DTAP:
-	stat.dtap_message_type[data_p->message_type]++;
+	stat_p->dtap_message_type[data_p->message_type]++;
 	break;
 
     default:
@@ -127,13 +124,11 @@ static void
 ansi_a_stat_draw(
     void		*tapdata)
 {
+    ansi_a_stat_t	*stat_p = tapdata;
     int			i, j;
     char		*strp;
 
-
-    tapdata = tapdata;
-
-    if (dlg_bsmap.win != NULL)
+    if (dlg_bsmap.win && tapdata)
     {
 	i = 0;
 
@@ -142,7 +137,7 @@ ansi_a_stat_draw(
 	    j = gtk_clist_find_row_from_data(GTK_CLIST(dlg_bsmap.table), (gpointer) i);
 
 	    strp = g_strdup_printf("%d",
-		    stat.bsmap_message_type[ansi_a_ios401_bsmap_strings[i].value]);
+		    stat_p->bsmap_message_type[ansi_a_ios401_bsmap_strings[i].value]);
 	    gtk_clist_set_text(GTK_CLIST(dlg_bsmap.table), j, 2, strp);
 	    g_free(strp);
 
@@ -152,7 +147,7 @@ ansi_a_stat_draw(
 	gtk_clist_sort(GTK_CLIST(dlg_bsmap.table));
     }
 
-    if (dlg_dtap.win != NULL)
+    if (dlg_dtap.win && tapdata)
     {
 	i = 0;
 
@@ -161,7 +156,7 @@ ansi_a_stat_draw(
 	    j = gtk_clist_find_row_from_data(GTK_CLIST(dlg_dtap.table), (gpointer) i);
 
 	    strp = g_strdup_printf("%d",
-		    stat.dtap_message_type[ansi_a_ios401_dtap_strings[i].value]);
+		    stat_p->dtap_message_type[ansi_a_ios401_dtap_strings[i].value]);
 	    gtk_clist_set_text(GTK_CLIST(dlg_dtap.table), j, 2, strp);
 	    g_free(strp);
 
@@ -235,8 +230,8 @@ ansi_a_stat_gtk_sort_column(
     gconstpointer	ptr1,
     gconstpointer	ptr2)
 {
-    GtkCListRow		*row1 = (GtkCListRow *) ptr1;
-    GtkCListRow		*row2 = (GtkCListRow *) ptr2;
+    const GtkCListRow	*row1 = ptr1;
+    const GtkCListRow	*row2 = ptr2;
     char		*text1 = NULL;
     char		*text2 = NULL;
     int			i1, i2;
@@ -465,12 +460,12 @@ register_tap_listener_gtkansi_a_stat(void)
     GString		*err_p;
 
 
-    register_ethereal_tap("ansi_a,", ansi_a_stat_gtk_init);
+    register_tap_listener_cmd_arg("ansi_a,", ansi_a_stat_gtk_init);
 
     memset((void *) &stat, 0, sizeof(ansi_a_stat_t));
 
     err_p =
-	register_tap_listener("ansi_a", NULL, NULL,
+	register_tap_listener("ansi_a", &stat, NULL,
 	    ansi_a_stat_reset,
 	    ansi_a_stat_packet,
 	    ansi_a_stat_draw);
@@ -483,8 +478,8 @@ register_tap_listener_gtkansi_a_stat(void)
 	exit(1);
     }
 
-    register_tap_menu_item("ANSI/A-Interface BSMAP", REGISTER_TAP_GROUP_NONE, 
+    register_tap_menu_item("ANSI/A-Interface BSMAP", REGISTER_TAP_GROUP_TELEPHONY, 
         ansi_a_stat_gtk_bsmap_cb, NULL, NULL ,NULL);
-    register_tap_menu_item("ANSI/A-Interface DTAP", REGISTER_TAP_GROUP_NONE,
+    register_tap_menu_item("ANSI/A-Interface DTAP", REGISTER_TAP_GROUP_TELEPHONY,
         ansi_a_stat_gtk_dtap_cb, NULL, NULL ,NULL);
 }
