@@ -1,7 +1,7 @@
 /* compat_macros.h
  * GTK-related Global defines, etc.
  *
- * $Id: compat_macros.h 12919 2005-01-01 23:29:05Z guy $
+ * $Id: compat_macros.h 14231 2005-04-29 14:51:52Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -165,6 +165,7 @@ gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), name)
 #define GTK_STOCK_GOTO_BOTTOM       "Bottom"
 #define GTK_STOCK_GOTO_TOP          "Top"
 #define GTK_STOCK_HELP              "Help"
+#define GTK_STOCK_HOME              "Home"
 #define GTK_STOCK_JUMP_TO           "GoTo"
 #define GTK_STOCK_NEW               "New"
 #define GTK_STOCK_NO                "No"
@@ -185,7 +186,11 @@ gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), name)
 #define GTK_STOCK_ZOOM_100          "Zoom 100%"
 
 #ifdef HAVE_LIBPCAP
-#define ETHEREAL_STOCK_CAPTURE_START            "New"
+#define ETHEREAL_STOCK_CAPTURE_INTERFACES       "Interfaces"
+#define ETHEREAL_STOCK_CAPTURE_OPTIONS          "Options"
+#define ETHEREAL_STOCK_CAPTURE_START            "Capture"
+#define ETHEREAL_STOCK_CAPTURE_STOP             "Stop"
+#define ETHEREAL_STOCK_CAPTURE_RESTART          "Restart"
 #define ETHEREAL_STOCK_CAPTURE_FILTER           "CFilter"
 #define ETHEREAL_STOCK_CAPTURE_FILTER_ENTRY     "Capture Filter:"
 #endif
@@ -199,6 +204,20 @@ gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), name)
 #define ETHEREAL_STOCK_EDIT                     "Edit..."
 #define ETHEREAL_STOCK_ADD_EXPRESSION           "Add Expression..."
 #define ETHEREAL_STOCK_DONT_SAVE                "Continue without Saving"
+#define ETHEREAL_STOCK_ABOUT                    "About"
+#define ETHEREAL_STOCK_COLORIZE                 "Colorize"
+#define ETHEREAL_STOCK_AUTOSCROLL               "Auto Scroll"
+#define ETHEREAL_STOCK_RESIZE_COLUMNS           "Resize Columns"
+#define ETHEREAL_STOCK_TIME                     "Time"
+#define ETHEREAL_STOCK_INTERNET                 "Internet"
+#define ETHEREAL_STOCK_WEB_SUPPORT              "Web Support"
+#define ETHEREAL_STOCK_WIKI                     "Wiki"
+#define ETHEREAL_STOCK_CONVERSATIONS            "Conversations"
+#define ETHEREAL_STOCK_ENDPOINTS                "Endpoints"
+#define ETHEREAL_STOCK_GRAPHS                   "Graphs"
+#define ETHEREAL_STOCK_TELEPHONY                "Telephony"
+#define ETHEREAL_STOCK_DECODE_AS                "Decode As"
+#define ETHEREAL_STOCK_CHECKBOX                 "Checkbox"
 
 /** Create a stock button. Will create a "normal" button for GTK1.
  *
@@ -229,6 +248,18 @@ dlg_radio_button_new_with_label_with_mnemonic( \
     radio_group ? gtk_radio_button_group(GTK_RADIO_BUTTON(radio_group)) : NULL, \
     label_text, accel_group)
 
+/** Create a radio button.
+ *
+ * @param radio_group group the radio buttons (another radio button or NULL for first one)
+ * @param label_text the text to display
+ * @param accel_group accelerator group (GTK1 only)
+ * @return the new radio button
+ */
+#define RADIO_BUTTON_NEW_WITH_LABEL(radio_group, label_text) \
+gtk_radio_button_new_with_label ( \
+    radio_group ? gtk_radio_button_group(GTK_RADIO_BUTTON(radio_group)) : NULL, \
+    label_text)
+
 /** Create a toggle button.
  *
  * @param label_text the text to display
@@ -285,7 +316,11 @@ g_signal_stop_emission_by_name(G_OBJECT(widget), name)
 {path, accelerator, GTK_MENU_FUNC(callback), action, "<StockItem>", data}
 
 #ifdef HAVE_LIBPCAP
-#define ETHEREAL_STOCK_LABEL_CAPTURE_START            "_New"
+#define ETHEREAL_STOCK_LABEL_CAPTURE_INTERFACES       "_Interfaces"
+#define ETHEREAL_STOCK_LABEL_CAPTURE_OPTIONS          "_Options"
+#define ETHEREAL_STOCK_LABEL_CAPTURE_START            "_Capture"
+#define ETHEREAL_STOCK_LABEL_CAPTURE_STOP             "_Stop"
+#define ETHEREAL_STOCK_LABEL_CAPTURE_RESTART          "_Restart"
 #define ETHEREAL_STOCK_LABEL_CAPTURE_FILTER           "_CFilter"
 #define ETHEREAL_STOCK_LABEL_CAPTURE_FILTER_ENTRY     "_Capture Filter:"
 #endif
@@ -299,9 +334,27 @@ g_signal_stop_emission_by_name(G_OBJECT(widget), name)
 #define ETHEREAL_STOCK_LABEL_EDIT                     "_Edit..."
 #define ETHEREAL_STOCK_LABEL_ADD_EXPRESSION           "_Expression..." /* plus sign coming from icon */
 #define ETHEREAL_STOCK_LABEL_DONT_SAVE                "Continue _without Saving"
+#define ETHEREAL_STOCK_LABEL_ABOUT                    "_About"
+#define ETHEREAL_STOCK_LABEL_COLORIZE                 "_Colorize"
+#define ETHEREAL_STOCK_LABEL_AUTOSCROLL               "_Auto Scroll in Live Capture"
+#define ETHEREAL_STOCK_LABEL_RESIZE_COLUMNS           "Resize Columns"
+#define ETHEREAL_STOCK_LABEL_TIME                     "Time"
+#define ETHEREAL_STOCK_LABEL_INTERNET                 "Internet"
+#define ETHEREAL_STOCK_LABEL_WEB_SUPPORT              "Web Support"
+#define ETHEREAL_STOCK_LABEL_WIKI                     "Wiki"
+#define ETHEREAL_STOCK_LABEL_CONVERSATIONS            "Conversations"
+#define ETHEREAL_STOCK_LABEL_ENDPOINTS                "Endpoints"
+#define ETHEREAL_STOCK_LABEL_GRAPHS                   "Graphs"
+#define ETHEREAL_STOCK_LABEL_TELEPHONY                "Telephony"
+#define ETHEREAL_STOCK_LABEL_DECODE_AS                "Decode As"
+#define ETHEREAL_STOCK_LABEL_CHECKBOX                 "Checkbox"
 
 #ifdef HAVE_LIBPCAP
+#define ETHEREAL_STOCK_CAPTURE_INTERFACES       "Ethereal_Stock_CaptureInterfaces"
+#define ETHEREAL_STOCK_CAPTURE_OPTIONS          "Ethereal_Stock_CaptureOptionss"
 #define ETHEREAL_STOCK_CAPTURE_START            "Ethereal_Stock_CaptureStart"
+#define ETHEREAL_STOCK_CAPTURE_STOP             "Ethereal_Stock_CaptureStop"
+#define ETHEREAL_STOCK_CAPTURE_RESTART          "Ethereal_Stock_CaptureRestart"
 #define ETHEREAL_STOCK_CAPTURE_FILTER           "Ethereal_Stock_CaptureFilter"
 #define ETHEREAL_STOCK_CAPTURE_FILTER_ENTRY     "Ethereal_Stock_CaptureFilter_Entry"
 #endif
@@ -315,6 +368,20 @@ g_signal_stop_emission_by_name(G_OBJECT(widget), name)
 #define ETHEREAL_STOCK_EDIT                     "Ethereal_Stock_Edit"
 #define ETHEREAL_STOCK_ADD_EXPRESSION           "Ethereal_Stock_Edit_Add_Expression"
 #define ETHEREAL_STOCK_DONT_SAVE                "Ethereal_Stock_Continue_without_Saving"
+#define ETHEREAL_STOCK_ABOUT                    "Ethereal_Stock_About"
+#define ETHEREAL_STOCK_COLORIZE                 "Ethereal_Stock_Colorize"
+#define ETHEREAL_STOCK_AUTOSCROLL               "Ethereal_Stock_Autoscroll"
+#define ETHEREAL_STOCK_RESIZE_COLUMNS           "Ethereal_Stock_Resize_Columns"
+#define ETHEREAL_STOCK_TIME                     "Ethereal_Stock_Time"
+#define ETHEREAL_STOCK_INTERNET                 "Ethereal_Stock_Internet"
+#define ETHEREAL_STOCK_WEB_SUPPORT              "Ethereal_Stock_Web_Support"
+#define ETHEREAL_STOCK_WIKI                     "Ethereal_Stock_Wiki"
+#define ETHEREAL_STOCK_CONVERSATIONS            "Ethereal_Stock_Conversations"
+#define ETHEREAL_STOCK_ENDPOINTS                "Ethereal_Stock_Endpoints"
+#define ETHEREAL_STOCK_GRAPHS                   "Ethereal_Stock_Graphs"
+#define ETHEREAL_STOCK_TELEPHONY                "Ethereal_Stock_Telephony"
+#define ETHEREAL_STOCK_DECODE_AS                "Ethereal_Stock_DecodeAs"
+#define ETHEREAL_STOCK_CHECKBOX                 "Ethereal_Stock_Checkbox"
 
 #define BUTTON_NEW_FROM_STOCK(stock_id) \
 gtk_button_new_from_stock(stock_id);
@@ -326,9 +393,14 @@ gtk_check_button_new_with_mnemonic(label_text)
 gtk_radio_button_new_with_mnemonic_from_widget( \
     radio_group ? GTK_RADIO_BUTTON(radio_group) : NULL, label_text)
 
+#define RADIO_BUTTON_NEW_WITH_LABEL(radio_group, label_text) \
+gtk_radio_button_new_with_label_from_widget( \
+    radio_group ? GTK_RADIO_BUTTON(radio_group) : NULL, label_text)
+
 #define TOGGLE_BUTTON_NEW_WITH_MNEMONIC(label_text, accel_group) \
 gtk_toggle_button_new_with_mnemonic(label_text)
 
+/* for details, see "Pango Text Attribute Markup" */
 #define PRIMARY_TEXT_START "<span weight=\"bold\" size=\"larger\">"
 #define PRIMARY_TEXT_END "</span>"
 
