@@ -1,7 +1,7 @@
 /* conversations_ip.c
  * conversations_ip   2003 Ronnie Sahlberg
  *
- * $Id: conversations_ip.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: conversations_ip.c 13993 2005-04-01 09:08:27Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -76,7 +76,7 @@ ip_endpoints_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_ip_conversation(void)
 {
-	register_ethereal_tap("conv,ip", ip_conversation_init);
+	register_tap_listener_cmd_arg("conv,ip", ip_conversation_init);
 
 	register_tap_menu_item("IPv4", REGISTER_TAP_GROUP_CONVERSATION_LIST,
 	    ip_endpoints_cb, NULL, NULL, NULL);
