@@ -4,7 +4,7 @@
  * conversations_table   2003 Ronnie Sahlberg
  * Helper routines common to all endpoint conversations tap.
  *
- * $Id: conversations_table.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: conversations_table.c 13463 2005-02-22 01:55:04Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,13 +48,14 @@
 #include "image/clist_descend.xpm"
 #include "simple_dialog.h"
 #include "globals.h"
-#include "gtk/find_dlg.h"
+#include "find_dlg.h"
 #include "color.h"
 #include "gtk/color_dlg.h"
 #include "gtkglobals.h"
 #include "main.h"
 #include "ui_util.h"
 #include "dlg_utils.h"
+#include "help_dlg.h"
 
 
 #define GTK_MENU_FUNC(a) ((GtkItemFactoryCallback)(a))
@@ -961,8 +962,6 @@ draw_ct_table_address(conversations_table *ct, int conversation_idx)
         entry=address_to_str(&ct->conversations[conversation_idx].src_address);
     else {
         entry=get_addr_name(&ct->conversations[conversation_idx].src_address);
-        if(!entry)
-            entry=address_to_str(&ct->conversations[conversation_idx].src_address);
     }
     gtk_clist_set_text(ct->table, rownum, 0, entry);
 
@@ -985,8 +984,6 @@ draw_ct_table_address(conversations_table *ct, int conversation_idx)
         entry=address_to_str(&ct->conversations[conversation_idx].dst_address);
     else {
         entry=get_addr_name(&ct->conversations[conversation_idx].dst_address);
-        if(!entry)
-            entry=address_to_str(&ct->conversations[conversation_idx].dst_address);
     }
     gtk_clist_set_text(ct->table, rownum, 2, entry);
 
@@ -1238,7 +1235,7 @@ init_conversation_table(gboolean hide_ports, char *table_name, char *tap_name, c
     char title[256];
     GtkWidget *vbox;
     GtkWidget *bbox;
-    GtkWidget *close_bt;
+    GtkWidget *close_bt, *help_bt;
     gboolean ret;
 
 
@@ -1261,19 +1258,28 @@ init_conversation_table(gboolean hide_ports, char *table_name, char *tap_name, c
     }
 
     /* Button row. */
-    bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    if(topic_available(HELP_STATS_CONVERSATIONS_DIALOG)) {
+        bbox = dlg_button_row_new(GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+    } else {
+        bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    }
     gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
     close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
     window_set_cancel_button(conversations->win, close_bt, window_cancel_button_cb);
 
+    if(topic_available(HELP_STATS_CONVERSATIONS_DIALOG)) {
+        help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
+        SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_STATS_CONVERSATIONS_DIALOG);
+    }
+
     SIGNAL_CONNECT(conversations->win, "delete_event", window_delete_event_cb, NULL);
     SIGNAL_CONNECT(conversations->win, "destroy", ct_win_destroy_cb, conversations);
 
     gtk_widget_show_all(conversations->win);
     window_present(conversations->win);
 
-    retap_packets(&cfile);
+    cf_retap_packets(&cfile);
 
 	
     /* Keep clist frozen to cause modifications to the clist (inserts, appends, others that are extremely slow
@@ -1385,7 +1391,7 @@ init_conversation_notebook_cb(GtkWidget *w _U_, gpointer d _U_)
     GtkWidget *vbox;
     GtkWidget *hbox;
     GtkWidget *bbox;
-    GtkWidget *close_bt;
+    GtkWidget *close_bt, *help_bt;
     GtkWidget *win;
     GtkWidget *resolv_cb;
     int page;
@@ -1440,19 +1446,28 @@ init_conversation_notebook_cb(GtkWidget *w _U_, gpointer d _U_)
     SIGNAL_CONNECT(resolv_cb, "toggled", ct_resolve_toggle_dest, pages);
 
     /* Button row. */
-    bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    if(topic_available(HELP_STATS_CONVERSATIONS_DIALOG)) {
+        bbox = dlg_button_row_new(GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+    } else {
+        bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    }
     gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
     close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
     window_set_cancel_button(win, close_bt, window_cancel_button_cb);
 
+    if(topic_available(HELP_STATS_CONVERSATIONS_DIALOG)) {
+        help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
+        SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_STATS_CONVERSATIONS_DIALOG);
+    }
+
     SIGNAL_CONNECT(win, "delete_event", window_delete_event_cb, NULL);
     SIGNAL_CONNECT(win, "destroy", ct_win_destroy_notebook_cb, pages);
 
     gtk_widget_show_all(win);
     window_present(win);
 
-    retap_packets(&cfile);
+    cf_retap_packets(&cfile);
 
     /* after retapping, redraw table */
     for (page=1; page<=GPOINTER_TO_INT(pages[0]); page++) {
