@@ -1,7 +1,7 @@
 /* dcerpc_stat.c
  * dcerpc_stat   2002 Ronnie Sahlberg
  *
- * $Id: dcerpc_stat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: dcerpc_stat.c 13993 2005-04-01 09:08:27Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -331,7 +331,7 @@ gtk_dcerpcstat_init(char *optarg)
     gtk_widget_show_all(rs->win);
     window_present(rs->win);
 
-    retap_packets(&cfile);
+    cf_retap_packets(&cfile);
 }
 
 
@@ -675,7 +675,7 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkdcerpcstat(void)
 {
-	register_ethereal_tap("dcerpc,srt,", gtk_dcerpcstat_init);
+	register_tap_listener_cmd_arg("dcerpc,srt,", gtk_dcerpcstat_init);
 
 	register_tap_menu_item("DCE-RPC...", REGISTER_TAP_GROUP_RESPONSE_TIME,
 	    gtk_dcerpcstat_cb, NULL, NULL, NULL);
