@@ -1,7 +1,7 @@
 /* fc_stat.c
  * fc_stat   2003 Ronnie Sahlberg
  *
- * $Id: fc_stat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: fc_stat.c 13993 2005-04-01 09:08:27Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -196,7 +196,7 @@ gtk_fcstat_init(char *optarg)
 	gtk_widget_show_all(fc->win);
     window_present(fc->win);
 
-    retap_packets(&cfile);
+    cf_retap_packets(&cfile);
 }
 
 
@@ -312,7 +312,7 @@ gtk_fcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkfcstat(void)
 {
-	register_ethereal_tap("fc,srt", gtk_fcstat_init);
+	register_tap_listener_cmd_arg("fc,srt", gtk_fcstat_init);
 
 	register_tap_menu_item("Fibre Channel...", REGISTER_TAP_GROUP_RESPONSE_TIME,
 	    gtk_fcstat_cb, NULL, NULL, NULL);
