@@ -1,7 +1,7 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c 12915 2005-01-01 23:19:11Z guy $
+ * $Id: file_dlg.c 14142 2005-04-19 21:02:01Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -54,8 +54,8 @@
 #include "gtk/color_dlg.h"
 #ifdef HAVE_LIBPCAP
 #include "capture_dlg.h"
-#include "range_utils.h"
 #endif
+#include "range_utils.h"
 #include "merge.h"
 #include "util.h"
 
@@ -676,7 +676,7 @@ file_open_ok_cb(GtkWidget *w, gpointer fs) {
   }
 
   /* Try to open the capture file. */
-  if ((err = cf_open(cf_name, FALSE, &cfile)) != 0) {
+  if (cf_open(&cfile, cf_name, FALSE, &err) != CF_OK) {
     /* We couldn't open it; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the open error,
@@ -715,14 +715,14 @@ file_open_ok_cb(GtkWidget *w, gpointer fs) {
 
   switch (cf_read(&cfile)) {
 
-  case READ_SUCCESS:
-  case READ_ERROR:
+  case CF_READ_OK:
+  case CF_READ_ERROR:
     /* Just because we got an error, that doesn't mean we were unable
        to read any of the file; we handle what we could get from the
        file. */
     break;
 
-  case READ_ABORTED:
+  case CF_READ_ABORTED:
     /* The user bailed out of re-reading the capture file; the
        capture file has been closed - just free the capture file name
        string and return (without changing the last containing
@@ -998,10 +998,9 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
   GtkWidget   *filter_te, *rb;
   dfilter_t   *rfcode = NULL;
   int          err;
-  gboolean     merge_ok;
+  cf_status_t  merge_status;
   char        *in_filenames[2];
-  int          out_fd;
-  char         tmpname[128+1];
+  char        *tmpname;
 
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
   cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
@@ -1027,39 +1026,37 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
     	return;
   }
 
-  out_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
-
   /* merge or append the two files */
   rb = OBJECT_GET_DATA(w, E_MERGE_CHRONO_KEY);
+  tmpname = NULL;
   if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (rb))) {
       /* chronological order */
       in_filenames[0] = cfile.filename;
       in_filenames[1] = cf_name;
-      merge_ok = cf_merge_files(tmpname, out_fd, 2, in_filenames,
-                                filetype, FALSE);
+      merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype, FALSE);
   } else {
       rb = OBJECT_GET_DATA(w, E_MERGE_PREPEND_KEY);
       if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (rb))) {
           /* prepend file */
           in_filenames[0] = cfile.filename;
           in_filenames[1] = cf_name;
-          merge_ok = cf_merge_files(tmpname, out_fd, 2, in_filenames,
-                                    filetype, TRUE);
+          merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype,
+                                        TRUE);
       } else {
           /* append file */
           in_filenames[0] = cf_name;
           in_filenames[1] = cfile.filename;
-          merge_ok = cf_merge_files(tmpname, out_fd, 2, in_filenames,
-                                    filetype, TRUE);
+          merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype,
+                                        TRUE);
       }
   }
 
   g_free(cf_name);
   
-  if (!merge_ok) {
-    close(out_fd);	/* XXX - it's already closed, right? */
+  if (merge_status != CF_OK) {
     if (rfcode != NULL)
       dfilter_free(rfcode);
+    g_free(tmpname);
     return;
   }
 
@@ -1069,15 +1066,17 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
   window_destroy(GTK_WIDGET (fs));
 
   /* Try to open the merged capture file. */
-  if ((err = cf_open(tmpname, TRUE /* temporary file */, &cfile)) != 0) {
+  if (cf_open(&cfile, tmpname, TRUE /* temporary file */, &err) != CF_OK) {
     /* We couldn't open it; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the open error,
        try again. */
     if (rfcode != NULL)
       dfilter_free(rfcode);
+    g_free(tmpname);
     return;
   }
+  g_free(tmpname);
 
   /* Attach the new read filter to "cf" ("cf_open()" succeeded, so
      it closed the previous capture file, and thus destroyed any
@@ -1086,14 +1085,14 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
 
   switch (cf_read(&cfile)) {
 
-  case READ_SUCCESS:
-  case READ_ERROR:
+  case CF_READ_OK:
+  case CF_READ_ERROR:
     /* Just because we got an error, that doesn't mean we were unable
        to read any of the file; we handle what we could get from the
        file. */
     break;
 
-  case READ_ABORTED:
+  case CF_READ_ABORTED:
     /* The user bailed out of re-reading the capture file; the
        capture file has been closed - just free the capture file name
        string and return (without changing the last containing
@@ -1395,11 +1394,115 @@ file_save_as_cmd_cb(GtkWidget *w _U_, gpointer data _U_)
   file_save_as_cmd(after_save_no_action, NULL);
 }
 
+
+/* all tests ok, we only have to save the file */
+/* (and probably continue with a pending operation) */
 static void
-file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
+file_save_as_cb(GtkWidget *w _U_, gpointer fs) {
   gchar	*cf_name;
   gchar	*dirname;
 
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+  cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
+#else
+  cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+#endif
+
+  /* Write out the packets (all, or only the ones from the current
+     range) to the file with the specified name. */
+  if (cf_save(&cfile, cf_name, &range, filetype) != CF_OK) {
+    /* The write failed; don't dismiss the open dialog box,
+       just leave it around so that the user can, after they
+       dismiss the alert box popped up for the error, try again. */
+    g_free(cf_name);
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
+     * as this will prevent the user from closing the now existing error
+     * message, simply close the dialog (this is the best we can do here). */
+    if (file_save_as_w)
+      window_destroy(GTK_WIDGET (fs));
+#else
+    gtk_widget_show(GTK_WIDGET (fs));
+#endif
+    return;
+  }
+
+  /* The write succeeded; get rid of the file selection box. */
+  /* cf_save() might already closed our dialog! */
+  if (file_save_as_w)
+    window_destroy(GTK_WIDGET (fs));
+
+  /* Save the directory name for future file dialogs. */
+  dirname = get_dirname(cf_name);  /* Overwrites cf_name */
+  set_last_open_dir(dirname);
+  g_free(cf_name);
+
+  /* we have finished saving, do we have pending things to do? */
+  switch(action_after_save_g) {
+  case(after_save_no_action):
+      break;
+  case(after_save_open_dialog):
+      file_open_cmd(action_after_save_data_g);
+      break;
+  case(after_save_open_recent_file):
+      menu_open_recent_file_cmd(action_after_save_data_g);
+      break;
+  case(after_save_open_dnd_file):
+      dnd_open_file_cmd(action_after_save_data_g);
+      break;
+  case(after_save_merge_dialog):
+      file_merge_cmd(action_after_save_data_g);
+      break;
+#ifdef HAVE_LIBPCAP
+  case(after_save_capture_dialog):
+      capture_start_confirmed();
+      break;
+#endif
+  case(after_save_close_file):
+      cf_close(&cfile);
+      break;
+  case(after_save_exit):
+      main_do_quit();
+      break;
+  default:
+      g_assert_not_reached();
+  }
+
+  action_after_save_g = after_save_no_action;
+}
+
+
+static void file_save_as_exists_answered_cb(gpointer dialog _U_, gint btn, gpointer data)
+{
+    gchar	*cf_name;
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(data)));
+#else
+    cf_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(data)));
+#endif
+
+    switch(btn) {
+    case(ESD_BTN_OK):
+        /* save file */
+        unlink(cf_name);
+        file_save_as_cb(NULL, data);
+        break;
+    case(ESD_BTN_CANCEL):
+        break;
+    default:
+        g_assert_not_reached();
+    }
+}
+
+
+/* user pressed "Save" dialog "Ok" button */
+static void
+file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
+  gchar	*cf_name;
+  gpointer  dialog;
+  
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
   cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
 #else
@@ -1435,15 +1538,18 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
     return;
   }
 
-  /* don't show the dialog while saving */
-  gtk_widget_hide(GTK_WIDGET (fs));
-
-  /* Write out the packets (all, or only the ones from the current
-     range) to the file with the specified name. */
-  if (! cf_save(cf_name, &cfile, &range, filetype)) {
-    /* The write failed; don't dismiss the open dialog box,
-       just leave it around so that the user can, after they
-       dismiss the alert box popped up for the error, try again. */
+  /*
+   * Check that the from file is not the same as to file
+   * We do it here so we catch all cases ...
+   * Unfortunately, the file requester gives us an absolute file
+   * name and the read file name may be relative (if supplied on
+   * the command line). From Joerg Mayer.
+   */
+  if (files_identical(cfile.filename, cf_name)) {
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+      "%sCapture file: \"%s\" identical to loaded file!%s\n\n"
+      "Please choose a different filename.",
+      simple_dialog_primary_start(), cf_name, simple_dialog_primary_end());
     g_free(cf_name);
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
     /* XXX - as we cannot start a new event loop (using gtk_dialog_run()),
@@ -1457,48 +1563,23 @@ file_save_as_ok_cb(GtkWidget *w _U_, gpointer fs) {
     return;
   }
 
-  /* The write succeeded; get rid of the file selection box. */
-  /* cf_save might already closed our dialog! */
-  if (file_save_as_w)
-    window_destroy(GTK_WIDGET (fs));
+  /* don't show the dialog while saving (or asking) */
+  gtk_widget_hide(GTK_WIDGET (fs));
 
-  /* Save the directory name for future file dialogs. */
-  dirname = get_dirname(cf_name);  /* Overwrites cf_name */
-  set_last_open_dir(dirname);
-  g_free(cf_name);
-
-  /* we have finished saving, do we have pending things to do? */
-  switch(action_after_save_g) {
-  case(after_save_no_action):
-      break;
-  case(after_save_open_dialog):
-      file_open_cmd(action_after_save_data_g);
-      break;
-  case(after_save_open_recent_file):
-      menu_open_recent_file_cmd(action_after_save_data_g);
-      break;
-  case(after_save_open_dnd_file):
-      dnd_open_file_cmd(action_after_save_data_g);
-      break;
-  case(after_save_merge_dialog):
-      file_merge_cmd(action_after_save_data_g);
-      break;
-#ifdef HAVE_LIBPCAP
-  case(after_save_capture_dialog):
-      capture_prep();
-      break;
-#endif
-  case(after_save_close_file):
-      cf_close(&cfile);
-      break;
-  case(after_save_exit):
-      main_do_quit();
-      break;
-  default:
-      g_assert_not_reached();
+  /* it the file doesn't exist, simply try to save it */
+  if (!file_exists(cf_name)) {
+    file_save_as_cb(NULL, fs);
+    g_free(cf_name);
+    return;
   }
 
-  action_after_save_g = after_save_no_action;
+  /* the file exists, ask the user to remove it first */
+  dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_OK_CANCEL,
+      PRIMARY_TEXT_START "Capture file: \"%s\" already exists!" PRIMARY_TEXT_END "\n\n"
+      "Replace the current file?", cf_name);
+  simple_dialog_set_cb(dialog, file_save_as_exists_answered_cb, fs);
+
+  g_free(cf_name);
 }
 
 void
@@ -1518,7 +1599,7 @@ file_save_as_destroy_cb(GtkWidget *win _U_, gpointer user_data _U_)
 /* Reload a file using the current read and display filters */
 void
 file_reload_cmd_cb(GtkWidget *w _U_, gpointer data _U_) {
-  cf_reload();
+  cf_reload(&cfile);
 }
 
 /******************** Color Filters *********************************/
@@ -1625,7 +1706,7 @@ file_color_import_ok_cb(GtkWidget *w, gpointer fs) {
   gchar     *cf_name, *s;
   gpointer  argument;
 
-  argument = OBJECT_GET_DATA(w, ARGUMENT_CL);     /* to be passed back into read_other_filters */
+  argument = OBJECT_GET_DATA(w, ARGUMENT_CL);     /* to be passed back into color_filters_import */
   
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
   cf_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
@@ -1636,16 +1717,16 @@ file_color_import_ok_cb(GtkWidget *w, gpointer fs) {
      Check whether they did. */
   if (test_for_directory(cf_name) == EISDIR) {
 	/* It's a directory - set the file selection box to display that
-	   directory, don't try to open the directory as a capture file. */
+	   directory, don't try to open the directory as a color filter file. */
         set_last_open_dir(cf_name);
         g_free(cf_name);
         file_selection_set_current_folder(fs, get_last_open_dir());
     	return;
   }
 
-  /* Try to open the capture file. */
+  /* Try to open the color filter file. */
 
-  if (!read_other_filters(cf_name, argument)) {
+  if (!color_filters_import(cf_name, argument)) {
     /* We couldn't open it; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the open error,
@@ -1797,7 +1878,7 @@ file_color_export_ok_cb(GtkWidget *w _U_, gpointer fs) {
   /* Write out the filters (all, or only the ones that are currently
      displayed or marked) to the file with the specified name. */
 
-   if (!write_other_filters(cf_name, color_marked))
+   if (!color_filters_export(cf_name, color_marked))
    {
     /* The write failed; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
