@@ -1,7 +1,7 @@
 /* filter_dlg.c
  * Dialog boxes for (display and capture) filter editing
  *
- * $Id: filter_dlg.c 12021 2004-09-16 22:07:03Z ulfl $
+ * $Id: filter_dlg.c 13890 2005-03-24 03:58:38Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -614,17 +614,9 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 
     help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
     if (list == CFILTER_LIST) {
-#ifdef ETHEREAL_EUG_DIR
-        SIGNAL_CONNECT(help_bt, "clicked", url_page_cb, HELP_CAPTURE_FILTERS_DIALOG);
-#else
-        SIGNAL_CONNECT(help_bt, "clicked", help_topic_cb, "Capture Filters");
-#endif
+        SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_CAPTURE_FILTERS_DIALOG);
     } else {
-#ifdef ETHEREAL_EUG_DIR
-        SIGNAL_CONNECT(help_bt, "clicked", url_page_cb, HELP_DISPLAY_FILTERS_DIALOG);
-#else
-        SIGNAL_CONNECT(help_bt, "clicked", help_topic_cb, "Display Filters");
-#endif
+        SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_DISPLAY_FILTERS_DIALOG);
     }
     gtk_tooltips_set_tip (tooltips, help_bt, ("Show topic specific help"), NULL);
 
@@ -1320,6 +1312,7 @@ color_filter_te(GtkWidget *w, guint16 red, guint16 green, guint16 blue)
     style = gtk_style_copy(gtk_widget_get_style(w));
     style->base[GTK_STATE_NORMAL] = bg;
     gtk_widget_set_style(w, style);
+    gtk_style_unref(style);
 }
 
 void
