@@ -1,7 +1,7 @@
 /* hostlist_fc.c   2004 Ian Schorr
  * modified from endpoint_talkers_fc.c   2003 Ronnie Sahlberg
  *
- * $Id: hostlist_fc.c 12128 2004-09-29 00:06:36Z guy $
+ * $Id: hostlist_fc.c 13993 2005-04-01 09:08:27Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -41,10 +41,10 @@
 
 
 static int
-fc_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, void *vip)
+fc_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, const void *vip)
 {
 	hostlist_table *hosts=(hostlist_table *)pit;
-	fc_hdr *fchdr=vip;
+	const fc_hdr *fchdr=vip;
 
 	/* Take two "add" passes per packet, adding for each direction, ensures that all
 	packets are counted properly (even if address is sending to itself) 
@@ -68,7 +68,7 @@ gtk_fc_hostlist_init(char *optarg)
 		filter=NULL;
 	}
 
-	init_hostlist_table(TRUE, "Fibre Channel Hosts", "fc", filter, (void *)fc_hostlist_packet);
+	init_hostlist_table(TRUE, "Fibre Channel Hosts", "fc", filter, fc_hostlist_packet);
 
 }
 
@@ -83,11 +83,10 @@ gtk_fc_hostlist_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_fc_hostlist(void)
 {
-	register_ethereal_tap("hosts,fc", gtk_fc_hostlist_init);
+	register_tap_listener_cmd_arg("hosts,fc", gtk_fc_hostlist_init);
 
 	register_tap_menu_item("Fibre Channel", REGISTER_TAP_GROUP_ENDPOINT_LIST,
 	    gtk_fc_hostlist_cb, NULL, NULL, NULL);
 
-	register_hostlist_table(TRUE, "Fibre Channel", "fc", NULL /*filter*/, (void *)fc_hostlist_packet);
+	register_hostlist_table(TRUE, "Fibre Channel", "fc", NULL /*filter*/, fc_hostlist_packet);
 }
-
