@@ -1,7 +1,7 @@
 /* hostlist_fddi.c   2004 Ian Schorr
  * modified from endpoint_talkers_fddi.c   2003 Ronnie Sahlberg
  *
- * $Id: hostlist_fddi.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: hostlist_fddi.c 13993 2005-04-01 09:08:27Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -83,7 +83,7 @@ gtk_fddi_hostlist_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_fddi_hostlist(void)
 {
-	register_ethereal_tap("hosts,fddi", gtk_fddi_hostlist_init);
+	register_tap_listener_cmd_arg("hosts,fddi", gtk_fddi_hostlist_init);
 
 	register_tap_menu_item("FDDI", REGISTER_TAP_GROUP_ENDPOINT_LIST,
 	    gtk_fddi_hostlist_cb, NULL, NULL, NULL);
