@@ -1,7 +1,7 @@
 /* io_stat.c
  * io_stat   2002 Ronnie Sahlberg
  *
- * $Id: io_stat.c 12921 2005-01-01 23:54:32Z guy $
+ * $Id: io_stat.c 14146 2005-04-20 08:58:49Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -52,6 +52,7 @@
 #include "compat_macros.h"
 #include "dlg_utils.h"
 #include "filter_dlg.h"
+#include "help_dlg.h"
 
 void protect_thread_critical_region(void);
 void unprotect_thread_critical_region(void);
@@ -1109,7 +1110,7 @@ gtk_iostat_init(char *optarg _U_)
 	/* build the GUI */
 	init_io_stat_window(io);
 
-	retap_packets(&cfile);
+	cf_retap_packets(&cfile);
 	io_stat_redraw(io);
 }
 
@@ -1269,7 +1270,7 @@ tick_interval_select(GtkWidget *item, gpointer key)
 	val=(int)OBJECT_GET_DATA(item, "tick_interval");
 
 	io->interval=val;
-	retap_packets(&cfile);
+	cf_retap_packets(&cfile);
 	io_stat_redraw(io);
 }
 
@@ -1633,7 +1634,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 	
 	io_stat_reset(gio->io);
 	enable_graph(gio, filter, field);
-	retap_packets(&cfile);
+	cf_retap_packets(&cfile);
 	io_stat_redraw(gio->io);
 
 	return 0;
@@ -1700,7 +1701,7 @@ static void
 create_advanced_field(io_stat_graph_t *gio, GtkWidget *box)
 {
 
-	gio->calc_field=gtk_entry_new_with_max_length(30);
+	gio->calc_field=gtk_entry_new_with_max_length(50);
 	gtk_box_pack_start(GTK_BOX(box), gio->calc_field, FALSE, FALSE, 0);
 	gtk_widget_show(gio->calc_field);
 	SIGNAL_CONNECT(gio->calc_field, "activate", filter_callback, gio);
@@ -1871,7 +1872,8 @@ init_io_stat_window(io_stat_t *io)
 {
 	GtkWidget *vbox;
 	GtkWidget *hbox;
-    GtkWidget *bt_close;
+	GtkWidget *bbox;
+    GtkWidget *close_bt, *help_bt;
 
 	/* create the main window */
 	io->window=window_new(GTK_WINDOW_TOPLEVEL, "I/O Graphs");
@@ -1893,12 +1895,21 @@ init_io_stat_window(io_stat_t *io)
 
 	io_stat_set_title(io);
 
-    hbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-    gtk_widget_show(hbox);
+    if(topic_available(HELP_STATS_IO_GRAPH_DIALOG)) {
+        bbox = dlg_button_row_new(GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+    } else {
+        bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    }
+	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
+    gtk_widget_show(bbox);
 
-    bt_close = OBJECT_GET_DATA(hbox, GTK_STOCK_CLOSE);
-    window_set_cancel_button(io->window, bt_close, window_cancel_button_cb);
+    close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    window_set_cancel_button(io->window, close_bt, window_cancel_button_cb);
+
+    if(topic_available(HELP_STATS_IO_GRAPH_DIALOG)) {
+        help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
+        SIGNAL_CONNECT(help_bt, "clicked", topic_cb, HELP_STATS_IO_GRAPH_DIALOG);
+    }
 
     SIGNAL_CONNECT(io->window, "delete_event", window_delete_event_cb, NULL);
 
@@ -1919,7 +1930,7 @@ gtk_iostat_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtk_iostat(void)
 {
-	register_ethereal_tap("io,stat", gtk_iostat_init);
+	register_tap_listener_cmd_arg("io,stat", gtk_iostat_init);
 
 	register_tap_menu_item("_IO Graphs", REGISTER_TAP_GROUP_GENERIC,
         gtk_iostat_cb, NULL, NULL, NULL);
