@@ -5,7 +5,7 @@
  *
  * MUCH code modified from service_response_time_table.c.
  *
- * $Id: isup_stat.c 13041 2005-01-14 20:27:34Z guy $
+ * $Id: isup_stat.c 14098 2005-04-16 10:53:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -398,7 +398,7 @@ register_tap_listener_gtkisup_stat(void)
     GString		*err_p;
 
 
-    register_ethereal_tap("isup,", isup_stat_gtk_init);
+    register_tap_listener_cmd_arg("isup,", isup_stat_gtk_init);
 
     memset((void *) &stat, 0, sizeof(isup_stat_t));
 
@@ -416,6 +416,6 @@ register_tap_listener_gtkisup_stat(void)
 	exit(1);
     }
 
-    register_tap_menu_item("ISUP Message Types", REGISTER_TAP_GROUP_NONE,
+    register_tap_menu_item("ISUP Message Types", REGISTER_TAP_GROUP_TELEPHONY,
         isup_stat_gtk_cb, NULL, NULL, NULL);
 }
