@@ -1,7 +1,7 @@
 /* menu.c
  * Menu routines
  *
- * $Id: menu.c 13091 2005-01-17 21:48:51Z ulfl $
+ * $Id: menu.c 14231 2005-04-29 14:51:52Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -46,6 +46,7 @@
 #include "filter_dlg.h"
 #include "dlg_utils.h"
 #include "file_dlg.h"
+#include "fileset_dlg.h"
 #include "find_dlg.h"
 #include "goto_dlg.h"
 #include "summary_dlg.h"
@@ -75,6 +76,7 @@
 #include "hostlist_table.h"
 #include "simple_dialog.h"
 #include "packet_history.h"
+#include "color_filters.h"
 
 GtkWidget *popup_menu_object;
 
@@ -117,6 +119,7 @@ static void name_resolution_transport_cb(GtkWidget *w _U_, gpointer d _U_);
 #ifdef HAVE_LIBPCAP
 static void auto_scroll_live_cb(GtkWidget *w _U_, gpointer d _U_);
 #endif
+static void colorize_cb(GtkWidget *w _U_, gpointer d _U_);
 
 /* This is the GtkItemFactoryEntry structure used to generate new menus.
        Item 1: The menu path. The letter after the underscore indicates an
@@ -177,11 +180,19 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_STOCK_ENTRY("/File/Save _As...", "<shift><control>S", file_save_as_cmd_cb,
                              0, GTK_STOCK_SAVE_AS),
     ITEM_FACTORY_ENTRY("/File/<separator>", NULL, NULL, 0, "<Separator>", NULL),
+    ITEM_FACTORY_ENTRY("/File/File Set", NULL, NULL, 0, "<Branch>", NULL),
+    ITEM_FACTORY_ENTRY("/File/File Set/List Files", NULL, fileset_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/File/File Set/Next File", NULL, fileset_next_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/File/File Set/Previous File", NULL, fileset_previous_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/File/<separator>", NULL, NULL, 0, "<Separator>", NULL),
     ITEM_FACTORY_ENTRY("/File/_Export", NULL, NULL, 0, "<Branch>", NULL),
     ITEM_FACTORY_ENTRY("/File/Export/as \"Plain _Text\" file...", NULL, export_text_cmd_cb,
                              0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/File/Export/as \"_PostScript\" file...", NULL, export_ps_cmd_cb,
                              0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/File/Export/as \"_CSV\" (Comma Separated Values packet summary) file...",
+                             NULL, export_csv_cmd_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/File/Export/<separator>", NULL, NULL, 0, "<Separator>", NULL),
     ITEM_FACTORY_ENTRY("/File/Export/as XML - \"P_SML\" (packet summary) file...", NULL, export_psml_cmd_cb,
                              0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/File/Export/as XML - \"P_DML\" (packet details) file...", NULL, export_pdml_cmd_cb,
@@ -212,10 +223,11 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_ENTRY("/Edit/Find Pre_vious", "<control>B", find_previous_cb, 0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Edit/<separator>", NULL, NULL, 0, "<Separator>", NULL),
     ITEM_FACTORY_ENTRY("/Edit/_Time Reference", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_ENTRY("/Edit/Time Reference/Set Time Reference (toggle)", "<control>T", reftime_frame_cb, REFTIME_TOGGLE, NULL, NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Edit/Time Reference/Set Time Reference (toggle)", "<control>T", reftime_frame_cb, 
+                        REFTIME_TOGGLE, ETHEREAL_STOCK_TIME),
     ITEM_FACTORY_ENTRY("/Edit/Time Reference/Find Next", NULL, reftime_frame_cb, REFTIME_FIND_NEXT, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Edit/Time Reference/Find Previous", NULL, reftime_frame_cb, REFTIME_FIND_PREV, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Edit/_Mark Packet", "<control>M", mark_frame_cb,
+    ITEM_FACTORY_ENTRY("/Edit/_Mark Packet (toggle)", "<control>M", mark_frame_cb,
                        0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Edit/Mark _All Packets", NULL, mark_all_frames_cb,
                        0, NULL, NULL),
@@ -248,6 +260,7 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_ENTRY("/View/Name Resolution/Enable for _MAC Layer", NULL, name_resolution_mac_cb, 0, "<CheckItem>", NULL),
     ITEM_FACTORY_ENTRY("/View/Name Resolution/Enable for _Network Layer", NULL, name_resolution_network_cb, 0, "<CheckItem>", NULL),
     ITEM_FACTORY_ENTRY("/View/Name Resolution/Enable for _Transport Layer", NULL, name_resolution_transport_cb, 0, "<CheckItem>", NULL),
+    ITEM_FACTORY_ENTRY("/View/Colorize Packet List", NULL, colorize_cb, 0, "<CheckItem>", NULL),
 #ifdef HAVE_LIBPCAP
     ITEM_FACTORY_ENTRY("/View/Auto Scroll in Li_ve Capture", NULL, auto_scroll_live_cb, 0, "<CheckItem>", NULL),
 #endif
@@ -258,6 +271,8 @@ static GtkItemFactoryEntry menu_items[] =
                              0, GTK_STOCK_ZOOM_OUT),
     ITEM_FACTORY_STOCK_ENTRY("/View/_Normal Size", "<control>equal", view_zoom_100_cb,
                              0, GTK_STOCK_ZOOM_100),
+    ITEM_FACTORY_STOCK_ENTRY("/View/Resize All Columns", NULL, packet_list_resize_columns_cb,
+                       0, ETHEREAL_STOCK_RESIZE_COLUMNS),
     ITEM_FACTORY_ENTRY("/View/<separator>", NULL, NULL, 0, "<Separator>", NULL),
     ITEM_FACTORY_ENTRY("/View/Collapse _All", NULL, collapse_all_cb,
                        0, NULL, NULL),
@@ -288,13 +303,17 @@ static GtkItemFactoryEntry menu_items[] =
                              goto_bottom_frame_cb, 0, GTK_STOCK_GOTO_BOTTOM),
 #ifdef HAVE_LIBPCAP
     ITEM_FACTORY_ENTRY("/_Capture", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_STOCK_ENTRY("/Capture/_Start...", "<control>K",
-                             capture_prep_cb, 0, ETHEREAL_STOCK_CAPTURE_START),
+    ITEM_FACTORY_STOCK_ENTRY("/Capture/_Interfaces...", NULL,
+                             capture_if_cb, 0, ETHEREAL_STOCK_CAPTURE_INTERFACES),
+    ITEM_FACTORY_STOCK_ENTRY("/Capture/_Options...", "<control>K",
+                             capture_prep_cb, 0, ETHEREAL_STOCK_CAPTURE_OPTIONS),
+    ITEM_FACTORY_STOCK_ENTRY("/Capture/_Start", NULL,
+                             capture_start_cb, 0, ETHEREAL_STOCK_CAPTURE_START),
     ITEM_FACTORY_STOCK_ENTRY("/Capture/S_top", "<control>E", capture_stop_cb,
-                             0, GTK_STOCK_STOP),
-    ITEM_FACTORY_ENTRY("/Capture/_Interfaces...", NULL,
-                             capture_if_cb, 0, NULL, NULL),
-    ITEM_FACTORY_STOCK_ENTRY("/Capture/_Capture Filters...", NULL, cfilter_dialog_cb,
+                             0, ETHEREAL_STOCK_CAPTURE_STOP),
+    ITEM_FACTORY_STOCK_ENTRY("/Capture/_Restart", NULL, capture_restart_cb,
+                             0, ETHEREAL_STOCK_CAPTURE_RESTART),
+    ITEM_FACTORY_STOCK_ENTRY("/Capture/Capture _Filters...", NULL, cfilter_dialog_cb,
                        0, ETHEREAL_STOCK_CAPTURE_FILTER),
 #endif /* HAVE_LIBPCAP */
     ITEM_FACTORY_ENTRY("/_Analyze", NULL, NULL, 0, "<Branch>", NULL),
@@ -327,48 +346,48 @@ static GtkItemFactoryEntry menu_items[] =
     ITEM_FACTORY_ENTRY("/Analyze/Prepare a Filter/... o_r not Selected", NULL, match_selected_ptree_cb, 
                        MATCH_SELECTED_OR_NOT, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Analyze/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/Analyze/_Enabled Protocols...", "<shift><control>R", proto_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Analyze/Decode _As...", NULL, decode_as_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Analyze/_User Specified Decodes...", NULL,
-                       decode_show_cb, 0, NULL, NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Analyze/_Enabled Protocols...", "<shift><control>R", proto_cb, 0, ETHEREAL_STOCK_CHECKBOX),
+    ITEM_FACTORY_STOCK_ENTRY("/Analyze/Decode _As...", NULL, decode_as_cb,
+                       0, ETHEREAL_STOCK_DECODE_AS),
+    ITEM_FACTORY_STOCK_ENTRY("/Analyze/_User Specified Decodes...", NULL,
+                       decode_show_cb, 0, ETHEREAL_STOCK_DECODE_AS),
     ITEM_FACTORY_ENTRY("/Analyze/<separator>", NULL, NULL, 0, "<Separator>", NULL),
     ITEM_FACTORY_ENTRY("/Analyze/_Follow TCP Stream", NULL,
                        follow_stream_cb, 0, NULL, NULL),    
     ITEM_FACTORY_ENTRY("/_Statistics", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_ENTRY("/Statistics/_Summary", NULL, summary_open_cb, 0, NULL, NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Statistics/_Summary", NULL, summary_open_cb, 0, GTK_STOCK_PROPERTIES),
     ITEM_FACTORY_ENTRY("/Statistics/_Protocol Hierarchy", NULL,
                        proto_hier_stats_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Statistics/Conversations", NULL,
-                       init_conversation_notebook_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Statistics/Endpoints", NULL,
-                       init_hostlist_notebook_cb, 0, NULL, NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Statistics/Conversations", NULL,
+                       init_conversation_notebook_cb, 0, ETHEREAL_STOCK_CONVERSATIONS),
+    ITEM_FACTORY_STOCK_ENTRY("/Statistics/Endpoints", NULL,
+                       init_hostlist_notebook_cb, 0, ETHEREAL_STOCK_ENDPOINTS),
     ITEM_FACTORY_ENTRY("/_Help", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_STOCK_ENTRY("/Help/_Contents", "F1", help_cb, 0, GTK_STOCK_HELP),
+    ITEM_FACTORY_STOCK_ENTRY("/Help/_Contents", "F1", topic_menu_cb, HELP_CONTENT, GTK_STOCK_HELP),
     ITEM_FACTORY_ENTRY("/Help/_Supported Protocols", NULL, supported_cb, 0, NULL, NULL),
 #if (GLIB_MAJOR_VERSION >= 2)
 #ifdef ETHEREAL_EUG_DIR
     ITEM_FACTORY_ENTRY("/Help/User's Guide", NULL, url_page_menu_cb, HELP_CONTENT, NULL, NULL),
 #endif
     ITEM_FACTORY_ENTRY("/Help/Manual Pages", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Ethereal", NULL, url_page_menu_cb, LOCALPAGE_MAN_ETHEREAL, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Ethereal Filter", NULL, url_page_menu_cb, LOCALPAGE_MAN_ETHEREAL_FILTER, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Ethereal", NULL, topic_menu_cb, LOCALPAGE_MAN_ETHEREAL, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Ethereal Filter", NULL, topic_menu_cb, LOCALPAGE_MAN_ETHEREAL_FILTER, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Help/Manual Pages/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Tethereal", NULL, url_page_menu_cb, LOCALPAGE_MAN_TETHEREAL, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Mergecap", NULL, url_page_menu_cb, LOCALPAGE_MAN_MERGECAP, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Editcap", NULL, url_page_menu_cb, LOCALPAGE_MAN_EDITCAP, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Text2pcap", NULL, url_page_menu_cb, LOCALPAGE_MAN_TEXT2PCAP, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Tethereal", NULL, topic_menu_cb, LOCALPAGE_MAN_TETHEREAL, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Mergecap", NULL, topic_menu_cb, LOCALPAGE_MAN_MERGECAP, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Editcap", NULL, topic_menu_cb, LOCALPAGE_MAN_EDITCAP, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Manual Pages/Text2pcap", NULL, topic_menu_cb, LOCALPAGE_MAN_TEXT2PCAP, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Help/Ethereal Online", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_STOCK_ENTRY("/Help/Ethereal Online/Home Page", NULL, url_page_menu_cb, ONLINEPAGE_HOME, GTK_STOCK_HOME),
-    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/Wiki", NULL, url_page_menu_cb, ONLINEPAGE_WIKI, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/User's Guide", NULL, url_page_menu_cb, ONLINEPAGE_USERGUIDE, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/FAQ's", NULL, url_page_menu_cb, ONLINEPAGE_FAQ, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/Downloads", NULL, url_page_menu_cb, ONLINEPAGE_DOWNLOAD, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/Example Files", NULL, url_page_menu_cb, ONLINEPAGE_SAMPLE_FILES, NULL, NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Help/Ethereal Online/Home Page", NULL, topic_menu_cb, ONLINEPAGE_HOME, GTK_STOCK_HOME),
+    ITEM_FACTORY_STOCK_ENTRY("/Help/Ethereal Online/Wiki", NULL, topic_menu_cb, ONLINEPAGE_WIKI, ETHEREAL_STOCK_WIKI),
+    ITEM_FACTORY_STOCK_ENTRY("/Help/Ethereal Online/User's Guide", NULL, topic_menu_cb, ONLINEPAGE_USERGUIDE, ETHEREAL_STOCK_WEB_SUPPORT),
+    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/FAQ's", NULL, topic_menu_cb, ONLINEPAGE_FAQ, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/Downloads", NULL, topic_menu_cb, ONLINEPAGE_DOWNLOAD, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Help/Ethereal Online/Example Files", NULL, topic_menu_cb, ONLINEPAGE_SAMPLE_FILES, NULL, NULL),
 #endif
     ITEM_FACTORY_ENTRY("/Help/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/Help/_About Ethereal", NULL, about_ethereal_cb,
-                       0, NULL, NULL)
+    ITEM_FACTORY_STOCK_ENTRY("/Help/_About Ethereal", NULL, about_ethereal_cb,
+                       0, ETHEREAL_STOCK_ABOUT)
 };
 
 
@@ -378,17 +397,14 @@ static int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
 /* packet list popup */
 static GtkItemFactoryEntry packet_list_menu_items[] =
 {
-    ITEM_FACTORY_ENTRY("/Follow TCP Stream", NULL, follow_stream_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Decode As...", NULL, decode_as_cb, 0, NULL, NULL),
-    ITEM_FACTORY_STOCK_ENTRY("/Display Filters...", NULL, dfilter_dialog_cb,
-                       0, ETHEREAL_STOCK_DISPLAY_FILTER),
-    ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/Mark Packet", NULL, mark_frame_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Mark Packet (toggle)", NULL, mark_frame_cb, 0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Time Reference", NULL, NULL, 0, "<Branch>", NULL),
-    ITEM_FACTORY_ENTRY("/Time Reference/Set Time Reference (toggle)", NULL, reftime_frame_cb, REFTIME_TOGGLE, NULL, NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Time Reference/Set Time Reference (toggle)", NULL, reftime_frame_cb, REFTIME_TOGGLE, ETHEREAL_STOCK_TIME),
     ITEM_FACTORY_ENTRY("/Time Reference/Find Next", NULL, reftime_frame_cb, REFTIME_FIND_NEXT, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Time Reference/Find Previous", NULL, reftime_frame_cb, REFTIME_FIND_PREV, NULL, NULL),
+
+    ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
+
     ITEM_FACTORY_ENTRY("/Apply as Filter", NULL, NULL, 0, "<Branch>", NULL),
     ITEM_FACTORY_ENTRY("/Apply as Filter/_Selected", NULL, match_selected_plist_cb, 
                        MATCH_SELECTED_REPLACE|MATCH_SELECTED_APPLY_NOW, NULL, NULL),
@@ -417,31 +433,25 @@ static GtkItemFactoryEntry packet_list_menu_items[] =
     ITEM_FACTORY_ENTRY("/Prepare a Filter/... o_r not Selected", NULL, match_selected_plist_cb,
                        MATCH_SELECTED_OR_NOT, NULL, NULL),
 
+    ITEM_FACTORY_ENTRY("/Follow TCP Stream", NULL, follow_stream_cb,
+                       0, NULL, NULL),
+
     ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/Coloring Rules...", NULL, color_display_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Print...", NULL, file_print_cmd_cb, 0, NULL, NULL),
+
+    ITEM_FACTORY_STOCK_ENTRY("/Decode As...", NULL, decode_as_cb, 0, ETHEREAL_STOCK_DECODE_AS),
+    ITEM_FACTORY_STOCK_ENTRY("/Print...", NULL, file_print_selected_cmd_cb, 0, GTK_STOCK_PRINT),
     ITEM_FACTORY_ENTRY("/Show Packet in New Window", NULL, new_window_cb,
-                       0, NULL, NULL),
+                       0, NULL, NULL)
 };
 
 static GtkItemFactoryEntry tree_view_menu_items[] =
 {
-/*    ITEM_FACTORY_STOCK_ENTRY("/Online Protocol Info", NULL, selected_ptree_info_cb,
-                       0, GTK_STOCK_DIALOG_QUESTION),
-    ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),*/
-    ITEM_FACTORY_ENTRY("/Follow TCP Stream", NULL, follow_stream_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Decode As...", NULL, decode_as_cb, 0, NULL, NULL),
-    ITEM_FACTORY_STOCK_ENTRY("/Display Filters...", NULL, dfilter_dialog_cb,
-                       0, ETHEREAL_STOCK_DISPLAY_FILTER),
+    ITEM_FACTORY_ENTRY("/Expand Subtrees", NULL, expand_tree_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Expand All", NULL, expand_all_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/Collapse All", NULL, collapse_all_cb, 0, NULL, NULL),
+
     ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/_Resolve Name", NULL, resolve_name_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/_Go to Corresponding Packet", NULL, goto_framenum_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/_Export Selected Packet Bytes...", NULL, savehex_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Protocol Preferences...", NULL, properties_cb,
-                       0, NULL, NULL),
+
     ITEM_FACTORY_ENTRY("/Apply as Filter", NULL, NULL, 0, "<Branch>", NULL),
     ITEM_FACTORY_ENTRY("/Apply as Filter/_Selected", NULL, match_selected_ptree_cb, 
                        MATCH_SELECTED_REPLACE|MATCH_SELECTED_APPLY_NOW, NULL, NULL),
@@ -469,23 +479,30 @@ static GtkItemFactoryEntry tree_view_menu_items[] =
                        MATCH_SELECTED_AND_NOT, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Prepare a Filter/... o_r not Selected", NULL, match_selected_ptree_cb,
                        MATCH_SELECTED_OR_NOT, NULL, NULL),
+
+    ITEM_FACTORY_ENTRY("/Follow TCP Stream", NULL, follow_stream_cb,
+                       0, NULL, NULL),
+
     ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
-    ITEM_FACTORY_ENTRY("/Collapse All", NULL, collapse_all_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Expand All", NULL, expand_all_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Expand Tree", NULL, expand_tree_cb, 0, NULL, NULL)
+
+    ITEM_FACTORY_STOCK_ENTRY("/Wiki Protocol Page", NULL, selected_ptree_info_cb,
+                       0, ETHEREAL_STOCK_WIKI),
+    ITEM_FACTORY_STOCK_ENTRY("/Filter Field Reference", NULL, selected_ptree_ref_cb,
+                       0, ETHEREAL_STOCK_INTERNET),
+    ITEM_FACTORY_ENTRY("/Protocol Preferences...", NULL, properties_cb,
+                       0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/<separator>", NULL, NULL, 0, "<Separator>", NULL),
+    ITEM_FACTORY_STOCK_ENTRY("/Decode As...", NULL, decode_as_cb, 0, ETHEREAL_STOCK_DECODE_AS),
+    ITEM_FACTORY_ENTRY("/_Resolve Name", NULL, resolve_name_cb, 0, NULL, NULL),
+    ITEM_FACTORY_ENTRY("/_Go to Corresponding Packet", NULL, goto_framenum_cb, 0, NULL, NULL),
 };
 
 static GtkItemFactoryEntry hexdump_menu_items[] =
 {
-    ITEM_FACTORY_ENTRY("/Follow TCP Stream", NULL, follow_stream_cb,
-                       0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Decode As...", NULL, decode_as_cb, 0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Display Filters...", NULL, dfilter_dialog_cb,
+    ITEM_FACTORY_ENTRY("/Copy", NULL, copy_hex_cb,
                        0, NULL, NULL),
     ITEM_FACTORY_ENTRY("/Export Selected Packet Bytes...", NULL, savehex_cb,
                        0, NULL, NULL),
-    ITEM_FACTORY_ENTRY("/Copy Packet Bytes into Clipboard", NULL, copy_hex_cb,
-                       0, NULL, NULL)
 };
 
 static int initialize = TRUE;
@@ -560,6 +577,7 @@ menus_init(void) {
     set_menus_for_captured_packets(FALSE);
     set_menus_for_selected_packet(&cfile);
     set_menus_for_selected_tree_row(&cfile);
+    set_menus_for_capture_in_progress(FALSE);
 
     /* init with an empty recent files list */
     clear_menu_recent_capture_file_cmd_cb(NULL, NULL);
@@ -657,6 +675,7 @@ register_tap_menu_item(
     case(REGISTER_TAP_GROUP_CONVERSATION_LIST): toolspath = "/Statistics/_Conversation List/"; break;
     case(REGISTER_TAP_GROUP_ENDPOINT_LIST): toolspath = "/Statistics/_Endpoint List/"; break;
     case(REGISTER_TAP_GROUP_RESPONSE_TIME): toolspath = "/Statistics/Service _Response Time/"; break;
+    case(REGISTER_TAP_GROUP_TELEPHONY): toolspath = "/Statistics/"; break;
     case(REGISTER_TAP_GROUP_NONE): toolspath = "/Statistics/"; break;
     default:
         g_assert(0);
@@ -759,6 +778,32 @@ guint merge_tap_menus_layered(GList *node, gint group) {
             entry = g_malloc0(sizeof (GtkItemFactoryEntry));
             entry->path = node_data->name;
             entry->callback = node_data->callback;
+#if GTK_MAJOR_VERSION >= 2
+            switch(group) {
+            case(REGISTER_TAP_GROUP_NONE):
+                break;
+            case(REGISTER_TAP_GROUP_GENERIC):
+                break;
+            case(REGISTER_TAP_GROUP_CONVERSATION_LIST):
+                entry->item_type = "<StockItem>";
+                entry->extra_data = ETHEREAL_STOCK_CONVERSATIONS;
+                break;
+            case(REGISTER_TAP_GROUP_ENDPOINT_LIST):
+                entry->item_type = "<StockItem>";
+                entry->extra_data = ETHEREAL_STOCK_ENDPOINTS;
+                break;
+            case(REGISTER_TAP_GROUP_RESPONSE_TIME):
+                entry->item_type = "<StockItem>";
+                entry->extra_data = ETHEREAL_STOCK_TIME;
+                break;
+            case(REGISTER_TAP_GROUP_TELEPHONY):
+                entry->item_type = "<StockItem>";
+                entry->extra_data = ETHEREAL_STOCK_TELEPHONY;
+                break;
+            default:
+                g_assert_not_reached();
+            }
+#endif
             gtk_item_factory_create_item(main_menu_factory, entry, node_data->callback_data, 2);
             set_menu_sensitivity(main_menu_factory, node_data->name, FALSE); /* no capture file yet */
             added++;
@@ -817,6 +862,9 @@ void merge_all_tap_menus(GList *node) {
     if (merge_tap_menus_layered(node, REGISTER_TAP_GROUP_RESPONSE_TIME)) {
         gtk_item_factory_create_item(main_menu_factory, entry, NULL, 2);
     }
+    if (merge_tap_menus_layered(node, REGISTER_TAP_GROUP_TELEPHONY)) {
+        gtk_item_factory_create_item(main_menu_factory, entry, NULL, 2);
+    }
     if (merge_tap_menus_layered(node, REGISTER_TAP_GROUP_NONE)) {
         /*gtk_item_factory_create_item(main_menu_factory, entry, NULL, 2);*/
     }
@@ -991,7 +1039,7 @@ menu_open_recent_file_cmd(GtkWidget *w)
 	gtk_label_get(GTK_LABEL(menu_item_child), &cf_name);
 
 	/* open and read the capture file (this will close an existing file) */
-	if ((err = cf_open(cf_name, FALSE, &cfile)) == 0) {
+	if (cf_open(&cfile, cf_name, FALSE, &err) == CF_OK) {
 		cf_read(&cfile);
 	} else {
 		/* the capture file isn't existing any longer, remove menu item */
@@ -1246,7 +1294,7 @@ timestamp_absolute_cb(GtkWidget *w _U_, gpointer d _U_)
     if (recent.gui_time_format != TS_ABSOLUTE) {
         set_timestamp_setting(TS_ABSOLUTE);
         recent.gui_time_format  = TS_ABSOLUTE;
-        change_time_formats(&cfile);
+        cf_change_time_formats(&cfile);
     }
 }
 
@@ -1256,7 +1304,7 @@ timestamp_absolute_date_cb(GtkWidget *w _U_, gpointer d _U_)
     if (recent.gui_time_format != TS_ABSOLUTE_WITH_DATE) {
         set_timestamp_setting(TS_ABSOLUTE_WITH_DATE);
         recent.gui_time_format  = TS_ABSOLUTE_WITH_DATE;
-        change_time_formats(&cfile);
+        cf_change_time_formats(&cfile);
     }
 }
 
@@ -1266,7 +1314,7 @@ timestamp_relative_cb(GtkWidget *w _U_, gpointer d _U_)
     if (recent.gui_time_format != TS_RELATIVE) {
         set_timestamp_setting(TS_RELATIVE);
         recent.gui_time_format  = TS_RELATIVE;
-        change_time_formats(&cfile);
+        cf_change_time_formats(&cfile);
     }
 }
 
@@ -1276,7 +1324,7 @@ timestamp_delta_cb(GtkWidget *w _U_, gpointer d _U_)
     if (recent.gui_time_format != TS_DELTA) {
         set_timestamp_setting(TS_DELTA);
         recent.gui_time_format  = TS_DELTA;
-        change_time_formats(&cfile);
+        cf_change_time_formats(&cfile);
     }
 }
 
@@ -1326,13 +1374,63 @@ name_resolution_transport_cb(GtkWidget *w _U_, gpointer d _U_)
 }
 
 #ifdef HAVE_LIBPCAP
+void
+menu_auto_scroll_live_changed(gboolean auto_scroll_live_in) {
+    GtkWidget *menu;
+
+
+    /* tell menu about it */
+    menu = gtk_item_factory_get_widget(main_menu_factory, "/View/Auto Scroll in Live Capture");
+    if( ((gboolean) GTK_CHECK_MENU_ITEM(menu)->active) != auto_scroll_live_in) {
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu), auto_scroll_live_in);
+    }
+
+    /* tell toolbar about it */
+    toolbar_auto_scroll_live_changed(auto_scroll_live_in);
+
+    /* change auto scroll */
+    if(auto_scroll_live_in != auto_scroll_live) {
+        auto_scroll_live  = auto_scroll_live_in;
+    }
+}
+
 static void 
 auto_scroll_live_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-    auto_scroll_live = GTK_CHECK_MENU_ITEM(w)->active;
+    menu_auto_scroll_live_changed(GTK_CHECK_MENU_ITEM(w)->active);
 }
 #endif
 
+
+void
+menu_colorize_changed(gboolean packet_list_colorize) {
+    GtkWidget *menu;
+
+
+    /* tell menu about it */
+    menu = gtk_item_factory_get_widget(main_menu_factory, "/View/Colorize Packet List");
+    if( ((gboolean) GTK_CHECK_MENU_ITEM(menu)->active) != packet_list_colorize) {
+        gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu), packet_list_colorize);
+    }
+
+    /* tell toolbar about it */
+    toolbar_colorize_changed(packet_list_colorize);
+
+    /* change colorization */
+    if(packet_list_colorize != recent.packet_list_colorize) {
+        recent.packet_list_colorize = packet_list_colorize;
+        color_filters_enable(packet_list_colorize);
+        cf_colorize_packets(&cfile);
+    }
+}
+
+static void 
+colorize_cb(GtkWidget *w, gpointer d _U_)
+{
+    menu_colorize_changed(GTK_CHECK_MENU_ITEM(w)->active);
+}
+
+
 /* the recent file read has finished, update the menu corresponding */
 void
 menu_recent_read_finished(void) {
@@ -1356,6 +1454,9 @@ menu_recent_read_finished(void) {
     menu = gtk_item_factory_get_widget(main_menu_factory, "/View/Packet Bytes");
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu), recent.byte_view_show);
 
+    menu = gtk_item_factory_get_widget(main_menu_factory, "/View/Colorize Packet List");
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu), recent.packet_list_colorize);
+
     menu_name_resolution_changed();
 
 #ifdef HAVE_LIBPCAP
@@ -1400,6 +1501,8 @@ menu_recent_read_finished(void) {
     default:
         g_assert_not_reached();
     }
+
+    menu_colorize_changed(recent.packet_list_colorize);
 }
 
 
@@ -1517,10 +1620,14 @@ set_menus_for_capture_in_progress(gboolean capture_in_progress)
   set_menu_sensitivity(main_menu_factory, "/File/Open Recent", 
       !capture_in_progress);
 #ifdef HAVE_LIBPCAP
-  set_menu_sensitivity(main_menu_factory, "/Capture/Start...",
+  set_menu_sensitivity(main_menu_factory, "/Capture/Options...",
+      !capture_in_progress);
+  set_menu_sensitivity(main_menu_factory, "/Capture/Start",
       !capture_in_progress);
   set_menu_sensitivity(main_menu_factory, "/Capture/Stop",
       capture_in_progress);
+  set_menu_sensitivity(main_menu_factory, "/Capture/Restart",
+      capture_in_progress);
   set_toolbar_for_capture_in_progress(capture_in_progress);
 
   set_capture_if_dialog_for_capture_in_progress(capture_in_progress);
@@ -1618,8 +1725,6 @@ set_menus_for_captured_packets(gboolean have_captured_packets)
       have_captured_packets);
   set_menu_sensitivity(main_menu_factory, "/View/Normal Size",
       have_captured_packets);
-  set_menu_sensitivity(packet_list_menu_factory, "/Coloring Rules...",
-      have_captured_packets);
   set_menu_sensitivity(main_menu_factory, "/Go/Go to Packet...",
       have_captured_packets);
   set_menu_sensitivity(main_menu_factory, "/Go/First Packet",
@@ -1698,9 +1803,9 @@ walk_menu_tree_for_selected_packet(GList *node, frame_data *fd,
 void
 set_menus_for_selected_packet(capture_file *cf)
 {
-  set_menu_sensitivity(main_menu_factory, "/Edit/Mark Packet",
+  set_menu_sensitivity(main_menu_factory, "/Edit/Mark Packet (toggle)",
       cf->current_frame != NULL);
-  set_menu_sensitivity(packet_list_menu_factory, "/Mark Packet",
+  set_menu_sensitivity(packet_list_menu_factory, "/Mark Packet (toggle)",
       cf->current_frame != NULL);
   set_menu_sensitivity(main_menu_factory, "/Edit/Time Reference",
       cf->current_frame != NULL);
@@ -1710,6 +1815,8 @@ set_menus_for_selected_packet(capture_file *cf)
       cf->current_frame != NULL);
   set_menu_sensitivity(main_menu_factory, "/Edit/Unmark All Packets",
       cf->current_frame != NULL);
+  set_menu_sensitivity(main_menu_factory, "/View/Resize All Columns",
+      cf->current_frame != NULL);
   set_menu_sensitivity(main_menu_factory, "/View/Collapse All",
       cf->current_frame != NULL);
   set_menu_sensitivity(tree_view_menu_factory, "/Collapse All",
@@ -1724,11 +1831,15 @@ set_menus_for_selected_packet(capture_file *cf)
       cf->current_frame != NULL);
   set_menu_sensitivity(main_menu_factory, "/Analyze/Follow TCP Stream",
       cf->current_frame != NULL ? (cf->edt->pi.ipproto == IP_PROTO_TCP) : FALSE);
-  set_menu_sensitivity(NULL, "/Follow TCP Stream",
+  set_menu_sensitivity(packet_list_menu_factory, "/Follow TCP Stream",
+      cf->current_frame != NULL ? (cf->edt->pi.ipproto == IP_PROTO_TCP) : FALSE);
+  set_menu_sensitivity(tree_view_menu_factory, "/Follow TCP Stream",
       cf->current_frame != NULL ? (cf->edt->pi.ipproto == IP_PROTO_TCP) : FALSE);
   set_menu_sensitivity(main_menu_factory, "/Analyze/Decode As...",
       cf->current_frame != NULL && decode_as_ok());
-  set_menu_sensitivity(NULL, "/Decode As...",
+  set_menu_sensitivity(packet_list_menu_factory, "/Decode As...",
+      cf->current_frame != NULL && decode_as_ok());
+  set_menu_sensitivity(tree_view_menu_factory, "/Decode As...",
       cf->current_frame != NULL && decode_as_ok());
   set_menu_sensitivity(main_menu_factory, "/View/Name Resolution/Resolve Name",
       cf->current_frame != NULL && (g_resolv_flags & RESOLV_ALL_ADDRS) != RESOLV_ALL_ADDRS);
@@ -1809,7 +1920,7 @@ set_menus_for_selected_tree_row(capture_file *cf)
 
   set_menu_sensitivity(main_menu_factory, "/File/Export/Selected Packet Bytes...", 
       cf->finfo_selected != NULL);  
-  set_menu_sensitivity(tree_view_menu_factory, "/Export Selected Packet Bytes...", 
+  set_menu_sensitivity(hexdump_menu_factory, "/Copy", 
       cf->finfo_selected != NULL);
   set_menu_sensitivity(hexdump_menu_factory, "/Export Selected Packet Bytes...", 
       cf->finfo_selected != NULL);
@@ -1836,9 +1947,10 @@ set_menus_for_selected_tree_row(capture_file *cf)
 	set_menu_sensitivity(tree_view_menu_factory, "/Protocol Preferences...",
 	  properties);
 	set_menu_sensitivity(main_menu_factory, "/View/Expand Tree", cf->finfo_selected->tree_type != -1);
-	set_menu_sensitivity(tree_view_menu_factory, "/Expand Tree", cf->finfo_selected->tree_type != -1);
-/*	set_menu_sensitivity(tree_view_menu_factory, "/Online Protocol Info",
-	  TRUE);*/
+	set_menu_sensitivity(tree_view_menu_factory, "/Wiki Protocol Page",
+	  TRUE);
+	set_menu_sensitivity(tree_view_menu_factory, "/Filter Field Reference",
+	  TRUE);
   } else {
 	set_menu_sensitivity(main_menu_factory,
 	    "/Go/Go to Corresponding Packet", FALSE);
@@ -1851,9 +1963,10 @@ set_menus_for_selected_tree_row(capture_file *cf)
 	set_menu_sensitivity(tree_view_menu_factory, "/Protocol Preferences...",
 	  FALSE);
 	set_menu_sensitivity(main_menu_factory, "/View/Expand Tree", FALSE);
-	set_menu_sensitivity(tree_view_menu_factory, "/Expand Tree", FALSE);
-/*	set_menu_sensitivity(tree_view_menu_factory, "/Online Protocol Info",
-	  FALSE);*/
+	set_menu_sensitivity(tree_view_menu_factory, "/Wiki Protocol Page",
+	  FALSE);
+	set_menu_sensitivity(tree_view_menu_factory, "/Filter Field Reference",
+	  FALSE);
   }
 
   walk_menu_tree_for_selected_tree_row(tap_menu_tree_root, cf->finfo_selected);
@@ -1867,3 +1980,10 @@ void set_menus_for_packet_history(gboolean back_history, gboolean forward_histor
   set_toolbar_for_packet_history(back_history, forward_history);
 }
 
+
+void set_menus_for_file_set(gboolean file_set, gboolean previous_file, gboolean next_file) {
+
+  set_menu_sensitivity(main_menu_factory, "/File/File Set/List Files", file_set);
+  set_menu_sensitivity(main_menu_factory, "/File/File Set/Previous File", previous_file);
+  set_menu_sensitivity(main_menu_factory, "/File/File Set/Next File", next_file);
+}
