@@ -5,7 +5,7 @@
  *
  * Modified from gsm_map_stat.c
  *
- * $Id: mtp3_stat.c 13041 2005-01-14 20:27:34Z guy $
+ * $Id: mtp3_stat.c 14098 2005-04-16 10:53:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -447,7 +447,7 @@ register_tap_listener_gtkmtp3_stat(void)
     GString		*err_p;
 
 
-    register_ethereal_tap("mtp3,", mtp3_stat_gtk_init);
+    register_tap_listener_cmd_arg("mtp3,", mtp3_stat_gtk_init);
 
     memset((void *) &mtp3_stat, 0, sizeof(mtp3_stat_t));
 
@@ -465,6 +465,6 @@ register_tap_listener_gtkmtp3_stat(void)
 	exit(1);
     }
 
-    register_tap_menu_item("MTP3/MSUs",  REGISTER_TAP_GROUP_NONE,
+    register_tap_menu_item("MTP3/MSUs",  REGISTER_TAP_GROUP_TELEPHONY,
         mtp3_stat_gtk_cb, NULL, NULL, NULL);
 }
