@@ -6,7 +6,7 @@
  *
  * Modified from gsm_map_summary.c
  *
- * $Id: mtp3_summary.c 13041 2005-01-14 20:27:34Z guy $
+ * $Id: mtp3_summary.c 14098 2005-04-16 10:53:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,6 +40,8 @@
 #include "epan/epan.h"
 #include "epan/value_string.h"
 #include "tap_menu.h"
+#include "globals.h"
+#include "file.h"
 #include "summary.h"
 #include "image/clist_ascend.xpm"
 #include "image/clist_descend.xpm"
@@ -251,7 +253,7 @@ mtp3_sum_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   int			i;
 
   /* initialize the tally */
-  summary_fill_in(&summary);
+  summary_fill_in(&cfile, &summary);
 
   /* initial compututations */
   seconds = summary.stop_time - summary.start_time;
@@ -433,6 +435,6 @@ mtp3_sum_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkmtp3_summary(void)
 {
-    register_tap_menu_item("MTP3/MSU Summary",  REGISTER_TAP_GROUP_NONE,
+    register_tap_menu_item("MTP3/MSU Summary",  REGISTER_TAP_GROUP_TELEPHONY,
         mtp3_sum_gtk_sum_cb, NULL, NULL, NULL);
 }
