@@ -1,7 +1,7 @@
 /* packet_list.c
  * packet list related functions   2002 Olivier Abad
  *
- * $Id: packet_list.c 13020 2005-01-13 17:31:33Z guy $
+ * $Id: packet_list.c 14082 2005-04-14 20:54:01Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -54,6 +54,11 @@
 #include "image/clist_ascend.xpm"
 #include "image/clist_descend.xpm"
 
+#include "progress_dlg.h"
+
+#define N_PROGBAR_UPDATES 100
+
+
 /*
  * XXX - gross hack.
  * This lets us use GtkCList in GTK+ 1.3[.x] and later, and EthCList on
@@ -263,7 +268,7 @@ packet_list_select_cb(GtkWidget *w _U_, gint row, gint col _U_, gpointer evt _U_
   while( (gtk_notebook_get_nth_page( GTK_NOTEBOOK(byte_nb_ptr), 0)))
     gtk_notebook_remove_page( GTK_NOTEBOOK(byte_nb_ptr), 0);
 
-  select_packet(&cfile, row);
+  cf_select_packet(&cfile, row);
   gtk_widget_grab_focus(packet_list);
   packet_history_add(row);
 }
@@ -271,7 +276,7 @@ packet_list_select_cb(GtkWidget *w _U_, gint row, gint col _U_, gpointer evt _U_
 static void
 packet_list_unselect_cb(GtkWidget *w _U_, gint row _U_, gint col _U_, gpointer evt _U_) {
 
-  unselect_packet(&cfile);
+  cf_unselect_packet(&cfile);
 }
 
 /* mark packets */
@@ -282,7 +287,7 @@ set_frame_mark(gboolean set, frame_data *frame, gint row) {
   if (row == -1)
     return;
   if (set) {
-    mark_frame(&cfile, frame);
+    cf_mark_frame(&cfile, frame);
     color_t_to_gdkcolor(&fg, &prefs.gui_marked_fg);
     color_t_to_gdkcolor(&bg, &prefs.gui_marked_bg);
     eth_clist_set_foreground(ETH_CLIST(packet_list), row, &fg);
@@ -290,7 +295,7 @@ set_frame_mark(gboolean set, frame_data *frame, gint row) {
   } else {
     color_filter_t *cfilter = frame->color_filter;
 
-    unmark_frame(&cfile, frame);
+    cf_unmark_frame(&cfile, frame);
     /* Restore the color from the matching color filter if any */
     if (cfilter) { /* The packet matches a color filter */
       color_t_to_gdkcolor(&fg, &cfilter->fg_color);
@@ -441,7 +446,7 @@ set_plist_sel_browse(gboolean val)
 	}
 
 	if (cfile.finfo_selected)
-		unselect_packet(&cfile);
+		cf_unselect_packet(&cfile);
 
         mode = new_mode;
         eth_clist_set_selection_mode(ETH_CLIST(packet_list), mode);
@@ -509,8 +514,8 @@ packet_list_new(e_prefs *prefs)
     SIGNAL_CONNECT(packet_list, "select-row", packet_list_select_cb, NULL);
     SIGNAL_CONNECT(packet_list, "unselect-row", packet_list_unselect_cb, NULL);
     for (i = 0; i < cfile.cinfo.num_cols; i++) {
-        /* Columns do not automatically resize, but are resizeable by
-           the user. */
+        /* For performance reasons, columns do not automatically resize, 
+           but are resizeable by the user. */
         eth_clist_set_column_auto_resize(ETH_CLIST(packet_list), i, FALSE);
         eth_clist_set_column_resizeable(ETH_CLIST(packet_list), i, TRUE);
 
@@ -590,11 +595,82 @@ packet_list_freeze(void)
     eth_clist_freeze(ETH_CLIST(packet_list));
 }
 
+void
+packet_list_resize_columns(void) {
+    int         i;
+    int         progbar_nextstep;
+    int         progbar_quantum;
+    gboolean    stop_flag;
+    GTimeVal    start_time;
+    float       prog_val;
+    progdlg_t  *progbar = NULL;
+    gchar       status_str[100];
+
+
+    progbar_nextstep = 0;
+    /* When we reach the value that triggers a progress bar update,
+       bump that value by this amount. */
+    progbar_quantum = cfile.cinfo.num_cols/N_PROGBAR_UPDATES;
+
+    stop_flag = FALSE;
+    g_get_current_time(&start_time);
+
+
+    main_window_update();
+
+    for (i = 0; i < cfile.cinfo.num_cols; i++) {
+      if (i >= progbar_nextstep) {
+        /* let's not divide by zero. I should never be started
+         * with count == 0, so let's assert that
+         */
+        g_assert(cfile.cinfo.num_cols > 0);
+
+        prog_val = (gfloat) i / cfile.cinfo.num_cols;
+
+        /* Create the progress bar if necessary */
+        if (progbar == NULL)
+           progbar = delayed_create_progress_dlg("Resizing", "Resize Columns", 
+             &stop_flag, &start_time, prog_val);
+
+        if (progbar != NULL) {
+          g_snprintf(status_str, sizeof(status_str),
+                     "%u of %u columns (%s)", i+1, cfile.cinfo.num_cols, cfile.cinfo.col_title[i]);
+          update_progress_dlg(progbar, prog_val, status_str);
+        }
+
+        progbar_nextstep += progbar_quantum;
+      }
+
+      if (stop_flag) {
+        /* Well, the user decided to abort the resizing... */
+        break;
+      }
+
+        /* auto resize the current column */
+        eth_clist_set_column_auto_resize(ETH_CLIST(packet_list), i, TRUE);
+
+        /* the current column should be resizeable by the user again */
+        /* (will turn off auto resize again) */
+        eth_clist_set_column_resizeable(ETH_CLIST(packet_list), i, TRUE);
+    }
+
+    /* We're done resizing the columns; destroy the progress bar if it
+       was created. */
+    if (progbar != NULL)
+      destroy_progress_dlg(progbar);
+}
+
+void packet_list_resize_columns_cb(GtkWidget *widget _U_, gpointer data _U_)
+{
+    packet_list_resize_columns();
+}
+
 void
 packet_list_thaw(void)
 {
     eth_clist_thaw(ETH_CLIST(packet_list));
     packets_bar_update();
+    /*packet_list_resize_columns();*/
 }
 
 void
