@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: proto_draw.c 14246 2005-04-30 17:23:33Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -235,7 +235,7 @@ redraw_hex_dump_all(void)
      The only workaround is to freshly select the frame, which will remove any 
      existing notebook tabs and "restart" the whole byte view again. */
   if (cfile.current_frame != NULL)
-    goto_frame(&cfile, cfile.current_frame->num);
+    cf_goto_frame(&cfile, cfile.current_frame->num);
 #endif
 }
 
@@ -1642,6 +1642,64 @@ void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
 #endif
 
 
+#if GTK_MAJOR_VERSION >= 2
+static int
+tree_view_key_pressed_cb(GtkCTree *ctree _U_, GdkEventKey *event, gpointer user_data _U_)
+{
+    GtkTreeSelection* selection;
+    GtkTreeIter iter;
+    GtkTreeIter parent;
+    GtkTreeModel* model;
+    GtkTreePath* path;
+    gboolean    expanded;
+    gboolean    has_parent;
+
+
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
+    if(!selection) {
+        return FALSE;
+    }
+
+    if(!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+        return FALSE;
+    }
+
+    path = gtk_tree_model_get_path(model, &iter);
+    if(!path) {
+        return FALSE;
+    }
+
+    if (event->keyval == GDK_Left) {
+        expanded = gtk_tree_view_row_expanded(GTK_TREE_VIEW(tree_view), path);
+        if(expanded) {
+            /* subtree is expanded, collapse it */
+            gtk_tree_view_collapse_row(GTK_TREE_VIEW(tree_view), path);
+            return TRUE;
+        } else {
+            /* subtree is already collapsed, jump to parent node */
+            has_parent = gtk_tree_model_iter_parent(model, &parent, &iter);
+            path = gtk_tree_model_get_path(model, &parent);
+            if(!path) {
+                return FALSE;
+            }
+            gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree_view), path,
+                                             NULL /* focus_column */,
+                                             FALSE /* !start_editing */);
+            return TRUE;
+        }
+    }
+    if (event->keyval == GDK_Right) {
+        /* try to expand the subtree */
+        gtk_tree_view_expand_row(GTK_TREE_VIEW(tree_view), path, FALSE /* !open_all */);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+#endif
+
+
+
 GtkWidget *
 main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
 {
@@ -1690,6 +1748,7 @@ main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
                                   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   SIGNAL_CONNECT(tree_view, "row-expanded", expand_tree, NULL);
   SIGNAL_CONNECT(tree_view, "row-collapsed", collapse_tree, NULL);
+  SIGNAL_CONNECT(tree_view, "key-press-event", tree_view_key_pressed_cb, NULL );
 #endif
   gtk_container_add( GTK_CONTAINER(tv_scrollw), tree_view );
   set_ptree_sel_browse(tree_view, prefs->gui_ptree_sel_browse);
@@ -1768,7 +1827,7 @@ void
 tree_view_follow_link(field_info   *fi)
 {
     if(fi->hfinfo->type == FT_FRAMENUM) {
-        goto_frame(&cfile, fi->value.value.integer);
+        cf_goto_frame(&cfile, fi->value.value.integer);
     }
 }
 
