@@ -2,7 +2,7 @@
  * Definitions for recent "preference" handling routines
  * Copyright 2004, Ulf Lamping <ulf.lamping@web.de>
  *
- * $Id: recent.h,v 1.11 2004/06/04 17:16:58 ulfl Exp $
+ * $Id: recent.h 13929 2005-03-27 02:19:51Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -47,6 +47,7 @@ typedef struct recent_settings_tag {
     gboolean    tree_view_show;
     gboolean    byte_view_show;
     gboolean    statusbar_show;
+    gboolean    packet_list_colorize;
     gint        gui_time_format;
     gint        gui_zoom_level;
 
@@ -67,10 +68,9 @@ extern recent_settings_t recent;
 
 /** Write recent settings file.
  *
- * @param rf_path_return path to recent file if function failed
- * @return 0 if succeeded, errno if failed
+ * @return TRUE if succeeded, FALSE if failed
  */
-extern int write_recent(char **rf_path_return);
+extern gboolean write_recent(void);
 
 /** Read recent settings file.
  *
