@@ -1,7 +1,7 @@
 /* rpc_progs.c
  * rpc_progs   2002 Ronnie Sahlberg
  *
- * $Id: rpc_progs.c 12913 2005-01-01 12:49:54Z guy $
+ * $Id: rpc_progs.c 13993 2005-04-01 09:08:27Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -406,7 +406,7 @@ gtk_rpcprogs_init(char *optarg _U_)
 	gtk_widget_show_all(win);
 	window_present(win);
 	
-	redissect_packets(&cfile);
+	cf_redissect_packets(&cfile);
 }
 
 static void
@@ -418,7 +418,7 @@ gtk_rpcprogs_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkrpcprogs(void)
 {
-	register_ethereal_tap("rpc,programs", gtk_rpcprogs_init);
+	register_tap_listener_cmd_arg("rpc,programs", gtk_rpcprogs_init);
 
 	register_tap_menu_item("ONC-RPC Programs", REGISTER_TAP_GROUP_NONE,
 	gtk_rpcprogs_cb, NULL, NULL, NULL);
