@@ -1,7 +1,7 @@
 /* sip_stat.c
  * sip_stat   2004 Martin Mathieson
  *
- * $Id: sip_stat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: sip_stat.c 14098 2005-04-16 10:53:09Z ulfl $
  * Copied from http_stat.c
  *
  * Ethereal - Network traffic analyzer
@@ -648,7 +648,7 @@ gtk_sipstat_init(char *optarg)
     window_present(sp->win);
 
     sip_init_hash(sp);
-    retap_packets(&cfile);
+    cf_retap_packets(&cfile);
 }
 
 static tap_dfilter_dlg sip_stat_dlg = {
@@ -662,8 +662,8 @@ static tap_dfilter_dlg sip_stat_dlg = {
 void
 register_tap_listener_gtksipstat(void)
 {
-    register_ethereal_tap("sip,stat", gtk_sipstat_init);
+    register_tap_listener_cmd_arg("sip,stat", gtk_sipstat_init);
 
-    register_tap_menu_item("SIP", REGISTER_TAP_GROUP_NONE,
+    register_tap_menu_item("SIP", REGISTER_TAP_GROUP_TELEPHONY,
                            gtk_tap_dfilter_dlg_cb, NULL, NULL, &(sip_stat_dlg));
 }
