@@ -2,7 +2,7 @@
  * Definitions for toolbar utility routines
  * Copyright 2003, Ulf Lamping <ulf.lamping@web.de>
  *
- * $Id: toolbar.h 12460 2004-11-01 11:42:08Z ulfl $
+ * $Id: toolbar.h 14098 2005-04-16 10:53:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -39,6 +39,16 @@ GtkWidget *toolbar_new(void);
 /** Redraw the main toolbar. Used, when user changes preferences. */
 void toolbar_redraw_all(void);
 
+/** The "Colorize Packet List" option has changed.
+ */
+void toolbar_colorize_changed(gboolean packet_list_colorize);
+
+#ifdef HAVE_LIBPCAP
+/** The "Auto Scroll in Live Capture" option has changed.
+ */
+void toolbar_auto_scroll_live_changed(gboolean auto_scroll_live);
+#endif
+
 /** We have (or don't have) a capture file now.
  *
  * @param have_capture_file TRUE, if we have a capture file
