@@ -1,7 +1,7 @@
 /* ui_util.c
  * UI utility routines
  *
- * $Id: ui_util.c 12918 2005-01-01 23:25:49Z guy $
+ * $Id: ui_util.c 13809 2005-03-19 16:36:23Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -167,7 +167,7 @@ window_new_with_geom(GtkWindowType type, const gchar *title, const gchar *geom_n
   window_geometry_t geom;
   GtkWidget *win = window_new(type, title);
 
-  OBJECT_SET_DATA(win, WINDOW_GEOM_KEY, (gpointer)geom_name);
+  OBJECT_SET_DATA(win, WINDOW_GEOM_KEY, (gpointer)g_strdup(geom_name));
 
   /* do we have a previously saved size and position of this window? */
   if(geom_name) {
@@ -527,6 +527,7 @@ window_destroy(GtkWidget *win)
       name = OBJECT_GET_DATA(win, WINDOW_GEOM_KEY);
       if(name) {
         window_geom_save(name, &geom);
+        g_free((gpointer)name);
       }
   }
 
@@ -580,14 +581,14 @@ set_main_window_name(gchar *window_name)
 }
 
 
-#ifdef HAVE_LIBPCAP
-
 /* update the main window */
 void main_window_update(void)
 {
   while (gtk_events_pending()) gtk_main_iteration();
 }
 
+#ifdef HAVE_LIBPCAP
+
 /* exit the main window */
 void main_window_exit(void)
 {
