@@ -1,7 +1,7 @@
 /* wsp_stat.c
  * wsp_stat   2003 Jean-Michel FAYARD
  *
- * $Id: wsp_stat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: wsp_stat.c 14098 2005-04-16 10:53:09Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -412,7 +412,7 @@ gtk_wspstat_init(char *optarg)
     gtk_widget_show_all(sp->win);
     window_present(sp->win);
 	
-    retap_packets(&cfile);
+    cf_retap_packets(&cfile);
 }
 
 static tap_dfilter_dlg wsp_stat_dlg = {
@@ -425,8 +425,8 @@ static tap_dfilter_dlg wsp_stat_dlg = {
 void
 register_tap_listener_gtkwspstat(void)
 {
-	register_ethereal_tap("wsp,stat", gtk_wspstat_init);
+	register_tap_listener_cmd_arg("wsp,stat", gtk_wspstat_init);
 
-	register_tap_menu_item("WAP-WSP...", REGISTER_TAP_GROUP_NONE,
+	register_tap_menu_item("WAP-WSP...", REGISTER_TAP_GROUP_TELEPHONY,
 	    gtk_tap_dfilter_dlg_cb, NULL, NULL, &(wsp_stat_dlg));
 }
