@@ -2,7 +2,7 @@
  * Routines for Ranging Request Message dissection
  * Copyright 2002, Anand V. Narwani <anand[AT]narwani.org>
  *
- * $Id: packet-rngreq.c 11400 2004-07-18 00:24:25Z guy $
+ * $Id: packet-rngreq.c 13596 2005-03-05 06:28:10Z lroland $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,9 +27,6 @@
 #include "config.h"
 #endif
 
-
-#include "plugins/plugin_api.h"
-#include "plugins/plugin_api_defs.h"
 #include "moduleinfo.h"
 
 #include <stdio.h>
