@@ -3,7 +3,7 @@
 *
 * Copyright 2004, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
 *
-* $Id: mate_setup.c 13001 2005-01-10 02:37:19Z lroland $
+* $Id: mate_setup.c 13464 2005-02-22 02:16:35Z lego $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
@@ -124,46 +124,52 @@ static mate_cfg_item* new_mate_cfg_item(guint8* name) {
 	new->type = MATE_UNK_TYPE;
 	new->transforms = g_ptr_array_new();
 	new->extra = new_avpl(name);
-	new->hfid_proto = -1;
-	new->discard_pdu_attributes = matecfg->discard_pdu_attributes;
-	new->last_to_be_created = matecfg->last_to_be_created;
-	new->drop_pdu = matecfg->drop_pdu;
-	new->drop_gop = matecfg->drop_gop;
-	new->expiration = matecfg->gog_expiration;
-	new->show_pdu_tree = matecfg->show_pdu_tree;
-	new->show_times = matecfg->show_times;
 	new->last_id = 0;
-	new->hfid_ranges = NULL;
-	new->hfids_attr = NULL;
-	new->criterium = NULL;
-	new->start = NULL;
-	new->stop = NULL;
-	new->key = NULL;
-	new->keys = NULL;
-	new->gop_as_subtree = FALSE;
-
 	new->hfid = -1;
-	new->hfid_pdu_rel_time = -1;
-	new->hfid_pdu_time_in_gop = -1;
 	new->my_hfids = g_hash_table_new(g_str_hash,g_str_equal);
 	new->items = g_hash_table_new(g_direct_hash,g_direct_equal);
-
-	new->hfid_gop_pdu = -1;
-	new->hfid_start_time = -1;
-	new->hfid_stop_time = -1;
-	new->hfid_last_time = -1;
-	new->hfid_gop_num_pdus = -1;
-
-	new->hfid_gog_num_of_gops = -1;
-	new->hfid_gog_gop = -1;
-
 	new->ett = -1;
 	new->ett_attr = -1;
 	new->ett_times = -1;
 	new->ett_children = -1;
+	
+	new->discard_pdu_attributes = matecfg->discard_pdu_attributes;
+	new->last_to_be_created = matecfg->last_to_be_created;
+	new->hfid_proto = -1;
+	new->transport_ranges = NULL;
+	new->payload_ranges = NULL;
+	new->hfids_attr = NULL;
+	new->drop_pdu = matecfg->drop_pdu;
+	new->criterium_match_mode = AVPL_NO_MATCH;
+	new->criterium = NULL;
+	new->hfid_pdu_rel_time = -1;
+	new->hfid_pdu_time_in_gop = -1;
 
+	new->expiration = -1.0;
+	new->hfid_start_time = -1;
+	new->hfid_stop_time = -1;
+	new->hfid_last_time = -1;
+	
+	new->start = NULL;
+	new->stop = NULL;
+	new->key = NULL;
+	new->show_pdu_tree = matecfg->show_pdu_tree;
+	new->show_times = matecfg->show_times;
+	new->drop_gop = matecfg->drop_gop;
+	new->idle_timeout = -1.0;
+	new->lifetime = -1.0;
+	new->hfid_gop_pdu = -1;
+	new->hfid_gop_num_pdus = -1;
+	new->ett_gog_gop = -1;
+	new->hfid_gog_gopstart = -1;
+	
 	new->gop_index = NULL;
 	new->gog_index = NULL;
+
+	new->gop_as_subtree = NULL;
+	new->keys = NULL;
+	new->hfid_gog_num_of_gops = -1;
+	new->hfid_gog_gop = -1;
 	
 	return new;
 }
@@ -190,8 +196,11 @@ static void delete_mate_cfg_item(mate_cfg_item* cfg, gboolean avp_items_too) {
 
 	if (cfg->transforms) g_ptr_array_free(cfg->transforms,TRUE);
 
-	if (cfg->hfid_ranges)
-		g_ptr_array_free(cfg->hfid_ranges,TRUE);
+	if (cfg->transport_ranges)
+		g_ptr_array_free(cfg->transport_ranges,TRUE);
+	
+	if (cfg->payload_ranges)
+		g_ptr_array_free(cfg->payload_ranges,TRUE);
 
 	if (cfg->hfids_attr)
 		g_hash_table_foreach_remove(cfg->hfids_attr,free_both, VALUE_TOO );
@@ -202,7 +211,8 @@ static mate_cfg_pdu* new_pducfg(guint8* name) {
 	mate_cfg_pdu* new = new_mate_cfg_item(name);
 
 	new->type = MATE_PDU_TYPE;
-	new->hfid_ranges = g_ptr_array_new();
+	new->transport_ranges = g_ptr_array_new();
+	
 	new->hfids_attr = g_hash_table_new(g_int_hash,g_int_equal);
 
 	g_ptr_array_add(matecfg->pducfglist,(gpointer) new);
@@ -214,8 +224,15 @@ static mate_cfg_pdu* new_pducfg(guint8* name) {
 
 static mate_cfg_gop* new_gopcfg(guint8* name) {
 	mate_cfg_gop* new = new_mate_cfg_item(name);
+	
 	new->type = MATE_GOP_TYPE;
-
+	new->expiration = matecfg->gop_expiration;
+	new->idle_timeout = matecfg->gop_idle_timeout;
+	new->lifetime = matecfg->gop_lifetime;
+	new->show_pdu_tree = matecfg->show_pdu_tree;
+	new->show_times = matecfg->show_times;
+	new->drop_gop = matecfg->drop_gop;
+	
 	g_hash_table_insert(matecfg->gopcfgs,(gpointer) new->name, (gpointer) new);
 
 	new->gop_index = g_hash_table_new(g_str_hash,g_str_equal);
@@ -230,7 +247,7 @@ static mate_cfg_gog* new_gogcfg(guint8* name) {
 
 	new->keys = new_loal(name);
 	new->expiration = matecfg->gog_expiration;
-
+	
 	g_hash_table_insert(matecfg->gogcfgs,new->name,new);
 
 	return new;
@@ -356,15 +373,43 @@ static gboolean add_hfid(guint8* what, guint8* how, GHashTable* where) {
 	return exists;
 }
 
+static guint8* add_ranges(guint8* range,GPtrArray* range_ptr_arr) {
+	gchar**  ranges;
+	guint i;
+	header_field_info* hfi;
+	int* hfidp;
+
+	ranges = g_strsplit(range,"/",0);
+	
+	if (ranges) {
+		for (i=0; ranges[i]; i++) {
+			hfi = proto_registrar_get_byname(ranges[i]);
+			if (hfi) {
+				hfidp = g_malloc(sizeof(int));
+				*hfidp = hfi->id;
+				g_ptr_array_add(range_ptr_arr,(gpointer)hfidp);
+				g_string_sprintfa(matecfg->mate_attrs_filter, "||%s",ranges[i]);
+			} else {
+				g_strfreev(ranges);
+				return g_strdup_printf("no such proto: '%s'",ranges[i]);;
+			}
+		}
+		
+		g_strfreev(ranges);
+	}
+	
+	return NULL;
+}
+
+
 static gboolean config_pdu(AVPL* avpl) {
 	guint8* name = NULL;
 	guint8* transport = extract_named_str(avpl,KEYWORD_TRANSPORT,NULL);
+	guint8* payload = extract_named_str(avpl,KEYWORD_PAYLOAD,NULL);
 	guint8* proto = extract_named_str(avpl,KEYWORD_PROTO,"no_protocol");
 	mate_cfg_pdu* cfg = lookup_using_index_avp(avpl,KEYWORD_NAME,matecfg->pducfgs,&name);
 	header_field_info* hfi;
-	int* hfidp;
-	gchar**  transports;
-	guint i;
+	guint8* range_err;
 	AVP* attr_avp;
 
 	if (! name ) {
@@ -395,31 +440,25 @@ static gboolean config_pdu(AVPL* avpl) {
 	g_string_sprintfa(matecfg->mate_protos_filter,"||%s",proto);
 
 	if ( transport ) {
-
-		transports = g_strsplit(transport,"/",0);
-
-		if (transports) {
-			for (i=0; transports[i]; i++) {
-				hfi = proto_registrar_get_byname(transports[i]);
-				if (hfi) {
-					hfidp = g_malloc(sizeof(int));
-					*hfidp = hfi->id;
-					g_ptr_array_add(cfg->hfid_ranges,(gpointer)hfidp);
-					g_string_sprintfa(matecfg->mate_attrs_filter, "||%s",transports[i]);
-				} else {
-					report_error("MATE: PduDef: no such proto: '%s' for Transport in: %s",proto,avpl->name);
-					g_strfreev(transports);
-					return FALSE;
-				}
-			}
-
-			g_strfreev(transports);
+		if (( range_err = add_ranges(transport,cfg->transport_ranges) )) {
+			report_error("MATE: PduDef: %s in Transport for '%s' in: %s",range_err, cfg->name,avpl->name);
+			g_free(range_err);
+			return FALSE;			
 		}
 	} else {
 		report_error("MATE: PduDef: no Transport for '%s' in: %s",cfg->name,avpl->name);
 		return FALSE;
 	}
 
+	if ( payload ) {
+		cfg->payload_ranges = g_ptr_array_new();
+		if (( range_err = add_ranges(payload,cfg->payload_ranges) )) {
+			report_error("MATE: PduDef: %s in Payload for '%s' in: %s",range_err, cfg->name,avpl->name);
+			g_free(range_err);
+			return FALSE;			
+		}
+	}
+	
 	while (( attr_avp = extract_first_avp(avpl) )) {
 		if ( ! add_hfid(attr_avp->v,attr_avp->n,cfg->hfids_attr) ) {
 			report_error("MATE: PduDef: failed to set PDU attribute '%s' in: %s",attr_avp->n,avpl->name);
@@ -569,6 +608,9 @@ static gboolean config_settings(AVPL*avpl) {
 
 
 	matecfg->gog_expiration = extract_named_float(avpl, KEYWORD_GOGEXPIRE,matecfg->gog_expiration);
+	matecfg->gop_expiration = extract_named_float(avpl, KEYWORD_GOPEXPIRATION,matecfg->gop_expiration);
+	matecfg->gop_idle_timeout = extract_named_float(avpl, KEYWORD_GOPIDLETIMEOUT,matecfg->gop_idle_timeout);
+	matecfg->gop_lifetime = extract_named_float(avpl, KEYWORD_GOPLIFETIME,matecfg->gop_lifetime);
 	matecfg->discard_pdu_attributes = extract_named_bool(avpl, KEYWORD_DISCARDPDU,matecfg->discard_pdu_attributes);
 	matecfg->drop_pdu = extract_named_bool(avpl, KEYWORD_DROPPDU,matecfg->drop_pdu);
 	matecfg->drop_gop = extract_named_bool(avpl, KEYWORD_DROPGOP,matecfg->drop_gop);
@@ -583,7 +625,7 @@ static gboolean config_settings(AVPL*avpl) {
 
 	matecfg->dbg_lvl = extract_named_int(avpl, KEYWORD_DBG_GENERAL,0);
 	matecfg->dbg_cfg_lvl = extract_named_int(avpl, KEYWORD_DBG_CFG,0);
-	matecfg->dbg_gop_lvl = extract_named_int(avpl, KEYWORD_DBG_PDU,0);
+	matecfg->dbg_pdu_lvl = extract_named_int(avpl, KEYWORD_DBG_PDU,0);
 	matecfg->dbg_gop_lvl = extract_named_int(avpl, KEYWORD_DBG_GOP,0);
 	matecfg->dbg_gog_lvl = extract_named_int(avpl, KEYWORD_DBG_GOG,0);
 
@@ -720,6 +762,11 @@ static gboolean config_gop(AVPL* avpl) {
 		report_error("MATE: GopDef: no On in: %s",avpl->name);
 		return FALSE;
 	}
+	
+	if (g_hash_table_lookup(matecfg->pducfgs,on) == NULL ) {
+		report_error("MATE: GopDef: Pdu '%s' does not exist in: %s",on,avpl->name);
+		return FALSE;		
+	}
 
 	if (g_hash_table_lookup(matecfg->gops_by_pduname,on) ) {
 		report_error("MATE: GopDef: Gop for Pdu '%s' exists already in: %s",on,avpl->name);
@@ -731,7 +778,10 @@ static gboolean config_gop(AVPL* avpl) {
 	cfg->drop_gop = extract_named_bool(avpl, KEYWORD_DROPGOP,matecfg->drop_gop);
 	cfg->show_pdu_tree = extract_named_str(avpl, KEYWORD_SHOWPDUTREE, matecfg->show_pdu_tree);
 	cfg->show_times = extract_named_bool(avpl, KEYWORD_SHOWGOPTIMES,matecfg->show_times);
-
+	cfg->expiration = extract_named_float(avpl, KEYWORD_GOPEXPIRATION,matecfg->gop_expiration);
+	cfg->idle_timeout = extract_named_float(avpl, KEYWORD_GOPIDLETIMEOUT,matecfg->gop_idle_timeout);
+	cfg->lifetime = extract_named_float(avpl, KEYWORD_GOPLIFETIME,matecfg->gop_lifetime);
+	
 	cfg->key = avpl;
 
 	return TRUE;
@@ -803,7 +853,10 @@ static gboolean config_gopextra(AVPL* avpl) {
 	cfg->drop_gop = extract_named_bool(avpl, KEYWORD_DROPGOP,cfg->drop_gop);
 	cfg->show_pdu_tree = extract_named_str(avpl, KEYWORD_SHOWPDUTREE, cfg->show_pdu_tree);
 	cfg->show_times = extract_named_bool(avpl, KEYWORD_SHOWGOPTIMES,cfg->show_times);
-
+	cfg->expiration = extract_named_float(avpl, KEYWORD_GOPEXPIRATION,cfg->expiration);
+	cfg->idle_timeout = extract_named_float(avpl, KEYWORD_GOPIDLETIMEOUT,cfg->idle_timeout);
+	cfg->lifetime = extract_named_float(avpl, KEYWORD_GOPLIFETIME,cfg->lifetime);
+	
 	merge_avpl(cfg->extra,avpl,TRUE);
 
 	return TRUE;
@@ -826,7 +879,7 @@ static gboolean config_gog(AVPL* avpl) {
 	cfg = new_gogcfg(name);
 
 	cfg->expiration = extract_named_float(avpl, KEYWORD_GOGEXPIRE,matecfg->gog_expiration);
-	cfg->gop_as_subtree = extract_named_bool(avpl, KEYWORD_GOPTREE,matecfg->gop_as_subtree);
+	cfg->gop_as_subtree = extract_named_str(avpl, KEYWORD_GOPTREE,matecfg->gop_as_subtree);
 	
 	return TRUE;
 }
@@ -840,7 +893,7 @@ static gboolean config_gogkey(AVPL* avpl) {
 
 	if ( ! name || ! cfg ) {
 		if ( ! name )
-			report_error("MATE: GogKey: no Name in %s",avpl->name);
+			report_error("MATE: GogKey: no For in %s",avpl->name);
 		else
 			report_error("MATE: GogKey: no such Gop '%s' in %s",name,avpl->name);
 
@@ -852,6 +905,11 @@ static gboolean config_gogkey(AVPL* avpl) {
 		return FALSE;
 	}
 
+	if (! g_hash_table_lookup(matecfg->gopcfgs,on) ) {
+		report_error("MATE: GogKey: no such Gop %s in On",on);
+		return FALSE;
+	}
+	
 	rename_avpl(avpl,name);
 
 	gogkeys = (LoAL*) g_hash_table_lookup(matecfg->gogs_by_gopname,on);
@@ -884,13 +942,14 @@ static gboolean config_gogextra(AVPL* avpl) {
 	}
 
 	cfg->expiration = extract_named_float(avpl, KEYWORD_GOGEXPIRE,cfg->expiration);
-	cfg->gop_as_subtree = extract_named_bool(avpl, KEYWORD_GOPTREE,cfg->gop_as_subtree);
+	cfg->gop_as_subtree = extract_named_str(avpl, KEYWORD_GOPTREE,cfg->gop_as_subtree);
 
 	merge_avpl(cfg->extra,avpl,TRUE);
 
 	return TRUE;
 }
 
+#define true_false_str(v) ((v) ? "TRUE" : "FALSE")
 
 static void print_xxx_transforms(mate_cfg_item* cfg) {
 	guint8* tr_name;
@@ -914,7 +973,7 @@ static void print_xxx_transforms(mate_cfg_item* cfg) {
 
 	for (i=0; i < cfg->transforms->len; i++) {
 		tr_name = ((AVPL_Transf*) g_ptr_array_index(cfg->transforms,i))->name;
-		dbg_print (dbg_cfg,0,dbg_facility,"Action=%s; For=%s; Name=%s;\n",cfg_name,cfg->name,tr_name);
+		dbg_print (dbg_cfg,0,dbg_facility,"Action=%s; For=%s; Name=%s;",cfg_name,cfg->name,tr_name);
 	}
 
 }
@@ -925,19 +984,19 @@ static void print_gog_config(gpointer k _U_, gpointer v, gpointer p _U_) {
 	void* cookie = NULL;
 	AVPL* avpl;
 
-	dbg_print (dbg_cfg,0,dbg_facility,"Action=GogDef; Name=%s; Expiration=%f;\n",cfg->name,cfg->expiration);
+	dbg_print (dbg_cfg,0,dbg_facility,"Action=GogDef; Name=%s; Expiration=%f;",cfg->name,cfg->expiration);
 
 	if (cfg->keys) {
 		while (( avpl = get_next_avpl(cfg->keys,&cookie) )) {
 			avplstr = avpl_to_str(avpl);
-			dbg_print (dbg_cfg,0,dbg_facility,"Action=GogKey; For=%s; On=%s; %s\n",cfg->name,avpl->name,avplstr);
+			dbg_print (dbg_cfg,0,dbg_facility,"Action=GogKey; For=%s; On=%s; %s",cfg->name,avpl->name,avplstr);
 			g_free(avplstr);
 		}
 	}
 
 	if (cfg->extra) {
 		avplstr = avpl_to_str(cfg->extra);
-		dbg_print (dbg_cfg,0,dbg_facility,"Action=GogExtra; For=%s; %s\n",cfg->name,avplstr);
+		dbg_print (dbg_cfg,0,dbg_facility,"Action=GogExtra; For=%s; %s",cfg->name,avplstr);
 		g_free(avplstr);
 	}
 
@@ -956,7 +1015,10 @@ static void print_gop_config(gpointer k _U_ , gpointer v, gpointer p _U_) {
 	gopdef = g_string_new("Action=GopDef; ");
 
 	show_pdu_tree = cfg->show_pdu_tree ? "TRUE" : "FALSE";
-	g_string_sprintfa(gopdef,"Name=%s; ShowPduTree=%s;",cfg->name,show_pdu_tree);
+	g_string_sprintfa(gopdef,"Name=%s; ShowPduTree=%s; ShowGopTimes=%s; "
+					  "GopExpiration=%f; GopIdleTimeout=%f GopLifetime=%f;",
+					  cfg->name,show_pdu_tree,true_false_str(cfg->show_times),
+					  cfg->expiration,cfg->idle_timeout,cfg->lifetime);
 
 	if (cfg->key) {
 		avplstr = avpl_to_str(cfg->key);
@@ -964,24 +1026,24 @@ static void print_gop_config(gpointer k _U_ , gpointer v, gpointer p _U_) {
 		g_free(avplstr);
 	}
 
-	dbg_print (dbg_cfg,0,dbg_facility,"%s\n",gopdef->str);
+	dbg_print (dbg_cfg,0,dbg_facility,"%s",gopdef->str);
 
 
 	if (cfg->start) {
 		avplstr = avpl_to_str(cfg->start);
-		dbg_print (dbg_cfg,0,dbg_facility,"Action=GopStart; For=%s; %s\n",cfg->name,avplstr);
+		dbg_print (dbg_cfg,0,dbg_facility,"Action=GopStart; For=%s; %s",cfg->name,avplstr);
 		g_free(avplstr);
 	}
 
 	if (cfg->stop) {
 		avplstr = avpl_to_str(cfg->stop);
-		dbg_print (dbg_cfg,0,dbg_facility,"Action=GopStop; For=%s; %s\n",cfg->name,avplstr);
+		dbg_print (dbg_cfg,0,dbg_facility,"Action=GopStop; For=%s; %s",cfg->name,avplstr);
 		g_free(avplstr);
 	}
 
 	if (cfg->extra) {
 		avplstr = avpl_to_str(cfg->extra);
-		dbg_print (dbg_cfg,0,dbg_facility,"Action=GopExtra; For=%s;  %s\n",cfg->name,avplstr);
+		dbg_print (dbg_cfg,0,dbg_facility,"Action=GopExtra; For=%s;  %s",cfg->name,avplstr);
 		g_free(avplstr);
 	}
 
@@ -1042,7 +1104,7 @@ static void print_transforms(gpointer k, gpointer v, gpointer p _U_) {
 				break;
 		}
 
-		dbg_print (dbg,0,dbg_facility,"\tAction=Transform; Name=%s; Match=%s; Mode=%s; %s %s\n",(guint8*) k,match,mode,match_s,replace_s);
+		dbg_print (dbg,0,dbg_facility,"\tAction=Transform; Name=%s; Match=%s; Mode=%s; %s %s",(guint8*) k,match,mode,match_s,replace_s);
 
 		g_free(match_s);
 		g_free(replace_s);
@@ -1064,13 +1126,25 @@ static void print_pdu_config(mate_cfg_pdu* cfg) {
 	g_string_sprintfa(s, "Name=%s; Proto=%s; DiscartAttribs=%s; Stop=%s;  Transport=",
 					  cfg->name,my_protoname(cfg->hfid_proto),discard,stop);
 
-	for (i = 0; i < cfg->hfid_ranges->len; i++) {
-		hfid = *((int*) g_ptr_array_index(cfg->hfid_ranges,i));
+	for (i = 0; i < cfg->transport_ranges->len; i++) {
+		hfid = *((int*) g_ptr_array_index(cfg->transport_ranges,i));
 		g_string_sprintfa(s,"%s/",my_protoname(hfid));
 	}
 
 	*(s->str + s->len - 1) = ';';
 
+	if (cfg->payload_ranges) {
+		g_string_sprintfa(s, " Payload=");
+		
+		for (i = 0; i < cfg->payload_ranges->len; i++) {
+			hfid = *((int*) g_ptr_array_index(cfg->payload_ranges,i));
+			g_string_sprintfa(s,"%s/",my_protoname(hfid));
+		}
+		
+		*(s->str + s->len - 1) = ';';
+
+	}
+	
 	g_hash_table_foreach(cfg->hfids_attr,print_hfid_hash,s);
 
 	dbg_print(dbg_cfg,0,dbg_facility,"%s",s->str);
@@ -1114,7 +1188,7 @@ static void print_gogs_by_gopname(gpointer k, gpointer v, gpointer p _U_) {
 
 	while(( avpl = get_next_avpl((LoAL*)v,&cookie) )) {
 		str = avpl_to_str(avpl);
-		dbg_print(dbg_cfg,0,dbg_facility,"Gop=%s; Gog=%s; --> %s\n",(guint8*)k,avpl->name,str);
+		dbg_print(dbg_cfg,0,dbg_facility,"Gop=%s; Gog=%s; --> %s",(guint8*)k,avpl->name,str);
 		g_free(str);
 	}
 
@@ -1122,10 +1196,10 @@ static void print_gogs_by_gopname(gpointer k, gpointer v, gpointer p _U_) {
 
 static void print_gops_by_pduname(gpointer k, gpointer v, gpointer p _U_) {
 	dbg_print(dbg_cfg,0,dbg_facility,
-			  "PduName=%s; GopName=%s;\n", (guint8*)k,((mate_cfg_gop*)v)->name);
+			  "PduName=%s; GopName=%s;", (guint8*)k,((mate_cfg_gop*)v)->name);
 }
 
-static void print_config() {
+static void print_config(void) {
 	guint i;
 
 	/* FIXME: print the settings */
@@ -1158,14 +1232,17 @@ static void print_config() {
 static void new_attr_hfri(mate_cfg_item* cfg, guint8* name) {
 	int* p_id = g_malloc(sizeof(int));
 
-	hf_register_info hfri = {
-		p_id,
-	{
-		g_strdup_printf("%s",name),
-		g_strdup_printf("mate.%s.%s",cfg->name,name),
-		FT_STRING,BASE_NONE,NULL,0,
-		g_strdup_printf("%s attribute of %s",name,cfg->name),HFILL
-	}};
+	hf_register_info hfri;
+
+	memset(&hfri, 0, sizeof hfri);
+	hfri.p_id = p_id;
+	hfri.hfinfo.name = g_strdup_printf("%s",name);
+	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.%s",cfg->name,name);
+	hfri.hfinfo.type = FT_STRING;
+	hfri.hfinfo.display = BASE_NONE;
+	hfri.hfinfo.strings = NULL;
+	hfri.hfinfo.bitmask = 0;
+	hfri.hfinfo.blurb = g_strdup_printf("%s attribute of %s",name,cfg->name);
 
 	*p_id = -1;
 	g_hash_table_insert(cfg->my_hfids,name,p_id);
@@ -1305,13 +1382,15 @@ static void analyze_gop_config(gpointer k _U_, gpointer v, gpointer p _U_) {
 		}
 	}
 
-	cookie = NULL;
-	while(( avp = get_next_avp(cfg->start,&cookie) )) {
-		if (! g_hash_table_lookup(cfg->my_hfids,avp->n))  {
-			new_attr_hfri(cfg,avp->n);
+	if(cfg->start) {
+		cookie = NULL;
+		while(( avp = get_next_avp(cfg->start,&cookie) )) {
+			if (! g_hash_table_lookup(cfg->my_hfids,avp->n))  {
+				new_attr_hfri(cfg,avp->n);
+			}
 		}
 	}
-
+	
 	if (cfg->stop) {
 		cookie = NULL;
 		while(( avp = get_next_avp(cfg->stop,&cookie) )) {
@@ -1373,6 +1452,24 @@ static void analyze_gog_config(gpointer k _U_, gpointer v, gpointer p _U_) {
 	
 	g_array_append_val(matecfg->hfrs,hfri);
 	
+	hfri.p_id = &(cfg->hfid_gog_gopstart);
+	hfri.hfinfo.name = "GopStart frame";
+	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.GopStart",cfg->name);
+	hfri.hfinfo.type = FT_FRAMENUM;
+	hfri.hfinfo.display = BASE_DEC;
+	hfri.hfinfo.blurb = g_strdup("The start frame of a GOP");
+	
+	g_array_append_val(matecfg->hfrs,hfri);
+	
+	hfri.p_id = &(cfg->hfid_gog_gopstop);
+	hfri.hfinfo.name = "GopStop frame";
+	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.GopStop",cfg->name);
+	hfri.hfinfo.type = FT_FRAMENUM;
+	hfri.hfinfo.display = BASE_DEC;
+	hfri.hfinfo.blurb = g_strdup("The stop frame of a GOP");
+	
+	g_array_append_val(matecfg->hfrs,hfri);
+	
 	hfri.p_id = &(cfg->hfid_start_time);
 	hfri.hfinfo.name = g_strdup_printf("%s start time",cfg->name);
 	hfri.hfinfo.abbrev = g_strdup_printf("mate.%s.StartTime",cfg->name);
@@ -1434,9 +1531,12 @@ static void analyze_gog_config(gpointer k _U_, gpointer v, gpointer p _U_) {
 	ett = &cfg->ett_times;
 	g_array_append_val(matecfg->ett,ett);
 	
+	ett = &cfg->ett_gog_gop;	
+	g_array_append_val(matecfg->ett,ett);
+	
 }
 
-static void analyze_config() {
+static void analyze_config(void) {
 	guint i;
 
 	for (i=0; i<matecfg->pducfglist->len; i++) {
@@ -1453,7 +1553,7 @@ static void new_action(guint8* name, config_action* action) {
 
 }
 
-static void init_actions() {
+static void init_actions(void) {
 	AVP* avp;
 
 	all_keywords = new_avpl("all_keywords");
@@ -1537,6 +1637,14 @@ static void init_actions() {
 	matecfg->pdu_tree = avp->n;
 	insert_avp(all_keywords,avp);
 	
+	avp = new_avp(KEYWORD_BASICTREE,"",'=');
+	matecfg->basic_tree = avp->n;
+	insert_avp(all_keywords,avp);
+
+	avp = new_avp(KEYWORD_FULLTREE,"",'=');
+	matecfg->full_tree = avp->n;
+	insert_avp(all_keywords,avp);
+	
 	if (actions) {
 		g_hash_table_destroy(actions);
 	}
@@ -1566,7 +1674,7 @@ extern mate_config* mate_cfg() {
 	return matecfg;
 }
 
-extern mate_config* mate_make_config(guint8* filename) {
+extern mate_config* mate_make_config(guint8* filename, int mate_hfid) {
 	gint* ett;
 
 	avp_init();
@@ -1602,7 +1710,8 @@ extern mate_config* mate_make_config(guint8* filename) {
 	matecfg->hfrs = g_array_new(FALSE,TRUE,sizeof(hf_register_info));
 	matecfg->ett = g_array_new(FALSE,TRUE,sizeof(gint*));
 	matecfg->ett_root = -1;
-
+	matecfg->hfid_mate = mate_hfid;
+	
 	ett = &matecfg->ett_root;
 	g_array_append_val(matecfg->ett,ett);
 
@@ -1611,7 +1720,8 @@ extern mate_config* mate_make_config(guint8* filename) {
 	init_actions();
 
 	matecfg->show_pdu_tree = matecfg->frame_tree;
-
+	matecfg->gop_as_subtree = matecfg->basic_tree;
+	
 	config_error = g_string_new("");
 	
 	if ( mate_load_config(filename) ) {
