@@ -2,7 +2,7 @@
 * Routines for megaco packet disassembly
 * RFC 3015
 *
-* $Id: packet-megaco.c 13004 2005-01-12 06:00:31Z etxrab $
+* $Id: packet-megaco.c 14291 2005-05-03 18:19:56Z gerald $
 *
 * Christian Falckenberg, 2002/10/17
 * Copyright (c) 2002 by Christian Falckenberg
@@ -39,7 +39,6 @@
 #include "config.h"
 #endif
 
-#include "plugins/plugin_api.h"
 #include "moduleinfo.h"
 
 #include <stdio.h>
@@ -55,8 +54,6 @@
 #include "sctpppids.h"
 #include <epan/dissectors/packet-tpkt.h>
 
-#include "plugins/plugin_api_defs.h"
-
 #ifndef ENABLE_STATIC
 G_MODULE_EXPORT const gchar version[] = VERSION;
 #endif
@@ -601,8 +598,13 @@ nextcontext:
 		tvb_previous_offset = tvb_find_guint8(tvb, tvb_current_offset,
 			tvb_len, '=')+1;
 		tvb_previous_offset = tvb_skip_wsp(tvb, tvb_previous_offset);
-		tvb_current_offset = tvb_find_guint8(tvb, tvb_previous_offset,
+		tvb_next_offset = tvb_find_guint8(tvb, tvb_previous_offset,
 			tvb_len, '{');
+		if (tvb_current_offset >= tvb_next_offset) {
+			proto_tree_add_text(megaco_tree, tvb, 0, 0, "[ Parse error: Invalid offset ]");
+			return;
+		}
+		tvb_current_offset = tvb_next_offset;
 		
 		
 		tokenlen = tvb_current_offset - tvb_previous_offset;
@@ -860,6 +862,7 @@ nextcontext:
 									
 									default:
 										tokenlen =  (tvb_RBRKT+1) - tvb_previous_offset;
+										tvb_ensure_bytes_exist(tvb, tvb_previous_offset, tokenlen);
 										proto_tree_add_string(megaco_tree, hf_megaco_error_Frame, tvb,
 											tvb_previous_offset, tokenlen,
 											"No Command detectable !");
@@ -886,6 +889,10 @@ nextcontext:
 					switch ( tempchar ){
 						
 					case 'E':
+						if ((tokenlen+1 > (int) sizeof(TermID)) || (tokenlen+1 <= 0)) {
+							proto_tree_add_text(megaco_tree, tvb, 0, 0, "[ Parse error: Invalid TermID length (%d) ]", tokenlen+1);
+							return;
+						}
 						tvb_get_nstringz0(tvb,tvb_offset,tokenlen+1,TermID);
 						TermID[0] = 'e';
 						my_proto_tree_add_string(megaco_tree_command_line, hf_megaco_termid, tvb,
@@ -999,6 +1006,10 @@ dissect_megaco_descriptors(tvbuff_t *tvb, proto_tree *megaco_tree_command_line,
 			tvb_current_offset = tvb_descriptors_end_offset;
 			
 		}
+		if (tvb_current_offset <= tvb_previous_offset) {
+			proto_tree_add_text(megaco_tree_command_line, tvb, 0, 0, "[ Parse error: Invalid offset ]");
+			return;
+		}
 		
 		
 		
@@ -2738,24 +2749,19 @@ static gint tvb_skip_wsp_return(tvbuff_t* tvb, gint offset){
 #ifndef ENABLE_STATIC
 
 G_MODULE_EXPORT void
-plugin_reg_handoff(void){
-	proto_reg_handoff_megaco();
-}
-
-G_MODULE_EXPORT void
-plugin_init(plugin_address_table_t *pat
-#ifndef PLUGINS_NEED_ADDRESS_TABLE
-			_U_
-#endif
-			){
-	/* initialise the table of pointers needed in Win32 DLLs */
-	plugin_address_table_init(pat);
+plugin_register(void)
+{
 	/* register the new protocol, protocol fields, and subtrees */
 	if (proto_megaco == -1) { /* execute protocol initialization only once */
 		proto_register_megaco();
 	}
 }
 
+G_MODULE_EXPORT void
+plugin_reg_handoff(void){
+	proto_reg_handoff_megaco();
+}
+
 #endif
 
 /* End the functions we need for plugin stuff */
