@@ -2,7 +2,7 @@
  * Routines for mgcp packet disassembly
  * RFC 2705
  *
- * $Id: packet-mgcp.h 11869 2004-09-01 07:07:23Z guy $
+ * $Id: packet-mgcp.h 14195 2005-04-26 23:14:35Z lroland $
  *
  * Copyright (c) 2000 by Ed Warnicke <hagbard@physics.rutgers.edu>
  *
@@ -26,28 +26,39 @@
  */
  
  /* A simple MGCP type that is occasionally handy */
-typedef enum _mgcp_type {
-  MGCP_REQUEST,
-  MGCP_RESPONSE,
-  MGCP_OTHERS
+typedef enum _mgcp_type
+{
+	MGCP_REQUEST,
+	MGCP_RESPONSE,
+	MGCP_OTHERS
 } mgcp_type_t;
 
 /* Container for tapping relevant data */
-typedef struct _mgcp_info_t {
-  mgcp_type_t mgcp_type;
-  char code[5];
-  guint32 transid;
-  nstime_t req_time;
-  gboolean is_duplicate;
-  gboolean request_available;
+typedef struct _mgcp_info_t
+{
+	mgcp_type_t mgcp_type;
+	char code[5];
+	guint32 transid;
+	nstime_t req_time;
+	gboolean is_duplicate;
+	gboolean request_available;
+	guint32 req_num; /* frame number request seen */
+	gchar *endpointId;
+	gchar *observedEvents;
+	guint32 rspcode;
+	gchar *signalReq;
+	gboolean hasDigitMap;
 } mgcp_info_t;
 
 /* Item of request list */
-typedef struct _mgcp_call_t {
+typedef struct _mgcp_call_t
+{
 	guint32 transid;
 	char code[5];
-	guint32	req_num;	/* frame number request seen */
-	guint32	rsp_num;	/* frame number response seen */
+	guint32 req_num; /* frame number request seen */
+	guint32 rsp_num; /* frame number response seen */
+	guint32 rspcode;
 	nstime_t req_time;
 	gboolean responded;
-} mgcp_call_t;	
+} mgcp_call_t;
+
