@@ -1,7 +1,7 @@
 /* packet-rtnet.c
  * Routines for RTnet packet disassembly
  *
- * $Id: packet-rtnet.c 12874 2004-12-30 23:19:17Z guy $
+ * $Id: packet-rtnet.c 13644 2005-03-07 02:44:41Z guy $
  *
  * Copyright (c) 2003 by Erwin Rol <erwin@erwinrol.com>
  * Copyright (c) 2004 by Jan Kiszka <jan.kiszka@web.de>
@@ -31,8 +31,6 @@
 #include "config.h"
 #endif
 
-#include "plugins/plugin_api.h"
-
 #include "moduleinfo.h"
 
 #include <stdio.h>
@@ -46,8 +44,6 @@
 #include "etypes.h"
 #include <epan/strutil.h>
 
-#include "plugins/plugin_api_defs.h"
-
 /* Define version if we are not building ethereal statically */
 
 #ifndef ENABLE_STATIC
@@ -1406,20 +1402,8 @@ proto_reg_handoff_rtcfg(void) {
 #ifndef ENABLE_STATIC
 
 G_MODULE_EXPORT void
-plugin_reg_handoff(void){
-  proto_reg_handoff_rtmac();
-  proto_reg_handoff_rtcfg();
-  data_handle = find_dissector("data");
-}
-
-G_MODULE_EXPORT void
-plugin_init(plugin_address_table_t *pat
-#ifndef PLUGINS_NEED_ADDRESS_TABLE
-_U_
-#endif
-){
-  /* initialise the table of pointers needed in Win32 DLLs */
-  plugin_address_table_init(pat);
+plugin_register(void)
+{
   /* register the new protocol, protocol fields, and subtrees */
   if (proto_rtmac == -1) { /* execute protocol initialization only once */
     proto_register_rtmac();
@@ -1429,6 +1413,13 @@ _U_
   }
 }
 
+G_MODULE_EXPORT void
+plugin_reg_handoff(void){
+  proto_reg_handoff_rtmac();
+  proto_reg_handoff_rtcfg();
+  data_handle = find_dissector("data");
+}
+
 #endif
 
 /* End the functions we need for plugin stuff */
