@@ -1,7 +1,7 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c 12875 2004-12-30 23:42:02Z guy $
+ * $Id: print.c 13724 2005-03-11 20:56:31Z ulfl $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -24,10 +24,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-/* With MSVC and a libethereal.dll this file needs to import some variables 
-   in a special way. Therefore _NEED_VAR_IMPORT_ is defined. */  
-#define _NEED_VAR_IMPORT_
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -517,6 +513,37 @@ write_psml_finale(FILE *fh)
 	fputs("</psml>\n", fh);
 }
 
+void
+write_csv_preamble(FILE *fh _U_)
+{
+
+}
+
+void
+proto_tree_write_csv(epan_dissect_t *edt, FILE *fh)
+{
+        gint    i;
+
+        /* if this is the first packet, we have to write the CSV header */
+        if(edt->pi.fd->num == 1) {
+            for(i=0; i < edt->pi.cinfo->num_cols - 1; i++)
+	        fprintf(fh, "\"%s\", ", edt->pi.cinfo->col_title[i]);
+
+            fprintf(fh, "\"%s\"\n", edt->pi.cinfo->col_title[i]);
+        }
+
+        for(i=0; i < edt->pi.cinfo->num_cols - 1; i++)
+            fprintf(fh, "\"%s\", ", edt->pi.cinfo->col_data[i]);
+
+        fprintf(fh, "\"%s\"\n", edt->pi.cinfo->col_data[i]);
+}
+
+void
+write_csv_finale(FILE *fh _U_)
+{
+
+}
+
 /*
  * Find the data source for a specified field, and return a pointer
  * to the data in it. Returns NULL if the data is out of bounds.
