@@ -2,7 +2,7 @@
  * Definitions for alert box routines with toolkit-independent APIs but
  * toolkit-dependent implementations.
  *
- * $Id: simple_dialog.h,v 1.15 2004/06/04 21:12:01 guy Exp $
+ * $Id: simple_dialog.h 14100 2005-04-16 19:18:05Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -55,7 +55,11 @@ typedef enum {
 /** display a "No" button */
 #define ESD_BTN_NO     0x08 
 /** display a "Clear" button */
-#define ESD_BTN_CLEAR  0x10 
+#define ESD_BTN_CLEAR  0x10
+/** display a "Save" button */
+#define ESD_BTN_SAVE   0x20
+/** display a "Continue without Saving" button */
+#define ESD_BTN_DONT_SAVE 0x40 
 
 /** Standard button combination "Ok" + "Cancel". */
 #define ESD_BTNS_OK_CANCEL	(ESD_BTN_OK|ESD_BTN_CANCEL)
@@ -63,6 +67,8 @@ typedef enum {
 #define ESD_BTNS_YES_NO		(ESD_BTN_YES|ESD_BTN_NO)
 /** Standard button combination "Yes" + "No" + "Cancel". */
 #define ESD_BTNS_YES_NO_CANCEL	(ESD_BTN_YES|ESD_BTN_NO|ESD_BTN_CANCEL)
+/** Standard button combination "No" + "Cancel" + "Save". */
+#define ESD_BTNS_SAVE_DONTSAVE_CANCEL (ESD_BTN_DONT_SAVE|ESD_BTN_CANCEL|ESD_BTN_SAVE)
 
 #if __GNUC__ >= 2
 /** Create and show a simple dialog.
@@ -95,7 +101,7 @@ extern gpointer vsimple_dialog(ESD_TYPE_E type, gint btn_mask,
  * @param ... printf like parameters
  * @return the newly created dialog
  */
-extern gpointer simple_dialog(gint type, gint btn_mask,
+extern gpointer simple_dialog(ESD_TYPE_E type, gint btn_mask,
     const gchar *msg_format, ...);
 /** Create and show a simple dialog using a va_list.
  *
@@ -105,7 +111,7 @@ extern gpointer simple_dialog(gint type, gint btn_mask,
  * @param ap parameters
  * @return the newly created dialog
  */
-extern gpointer vsimple_dialog(gint type, gint btn_mask,
+extern gpointer vsimple_dialog(ESD_TYPE_E type, gint btn_mask,
     const gchar *msg_format, va_list ap);
 #endif
 
@@ -120,6 +126,20 @@ typedef void (* simple_dialog_cb_t) (gpointer dialog, gint btn, gpointer data);
  */
 extern void simple_dialog_set_cb(gpointer dialog, simple_dialog_cb_t callback_fct, gpointer data);
 
+/** Add a check button to the dialog (e.g. "Don't show this message again")
+ * 
+ * @param dialog the dialog from simple_dialog()
+ * @param text the text to display 
+ */
+extern void simple_dialog_check_set(gpointer dialog, gchar *text);
+
+/** Get the check buttons state.
+ * 
+ * @param dialog the dialog from simple_dialog()
+ * @return current button state (TRUE is checked)
+ */
+extern gboolean simple_dialog_check_get(gpointer dialog);
+
 /** Surround the primary dialog message text by 
  *  simple_dialog_primary_start() and simple_dialog_primary_end().
  *  To highlight the first sentence (will take effect on GTK2 only).
