@@ -1,7 +1,7 @@
 /* tap-smbstat.c
  * smbstat   2003 Ronnie Sahlberg
  *
- * $Id: tap-smbstat.c 12910 2005-01-01 02:57:02Z guy $
+ * $Id: tap-smbstat.c 14107 2005-04-16 21:54:32Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -22,10 +22,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-/* With MSVC and a libethereal.dll this file needs to import some variables 
-   in a special way. Therefore _NEED_VAR_IMPORT_ is defined. */  
-#define _NEED_VAR_IMPORT_
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -71,14 +67,14 @@ smbstat_packet(void *pss, packet_info *pinfo, epan_dissect_t *edt _U_, const voi
 		return 0;
 	}
 
-	if(si->cmd==0xA0){
+	if(si->cmd==0xA0 && si->sip->extra_info_type == SMB_EI_NTI){
 		smb_nt_transact_info_t *sti=(smb_nt_transact_info_t *)si->sip->extra_info;
 
 		/*nt transaction*/
 		if(sti){
 			sp=&(ss->nt_trans[sti->subcmd]);
 		}
-	} else if(si->cmd==0x32){
+	} else if(si->cmd==0x32 && si->sip->extra_info_type == SMB_EI_T2I){
 		smb_transact2_info_t *st2i=(smb_transact2_info_t *)si->sip->extra_info;
 
 		/*transaction2*/
@@ -283,6 +279,6 @@ smbstat_init(char *optarg)
 void
 register_tap_listener_smbstat(void)
 {
-	register_ethereal_tap("smb,rtt", smbstat_init);
+	register_tap_listener_cmd_arg("smb,rtt", smbstat_init);
 }
 
