@@ -1,10 +1,13 @@
 /* nettl.c
  *
- * $Id: nettl.c 12351 2004-10-19 22:28:27Z guy $
+ * $Id: nettl.c 14025 2005-04-07 10:04:50Z sahlberg $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
+ * Enhancements by Mark C. Brown <mbrown@hp.com>
+ * Copyright (C) 2003, 2005 Hewlett-Packard Development Company, L.P.
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
@@ -69,15 +72,17 @@ struct nettlrec_sx25l2_hdr {
 /* This also works for BASE100 and GSC100BT */
 /* see /usr/include/sys/netdiag1.h for hints */
 struct nettlrec_ns_ls_ip_hdr {
-    guint32	kind;
+    guint32	devid;
     guint8	xxa[4];
-    guint8	rectype;
-    guint8	xxb[19];
+    guint32	kind;
+    guint8	xxb[16];
     guint32	caplen;
     guint32	length;
     guint32	sec;
     guint32	usec;
-    guint8	xxc[16];
+    guint32	pid;
+    guint8	xxc[10];
+    guint16	uid;
 };
 
 /* Full record header for writing out a nettl file */
@@ -164,7 +169,8 @@ static gboolean nettl_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 int nettl_open(wtap *wth, int *err, gchar **err_info _U_)
 {
     char magic[12], os_vers[2];
-    guint8 dummy[4];
+    guint16 dummy[2];
+    int subsys;
     int bytes_read;
 
     /* Read in the string that should be at the start of a HP file */
@@ -221,27 +227,28 @@ int nettl_open(wtap *wth, int *err, gchar **err_info _U_)
         return 0;
     }
 
-    switch (dummy[3]) {
+    subsys = g_ntohs(dummy[1]);
+    switch (subsys) {
         case NETTL_SUBSYS_HPPB_FDDI :
         case NETTL_SUBSYS_EISA_FDDI :
         case NETTL_SUBSYS_PCI_FDDI :
         case NETTL_SUBSYS_HSC_FDDI :
-		wth->file_encap = WTAP_ENCAP_FDDI_BITSWAPPED;
+		wth->file_encap = WTAP_ENCAP_NETTL_FDDI;
 		break;
         case NETTL_SUBSYS_TOKEN :
         case NETTL_SUBSYS_PCI_TR :
-		wth->file_encap = WTAP_ENCAP_TOKEN_RING;
+		wth->file_encap = WTAP_ENCAP_NETTL_TOKEN_RING;
 		break;
         case NETTL_SUBSYS_NS_LS_IP :
         case NETTL_SUBSYS_NS_LS_LOOPBACK :
         case NETTL_SUBSYS_NS_LS_TCP :
         case NETTL_SUBSYS_NS_LS_UDP :
         case NETTL_SUBSYS_NS_LS_IPV6 :
-		wth->file_encap = WTAP_ENCAP_RAW_IP;
+		wth->file_encap = WTAP_ENCAP_NETTL_RAW_IP;
 		break;
 	default:
-		/* if assumption is bad, the read will catch it */
-		wth->file_encap = WTAP_ENCAP_ETHERNET;
+		/* If this assumption is bad, the read will catch it */
+		wth->file_encap = WTAP_ENCAP_NETTL_ETHERNET;
     }
 
     if (file_seek(wth->fh, 0x80, SEEK_SET, err) == -1)
@@ -335,9 +342,9 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
     struct nettlrec_ns_ls_drv_eth_hdr drv_eth_hdr;
     guint16 length;
     int offset = 0;
-    int encap;
+    int subsys;
     int padlen;
-    guint8 dummy[4];
+    guint16 dummy[2];
     guchar dummyc[12];
 
     errno = WTAP_ERR_CANT_READ;
@@ -353,9 +360,9 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 	return 0;
     }
     offset += 4;
-    encap=dummy[3];
 
-    switch (encap) {
+    subsys = g_ntohs(dummy[1]);
+    switch (subsys) {
 	case NETTL_SUBSYS_LAN100 :
 	case NETTL_SUBSYS_EISA100BT :
 	case NETTL_SUBSYS_BASE100 :
@@ -368,6 +375,7 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 	case NETTL_SUBSYS_INTL100 :
 	case NETTL_SUBSYS_IGELAN :
 	case NETTL_SUBSYS_IETHER :
+	case NETTL_SUBSYS_IXGBE :
 	case NETTL_SUBSYS_HPPB_FDDI :
 	case NETTL_SUBSYS_EISA_FDDI :
         case NETTL_SUBSYS_PCI_FDDI :
@@ -383,28 +391,26 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 	case NETTL_SUBSYS_NS_LS_IPV6 :
 	case NETTL_SUBSYS_NS_LS_ICMPV6 :
 	case NETTL_SUBSYS_NS_LS_ICMP :
-	    if( (encap == NETTL_SUBSYS_NS_LS_IP)
-	     || (encap == NETTL_SUBSYS_NS_LS_LOOPBACK)
-	     || (encap == NETTL_SUBSYS_NS_LS_UDP)
-	     || (encap == NETTL_SUBSYS_NS_LS_TCP)
-	     || (encap == NETTL_SUBSYS_NS_LS_IPV6)) {
-		phdr->pkt_encap = WTAP_ENCAP_RAW_IP;
-	    } else if (encap == NETTL_SUBSYS_NS_LS_ICMP) {
-		phdr->pkt_encap = WTAP_ENCAP_RAW_ICMP;
-	    } else if (encap == NETTL_SUBSYS_NS_LS_ICMPV6) {
-		phdr->pkt_encap = WTAP_ENCAP_RAW_ICMPV6;
-	    } else if( (encap == NETTL_SUBSYS_HPPB_FDDI)
-		    || (encap == NETTL_SUBSYS_EISA_FDDI)
-		    || (encap == NETTL_SUBSYS_PCI_FDDI)
-		    || (encap == NETTL_SUBSYS_HSC_FDDI) ) {
-		phdr->pkt_encap = WTAP_ENCAP_FDDI_BITSWAPPED;
-	    } else if( (encap == NETTL_SUBSYS_PCI_TR)
-		    || (encap == NETTL_SUBSYS_TOKEN) ) {
-		phdr->pkt_encap = WTAP_ENCAP_TOKEN_RING;
+	    if( (subsys == NETTL_SUBSYS_NS_LS_IP)
+	     || (subsys == NETTL_SUBSYS_NS_LS_LOOPBACK)
+	     || (subsys == NETTL_SUBSYS_NS_LS_UDP)
+	     || (subsys == NETTL_SUBSYS_NS_LS_TCP)
+	     || (subsys == NETTL_SUBSYS_NS_LS_IPV6)) {
+		phdr->pkt_encap = WTAP_ENCAP_NETTL_RAW_IP;
+	    } else if (subsys == NETTL_SUBSYS_NS_LS_ICMP) {
+		phdr->pkt_encap = WTAP_ENCAP_NETTL_RAW_ICMP;
+	    } else if (subsys == NETTL_SUBSYS_NS_LS_ICMPV6) {
+		phdr->pkt_encap = WTAP_ENCAP_NETTL_RAW_ICMPV6;
+	    } else if( (subsys == NETTL_SUBSYS_HPPB_FDDI)
+		    || (subsys == NETTL_SUBSYS_EISA_FDDI)
+		    || (subsys == NETTL_SUBSYS_PCI_FDDI)
+		    || (subsys == NETTL_SUBSYS_HSC_FDDI) ) {
+		phdr->pkt_encap = WTAP_ENCAP_NETTL_FDDI;
+	    } else if( (subsys == NETTL_SUBSYS_PCI_TR)
+		    || (subsys == NETTL_SUBSYS_TOKEN) ) {
+		phdr->pkt_encap = WTAP_ENCAP_NETTL_TOKEN_RING;
 	    } else {
-		phdr->pkt_encap = WTAP_ENCAP_ETHERNET;
-		/* We assume there's no FCS in this frame. */
-		pseudo_header->eth.fcs_len = 0;
+		phdr->pkt_encap = WTAP_ENCAP_NETTL_ETHERNET;
 	    }
 
 	    bytes_read = file_read(&ip_hdr, 1, sizeof ip_hdr, fh);
@@ -438,8 +444,8 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 	    }
 
 	    /* HPPB FDDI has different inbound vs outbound trace records */
-	    if (encap == NETTL_SUBSYS_HPPB_FDDI) {
-                if (ip_hdr.rectype == NETTL_HDR_PDUIN) {
+	    if (subsys == NETTL_SUBSYS_HPPB_FDDI) {
+                if (pntohl(&ip_hdr.kind) == NETTL_HDR_PDUIN) {
                    /* inbound is very strange...
                       there are an extra 3 bytes after the DSAP and SSAP
                       for SNAP frames ???
@@ -449,8 +455,7 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 		   if (length <= 0)
 		       return 0;
 		   phdr->len = length;
-		   length = pntohl(&ip_hdr.caplen);
-		   phdr->caplen = length;
+		   phdr->caplen = pntohl(&ip_hdr.caplen);
                 } else {
 	           /* outbound appears to have variable padding */
 		   bytes_read = file_read(dummyc, 1, 9, fh);
@@ -486,9 +491,9 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 		   length = pntohl(&ip_hdr.caplen);
 		   phdr->caplen = length - padlen;
                }
-	    } else if ( (encap == NETTL_SUBSYS_PCI_FDDI)
-	             || (encap == NETTL_SUBSYS_EISA_FDDI)
-	             || (encap == NETTL_SUBSYS_HSC_FDDI) ) {
+	    } else if ( (subsys == NETTL_SUBSYS_PCI_FDDI)
+	             || (subsys == NETTL_SUBSYS_EISA_FDDI)
+	             || (subsys == NETTL_SUBSYS_HSC_FDDI) ) {
 	        /* other flavor FDDI cards have an extra 3 bytes of padding */
 		bytes_read = file_read(dummy, 1, 3, fh);
 		if (bytes_read != 3) {
@@ -508,7 +513,7 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 		phdr->len = length - 3;
 		length = pntohl(&ip_hdr.caplen);
 		phdr->caplen = length - 3;
-	    } else if (encap == NETTL_SUBSYS_NS_LS_LOOPBACK) {
+	    } else if (subsys == NETTL_SUBSYS_NS_LS_LOOPBACK) {
 	        /* LOOPBACK has an extra 26 bytes of padding */
 		bytes_read = file_read(dummy, 1, 26, fh);
 		if (bytes_read != 26) {
@@ -533,8 +538,7 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 		if (length <= 0)
 		    return 0;
 		phdr->len = length;
-		length = pntohl(&ip_hdr.caplen);
-		phdr->caplen = length;
+		phdr->caplen = pntohl(&ip_hdr.caplen);
 	    }
 
 	    phdr->ts.tv_sec = pntohl(&ip_hdr.sec);
@@ -574,9 +578,7 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 	    /* XXX we dont know how to identify this as ethernet frames, so
 	       we assumes everything is. We will crash and burn for anything else */
 	    /* for encapsulated 100baseT we do this */
-	    phdr->pkt_encap = WTAP_ENCAP_ETHERNET;
-	    /* We assume there's no FCS in this frame. */
-	    pseudo_header->eth.fcs_len = 0;
+	    phdr->pkt_encap = WTAP_ENCAP_NETTL_ETHERNET;
 	    bytes_read = file_read(&drv_eth_hdr, 1, sizeof drv_eth_hdr, fh);
 	    if (bytes_read != sizeof drv_eth_hdr) {
 		*err = file_error(fh);
@@ -593,8 +595,7 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 	    length = pntohs(&drv_eth_hdr.length); 
 	    if (length <= 0) return 0;
 	    phdr->len = length;
-	    length = pntohs(&drv_eth_hdr.caplen);
-	    phdr->caplen = length;
+	    phdr->caplen = pntohs(&drv_eth_hdr.caplen);
 
 	    phdr->ts.tv_sec = pntohl(&ip_hdr.sec);
 	    phdr->ts.tv_usec = pntohl(&ip_hdr.usec);
@@ -640,11 +641,48 @@ nettl_read_rec_header(wtap *wth, FILE_T fh, struct wtap_pkthdr *phdr,
 		(lapb_hdr.from_dce & 0x20 ? FROM_DCE : 0x00);
 	    break;
 	default:
-	    *err = WTAP_ERR_UNSUPPORTED_ENCAP;
-	    *err_info = g_strdup_printf("nettl: subsystem %u unknown or unsupported",
-		    encap);
-	    return -1;
+	    wth->file_encap = WTAP_ENCAP_PER_PACKET;
+	    phdr->pkt_encap = WTAP_ENCAP_NETTL_UNKNOWN;
+            bytes_read = file_read(&ip_hdr, 1, sizeof ip_hdr, fh);
+            if (bytes_read != sizeof ip_hdr) {
+                *err = file_error(fh);
+                if (*err != 0)
+                    return -1;
+                if (bytes_read != 0) {
+                    *err = WTAP_ERR_SHORT_READ;
+                    return -1;
+                }
+                return 0;
+            }
+            offset += sizeof ip_hdr;
+            length = pntohl(&ip_hdr.length);
+            if (length <= 0) return 0;
+            phdr->len = length;
+            phdr->caplen = pntohl(&ip_hdr.caplen);
+            phdr->ts.tv_sec = pntohl(&ip_hdr.sec);
+            phdr->ts.tv_usec = pntohl(&ip_hdr.usec);
+            if (wth->capture.nettl->is_hpux_11) {
+                bytes_read = file_read(dummy, 1, 4, fh);
+                if (bytes_read != 4) {
+                    *err = file_error(fh);
+                    if (*err != 0)
+                        return -1;
+                    if (bytes_read != 0) {
+                        *err = WTAP_ERR_SHORT_READ;
+                        return -1;
+                    }
+                    return 0;
+                }
+                offset += 4;
+            }
     }
+
+    pseudo_header->nettl.subsys   = subsys;
+    pseudo_header->nettl.devid    = pntohl(&ip_hdr.devid);
+    pseudo_header->nettl.kind     = pntohl(&ip_hdr.kind);
+    pseudo_header->nettl.pid      = pntohl(&ip_hdr.pid);
+    pseudo_header->nettl.uid      = pntohs(&ip_hdr.uid);
+
     return offset;
 }
 
@@ -705,11 +743,15 @@ int nettl_dump_can_write_encap(int encap)
 		case WTAP_ENCAP_ETHERNET:
 		case WTAP_ENCAP_FDDI_BITSWAPPED:
 		case WTAP_ENCAP_TOKEN_RING:
-		case WTAP_ENCAP_RAW_IP:
-		case WTAP_ENCAP_RAW_ICMP:
-		case WTAP_ENCAP_RAW_ICMPV6:
+		case WTAP_ENCAP_NETTL_ETHERNET:
+		case WTAP_ENCAP_NETTL_FDDI:
+		case WTAP_ENCAP_NETTL_TOKEN_RING:
+		case WTAP_ENCAP_NETTL_RAW_IP:
+		case WTAP_ENCAP_NETTL_RAW_ICMP:
+		case WTAP_ENCAP_NETTL_RAW_ICMPV6:
 		case WTAP_ENCAP_PER_PACKET:
 		case WTAP_ENCAP_UNKNOWN:
+		case WTAP_ENCAP_NETTL_UNKNOWN:
 			return 0;
 		default:
 			return WTAP_ERR_UNSUPPORTED_ENCAP;
@@ -764,14 +806,35 @@ static gboolean nettl_dump(wtap_dumper *wdh,
 
 	memset(&rec_hdr,0,sizeof(rec_hdr));
 	rec_hdr.hdr_len = g_htons(sizeof(rec_hdr));
-	rec_hdr.hdr.rectype = NETTL_HDR_PDUIN;
+	rec_hdr.hdr.kind = g_htonl(NETTL_HDR_PDUIN);
 	rec_hdr.hdr.sec = g_htonl(phdr->ts.tv_sec);
 	rec_hdr.hdr.usec = g_htonl(phdr->ts.tv_usec);
 	rec_hdr.hdr.caplen = g_htonl(phdr->caplen);
 	rec_hdr.hdr.length = g_htonl(phdr->len);
+	rec_hdr.hdr.devid = -1;
+	rec_hdr.hdr.pid = -1;
+	rec_hdr.hdr.uid = -1;
 
 	switch (phdr->pkt_encap) {
 
+		case WTAP_ENCAP_NETTL_FDDI:
+			/* account for pad bytes */
+			rec_hdr.hdr.caplen = g_htonl(phdr->caplen + 3);
+			rec_hdr.hdr.length = g_htonl(phdr->len + 3);
+                        /* fall through and fill the rest of the fields */
+		case WTAP_ENCAP_NETTL_ETHERNET:
+		case WTAP_ENCAP_NETTL_TOKEN_RING:
+		case WTAP_ENCAP_NETTL_RAW_IP:
+		case WTAP_ENCAP_NETTL_RAW_ICMP:
+		case WTAP_ENCAP_NETTL_RAW_ICMPV6:
+		case WTAP_ENCAP_NETTL_UNKNOWN:
+			rec_hdr.subsys = g_htons(pseudo_header->nettl.subsys);
+			rec_hdr.hdr.devid = g_htonl(pseudo_header->nettl.devid);
+			rec_hdr.hdr.kind = g_htonl(pseudo_header->nettl.kind);
+			rec_hdr.hdr.pid = g_htonl(pseudo_header->nettl.pid);
+			rec_hdr.hdr.uid = g_htons(pseudo_header->nettl.uid);
+			break;
+
 		case WTAP_ENCAP_RAW_IP:
 			rec_hdr.subsys = g_htons(NETTL_SUBSYS_NS_LS_IP);
 			break;
@@ -791,14 +854,6 @@ static gboolean nettl_dump(wtap_dumper *wdh,
 			rec_hdr.subsys = g_htons(NETTL_SUBSYS_PCI_TR);
 			break;
 	
-		case WTAP_ENCAP_RAW_ICMP:
-			rec_hdr.subsys = g_htons(NETTL_SUBSYS_NS_LS_ICMP);
-			break;
-	
-		case WTAP_ENCAP_RAW_ICMPV6:
-			rec_hdr.subsys = g_htons(NETTL_SUBSYS_NS_LS_ICMPV6);
-			break;
-	
 		default:
 			/* found one we don't support */
 			*err = WTAP_ERR_UNSUPPORTED_ENCAP;
@@ -815,7 +870,8 @@ static gboolean nettl_dump(wtap_dumper *wdh,
 	}
 	wdh->bytes_dumped += sizeof(rec_hdr);
 
-	if (phdr->pkt_encap == WTAP_ENCAP_FDDI_BITSWAPPED) {
+	if ((phdr->pkt_encap == WTAP_ENCAP_FDDI_BITSWAPPED) ||
+	    (phdr->pkt_encap == WTAP_ENCAP_NETTL_FDDI)) {
 		/* add those weird 3 bytes of padding */
 		nwritten = fwrite(&dummy, 1, 3, wdh->fh);
 		if (nwritten != 3) {
