@@ -1,6 +1,6 @@
 /* wtap.c
  *
- * $Id: wtap.c 12678 2004-12-07 17:54:08Z guy $
+ * $Id: wtap.c 14265 2005-05-02 14:07:33Z tuexen $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -252,14 +252,41 @@ static const struct encap_type_info {
 	/* WTAP_ENCAP_BACNET_MS_TP */
 	{ "BACnet MS/TP", "bacnet-ms-tp" },
 
-	/* WTAP_ENCAP_RAW_ICMP */
-	{ "Raw ICMP", "raw-icmp" },
+	/* WTAP_ENCAP_NETTL_RAW_ICMP */
+	{ "Raw ICMP with nettl headers", "raw-icmp-nettl" },
 
-	/* WTAP_ENCAP_RAW_ICMPV6 */
-	{ "Raw ICMPv6", "raw-icmpv6" },
+	/* WTAP_ENCAP_NETTL_RAW_ICMPV6 */
+	{ "Raw ICMPv6 with nettl headers", "raw-icmpv6-nettl" },
 
 	/* WTAP_ENCAP_GPRS_LLC */
 	{ "GPRS LLC", "gprs-llc" },
+
+	/* WTAP_ENCAP_JUNIPER_ATM1 */
+	{ "Juniper ATM1", "juniper-atm1" },
+
+	/* WTAP_ENCAP_JUNIPER_ATM2 */
+	{ "Juniper ATM2", "juniper-atm2" },
+
+	/* WTAP_ENCAP_REDBACK */
+	{ "Redback SmartEdge", "redback" },
+
+	/* WTAP_ENCAP_NETTL_RAW_IP */
+	{ "Raw IP with nettl headers", "rawip-nettl" },
+
+	/* WTAP_ENCAP_NETTL_ETHERNET */
+	{ "Ethernet with nettl headers", "ether-nettl" },
+
+	/* WTAP_ENCAP_NETTL_TOKEN_RING */
+	{ "Token Ring with nettl headers", "tr-nettl" },
+
+	/* WTAP_ENCAP_NETTL_FDDI */
+	{ "FDDI with nettl headers", "fddi-nettl" },
+
+	/* WTAP_ENCAP_NETTL_UNKNOWN */
+	{ "Unknown link-layer type with nettl headers", "unknown-nettl" },
+
+	/* WTAP_ENCAP_MTP2_WITH_PHDR */
+	{ "MTP2 with pseudoheader", "mtp2-with-phdr" },
 };
 
 /* Name that should be somewhat descriptive. */
@@ -314,6 +341,7 @@ static const char *wtap_errlist[] = {
 	"Uncompression error: data oddly truncated",
 	"Uncompression error: data would overflow buffer",
 	"Uncompression error: bad LZ77 offset",
+	"The standard input cannot be opened for random access",
 };
 #define	WTAP_ERRLIST_SIZE	(sizeof wtap_errlist / sizeof wtap_errlist[0])
 
@@ -436,29 +464,6 @@ wtap_buf_ptr(wtap *wth)
 	return buffer_start_ptr(wth->frame_buffer);
 }
 
-gboolean
-wtap_loop(wtap *wth, int count, wtap_handler callback, guchar* user, int *err,
-    gchar **err_info)
-{
-	long		data_offset;
-	int		loop = 0;
-
-	/* Start by clearing error flag */
-	*err = 0;
-
-	while ( (wtap_read(wth, err, err_info, &data_offset)) ) {
-		callback(user, &wth->phdr, data_offset,
-		    &wth->pseudo_header, buffer_start_ptr(wth->frame_buffer));
-		if (count > 0 && ++loop >= count)
-			break;
-	}
-
-	if (*err == 0)
-		return TRUE;	/* success */
-	else
-		return FALSE;	/* failure */
-}
-
 gboolean
 wtap_seek_read(wtap *wth, long seek_off,
 	union wtap_pseudo_header *pseudo_header, guint8 *pd, int len,
