@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h 12678 2004-12-07 17:54:08Z guy $
+ * $Id: wtap.h 14265 2005-05-02 14:07:33Z tuexen $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -153,12 +153,20 @@
 #define WTAP_ENCAP_SYMANTEC			61
 #define WTAP_ENCAP_APPLE_IP_OVER_IEEE1394	62
 #define WTAP_ENCAP_BACNET_MS_TP			63
-#define WTAP_ENCAP_RAW_ICMP			64
-#define WTAP_ENCAP_RAW_ICMPV6			65
+#define WTAP_ENCAP_NETTL_RAW_ICMP		64
+#define WTAP_ENCAP_NETTL_RAW_ICMPV6		65
 #define WTAP_ENCAP_GPRS_LLC			67
-
+#define WTAP_ENCAP_JUNIPER_ATM1			68
+#define WTAP_ENCAP_JUNIPER_ATM2			69
+#define WTAP_ENCAP_REDBACK			70
+#define WTAP_ENCAP_NETTL_RAW_IP			71
+#define WTAP_ENCAP_NETTL_ETHERNET		72
+#define WTAP_ENCAP_NETTL_TOKEN_RING		73
+#define WTAP_ENCAP_NETTL_FDDI			74
+#define WTAP_ENCAP_NETTL_UNKNOWN		75
+#define WTAP_ENCAP_MTP2_WITH_PHDR               76
 /* last WTAP_ENCAP_ value + 1 */
-#define WTAP_NUM_ENCAP_TYPES			67
+#define WTAP_NUM_ENCAP_TYPES			77
 
 /* File types that can be read by wiretap.
    We support writing some many of these file types, too, so we
@@ -339,6 +347,9 @@ struct atm_phdr {
 #define ASCEND_PFX_WDS_X 1
 #define ASCEND_PFX_WDS_R 2
 #define ASCEND_PFX_WDD   3
+#define ASCEND_PFX_ISDN_X 4
+#define ASCEND_PFX_ISDN_R 5
+#define ASCEND_PFX_ETHER 6
 
 struct ascend_phdr {
 	guint16	type;			/* ASCEND_PFX_*, as defined above */
@@ -416,6 +427,24 @@ struct irda_phdr {
 	guint16 pkttype;    /* packet type */
 };
 
+/* Packet "pseudo-header" for nettl (HP-UX) capture files. */
+
+struct nettl_phdr {
+	guint16 subsys;
+	guint32 devid;
+	guint32 kind;
+	gint32  pid;
+	gint16  uid;
+};
+
+/* Packet "pseudo-header" for MTP2 files. */
+
+struct mtp2_phdr {
+	guint8  sent;         
+	guint8  annex_a_used;   
+	guint16 link_number;  
+};
+
 union wtap_pseudo_header {
 	struct eth_phdr		eth;
 	struct x25_phdr		x25;
@@ -426,6 +455,8 @@ union wtap_pseudo_header {
 	struct ieee_802_11_phdr	ieee_802_11;
 	struct cosine_phdr	cosine;
 	struct irda_phdr	irda;
+	struct nettl_phdr	nettl;
+	struct mtp2_phdr        mtp2;
 };
 
 struct wtap_pkthdr {
@@ -435,9 +466,6 @@ struct wtap_pkthdr {
 	int pkt_encap;
 };
 
-typedef void (*wtap_handler)(guchar*, const struct wtap_pkthdr*,
-		long, union wtap_pseudo_header *pseudo_header, const guchar *);
-
 struct wtap;
 struct Buffer;
 struct wtap_dumper;
@@ -456,11 +484,6 @@ typedef struct wtap_dumper wtap_dumper;
 struct wtap* wtap_open_offline(const char *filename, int *err,
     gchar **err_info, gboolean do_random);
 
-/* Returns TRUE if entire loop-reading was successful. If read failure
- * happened, FALSE is returned and err is set. */
-gboolean wtap_loop(wtap *wth, int, wtap_handler, guchar*, int *err,
-    gchar **err_info);
-
 /* Returns TRUE if read was successful. FALSE if failure. data_offset is
  * set the the offset in the file where the data for the read packet is
  * located. */
@@ -543,6 +566,8 @@ void wtap_set_bytes_dumped(wtap_dumper *wdh, long bytes_dumped);
 	/* Uncompressing Sniffer data would overflow buffer */
 #define	WTAP_ERR_UNC_BAD_OFFSET			-17
 	/* LZ77 compressed data has bad offset to string */
+#define	WTAP_ERR_RANDOM_OPEN_STDIN		-18
+	/* We're trying to open the standard input for random access */
 
 /* Errors from zlib; zlib error Z_xxx turns into Wiretap error
    WTAP_ERR_ZLIB + Z_xxx.
